@@ -17,7 +17,7 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 #include <string.h>
-#include <assert.h>
+// #include <assert.h>
 #include <limits.h>
 
 #include "xmalloc.h"
@@ -26,6 +26,7 @@
 #include "tre-stack.h"
 #include "tre-parse.h"
 
+#define assert(a) R_assert(a)
 
 /* Characters with special meanings in regexp syntax. */
 #define CHAR_PIPE	   L'|'
@@ -185,11 +186,6 @@ int tre_isascii_func(tre_cint_t c) { return !(c >> 7); }
 #endif /* !tre_isascii */
 
 
-/* R addition:  this platform has the function but not the declaration */
-#ifdef WIN64
-int iswblank(wint_t wc);
-#endif
-
 #ifdef tre_isblank
 int tre_isblank_func(tre_cint_t c) { return tre_isblank(c); }
 #else /* !tre_isblank */
