@@ -11,6 +11,25 @@
  * be used for encoding validation purpose.
  */
 
+/* Primary source is apparently
+   http://code.google.com/p/win-iconv/source/checkout
+
+   Original R version from http://www.gtk.org/download-windows.html
+
+   R changes:
+
+   - add some missing encoding names, remove duplicate names.
+   - add iconvlist()
+   - set errno on error
+   - XP-compatibility for WC_NO_BEST_FIT_CHARS -- use only for ASCII
+
+ */
+
+/* for WC_NO_BEST_FIT_CHARS */
+#ifndef WINVER
+# define WINVER 0x0500
+#endif
+
 #include <windows.h>
 #include <errno.h>
 #include <string.h>
@@ -21,38 +40,20 @@
 
 #define MB_CHAR_MAX 16
 
-#define UNICODE_MODE_BOM_DONE 1
-#define UNICODE_MODE_SWAPPED 2
+#define UNICODE_MODE_BOM_DONE   1
+#define UNICODE_MODE_SWAPPED    2
 
-#define UNICODE_FLAG_USE_BOM_ENDIAN 1
-
-#define return_error(code)  \
-    do {                    \
-	errno = code;       \
-	return -1;          \
-    } while (0)
-
-#define xstrlcpy(dst, src, size)    \
-    do {                            \
-	strncpy(dst, src, size);    \
-	dst[size - 1] = 0;          \
-    } while (0)
-
-#define xstrlcpyn(dst, src, srclen, size) \
-    xstrlcpy(dst, src, xmin((srclen) + 1, size))
-
-#define xmin(a, b) ((a) < (b) ? (a) : (b))
-#define xmax(a, b) ((a) > (b) ? (a) : (b))
-
-#define STATIC_STRLEN(arr) (sizeof(arr) - 1)
+#define FLAG_USE_BOM            1
+#define FLAG_TRANSLIT           2 /* //TRANSLIT */
+#define FLAG_IGNORE             4 /* //IGNORE (not implemented) */
 
 typedef unsigned char uchar;
 typedef unsigned short ushort;
 typedef unsigned int uint;
 
-//typedef void* iconv_t;
-
 /*
+typedef void* iconv_t;
+
 iconv_t iconv_open(const char *tocode, const char *fromcode);
 int iconv_close(iconv_t cd);
 size_t iconv(iconv_t cd, const char **inbuf, size_t *inbytesleft, char **outbuf, size_t *outbytesleft);
@@ -107,15 +108,14 @@ static int win_iconv_close(iconv_t cd);
 static size_t win_iconv(iconv_t cd, const char **inbuf, size_t *inbytesleft, char **outbuf, size_t *outbytesleft);
 
 static int load_mlang();
-static csconv_t make_csconv(const char *name);
+static int make_csconv(const char *name, csconv_t *cv);
 static int name_to_codepage(const char *name);
 static uint utf16_to_ucs4(const ushort *wbuf);
 static void ucs4_to_utf16(uint wc, ushort *wbuf, int *wbufsize);
-static int is_unicode(int codepage);
 static int mbtowc_flags(int codepage);
 static int must_use_null_useddefaultchar(int codepage);
-static void check_utf_bom(rec_iconv_t *cd, ushort *wbuf, int *wbufsize);
 static char *strrstr(const char *str, const char *token);
+static int seterror(int err);
 
 
 static int sbcs_mblen(csconv_t *cv, const uchar *buf, int bufsize);
@@ -178,7 +178,9 @@ static struct {
     {1200, "UTF-16"},
     {1200, "UCS-2"},
     {12000, "UTF32"},
+    {12000, "UTF-32"},
 #endif
+    /* R additions */
     {12000, "UCS-4LE"},
     {12001, "UCS-4BE"},
 
@@ -277,7 +279,7 @@ static struct {
     {51932, "CP51932"},
     {51932, "MS51932"},
     {51932, "WINDOWS-51932"},
-    // {51932, "EUC-JP"},
+    // {51932, "EUC-JP"}, /* duplicate */
 
     {932, "CP932"},
     {932, "MS932"},
@@ -306,6 +308,7 @@ static struct {
 
     {950, "CP950"},
     {950, "BIG5"},
+    /* R additions */
     {950, "BIG-5"},
     {950, "BIG-FIVE"},
 
@@ -464,7 +467,7 @@ static struct {
     {20106, "x-IA5-German"}, /* IA5 German (7-bit) */
     {20107, "x-IA5-Swedish"}, /* IA5 Swedish (7-bit) */
     {20108, "x-IA5-Norwegian"}, /* IA5 Norwegian (7-bit) */
-    // {20127, "us-ascii"}, /* US-ASCII (7-bit) */
+    // {20127, "us-ascii"}, /* US-ASCII (7-bit), duplicate */
     {20261, "x-cp20261"}, /* T.61 */
     {20269, "x-cp20269"}, /* ISO 6937 Non-Spacing Accent */
     {20273, "IBM273"}, /* IBM EBCDIC Germany */
@@ -495,6 +498,7 @@ static struct {
     {28591, "iso8859-1"},
     {28592, "iso-8859-2"}, /* ISO 8859-2 Central European; Central European (ISO) */
     {28592, "iso8859-2"},
+    /* latin2 etc are R additions */
     {28592, "latin2"},
     {28593, "iso-8859-3"}, /* ISO 8859-3 Latin 3 */
     {28593, "iso8859-3"},
@@ -667,19 +671,19 @@ load_mlang()
     HMODULE h;
     if (ConvertINetString != NULL)
 	return TRUE;
-    h = LoadLibrary("mlang.dll");
+    h = LoadLibrary(TEXT("mlang.dll"));
     if (!h)
 	return FALSE;
-    ConvertINetString = (CONVERTINETSTRING)GetProcAddress(h, "ConvertINetString");
-    ConvertINetMultiByteToUnicode = (CONVERTINETMULTIBYTETOUNICODE)GetProcAddress(h, "ConvertINetMultiByteToUnicode");
-    ConvertINetUnicodeToMultiByte = (CONVERTINETUNICODETOMULTIBYTE)GetProcAddress(h, "ConvertINetUnicodeToMultiByte");
-    IsConvertINetStringAvailable = (ISCONVERTINETSTRINGAVAILABLE)GetProcAddress(h, "IsConvertINetStringAvailable");
-    LcidToRfc1766A = (LCIDTORFC1766A)GetProcAddress(h, "LcidToRfc1766A");
-    Rfc1766ToLcidA = (RFC1766TOLCIDA)GetProcAddress(h, "Rfc1766ToLcidA");
+    ConvertINetString = (CONVERTINETSTRING)GetProcAddress(h, TEXT("ConvertINetString"));
+    ConvertINetMultiByteToUnicode = (CONVERTINETMULTIBYTETOUNICODE)GetProcAddress(h, TEXT("ConvertINetMultiByteToUnicode"));
+    ConvertINetUnicodeToMultiByte = (CONVERTINETUNICODETOMULTIBYTE)GetProcAddress(h, TEXT("ConvertINetUnicodeToMultiByte"));
+    IsConvertINetStringAvailable = (ISCONVERTINETSTRINGAVAILABLE)GetProcAddress(h, TEXT("IsConvertINetStringAvailable"));
+    LcidToRfc1766A = (LCIDTORFC1766A)GetProcAddress(h, TEXT("LcidToRfc1766A"));
+    Rfc1766ToLcidA = (RFC1766TOLCIDA)GetProcAddress(h, TEXT("Rfc1766ToLcidA"));
     return TRUE;
 }
 
-iconv_t LIBICONV_DLL_EXPORTED
+iconv_t
 iconv_open(const char *tocode, const char *fromcode)
 {
     rec_iconv_t *cd;
@@ -691,6 +695,9 @@ iconv_open(const char *tocode, const char *fromcode)
 	return (iconv_t)(-1);
     }
 
+    /* reset the errno to prevent reporting wrong error code.
+     * 0 for unsorted error. */
+    errno = 0;
     if (win_iconv_open(cd, tocode, fromcode))
 	return (iconv_t)cd;
 
@@ -699,7 +706,7 @@ iconv_open(const char *tocode, const char *fromcode)
     return (iconv_t)(-1);
 }
 
-int LIBICONV_DLL_EXPORTED
+int
 iconv_close(iconv_t _cd)
 {
     rec_iconv_t *cd = (rec_iconv_t *)_cd;
@@ -710,7 +717,7 @@ iconv_close(iconv_t _cd)
     return r;
 }
 
-size_t LIBICONV_DLL_EXPORTED
+size_t
 iconv(iconv_t _cd, const char **inbuf, size_t *inbytesleft,
       char **outbuf, size_t *outbytesleft)
 {
@@ -723,9 +730,7 @@ iconv(iconv_t _cd, const char **inbuf, size_t *inbytesleft,
 static int
 win_iconv_open(rec_iconv_t *cd, const char *tocode, const char *fromcode)
 {
-    cd->from = make_csconv(fromcode);
-    cd->to = make_csconv(tocode);
-    if (cd->from.codepage == -1 || cd->to.codepage == -1)
+    if (!make_csconv(fromcode, &cd->from) || !make_csconv(tocode, &cd->to))
 	return FALSE;
     cd->iconv_close = win_iconv_close;
     cd->iconv = win_iconv;
@@ -749,7 +754,8 @@ win_iconv(iconv_t _cd, const char **inbuf, size_t *inbytesleft,
     int insize;
     int outsize;
     int wsize;
-    DWORD mode;
+    DWORD frommode;
+    DWORD tomode;
     uint wc;
     compat_t *cp;
     int i;
@@ -758,14 +764,16 @@ win_iconv(iconv_t _cd, const char **inbuf, size_t *inbytesleft,
     {
 	if (outbuf != NULL && *outbuf != NULL && cd->to.flush != NULL)
 	{
+	    tomode = cd->to.mode;
 	    outsize = cd->to.flush(&cd->to, (uchar *)*outbuf, *outbytesleft);
 	    if (outsize == -1)
+	    {
+		cd->to.mode = tomode;
 		return (size_t)(-1);
+	    }
 	    *outbuf += outsize;
 	    *outbytesleft -= outsize;
 	}
-	if (is_unicode(cd->from.codepage) && (cd->from.mode & UNICODE_MODE_SWAPPED))
-	    cd->from.codepage ^= 1;
 	cd->from.mode = 0;
 	cd->to.mode = 0;
 	return 0;
@@ -773,17 +781,15 @@ win_iconv(iconv_t _cd, const char **inbuf, size_t *inbytesleft,
 
     while (*inbytesleft != 0)
     {
-	mode = cd->from.mode;
+	frommode = cd->from.mode;
+	tomode = cd->to.mode;
 	wsize = MB_CHAR_MAX;
 
 	insize = cd->from.mbtowc(&cd->from, (const uchar *)*inbuf, *inbytesleft, wbuf, &wsize);
 	if (insize == -1)
-	    return (size_t)(-1);
-
-	if (is_unicode(cd->from.codepage) && !(cd->from.mode & UNICODE_MODE_BOM_DONE))
 	{
-	    check_utf_bom(cd, wbuf, &wsize);
-	    cd->from.mode |= UNICODE_MODE_BOM_DONE;
+	    cd->from.mode = frommode;
+	    return (size_t)(-1);
 	}
 
 	if (wsize == 0)
@@ -825,7 +831,8 @@ win_iconv(iconv_t _cd, const char **inbuf, size_t *inbytesleft,
 				*outbytesleft);
 	if (outsize == -1)
 	{
-	    cd->from.mode = mode;
+	    cd->from.mode = frommode;
+	    cd->to.mode = tomode;
 	    return (size_t)(-1);
 	}
 
@@ -838,94 +845,104 @@ win_iconv(iconv_t _cd, const char **inbuf, size_t *inbytesleft,
     return 0;
 }
 
-static csconv_t
-make_csconv(const char *_name)
+static int
+make_csconv(const char *_name, csconv_t *cv)
 {
-    CPINFOEX cpinfoex;
-    csconv_t cv = {0, 0}; /* -Wall */
+    CPINFO cpinfo;
     int use_compat = TRUE;
-    char name[128];
+    int flag = 0;
+    char *name;
     char *p;
 
-    xstrlcpy(name, _name, sizeof(name));
+    name = strdup(_name);
+    if (name == NULL)
+	return FALSE;
 
     /* check for option "enc_name//opt1//opt2" */
     while ((p = strrstr(name, "//")) != NULL)
     {
 	if (_stricmp(p + 2, "nocompat") == 0)
 	    use_compat = FALSE;
+	else if (_stricmp(p + 2, "translit") == 0)
+	    flag |= FLAG_TRANSLIT;
+	else if (_stricmp(p + 2, "ignore") == 0)
+	    flag |= FLAG_IGNORE;
 	*p = 0;
     }
 
-    cv.mode = 0;
-    cv.flags = 0;
-    cv.mblen = NULL;
-    cv.flush = NULL;
-    cv.compat = NULL;
-    cv.codepage = name_to_codepage(name);
-    if (cv.codepage == 1200 || cv.codepage == 1201)
+    cv->mode = 0;
+    cv->flags = flag;
+    cv->mblen = NULL;
+    cv->flush = NULL;
+    cv->compat = NULL;
+    cv->codepage = name_to_codepage(name);
+    if (cv->codepage == 1200 || cv->codepage == 1201)
     {
-	cv.mbtowc = utf16_mbtowc;
-	cv.wctomb = utf16_wctomb;
-	if (_stricmp(name, "UTF-16") == 0 ||
-	    _stricmp(name, "UTF16") == 0 ||
-	    _stricmp(name, "UCS-2") == 0)
-	    cv.flags |= UNICODE_FLAG_USE_BOM_ENDIAN;
+	cv->mbtowc = utf16_mbtowc;
+	cv->wctomb = utf16_wctomb;
+	if (_stricmp(name, "UTF-16") == 0 || _stricmp(name, "UTF16") == 0)
+	    cv->flags |= FLAG_USE_BOM;
     }
-    else if (cv.codepage == 12000 || cv.codepage == 12001)
+    else if (cv->codepage == 12000 || cv->codepage == 12001)
     {
-	cv.mbtowc = utf32_mbtowc;
-	cv.wctomb = utf32_wctomb;
+	cv->mbtowc = utf32_mbtowc;
+	cv->wctomb = utf32_wctomb;
 	if (_stricmp(name, "UTF-32") == 0 || _stricmp(name, "UTF32") == 0)
-	    cv.flags |= UNICODE_FLAG_USE_BOM_ENDIAN;
+	    cv->flags |= FLAG_USE_BOM;
     }
-    else if (cv.codepage == 65001)
+    else if (cv->codepage == 65001)
     {
-	cv.mbtowc = kernel_mbtowc;
-	cv.wctomb = kernel_wctomb;
-	cv.mblen = utf8_mblen;
+	cv->mbtowc = kernel_mbtowc;
+	cv->wctomb = kernel_wctomb;
+	cv->mblen = utf8_mblen;
     }
-    else if ((cv.codepage == 50220 || cv.codepage == 50221
-	      || cv.codepage == 50222) && load_mlang())
+    else if ((cv->codepage == 50220 || cv->codepage == 50221 || 
+	      cv->codepage == 50222) && load_mlang())
     {
-	cv.mbtowc = iso2022jp_mbtowc;
-	cv.wctomb = iso2022jp_wctomb;
-	cv.flush = iso2022jp_flush;
+	cv->mbtowc = iso2022jp_mbtowc;
+	cv->wctomb = iso2022jp_wctomb;
+	cv->flush = iso2022jp_flush;
     }
-    else if (cv.codepage == 51932 && load_mlang())
+    else if (cv->codepage == 51932 && load_mlang())
     {
-	cv.mbtowc = mlang_mbtowc;
-	cv.wctomb = mlang_wctomb;
-	cv.mblen = eucjp_mblen;
+	cv->mbtowc = mlang_mbtowc;
+	cv->wctomb = mlang_wctomb;
+	cv->mblen = eucjp_mblen;
     }
-    else if (IsValidCodePage(cv.codepage)
-	     && GetCPInfoEx(cv.codepage, 0, &cpinfoex) != 0)
+    else if (IsValidCodePage(cv->codepage)
+	     && GetCPInfo(cv->codepage, &cpinfo) != 0)
     {
-	cv.mbtowc = kernel_mbtowc;
-	cv.wctomb = kernel_wctomb;
-	if (cpinfoex.MaxCharSize == 1)
-	    cv.mblen = sbcs_mblen;
-	else if (cpinfoex.MaxCharSize == 2)
-	    cv.mblen = dbcs_mblen;
+	cv->mbtowc = kernel_mbtowc;
+	cv->wctomb = kernel_wctomb;
+	if (cpinfo.MaxCharSize == 1)
+	    cv->mblen = sbcs_mblen;
+	else if (cpinfo.MaxCharSize == 2)
+	    cv->mblen = dbcs_mblen;
 	else
-	    cv.mblen = mbcs_mblen;
+	    cv->mblen = mbcs_mblen;
     }
     else
     {
 	/* not supported */
-	cv.codepage = -1;
+	free(name);
+	errno = EINVAL;
+	return FALSE;
     }
+
     if (use_compat)
     {
-	switch (cv.codepage)
+	switch (cv->codepage)
 	{
-	case 932: cv.compat = cp932_compat; break;
-	case 20932: cv.compat = cp20932_compat; break;
-	case 51932: cv.compat = cp51932_compat; break;
-	case 50220: case 50221: case 50222: cv.compat = cp5022x_compat; break;
+	case 932: cv->compat = cp932_compat; break;
+	case 20932: cv->compat = cp20932_compat; break;
+	case 51932: cv->compat = cp51932_compat; break;
+	case 50220: case 50221: case 50222: cv->compat = cp5022x_compat; break;
 	}
     }
-    return cv;
+
+    free(name);
+
+    return TRUE;
 }
 
 static int
@@ -980,14 +997,6 @@ ucs4_to_utf16(uint wc, ushort *wbuf, int *wbufsize)
     }
 }
 
-static int
-is_unicode(int codepage)
-{
-    return (codepage == 1200 || codepage == 1201 ||
-	    codepage == 12000 || codepage == 12001 ||
-	    codepage == 65000 || codepage == 65001);
-}
-
 /*
  * Check if codepage is one of those for which the dwFlags parameter
  * to MultiByteToWideChar() must be zero. Return zero or
@@ -1029,26 +1038,11 @@ must_use_null_useddefaultchar(int codepage)
 	    codepage == 42);
 }
 
-static void
-check_utf_bom(rec_iconv_t *cd, ushort *wbuf, int *wbufsize)
+static int
+seterror(int err)
 {
-    /* If we have a BOM, trust it, despite what the caller said */
-    if (wbuf[0] == 0xFFFE && (cd->from.flags & UNICODE_FLAG_USE_BOM_ENDIAN))
-    {
-	/* swap endian: 1200 <-> 1201 or 12000 <-> 12001 */
-	cd->from.codepage ^= 1;
-	cd->from.mode |= UNICODE_MODE_SWAPPED;
-	wbuf[0] = 0xFEFF;
-    }
-
-    /*
-     * Remove BOM.
-     * Don't do this if "to" is Unicode,
-     * except if "to" is UTF-8.
-     */
-    if (wbuf[0] == 0xFEFF &&
-	(!is_unicode(cd->to.codepage) || cd->to.codepage == 65001))
-	*wbufsize = 0;
+    errno = err;
+    return -1;
 }
 
 static char *
@@ -1075,7 +1069,7 @@ dbcs_mblen(csconv_t *cv, const uchar *buf, int bufsize)
 {
     int len = IsDBCSLeadByteEx(cv->codepage, buf[0]) ? 2 : 1;
     if (bufsize < len)
-	return_error(EINVAL);
+	return seterror(EINVAL);
     return len;
 }
 
@@ -1094,11 +1088,11 @@ mbcs_mblen(csconv_t *cv, const uchar *buf, int bufsize)
 		 bufsize >= 4 &&
 		 buf[1] >= 0x30 && buf[1] <= 0x39) len = 4;
 	else
-	    return_error(EINVAL);
+	    return seterror(EINVAL);
 	return len;
     }
     else
-	return_error(EINVAL);
+	return seterror(EINVAL);
 }
 
 static int
@@ -1114,9 +1108,9 @@ utf8_mblen(csconv_t *cv, const uchar *buf, int bufsize)
     else if ((buf[0] & 0xFE) == 0xFC) len = 6;
 
     if (len == 0)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     else if (bufsize < len)
-	return_error(EINVAL);
+	return seterror(EINVAL);
     return len;
 }
 
@@ -1128,27 +1122,27 @@ eucjp_mblen(csconv_t *cv, const uchar *buf, int bufsize)
     else if (buf[0] == 0x8E) /* JIS X 0201 */
     {
 	if (bufsize < 2)
-	    return_error(EINVAL);
+	    return seterror(EINVAL);
 	else if (!(0xA1 <= buf[1] && buf[1] <= 0xDF))
-	    return_error(EILSEQ);
+	    return seterror(EILSEQ);
 	return 2;
     }
     else if (buf[0] == 0x8F) /* JIS X 0212 */
     {
 	if (bufsize < 3)
-	    return_error(EINVAL);
+	    return seterror(EINVAL);
 	else if (!(0xA1 <= buf[1] && buf[1] <= 0xFE)
 		|| !(0xA1 <= buf[2] && buf[2] <= 0xFE))
-	    return_error(EILSEQ);
+	    return seterror(EILSEQ);
 	return 3;
     }
     else /* JIS X 0208 */
     {
 	if (bufsize < 2)
-	    return_error(EINVAL);
+	    return seterror(EINVAL);
 	else if (!(0xA1 <= buf[0] && buf[0] <= 0xFE)
 		|| !(0xA1 <= buf[1] && buf[1] <= 0xFE))
-	    return_error(EILSEQ);
+	    return seterror(EILSEQ);
 	return 2;
     }
 }
@@ -1164,7 +1158,7 @@ kernel_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int *wb
     *wbufsize = MultiByteToWideChar(cv->codepage, mbtowc_flags (cv->codepage),
 	    (const char *)buf, len, (wchar_t *)wbuf, *wbufsize);
     if (*wbufsize == 0)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     return len;
 }
 
@@ -1172,23 +1166,37 @@ static int
 kernel_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
 {
     BOOL usedDefaultChar = 0;
+    BOOL *p = NULL;
+    int flags = 0;
     int len;
 
     if (bufsize == 0)
-	return_error(E2BIG);
-    len = WideCharToMultiByte(cv->codepage, 0,
-	    (const wchar_t *)wbuf, wbufsize, (char *)buf, bufsize, NULL,
-	    must_use_null_useddefaultchar(cv->codepage) ? NULL : &usedDefaultChar);
+	return seterror(E2BIG);
+    if (!must_use_null_useddefaultchar(cv->codepage))
+    {
+	p = &usedDefaultChar;
+#ifdef WC_NO_BEST_FIT_CHARS
+	/* http://msdn.microsoft.com/en-us/library/dd374130%28VS.85%29.aspx
+	   says this cannot be used for 65001 and 54936, but it also
+	   says 'for Vista only', and 65001 fails on XP.
+	   We definitely want this for ASCII, which is 20127.
+	 */
+    if ( !(cv->flags & FLAG_TRANSLIT) && (cv->codepage == 20127) )
+	flags |= WC_NO_BEST_FIT_CHARS;
+#endif
+    }
+    len = WideCharToMultiByte(cv->codepage, flags,
+	    (const wchar_t *)wbuf, wbufsize, (char *)buf, bufsize, NULL, p);
     if (len == 0)
     {
 	if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
-	    return_error(E2BIG);
-	return_error(EILSEQ);
+	    return seterror(E2BIG);
+	return seterror(EILSEQ);
     }
     else if (usedDefaultChar)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     else if (cv->mblen(cv, buf, len) != len) /* validate result */
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     return len;
 }
 
@@ -1214,7 +1222,7 @@ mlang_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int *wbu
     hr = ConvertINetMultiByteToUnicode(&cv->mode, cv->codepage,
 	    (const char *)buf, &insize, (wchar_t *)wbuf, wbufsize);
     if (hr != S_OK || insize != len)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     return len;
 }
 
@@ -1229,11 +1237,11 @@ mlang_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
     hr = ConvertINetUnicodeToMultiByte(&cv->mode, cv->codepage,
 	    (const wchar_t *)wbuf, &wbufsize, tmpbuf, &tmpsize);
     if (hr != S_OK || insize != wbufsize)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     else if (bufsize < tmpsize)
-	return_error(E2BIG);
+	return seterror(E2BIG);
     else if (cv->mblen(cv, (uchar *)tmpbuf, tmpsize) != tmpsize)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     memcpy(buf, tmpbuf, tmpsize);
     return tmpsize;
 }
@@ -1241,24 +1249,47 @@ mlang_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
 static int
 utf16_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int *wbufsize)
 {
+    int codepage = cv->codepage;
+
+    /* swap endian: 1200 <-> 1201 */
+    if (cv->mode & UNICODE_MODE_SWAPPED)
+	codepage ^= 1;
+
     if (bufsize < 2)
-	return_error(EINVAL);
-    if (cv->codepage == 1200) /* little endian */
+	return seterror(EINVAL);
+    if (codepage == 1200) /* little endian */
 	wbuf[0] = (buf[1] << 8) | buf[0];
-    else if (cv->codepage == 1201) /* big endian */
+    else if (codepage == 1201) /* big endian */
 	wbuf[0] = (buf[0] << 8) | buf[1];
+
+    if ((cv->flags & FLAG_USE_BOM) && !(cv->mode & UNICODE_MODE_BOM_DONE))
+    {
+	cv->mode |= UNICODE_MODE_BOM_DONE;
+	if (wbuf[0] == 0xFFFE)
+	{
+	    cv->mode |= UNICODE_MODE_SWAPPED;
+	    *wbufsize = 0;
+	    return 2;
+	}
+	else if (wbuf[0] == 0xFEFF)
+	{
+	    *wbufsize = 0;
+	    return 2;
+	}
+    }
+
     if (0xDC00 <= wbuf[0] && wbuf[0] <= 0xDFFF)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     if (0xD800 <= wbuf[0] && wbuf[0] <= 0xDBFF)
     {
 	if (bufsize < 4)
-	    return_error(EINVAL);
-	if (cv->codepage == 1200) /* little endian */
+	    return seterror(EINVAL);
+	if (codepage == 1200) /* little endian */
 	    wbuf[1] = (buf[3] << 8) | buf[2];
-	else if (cv->codepage == 1201) /* big endian */
+	else if (codepage == 1201) /* big endian */
 	    wbuf[1] = (buf[2] << 8) | buf[3];
 	if (!(0xDC00 <= wbuf[1] && wbuf[1] <= 0xDFFF))
-	    return_error(EILSEQ);
+	    return seterror(EILSEQ);
 	*wbufsize = 2;
 	return 4;
     }
@@ -1269,8 +1300,26 @@ utf16_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int *wbu
 static int
 utf16_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
 {
+    if ((cv->flags & FLAG_USE_BOM) && !(cv->mode & UNICODE_MODE_BOM_DONE))
+    {
+	int r;
+
+	cv->mode |= UNICODE_MODE_BOM_DONE;
+	if (bufsize < 2)
+	    return seterror(E2BIG);
+	if (cv->codepage == 1200) /* little endian */
+	    memcpy(buf, "\xFF\xFE", 2);
+	else if (cv->codepage == 1201) /* big endian */
+	    memcpy(buf, "\xFE\xFF", 2);
+
+	r = utf16_wctomb(cv, wbuf, wbufsize, buf + 2, bufsize - 2);
+	if (r == -1)
+	    return -1;
+	return r + 2;
+    }
+
     if (bufsize < 2)
-	return_error(E2BIG);
+	return seterror(E2BIG);
     if (cv->codepage == 1200) /* little endian */
     {
 	buf[0] = (wbuf[0] & 0x00FF);
@@ -1284,7 +1333,7 @@ utf16_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
     if (0xD800 <= wbuf[0] && wbuf[0] <= 0xDBFF)
     {
 	if (bufsize < 4)
-	    return_error(E2BIG);
+	    return seterror(E2BIG);
 	if (cv->codepage == 1200) /* little endian */
 	{
 	    buf[2] = (wbuf[1] & 0x00FF);
@@ -1301,19 +1350,41 @@ utf16_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
 }
 
 static int
-utf32_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, 
+utf32_mbtowc(csconv_t *cv, const uchar *buf, int bufsize,
 	     ushort *wbuf, int *wbufsize)
 {
-    uint wc = 0;
+    int codepage = cv->codepage;
+    uint wc = 0 /* -Wall */;
+
+    /* swap endian: 12000 <-> 12001 */
+    if (cv->mode & UNICODE_MODE_SWAPPED)
+	codepage ^= 1;
 
     if (bufsize < 4)
-	return_error(EINVAL);
-    if (cv->codepage == 12000) /* little endian */
+	return seterror(EINVAL);
+    if (codepage == 12000) /* little endian */
 	wc = (buf[3] << 24) | (buf[2] << 16) | (buf[1] << 8) | buf[0];
-    else if (cv->codepage == 12001) /* big endian */
+    else if (codepage == 12001) /* big endian */
 	wc = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
+
+    if ((cv->flags & FLAG_USE_BOM) && !(cv->mode & UNICODE_MODE_BOM_DONE))
+    {
+	cv->mode |= UNICODE_MODE_BOM_DONE;
+	if (wc == 0xFFFE0000)
+	{
+	    cv->mode |= UNICODE_MODE_SWAPPED;
+	    *wbufsize = 0;
+	    return 4;
+	}
+	else if (wc == 0x0000FEFF)
+	{
+	    *wbufsize = 0;
+	    return 4;
+	}
+    }
+
     if ((0xD800 <= wc && wc <= 0xDFFF) || 0x10FFFF < wc)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     ucs4_to_utf16(wc, wbuf, wbufsize);
     return 4;
 }
@@ -1323,8 +1394,26 @@ utf32_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
 {
     uint wc;
 
+    if ((cv->flags & FLAG_USE_BOM) && !(cv->mode & UNICODE_MODE_BOM_DONE))
+    {
+	int r;
+
+	cv->mode |= UNICODE_MODE_BOM_DONE;
+	if (bufsize < 4)
+	    return seterror(E2BIG);
+	if (cv->codepage == 12000) /* little endian */
+	    memcpy(buf, "\xFF\xFE\x00\x00", 4);
+	else if (cv->codepage == 12001) /* big endian */
+	    memcpy(buf, "\x00\x00\xFE\xFF", 4);
+
+	r = utf32_wctomb(cv, wbuf, wbufsize, buf + 4, bufsize - 4);
+	if (r == -1)
+	    return -1;
+	return r + 4;
+    }
+
     if (bufsize < 4)
-	return_error(E2BIG);
+	return seterror(E2BIG);
     wc = utf16_to_ucs4(wbuf);
     if (cv->codepage == 12000) /* little endian */
     {
@@ -1415,7 +1504,7 @@ iso2022jp_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int
 	    if (bufsize < esc_len)
 	    {
 		if (strncmp((char *)buf, iesc[i].esc, bufsize) == 0)
-		    return_error(EINVAL);
+		    return seterror(EINVAL);
 	    }
 	    else
 	    {
@@ -1428,7 +1517,7 @@ iso2022jp_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int
 	    }
 	}
 	/* not supported escape sequence */
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     }
     else if (buf[0] == iso2022_SO_seq[0])
     {
@@ -1455,10 +1544,10 @@ iso2022jp_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int
 
     len = iesc[cs].len;
     if (bufsize < len)
-	return_error(EINVAL);
+	return seterror(EINVAL);
     for (i = 0; i < len; ++i)
 	if (!(buf[i] < 0x80))
-	    return_error(EILSEQ);
+	    return seterror(EILSEQ);
     esc_len = iesc[cs].esc_len;
     memcpy(tmp, iesc[cs].esc, esc_len);
     if (shift == ISO2022_SO)
@@ -1482,13 +1571,13 @@ iso2022jp_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int
     hr = ConvertINetMultiByteToUnicode(&dummy, cv->codepage,
 	    (const char *)tmp, &insize, (wchar_t *)wbuf, wbufsize);
     if (hr != S_OK || insize != len + esc_len)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
 
     /* Check for conversion error.  Assuming defaultChar is 0x3F. */
     /* ascii should be converted from ascii */
     if (wbuf[0] == buf[0]
 	    && cv->mode != ISO2022_MODE(ISO2022JP_CS_ASCII, ISO2022_SI))
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
 
     /* reset the mode for informal sequence */
     if (cv->mode != ISO2022_MODE(cs, shift))
@@ -1521,9 +1610,9 @@ iso2022jp_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsi
     hr = ConvertINetUnicodeToMultiByte(&dummy, cv->codepage,
 	    (const wchar_t *)wbuf, &wbufsize, tmp, &tmpsize);
     if (hr != S_OK || insize != wbufsize)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     else if (bufsize < tmpsize)
-	return_error(E2BIG);
+	return seterror(E2BIG);
 
     if (tmpsize == 1)
     {
@@ -1543,7 +1632,7 @@ iso2022jp_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsi
 	}
 	if (iesc[i].esc == NULL)
 	    /* not supported escape sequence */
-	    return_error(EILSEQ);
+	    return seterror(EILSEQ);
     }
 
     shift = ISO2022_SI;
@@ -1558,9 +1647,9 @@ iso2022jp_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsi
     /* Check for converting error.  Assuming defaultChar is 0x3F. */
     /* ascii should be converted from ascii */
     if (cs == ISO2022JP_CS_ASCII && !(wbuf[0] < 0x80))
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
     else if (tmpsize < esc_len + len)
-	return_error(EILSEQ);
+	return seterror(EILSEQ);
 
     if (cv->mode == ISO2022_MODE(cs, shift))
     {
@@ -1587,7 +1676,7 @@ iso2022jp_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsi
     }
 
     if (bufsize < len + esc_len)
-	return_error(E2BIG);
+	return seterror(E2BIG);
     memcpy(buf, tmp, len + esc_len);
     cv->mode = ISO2022_MODE(cs, shift);
     return len + esc_len;
@@ -1607,7 +1696,7 @@ iso2022jp_flush(csconv_t *cv, uchar *buf, int bufsize)
 	if (ISO2022_MODE_CS(cv->mode) != ISO2022JP_CS_ASCII)
 	    esc_len += iesc[ISO2022JP_CS_ASCII].esc_len;
 	if (bufsize < esc_len)
-	    return_error(E2BIG);
+	    return seterror(E2BIG);
 
 	esc_len = 0;
 	if (ISO2022_MODE_SHIFT(cv->mode) != ISO2022_SI)
@@ -1626,7 +1715,6 @@ iso2022jp_flush(csconv_t *cv, uchar *buf, int bufsize)
     return 0;
 }
 
-LIBICONV_DLL_EXPORTED
 void iconvlist (int (*do_one) (unsigned int namescount,
 			       const char * const * names,
 			       void* data),
