@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2007  Robert Gentleman, Ross Ihaka and the
- *                            R Development Core Team
+ *  Copyright (C) 1998--2011  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,6 +34,8 @@
 
 #include "Defn.h"
 #include "Print.h"
+#include <Rinternals.h>
+#include <R_ext/Parse.h>  /* parsing is used in handling escape codes */
 
 #include "graphapp/ga.h"
 #include "console.h"
@@ -273,10 +274,7 @@ SEXP do_dataentry(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
     }
 
-    /* scale scrollbars as needed */
     DE->xScrollbarScale = DE->yScrollbarScale = 1;
-    if (DE->xmaxused > 10000) DE->xScrollbarScale = DE->xmaxused/1000;
-    if (DE->ymaxused > 10000) DE->yScrollbarScale = DE->ymaxused/1000;
 
     /* start up the window, more initializing in here */
     if (initwin(DE, G_("Data Editor")))
@@ -397,7 +395,7 @@ static void doHscroll(DEstruct DE, int oldcol)
 	for (i = oldcol; i < DE->colmin; i++) dw += BOXW(i);
 	copyH(DE, dw, DE->boxw[0], oldwindowWidth - dw + 1);
 	dw = oldwindowWidth - BOXW(oldcol) + 1;
-	cleararea(DE, dw, DE->hwidth, DE->p->w-dw, DE->p->h, DE->p->bg);
+	cleararea(DE, dw, DE->hwidth, DE->p->w-dw, DE->p->h, DE->p->guiColors[dataeditbg]);
 	/* oldnwide includes the row labels */
 	for (i = oldcol+oldnwide-1; i <= DE->colmax; i++) drawcol(DE, i);
     } else {
@@ -405,7 +403,7 @@ static void doHscroll(DEstruct DE, int oldcol)
 	dw = BOXW(DE->colmin);
 	copyH(DE, DE->boxw[0], DE->boxw[0] + dw, DE->windowWidth - dw + 1);
 	dw = DE->windowWidth + 1;
-	cleararea(DE, dw, DE->hwidth, DE->p->w-dw, DE->p->h, DE->p->bg);
+	cleararea(DE, dw, DE->hwidth, DE->p->w-dw, DE->p->h, DE->p->guiColors[dataeditbg]);
 	drawcol(DE, DE->colmin);
     }
     gchangescrollbar(DE->de, HWINSB, (DE->colmin - 1)/DE->xScrollbarScale,
@@ -525,7 +523,7 @@ static int get_col_width(DEstruct DE, int col)
 	/* don't use NA labels */
 	lab = STRING_ELT(DE->names, col - 1);
 	if(lab != NA_STRING) w = strlen(CHAR(lab)); else w = fw;
-	PrintDefaults(R_NilValue);
+	PrintDefaults();
 	for (i = 0; i < INTEGER(DE->lens)[col - 1]; i++) {
 	    strp = EncodeElement(tmp, i, 0, '.');
 	    w1 = strlen(strp);
@@ -562,7 +560,7 @@ static void drawcol(DEstruct DE, int whichcol)
     SEXP tmp;
 
     find_coords(DE, 0, col, &src_x, &src_y);
-    cleararea(DE, src_x, src_y, bw, DE->windowHeight, DE->p->bg);
+    cleararea(DE, src_x, src_y, bw, DE->windowHeight, DE->p->guiColors[dataeditbg]);
     cleararea(DE, src_x, src_y, bw, DE->box_h, bbg);
     for (i = 0; i < DE->nhigh; i++)
 	drawrectangle(DE, src_x, DE->hwidth + i * DE->box_h, bw, DE->box_h, 1, 1);
@@ -591,7 +589,7 @@ static void drawrow(DEstruct DE, int whichrow)
 
     find_coords(DE, row, 0, &src_x, &src_y);
     cleararea(DE, src_x, src_y, DE->windowWidth, DE->box_h,
-	      (whichrow > 0) ? DE->p->bg : bbg);
+	      (whichrow > 0) ? DE->p->guiColors[dataeditbg] : bbg);
     drawrectangle(DE, src_x, src_y, DE->boxw[0], DE->box_h, 1, 1);
 
     sprintf(rlab, DE->labform, whichrow);
@@ -620,7 +618,7 @@ static void drawrow(DEstruct DE, int whichrow)
 static void printelt(DEstruct DE, SEXP invec, int vrow, int ssrow, int sscol)
 {
     const char *strp;
-    PrintDefaults(R_NilValue);
+    PrintDefaults();
     if (TYPEOF(invec) == REALSXP) {
 	strp = EncodeElement(invec, vrow, 0, '.');
 	printstring(DE, strp, strlen(strp), ssrow, sscol, 0);
@@ -774,6 +772,46 @@ static Rboolean getccol(DEstruct DE)
     return newcol;
 }
 
+static SEXP processEscapes(SEXP x)
+{
+    SEXP newval, pattern, replacement, expr;
+    ParseStatus status;
+    
+    /* We process escape sequences in a scalar string by escaping
+       unescaped quotes, then quoting the whole thing and parsing it.  This
+       is supposed to be equivalent to the R code
+
+       newval <- gsub(perl=TRUE, "(?<!\\\\)((\\\\\\\\)*)\"", "\\1\\\\\"", x)
+       newval <- sub('(^.*$)', '"\1"', newval)
+       newval <- eval(parse(text=newval))
+
+       We do it this way to avoid extracting the escape handling
+       code from the parser.  We need it in C code because this may be executed
+       numerous times from C in dataentry.c */
+    	
+    PROTECT( pattern = mkString("(?<!\\\\)((\\\\\\\\)*)\"") );
+    PROTECT( replacement = mkString("\\1\\\\\"") );
+    PROTECT( expr = lang5(install("gsub"), ScalarLogical(1), pattern, replacement, x) );
+    SET_TAG( CDR(expr), install("perl") );
+
+    PROTECT( newval = eval(expr, R_BaseEnv) );
+    PROTECT( pattern = mkString("(^.*$)") );
+    PROTECT( replacement = mkString("\"\\1\"") );
+    PROTECT( expr = lang4(install("sub"), pattern, replacement, newval) );
+    PROTECT( newval = eval(expr, R_BaseEnv) );
+    PROTECT( expr = R_ParseVector( newval, 1, &status, R_NilValue) );
+    
+    /* We only handle the first entry. If this were available more generally,
+       we'd probably want to loop over all of expr */
+       
+    if (status == PARSE_OK && length(expr))
+	PROTECT( newval = eval(VECTOR_ELT(expr, 0), R_BaseEnv) );
+    else
+	PROTECT( newval = R_NilValue );  /* protect just so the count doesn't change */
+    UNPROTECT(10);
+    return newval;
+}
+
 /* close up the entry to a cell, put the value that has been entered
    into the correct place and as the correct type */
 
@@ -803,9 +841,16 @@ static void closerect(DEstruct DE)
 	    char *endp;
 	    double new = R_strtod(DE->buf, &endp);
 	    int warn = !isBlankString(endp);
-	    if (TYPEOF(cvec) == STRSXP)
-		SET_STRING_ELT(cvec, wrow - 1, mkChar(DE->buf));
-	    else
+	    if (TYPEOF(cvec) == STRSXP) {
+	    	SEXP newval;
+	    	PROTECT( newval = mkString(DE->buf) );
+	    	PROTECT( newval = processEscapes(newval) );
+	    	if (TYPEOF(newval) == STRSXP && length(newval) == 1)
+		    SET_STRING_ELT(cvec, wrow - 1, STRING_ELT(newval, 0));
+		else
+		    warning(G_("dataentry: parse error on string"));
+		UNPROTECT(2);
+	    } else
 		REAL(cvec)[wrow - 1] = new;
 	    if (newcol && warn) {
 		/* change mode to character */
@@ -851,7 +896,7 @@ static void printstring(DEstruct DE, const char *ibuf, int buflen,
     find_coords(DE, row, col, &x_pos, &y_pos);
     if (col == 0) bw = DE->boxw[0]; else bw = BOXW(col+DE->colmin-1);
     cleararea(DE, x_pos + 1, y_pos + 1, bw - 1, DE->box_h - 1,
-	      (row==0 || col==0) ? bbg:DE->p->bg);
+	      (row==0 || col==0) ? bbg:DE->p->guiColors[dataeditbg]);
     fw = min(BUFSIZE, (bw - 8)/(DE->p->fw));
     bufw = min(fw, buflen);
     strncpy(buf, ibuf, bufw);
@@ -875,7 +920,7 @@ static void clearrect(DEstruct DE)
 
     find_coords(DE, DE->crow, DE->ccol, &x_pos, &y_pos);
     cleararea(DE, x_pos, y_pos, BOXW(DE->ccol+DE->colmin-1),
-	      DE->box_h, DE->p->bg);
+	      DE->box_h, DE->p->guiColors[dataeditbg]);
 }
 
 /* handlechar has to be able to parse decimal numbers and strings,
@@ -990,7 +1035,7 @@ static void cleararea(DEstruct DE,
 
 static void clearwindow(DEstruct DE)
 {
-    gfillrect(DE->de, DE->p->bg, rect(0, 0, DE->p->w, DE->p->h));
+    gfillrect(DE->de, DE->p->guiColors[dataeditbg], rect(0, 0, DE->p->w, DE->p->h));
 }
 
 
@@ -999,14 +1044,14 @@ static void drawrectangle(DEstruct DE,
 			  int lwd, int fore)
 {
     /* only used on screen, so always fast */
-    gdrawrect(DE->de, lwd, 0, (fore==1)? DE->p->ufg: DE->p->bg,
+    gdrawrect(DE->de, lwd, 0, (fore==1)? DE->p->guiColors[dataedituser]: DE->p->guiColors[dataeditbg],
 	      rect(xpos, ypos, width, height), 1, PS_ENDCAP_SQUARE,
 	      PS_JOIN_BEVEL, 10);
 }
 
 static void de_drawtext(DEstruct DE, int xpos, int ypos, const char *text)
 {
-    gdrawstr(DE->de, DE->p->f, DE->p->fg, pt(xpos, ypos), text);
+    gdrawstr(DE->de, DE->p->f, DE->p->guiColors[dataeditfg], pt(xpos, ypos), text);
 }
 
 /* Keypress callbacks */
@@ -1166,7 +1211,7 @@ static const char *get_cell_text(DEstruct DE)
     if (wcol <= DE->xmaxused) {
 	tvec = VECTOR_ELT(DE->work, wcol - 1);
 	if (!isNull(tvec) && wrow < INTEGER(DE->lens)[wcol - 1]) {
-	    PrintDefaults(R_NilValue);
+	    PrintDefaults();
 	    if (TYPEOF(tvec) == REALSXP) {
 		prev = EncodeElement(tvec, wrow, 0, '.');
 	    } else if (TYPEOF(tvec) == STRSXP) {
@@ -1264,9 +1309,10 @@ static void de_mousedown(control c, int buttons, point xy)
 		rr.width = (strlen(prev) + 2) * (DE->p->fw);
 	    addto(DE->de);
 	    DE->celledit = newfield_no_border(prev, rr);
-	    setbackground(DE->celledit, DE->p->bg);
-	    setforeground(DE->celledit, DE->p->ufg);
 	    settextfont(DE->celledit, DE->p->f);
+	    setbackground(DE->celledit, DE->p->guiColors[dataeditbg]);
+	    setforeground(DE->celledit, DE->p->guiColors[dataedituser]);
+
 	    show(DE->celledit);
 	    DE->CellEditable = TRUE;
 	} else if (buttons & LeftButton) {
@@ -1326,7 +1372,7 @@ static void deredraw(DEstruct DE)
     }
     printlabs(DE);
     for (i = DE->colmin; i <= DE->colmax; i++) drawcol(DE,i);
-    gfillrect(DE->de, DE->p->bg, rect(DE->windowWidth+1, DE->hwidth,
+    gfillrect(DE->de, DE->p->guiColors[dataeditbg], rect(DE->windowWidth+1, DE->hwidth,
 				      DE->p->w - DE->windowWidth-1,
 				      DE->p->h - DE->hwidth));
     highlightrect(DE);
@@ -1362,7 +1408,7 @@ static Rboolean initwin(DEstruct DE, const char *title)
     DE->oldWIDTH = DE->oldHEIGHT = 0;
     DE->nboxchars = 5;
 
-    DE->nboxchars = asInteger(GetOption(install("de.cellwidth"), R_GlobalEnv));
+    DE->nboxchars = asInteger(GetOption1(install("de.cellwidth")));
     if (DE->nboxchars == NA_INTEGER || DE->nboxchars < 0) DE->nboxchars = 0;
     if (DE->nboxchars > 0) check(DE->de_mvw);
     DE->box_w = ((DE->nboxchars >0)?DE->nboxchars:FIELDWIDTH)*(DE->p->fw) + 8;
@@ -1547,7 +1593,7 @@ static void de_sbf(control c, int pos)
 	DE->rowmin = 1 + pos*DE->yScrollbarScale;
 	if(DE->rowmin > DE->ymaxused - DE->nhigh + 2)
 	    DE->rowmin = max(1,DE->ymaxused - DE->nhigh + 2);
-	printf("pos %d, rowmin %d, scale %d\n", pos, DE->rowmin, DE->yScrollbarScale);
+/*	printf("pos %d, rowmin %d, scale %d\n", pos, DE->rowmin, DE->yScrollbarScale); */
     }
     drawwindow(DE);
 }
@@ -1692,8 +1738,8 @@ static dataeditor newdataeditor(DEstruct DE, const char *title)
 
     DE->p = newconsoledata((consolefn) ? consolefn : FixedFont,
 			   pagerrow, pagercol, 0, 0,
-			   consolefg, consoleuser, consolebg,
-			   DATAEDITOR, 0);
+			   guiColors,
+			   DATAEDITOR, 0, 0);
     if (!DE->p) return NULL;
 
     w = DE->p->w ;
@@ -1721,7 +1767,7 @@ static dataeditor newdataeditor(DEstruct DE, const char *title)
     DE->p->right = (DE->p->w - (DE->p->cols)*(DE->p->fw)) / 2;
     DE->p->top = (DE->p->h - (DE->p->rows)*(DE->p->fh)) / 2;
     gsetcursor(c, ArrowCursor);
-    setbackground(c, consolebg);
+    setbackground(c, guiColors[dataeditbg]);
     if (ismdi() && (RguiMDI & RW_TOOLBAR)) {
 	/* blank toolbar to stop windows jumping around */
 	int btsize = 24;
@@ -1838,10 +1884,7 @@ SEXP do_dataviewer(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    errorcall(call, G_("invalid argument"));
     }
 
-    /* scale scrollbars as needed */
     DE->xScrollbarScale = DE->yScrollbarScale = 1;
-    if (DE->xmaxused > 10000) DE->xScrollbarScale = DE->xmaxused/1000;
-    if (DE->ymaxused > 10000) DE->yScrollbarScale = DE->ymaxused/1000;
 
     /* start up the window, more initializing in here */
     if (initwin(DE, CHAR(STRING_ELT(stitle, 0))))
