@@ -19,6 +19,8 @@
  *  http://www.r-project.org/Licenses/
  */
  
+#include "guicolors.h"
+
 /*                configuration editor                        */
 
 /* current state */
@@ -35,10 +37,13 @@ struct structGUI
     char style[20];
     int crows, ccols, cx, cy, setWidthOnResize, prows, pcols,
 	cbb, cbl, grx, gry;
-    rgb bg, fg, user, hlt;
+    int cursor_blink; /* 0=none, 1=partial, 2=full */
+    rgb guiColors[numGuiColors];
     rect MDIsize;
     char language[20];
     int buffered;
+    char warning[256];  /* When loading the Rconsole, the warning system isn't set
+                           up, so save one warning here. */
 };
 typedef struct structGUI *Gui;
 
