@@ -13,8 +13,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifndef R_PARSE_H
@@ -27,19 +27,31 @@
 /* SEXP R_ParseVector(SEXP, int, ParseStatus *, SEXP); in R_ext/Parse.h */
 
 /* Private interface */
+
+typedef struct {
+
+    Rboolean keepSrcRefs;	/* Whether to attach srcrefs to objects as they are parsed */
+    Rboolean didAttach;		/* Record of whether a srcref was attached */
+    SEXP SrcFile;		/* The srcfile object currently being parsed */
+    SEXP Original;		/* The underlying srcfile object */
+    PROTECT_INDEX SrcFileProt;	/* The SrcFile may change */
+    PROTECT_INDEX OriginalProt; /* ditto */
+    				/* Position information about the current parse */
+    int xxlineno;		/* Line number according to #line directives */
+    int xxcolno;		/* Character number on line */
+    int xxbyteno;		/* Byte number on line */
+    int xxparseno;              /* Line number ignoring #line directives */
+} SrcRefState;
+
+void R_InitSrcRefState(SrcRefState *state);
+void R_FinalizeSrcRefState(SrcRefState *state);
+
 SEXP R_Parse1Buffer(IoBuffer*, int, ParseStatus *); /* in ReplIteration,
 						       R_ReplDLLdo1 */
 SEXP R_ParseBuffer(IoBuffer*, int, ParseStatus *, SEXP, SEXP); /* in source.c */
-SEXP R_Parse1File(FILE*, int, ParseStatus *); /* in R_ReplFile */
+SEXP R_Parse1File(FILE*, int, ParseStatus *, SrcRefState *); /* in R_ReplFile */
 SEXP R_ParseFile(FILE*, int, ParseStatus *, SEXP);  /* in edit.c */
 
-/* Unused */
-#ifdef PARSE_UNUSED
-SEXP R_Parse1General(int (*)(), int (*)(), int, ParseStatus *);
-SEXP R_ParseGeneral(int (*)(), int (*)(), int, ParseStatus *, SEXP);
-SEXP R_Parse1Vector(TextBuffer*, int, ParseStatus *);
-#endif
-
 #ifndef HAVE_RCONNECTION_TYPEDEF
 typedef struct Rconn  *Rconnection;
 #define HAVE_RCONNECTION_TYPEDEF
