@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2004  The R Development Core Team.
+ *  Copyright (C) 1998--2007  The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -14,18 +14,25 @@
  *  GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifndef R_ARITH_H_
 #define R_ARITH_H_
 
-#include <math.h>
+/* Only for use where config.h has not already been included */
+#if defined(HAVE_GLIBC2) && !defined(_BSD_SOURCE)
+/* ensure that finite and isnan are declared */
+# define _BSD_SOURCE 1
+#endif
 
 #include <R_ext/libextern.h>
 #ifdef  __cplusplus
 extern "C" {
+#elif !defined(NO_C_HEADERS)
+/* needed for isnan and isfinite, neither of which are used under C++ */
+# include <math.h>
 #endif
 
 /* implementation of these : ../../main/arithmetic.c */
@@ -48,24 +55,25 @@ LibExtern int	 R_NaInt;	/* NA_INTEGER:= INT_MIN currently */
 int R_IsNA(double);		/* True for R's NA only */
 int R_IsNaN(double);		/* True for special NaN, *not* for NA */
 int R_finite(double);		/* True if none of NA, NaN, +/-Inf */
-
 #define ISNA(x)	       R_IsNA(x)
-/* True for *both* NA and NaN.
-   NOTE: some systems do not return 1 for TRUE. */
-#define ISNAN(x)       (isnan(x)!=0)
+
+/* ISNAN(): True for *both* NA and NaN.
+   NOTE: some systems do not return 1 for TRUE.
+   Also note that C++ math headers specifically undefine
+   isnan if it is a macro (it is on OS X and in C99),
+   hence the workaround.  This code also appears in Rmath.h
+*/
+#ifdef __cplusplus
+  int R_isnancpp(double); /* in arithmetic.c */
+#  define ISNAN(x)     R_isnancpp(x)
+#else
+#  define ISNAN(x)     (isnan(x)!=0)
+#endif
+
+/* The following is only defined inside R */
 #ifdef HAVE_WORKING_ISFINITE
 /* isfinite is defined in <math.h> according to C99 */
 # define R_FINITE(x)    isfinite(x)
-#elif HAVE_WORKING_FINITE
-/* include header needed to define finite() */
-#  ifdef HAVE_IEEE754_H
-#   include <ieee754.h>		/* newer Linuxen */
-#  else
-#   ifdef HAVE_IEEEFP_H
-#    include <ieeefp.h>		/* others [Solaris], .. */
-#   endif
-#  endif
-# define R_FINITE(x)    finite(x)
 #else
 # define R_FINITE(x)    R_finite(x)
 #endif
