@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001-10 The R Development Core Team.
+ *  Copyright (C) 2001-11 The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -141,7 +141,7 @@ struct _DevDesc {
      * Event handling entries
      ********************************************************/
 
-    /* These determine whether getGraphicsEvent will try to set an event handler */
+    /* Used in do_setGraphicsEventEnv */
 
     Rboolean canGenMouseDown; /* can the device generate mousedown events */
     Rboolean canGenMouseMove; /* can the device generate mousemove events */
@@ -150,7 +150,7 @@ struct _DevDesc {
 
     Rboolean gettingEvent;    /* This is set while getGraphicsEvent
 				 is actively looking for events */
-
+    
     /********************************************************
      * Device procedures.
      ********************************************************/
@@ -184,6 +184,7 @@ struct _DevDesc {
      *
      * static void   X11_Activate(pDevDesc dd);
      *
+     * As from R 2.14.0 this can be omitted or set to NULL.
      */
 #if R_USE_PROTOTYPES
     void (*activate)(const pDevDesc );
@@ -261,6 +262,7 @@ struct _DevDesc {
      *
      * static void X11_Deactivate(pDevDesc dd)
      *
+     * As from R 2.14.0 this can be omitted or set to NULL.
      */
 #if R_USE_PROTOTYPES
     void (*deactivate)(pDevDesc );
@@ -277,6 +279,7 @@ struct _DevDesc {
      *
      * static Rboolean X11_Locator(double *x, double *y, pDevDesc dd)
      *
+     * As from R 2.14.0 this can be omitted or set to NULL.
      */
 #if R_USE_PROTOTYPES
     Rboolean (*locator)(double *x, double *y, pDevDesc dd);
@@ -339,6 +342,7 @@ struct _DevDesc {
      *
      * static void X11_Mode(int mode, pDevDesc dd);
      *
+     * As from R 2.14.0 this can be omitted or set to NULL.
      */
 #if R_USE_PROTOTYPES
     void (*mode)(int mode, pDevDesc dd);
@@ -423,6 +427,33 @@ struct _DevDesc {
 		 const pGEcontext gc, pDevDesc dd);
 #else
     void (*rect)();
+#endif
+    /* 
+     * device_Path should draw one or more sets of points 
+     * as a single path
+     * 
+     * 'x' and 'y' give the points
+     *
+     * 'npoly' gives the number of polygons in the path
+     * MUST be at least 1
+     *
+     * 'nper' gives the number of points in each polygon
+     * each value MUST be at least 2
+     *
+     * 'winding' says whether to fill using the nonzero 
+     * winding rule or the even-odd rule
+     *
+     * Added 2010-06-27
+     *
+     * As from R 2.13.2 this can be left unimplemented as NULL.
+     */
+#if R_USE_PROTOTYPES
+    void (*path)(double *x, double *y, 
+                 int npoly, int *nper,
+                 Rboolean winding,
+                 const pGEcontext gc, pDevDesc dd);
+#else
+    void (*path)();
 #endif
     /* 
      * device_Raster should draw a raster image justified 
@@ -436,6 +467,8 @@ struct _DevDesc {
      *
      * 'rot' is in degrees (as per device_Text), with positive
      * rotation anticlockwise from the positive x-axis.
+     *
+     * As from R 2.13.2 this can be left unimplemented as NULL.
      */
 #if R_USE_PROTOTYPES
     void (*raster)(unsigned int *raster, int w, int h,
@@ -455,6 +488,11 @@ struct _DevDesc {
      *
      * This will only make sense for raster devices and can 
      * probably only be implemented for screen devices.
+     *
+     * added 2010-06-27
+     *
+     * As from R 2.13.2 this can be left unimplemented as NULL.
+     * For earlier versions of R it should return R_NilValue.
      */
 #if R_USE_PROTOTYPES
     SEXP (*cap)(pDevDesc dd);
@@ -478,6 +516,8 @@ struct _DevDesc {
      *
      * R_GE_gcontext parameters that should be honoured (if possible):
      *   col, fill, gamma, lty, lwd
+     *
+     * As from R 2.13.2 this can be left unimplemented as NULL.
      */
 #if R_USE_PROTOTYPES
     void (*size)(double *left, double *right, double *bottom, double *top,
@@ -537,13 +577,8 @@ struct _DevDesc {
     void (*onExit)();
 #endif
     /*
-     * device_getEvent is called by do_getGraphicsEvent to get a modal
-     * graphics event.  It should call R_ProcessEvents() until one
-     * of the event handlers sets eventResult to a non-null value,
-     * and then return it
-     * An example is ...
-     *
-     * static SEXP GA_getEvent(SEXP eventRho, const char *prompt);
+     * device_getEvent is no longer used, but the slot is kept for back
+     * compatibility of the structure.
      */
     SEXP (*getEvent)(SEXP, const char *);
 
@@ -556,6 +591,8 @@ struct _DevDesc {
        FALSE if it wants the engine to do so. 
 
        There is an example in the windows() device.
+
+       Can be left unimplemented as NULL.
     */
 #if R_USE_PROTOTYPES
     Rboolean (*newFrameConfirm)(pDevDesc dd);
@@ -591,6 +628,49 @@ struct _DevDesc {
 
     /* --------- Post-2.7.0 features --------- */
 
+    /* Added in 2.12.0:  Changed graphics event handling. */
+    
+    SEXP eventEnv;   /* This is an environment holding event handlers. */
+    /*
+     * eventHelper(dd, 1) is called by do_getGraphicsEvent before looking for a 
+     * graphics event.  It will then call R_ProcessEvents() and eventHelper(dd, 2)
+     * until this or another device returns sets a non-null result value in eventEnv,
+     * at which time eventHelper(dd, 0) will be called.
+     * 
+     * An example is ...
+     *
+     * static SEXP GA_eventHelper(pDevDesc dd, int code);
+
+     * Can be left unimplemented as NULL
+     */
+#if R_USE_PROTOTYPES
+    void (*eventHelper)(pDevDesc dd, int code);
+#else
+    void (*eventHelper)();
+#endif
+
+    /* added in 2.14.0, only used by screen devices.
+
+       Allows graphics devices to have multiple levels of suspension: 
+       when this reaches zero output is flushed.
+
+       Can be left unimplemented as NULL.
+     */
+#if R_USE_PROTOTYPES
+    int (*holdflush)(pDevDesc dd, int level);
+#else
+    int (*holdflush)();
+#endif
+
+    /* added in 2.14.0, for dev.capabilities.
+       In all cases 0 means NA (unset).
+    */
+    int haveTransparency; /* 1 = no, 2 = yes */
+    int haveTransparentBg; /* 1 = no, 2 = fully, 3 = semi */
+    int haveRaster; /* 1 = no, 2 = yes, 3 = except for missing values */
+    int haveCapture, haveLocator;  /* 1 = no, 2 = yes */
+
+
     /* Area for future expansion.
        By zeroing this, devices are more likely to work if loaded
        into a later version of R than that they were compiled under.
@@ -743,9 +823,9 @@ typedef enum {meMouseDown = 0,
 #define doKeybd			Rf_doKeybd
 #define doMouseEvent		Rf_doMouseEvent
 
-SEXP doMouseEvent(SEXP eventRho, pDevDesc dd, R_MouseEvent event,
+void doMouseEvent(pDevDesc dd, R_MouseEvent event,
                   int buttons, double x, double y);
-SEXP doKeybd(SEXP eventRho, pDevDesc dd, R_KeyName rkey,
+void doKeybd(pDevDesc dd, R_KeyName rkey,
 	     const char *keyname);
 
 
