@@ -1,3 +1,19 @@
+#  File src/library/grid/R/grab.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 #########
 # Generate a gTree from the current display list
 #
@@ -8,31 +24,17 @@ rootVP <- function(pvp) {
   match(pvp$name, "ROOT", nomatch=FALSE)
 }
 
-# Return the full vpPath to the current viewport
-current.vpPath <- function() {
-  names <- NULL
-  pvp <- grid.Call("L_currentViewport")
-  while (!rootVP(pvp)) {
-    names <- c(names, pvp$name)
-    pvp <- pvp$parent
-  }
-  if (!is.null(names))
-    vpPathFromVector(rev(names))
-  else
-    names
-}
-
 # List the children of the current vp (as a vpList)
 current.vpList <- function() {
-  cpvp <- grid.Call("L_currentViewport")
-  if (length(ls(env=cpvp$children, all.names=TRUE)) == 0)
+  cpvp <- grid.Call(L_currentViewport)
+  if (length(ls(cpvp$children, all.names=TRUE)) == 0)
     NULL
   else
     vpListFromNode(cpvp)
 }
 
 current.vpNames <-function() {
-  ls(env=grid.Call("L_currentViewport")$children)
+  ls(grid.Call(L_currentViewport)$children)
 }
 
 # vp might be a viewport, or a vpList, or a vpStack, or a vpTree
@@ -41,11 +43,11 @@ vpExists <- function(vp) {
 }
 
 vpExists.viewport <- function(vp) {
-  vp$name %in% ls(env=.Call("L_currentViewport", PACKAGE="grid")$children)
+  vp$name %in% ls(.Call(L_currentViewport)$children)
 }
 
 vpExists.vpStack <- function(vp) {
-  vpExists(vp[[1]])
+  vpExists(vp[[1L]])
 }
 
 vpExists.vpList <- function(vp) {
@@ -82,7 +84,7 @@ wrap.up <- function(x) {
   recordGrob(upViewport(n), list(n=x))
 }
 
-wrap.down <- function(x) {
+wrap.vpPath <- function(x) {
   recordGrob(downViewport(path), list(path=x))
 }
 
@@ -97,7 +99,7 @@ wrap.down <- function(x) {
 #   in a gTree
 grabDL <- function(warn, wrap, ...) {
   gList <- NULL
-  dl.index <- grid.Call("L_getDLindex")
+  dl.index <- grid.Call(L_getDLindex)
   if (dl.index > 1) {
     if (warn > 0) {
       names <- getNames()
@@ -112,7 +114,7 @@ grabDL <- function(warn, wrap, ...) {
       # dispatching to a function call per element because
       # we need to work with whole DL at times, not
       # just individual elements
-      elt <- grid.Call("L_getDLelt", as.integer(i - 1))
+      elt <- grid.Call(L_getDLelt, as.integer(i - 1))
       if (wrap)
         gList <- addToGList(wrap(elt), gList)
       else {
@@ -201,12 +203,24 @@ grid.grab <- function(warn=2, wrap=FALSE, ...) {
 }
 
 grid.grabExpr <- function(expr, warn=2, wrap=FALSE, ...) {
-  # Start a new null device
-  .Call("L_nullDevice", PACKAGE="grDevices")
-  # If something goes wrong, want to revert to the current device
+  # Start a new null device for this function
+  .Call("GD_nullDevice", PACKAGES = "grDevices")
   on.exit(dev.off())
   # Run the graphics code in expr
-  eval(substitute(expr))
+  # Rely on lazy evaluation for correct "timing"
+  eval(expr)
   # Grab the DL on the new device
   grabDL(warn, wrap, ...)
 }
+
+#########################
+# A different sort of capture ...
+# Just grab the screen raster image
+#########################
+
+grid.cap <- function() {
+    # This does not need recording on the display list
+    grid.Call(L_cap)
+}
+
+
