@@ -13,13 +13,14 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
- *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 59 Temple Place,
- *  Suite 330, Boston, MA  02111-1307  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #include "grid.h"
+#include <string.h>
+
 
 extern int gridRegisterIndex;
 
@@ -42,13 +43,14 @@ double gpLineHeight(SEXP gp, int i) {
     return REAL(lineheight)[i % LENGTH(lineheight)];
 }
 
+/* grid has no concept of 'colour 0' (bg in base) */
 int gpCol(SEXP gp, int i) {
     SEXP col = VECTOR_ELT(gp, GP_COL);
     int result;
     if (isNull(col))
 	result = R_TRANWHITE;
     else
-	result = RGBpar(col, i % LENGTH(col));
+	result = RGBpar3(col, i % LENGTH(col), R_TRANWHITE);
     return result;
 }
 
@@ -62,7 +64,7 @@ int gpFill(SEXP gp, int i) {
     if (isNull(fill))
 	result = R_TRANWHITE;
     else
-	result = RGBpar(fill, i % LENGTH(fill));
+	result = RGBpar3(fill, i % LENGTH(fill), R_TRANWHITE);
     return result;
 }
 
@@ -81,7 +83,7 @@ SEXP gpLineTypeSXP(SEXP gp) {
 
 int gpLineType(SEXP gp, int i) {
     SEXP linetype = gpLineTypeSXP(gp);
-    return LTYpar(linetype, i % LENGTH(linetype));
+    return GE_LTYpar(linetype, i % LENGTH(linetype));
 }
 
 SEXP gpLineWidthSXP(SEXP gp) {
@@ -115,7 +117,7 @@ SEXP gpFontFamilySXP(SEXP gp) {
     return VECTOR_ELT(gp, GP_FONTFAMILY);
 }
 
-char* gpFontFamily(SEXP gp, int i) {
+const char* gpFontFamily(SEXP gp, int i) {
     SEXP fontfamily = gpFontFamilySXP(gp);
     return CHAR(STRING_ELT(fontfamily, i % LENGTH(fontfamily)));
 }
@@ -135,7 +137,7 @@ SEXP gpLineEndSXP(SEXP gp) {
 
 R_GE_lineend gpLineEnd(SEXP gp, int i) {
     SEXP lineend = gpLineEndSXP(gp);
-    return LENDpar(lineend, i % LENGTH(lineend));
+    return GE_LENDpar(lineend, i % LENGTH(lineend));
 }
 
 SEXP gpLineJoinSXP(SEXP gp) {
@@ -144,7 +146,7 @@ SEXP gpLineJoinSXP(SEXP gp) {
 
 R_GE_linejoin gpLineJoin(SEXP gp, int i) {
     SEXP linejoin = gpLineJoinSXP(gp);
-    return LJOINpar(linejoin, i % LENGTH(linejoin));
+    return GE_LJOINpar(linejoin, i % LENGTH(linejoin));
 }
 
 SEXP gpLineMitreSXP(SEXP gp) {
@@ -195,7 +197,7 @@ static unsigned int combineAlpha(double alpha, int col)
 /* 
  * Generate an R_GE_gcontext from a gpar
  */
-void gcontextFromgpar(SEXP gp, int i, R_GE_gcontext *gc) 
+void gcontextFromgpar(SEXP gp, int i, const pGEcontext gc, pGEDevDesc dd) 
 {
     /* 
      * Combine gpAlpha with col and fill
@@ -205,14 +207,19 @@ void gcontextFromgpar(SEXP gp, int i, R_GE_gcontext *gc)
     gc->gamma = gpGamma(gp, i);
     /*
      * Combine gpLex with lwd
+     * Also scale by GSS_SCALE (a "zoom" factor)
      */
-    gc->lwd = gpLineWidth(gp, i) * gpLex(gp, i);
+    gc->lwd = gpLineWidth(gp, i) * gpLex(gp, i) * 
+	REAL(gridStateElement(dd, GSS_SCALE))[0];
     gc->lty = gpLineType(gp, i);
     gc->lend = gpLineEnd(gp, i);
     gc->ljoin = gpLineJoin(gp, i);
     gc->lmitre = gpLineMitre(gp, i);
     gc->cex = gpCex(gp, i);
-    gc->ps = gpFontSize(gp, i);
+    /*
+     * Scale by GSS_SCALE (a "zoom" factor)
+     */
+    gc->ps = gpFontSize(gp, i) * REAL(gridStateElement(dd, GSS_SCALE))[0];
     gc->lineheight = gpLineHeight(gp, i);
     gc->fontface = gpFont(gp, i);
     strcpy(gc->fontfamily, gpFontFamily(gp, i));
@@ -226,7 +233,7 @@ SEXP L_setGPar(SEXP gpars)
      */
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     setGridStateElement(dd, GSS_GPAR, gpars);
     return R_NilValue;
 }
@@ -239,7 +246,7 @@ SEXP L_getGPar(void)
      */
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_GPAR);
 }
 
@@ -247,7 +254,7 @@ SEXP L_getGPsaved()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_GPSAVED);
 }
 
@@ -255,14 +262,14 @@ SEXP L_setGPsaved(SEXP gpars)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     setGridStateElement(dd, GSS_GPSAVED, gpars);
     return R_NilValue;
 }
 
-void initGPar(GEDevDesc *dd)
+void initGPar(pGEDevDesc dd)
 {
-    NewDevDesc *dev = dd->dev;
+    pDevDesc dev = dd->dev;
     SEXP gpar, gparnames, class;
     SEXP gpfill, gpcol, gpgamma, gplty, gplwd, gpcex, gpfs, gplh, gpfont;
     SEXP gpfontfamily, gpalpha, gplineend, gplinejoin, gplinemitre, gplex;
@@ -285,21 +292,16 @@ void initGPar(GEDevDesc *dd)
     SET_STRING_ELT(gparnames, GP_LINEMITRE, mkChar("linemitre"));
     SET_STRING_ELT(gparnames, GP_LEX, mkChar("lex"));
     setAttrib(gpar, R_NamesSymbol, gparnames);
-    /* FIXME:  Need to export col2name via (probably) GraphicsEngine.h
-     * In the meantime I just have to override the device settings
-     */
     PROTECT(gpfill = allocVector(STRSXP, 1));
-    /* SET_STRING_ELT(gpfill, 0, mkChar(col2name(dev->startfill))); */
-    SET_STRING_ELT(gpfill, 0, mkChar("transparent"));
+    SET_STRING_ELT(gpfill, 0, mkChar(col2name(dev->startfill)));
     SET_VECTOR_ELT(gpar, GP_FILL, gpfill);
     PROTECT(gpcol = allocVector(STRSXP, 1));
-    /* SET_STRING_ELT(gpcol, 0, mkChar(col2name(dev->startcol))); */
-    SET_STRING_ELT(gpcol, 0, mkChar("black"));
+    SET_STRING_ELT(gpcol, 0, mkChar(col2name(dev->startcol)));
     SET_VECTOR_ELT(gpar, GP_COL, gpcol);
     PROTECT(gpgamma = allocVector(REALSXP, 1));
     REAL(gpgamma)[0] = dev->startgamma;
     SET_VECTOR_ELT(gpar, GP_GAMMA, gpgamma);
-    PROTECT(gplty = LTYget(dev->startlty));
+    PROTECT(gplty = GE_LTYget(dev->startlty));
     SET_VECTOR_ELT(gpar, GP_LTY, gplty);
     PROTECT(gplwd = allocVector(REALSXP, 1));
     REAL(gplwd)[0] = 1;
