@@ -126,7 +126,7 @@ aov <- function(formula, data = NULL, projections = FALSE, qr = TRUE,
                 fiti$terms <- Terms
             } else {
                 y <- qty[select,,drop=FALSE]
-                fiti <- list(coefficients = numeric(0L), residuals = y,
+                fiti <- list(coefficients = numeric(), residuals = y,
                              fitted.values = 0 * y, weights = wts, rank = 0L,
                              df.residual = NROW(y))
             }
@@ -154,7 +154,8 @@ function(x, intercept = FALSE, tol = .Machine$double.eps^0.5, ...)
         cat("Call:\n   ")
         dput(cl, control=NULL)
     }
-    asgn <- x$assign[x$qr$pivot[1L:x$rank]]
+    qrx <- if(x$rank) qr(x)
+    asgn <- x$assign[qrx$pivot[1L:x$rank]]
     effects <- x$effects
     if(!is.null(effects))
         effects <- as.matrix(effects)[seq_along(asgn),,drop=FALSE]
@@ -226,7 +227,7 @@ function(x, intercept = FALSE, tol = .Machine$double.eps^0.5, ...)
             cat("Residual standard error:", sapply(rs, format), "\n")
         }
         coef <- as.matrix(x$coefficients)[, 1L]
-        R <- x$qr$qr
+        R <- qrx$qr
         R <- R[1L:min(dim(R)), ,drop=FALSE]
         R[lower.tri(R)] <- 0
         if(rank < (nc <- length(coef))) {
@@ -328,11 +329,11 @@ summary.aov <- function(object, intercept = FALSE, split,
     for (y in 1L:nresp) {
         if(is.null(effects)) {
             nterms <- 0
-            df <- ss <- ms <- numeric(0L)
-            nmrows <- character(0L)
+            df <- ss <- ms <- numeric()
+            nmrows <- character()
         } else {
-            df <- ss <- numeric(0L)
-            nmrows <- character(0L)
+            df <- ss <- numeric()
+            nmrows <- character()
             for(i in seq(nterms)) {
                 ai <- (asgn == uasgn[i])
                 df <- c(df, sum(ai))
@@ -380,9 +381,11 @@ summary.aov <- function(object, intercept = FALSE, split,
 
 print.summary.aov <-
     function(x, digits = max(3, getOption("digits") - 3), symbolic.cor = FALSE,
-             signif.stars= getOption("show.signif.stars"),	...)
+             signif.stars = getOption("show.signif.stars"),	...)
 {
-    if (length(x) == 1L)  print(x[[1L]], ...)
+    if (length(x) == 1L)
+        print(x[[1L]], digits = digits, symbolic.cor = symbolic.cor,
+              signif.stars = signif.stars)
     else NextMethod()
     if(nzchar(mess <- naprint(attr(x, "na.action")))) cat(mess, "\n", sep="")
     invisible(x)
@@ -428,7 +431,7 @@ alias.lm <- function(object, complete = TRUE, partial = FALSE,
     Model <- object$terms
     attributes(Model) <- NULL
     value <- list(Model = Model)
-    R <- object$qr$qr
+    R <- qr(object)$qr
     R <- R[1L:min(dim(R)), , drop=FALSE]
     R[lower.tri(R)] <- 0
     d <- dim(R)
@@ -532,12 +535,13 @@ se.contrast.aov <-
 {
     contrast.weight.aov <- function(object, contrast)
     {
-        asgn <- object$assign[object$qr$pivot[1L:object$rank]]
+        qro <- qr(object)
+        asgn <- object$assign[qro$pivot[1L:object$rank]]
         uasgn <- unique(asgn)
         nterms <- length(uasgn)
         nmeffect <- c("(Intercept)",
                       attr(object$terms, "term.labels"))[1L + uasgn]
-        effects <- as.matrix(qr.qty(object$qr, contrast))
+        effects <- as.matrix(qr.qty(qro, contrast))
         res <- matrix(0, nrow = nterms, ncol = ncol(effects),
                       dimnames = list(nmeffect, colnames(contrast)))
         for(i in seq(nterms)) {
