@@ -19,7 +19,7 @@ lowess <- function(x, y=NULL, f=2/3, iter=3, delta=.01*diff(range(xy$x[o]))) {
     n <- as.integer(length(xy$x))
     if(n == 0) stop("'x' is empty")
     o <- order(xy$x)
-    .C("lowess",
+    .C(C_lowess,
        x=as.double(xy$x[o]),
        as.double(xy$y[o]),
        n,
