@@ -1,3 +1,19 @@
+#  File src/library/stats/R/prcomp.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 prcomp <- function (x, ...) UseMethod("prcomp")
 
 prcomp.default <-
@@ -13,14 +29,14 @@ prcomp.default <-
     s$d <- s$d / sqrt(max(1, nrow(x) - 1))
     if (!is.null(tol)) {
         ## we get rank at least one even for a 0 matrix.
-        rank <- sum(s$d > (s$d[1]*tol))
+        rank <- sum(s$d > (s$d[1L]*tol))
         if (rank < ncol(x)) {
-            s$v <- s$v[, 1:rank, drop = FALSE]
-            s$d <- s$d[1:rank]
+            s$v <- s$v[, 1L:rank, drop = FALSE]
+            s$d <- s$d[1L:rank]
         }
     }
     dimnames(s$v) <-
-        list(colnames(x), paste("PC", seq(len = ncol(s$v)), sep = ""))
+        list(colnames(x), paste("PC", seq_len(ncol(s$v)), sep = ""))
     r <- list(sdev = s$d, rotation = s$v,
               center = if(is.null(cen)) FALSE else cen,
               scale = if(is.null(sc)) FALSE else sc)
@@ -32,12 +48,12 @@ prcomp.default <-
 prcomp.formula <- function (formula, data = NULL, subset, na.action, ...)
 {
     mt <- terms(formula, data = data)
-    if (attr(mt, "response") > 0)
+    if (attr(mt, "response") > 0L)
         stop("response not allowed in formula")
     cl <- match.call()
     mf <- match.call(expand.dots = FALSE)
     mf$... <- NULL
-    mf[[1]] <- as.name("model.frame")
+    mf[[1L]] <- as.name("model.frame")
     mf <- eval.parent(mf)
     ## this is not a `standard' model-fitting function,
     ## so no need to consider contrasts or levels
@@ -45,11 +61,11 @@ prcomp.formula <- function (formula, data = NULL, subset, na.action, ...)
         stop("PCA applies only to numerical variables")
     na.act <- attr(mf, "na.action")
     mt <- attr(mf, "terms")
-    attr(mt, "intercept") <- 0
+    attr(mt, "intercept") <- 0L
     x <- model.matrix(mt, mf)
     res <- prcomp.default(x, ...)
     ## fix up call to refer to the generic, but leave arg name as `formula'
-    cl[[1]] <- as.name("prcomp")
+    cl[[1L]] <- as.name("prcomp")
     res$call <- cl
     if (!is.null(na.act)) {
         res$na.action <- na.act
@@ -60,7 +76,7 @@ prcomp.formula <- function (formula, data = NULL, subset, na.action, ...)
 }
 
 plot.prcomp <- function(x, main = deparse(substitute(x)), ...)
-    screeplot(x, main = main, ...)
+    screeplot.default(x, main = main, ...)
 
 print.prcomp <- function(x, print.x = FALSE, ...) {
     cat("Standard deviations:\n")
@@ -88,7 +104,7 @@ summary.prcomp <- function(object, ...)
 }
 
 print.summary.prcomp <-
-function(x, digits = min(3, getOption("digits")-3), ...)
+function(x, digits = max(3, getOption("digits") - 3), ...)
 {
     cat("Importance of components:\n")
     print(x$importance, digits = digits)
@@ -101,16 +117,15 @@ predict.prcomp <- function(object, newdata, ...)
         if(!is.null(object$x)) return(object$x)
         else stop("no scores are available: refit with 'retx=TRUE'")
     }
-    if(length(dim(newdata)) != 2)
+    if(length(dim(newdata)) != 2L)
         stop("'newdata' must be a matrix or data frame")
-    p <- NCOL(object$rotation)
     nm <- rownames(object$rotation)
     if(!is.null(nm)) {
         if(!all(nm %in% colnames(newdata)))
             stop("'newdata' does not have named columns matching one or more of the original columns")
         newdata <- newdata[, nm, drop = FALSE]
     } else {
-        if(NCOL(newdata) != p)
+        if(NCOL(newdata) != NROW(object$rotation) )
             stop("'newdata' does not have the correct number of columns")
     }
     ## next line does as.matrix
@@ -122,6 +137,6 @@ predict.prcomp <- function(object, newdata, ...)
     ## we need to test just the columns which are actually used.
     mt <- attr(mf, "terms")
     mterms <- attr(mt, "factors")
-    mterms <- rownames(mterms)[apply(mterms, 1, any)]
+    mterms <- rownames(mterms)[apply(mterms, 1L, function(x) any(x > 0L))]
     any(sapply(mterms, function(x) is.factor(mf[,x]) || !is.numeric(mf[,x])))
 }
