@@ -49,6 +49,7 @@
     rarch <- Sys.getenv("R_ARCH") # unix only
     if (WINDOWS && nzchar(.Platform$r_arch))
         rarch <- paste0("/", .Platform$r_arch)
+    test_archs <- rarch
 
     SHLIB_EXT <- if (WINDOWS) ".dll" else {
         ## can we do better?
@@ -64,9 +65,7 @@
         ## These might be needed for configure.win and Make{file,vars}.win
         ## Some people have *assumed* that R_HOME uses /
         Sys.setenv(R_HOME = rhome)
-        if (nzchar(rarch)) {
-            Sys.setenv(R_ARCH = rarch, R_ARCH_BIN = rarch)
-        }
+        if (nzchar(rarch)) Sys.setenv(R_ARCH = rarch, R_ARCH_BIN = rarch)
     }
 
     Usage <- function() {
@@ -192,7 +191,7 @@
         full
     }
 
-    ## used for LazyData, LazyLoad, KeepSource
+    ## used for LazyData, LazyLoad, KeepSource, ByteCompile
     parse_description_field <- function(desc, field, default=TRUE)
     {
         tmp <- desc[field]
@@ -608,23 +607,18 @@
                     file.copy(f, instdir, TRUE)
                     Sys.chmod(file.path(instdir, f), "644")
                 }
-            descdir <- "."
-            if (!file.exists(namespace <- file.path(instdir, "NAMESPACE")) ) {
-                starsmsg(stars, "Creating default NAMESPACE file")
-                descdir <- tempdir()
-                file.copy("DESCRIPTION", descdir)
-                add_namespace_is_auto_to_description_file(file.path(descdir, "DESCRIPTION"))
-                file.copy(file.path(descdir, "DESCRIPTION"), instdir)
-                writeDefaultNamespace(namespace) # from build.R
-            }
 
             ## This cannot be done in a MBCS: write.dcf fails
             ctype <- Sys.getlocale("LC_CTYPE")
             Sys.setlocale("LC_CTYPE", "C")
-            res <- try(.install_package_description(descdir, instdir))
+            res <- try(.install_package_description('.', instdir))
             Sys.setlocale("LC_CTYPE", ctype)
             if (inherits(res, "try-error"))
                 pkgerrmsg("installing package DESCRIPTION failed", pkg_name)
+            if (!file.exists(namespace <- file.path(instdir, "NAMESPACE")) ) {
+                starsmsg(stars, "Creating default NAMESPACE file")
+                writeDefaultNamespace(namespace) # from build.R
+            }
         }
 
         if (install_libs && dir.exists("src") &&
@@ -680,7 +674,7 @@
                                             "igraph", "jpeg", "png", "proj4",
                                             "randtoolbox", "rgdal", "rngWELL",
                                             "rphast", "rtfbs", "tcltk2"))
-                            one_only <- sum(nchar(readLines("../configure.win"), "bytes")) > 0
+                            one_only <- sum(nchar(readLines("../configure.win", warn = FALSE), "bytes")) > 0
                         if(one_only && !force_biarch)
                             warning("this package has a non-empty 'configure.win' file,\nso building only the main architecture\n", call. = FALSE, domain=NA)
                     }
@@ -689,6 +683,7 @@
                         has_error <- run_shlib(pkg_name, srcs, instdir, rarch)
                     else {
                         setwd(owd)
+                        test_archs <- archs
                         for(arch in archs) {
                             message("", domain = NA) # a blank line
                             starsmsg("***", "arch - ", arch)
@@ -698,15 +693,12 @@
                             ## avoid read-only files/dir such as nested .svn
                             .Internal(dirchmod(ss))
                             setwd(ss)
+
                             ra <- paste0("/", arch)
                             Sys.setenv(R_ARCH = ra, R_ARCH_BIN = ra)
-                            has_error0 <- run_shlib(pkg_name, srcs, instdir, ra)
+                            has_error <- run_shlib(pkg_name, srcs, instdir, ra)
                             setwd(owd)
-                            ## allow archs other than the current one to fail.
-                            if (has_error0 && ra == rarch) {
-                                has_error <- TRUE
-                                break
-                            }
+                            if (has_error) break
                         }
                     }
                 }
@@ -753,6 +745,7 @@
                             has_error <- run_shlib(pkg_name, srcs, instdir, rarch)
                         } else {
                             setwd(owd)
+                            test_archs <- archs
                             for(arch in archs) {
                                 if (arch == "R") {
                                     ## top-level, so one arch without subdirs
@@ -787,7 +780,7 @@
             if(WINDOWS) dirs <- dirs[dirs %in% c("i386", "x64")]
             if (length(dirs)) {
                 descfile <- file.path(instdir, "DESCRIPTION")
-                olddesc <- readLines(descfile)
+                olddesc <- readLines(descfile, warn = FALSE)
                 olddesc <- grep("^Archs:", olddesc,
                                 invert = TRUE, value = TRUE, useBytes = TRUE)
                 newdesc <- c(olddesc,
@@ -939,7 +932,7 @@
             ## handle .Rinstignore:
             ignore_file <- ".Rinstignore"
             ignore <- if (file.exists(ignore_file)) {
-                ignore <- readLines(ignore_file)
+                ignore <- readLines(ignore_file, warn = FALSE)
                 ignore[nzchar(ignore)]
             } else character()
             for(e in ignore)
@@ -968,6 +961,7 @@
                 pdfs <- dir(file.path(instdir, "doc"), pattern="\\.pdf",
                             recursive = TRUE, full.names = TRUE,
                             all.files = TRUE)
+                res <- compactPDF(pdfs, gs_cmd = "")
                 ## print selectively
                 res <- compactPDF(pdfs)
                 print(res[res$old > 1e5, ])
@@ -980,14 +974,11 @@
 	    file.copy("tests", instdir, recursive = TRUE)
 	}
 
-	## LazyLoading
+	## LazyLoading/Compiling
 	value <- parse_description_field(desc, "LazyLoad", default = TRUE)
-        if(!value) {
-            value <- TRUE
-            warning("LazyLoad != TRUE is deprecated and ignored",
-                    call. = FALSE, domain = NA)
-        }
-	if (install_R && dir.exists("R") && length(dir("R")) && value) {
+        if(!value)
+            message("Note: LazyLoad != TRUE is deprecated and ignored", domain = NA)
+	if (install_R && dir.exists("R") && length(dir("R"))) {
             BC <- parse_description_field(desc, "ByteCompile",
                                           default = byte_compile)
             rcp <- as.numeric(Sys.getenv("R_COMPILE_PKGS"))
@@ -1069,23 +1060,37 @@
         if (clean) run_clean()
 
         if (test_load) {
-            ## As from R 2.13.0 do this in a separate R process, in case
-            ## it brings down the R process running .install.packages()
-            ## and so do_exit_on_error() is not called.
+            ## Do this in a separate R process, in case it crashes R.
 	    starsmsg(stars, "testing if installed package can be loaded")
-            ## FIXME: maybe 'lib' is not quite good enough
+            ## FIXME: maybe the quoting as 'lib' is not quite good enough
             ## On a Unix-alike this calls system(input=)
             ## and that uses a temporary file and redirection.
             cmd <- paste("tools:::.test_load_package('", pkg_name, "', '", lib, "')",
                          sep = "")
             ## R_LIBS was set already.  R_runR is in check.R
-            res <- R_runR(cmd, "--no-save --slave", stdout = "", stderr = "")
-            if (res) errmsg("loading failed")
+            if (length(test_archs) > 1L) {
+                msgs <- character()
+                for (arch in test_archs) {
+                    starsmsg("***", "arch - ", arch)
+                    res <- R_runR(cmd, "--no-save --slave",
+                                  stdout = "", stderr = "", arch = arch)
+                    if (res) msgs <- c(msgs, arch)
+                }
+                if (length(msgs)) {
+                    msg <- paste("loading failed for",
+                                 paste(sQuote(msgs), collapse = ", "))
+                    errmsg(msg) # does not return
+                }
+            } else {
+                res <- R_runR(cmd, "--no-save --slave",
+                              stdout = "", stderr = "")
+                if (res) errmsg("loading failed") # does not return
+            }
         }
     }
 
     options(showErrorCalls=FALSE)
-    pkgs <- character(0)
+    pkgs <- character()
     if (is.null(args)) {
         args <- commandArgs(TRUE)
         ## it seems that splits on spaces, so try harder.
@@ -1107,8 +1112,8 @@
     build_example <- FALSE
     use_configure <- TRUE
     auto_zip <- FALSE
-    configure_args <- character(0)
-    configure_vars <- character(0)
+    configure_args <- character()
+    configure_vars <- character()
     fake <- FALSE
     lazy <- TRUE
     lazy_data <- FALSE
@@ -1118,7 +1123,7 @@
     pkglock <- FALSE  # set for per-package locking
     libs_only <- FALSE
     tar_up <- zip_up <- FALSE
-    shargs <- character(0)
+    shargs <- character()
     multiarch <- TRUE
     force_biarch <- FALSE
     test_load <- TRUE
@@ -1442,7 +1447,7 @@
 	install_inst <- FALSE
     }
 
-    build_help_types <- character(0)
+    build_help_types <- character()
     if (build_html) build_help_types <- c(build_help_types, "html")
     if (build_latex) build_help_types <- c(build_help_types, "latex")
     if (build_example) build_help_types <- c(build_help_types, "example")
