@@ -84,7 +84,10 @@ function(object, filename = NULL, name = NULL,
         Call <- paste0(Call, arg.names[i],
                        if(!is.missing.arg(argls[[i]]))
                        paste0(" = ",
-                              paste(deparse(argls[[i]], width.cutoff= 500L),
+                              ## need to backtick symbols
+                              paste(deparse(argls[[i]],
+                                            backtick = TRUE,
+                                            width.cutoff= 500L),
                                     collapse="\n")))
         if(i != n) Call <- paste0(Call, ", ")
     }
@@ -159,7 +162,7 @@ function(object, filename = NULL, name = NULL,
 
     cat(unlist(Rdtxt), file = filename, sep = "\n")
 
-    message(gettextf("Created file named '%s'.", filename),
+    message(gettextf("Created file named %s.", sQuote(filename)),
             "\n",
             gettext("Edit the file and move it to the appropriate directory."),
             domain = NA)
@@ -303,7 +306,7 @@ function(object, filename = NULL, name = NULL)
 
     cat(unlist(Rdtxt), file = filename, sep = "\n")
 
-    message(gettextf("Created file named '%s'.", filename),
+    message(gettextf("Created file named %s.", sQuote(filename)),
             "\n",
             gettext("Edit the file and move it to the appropriate directory."),
             domain = NA)
@@ -407,7 +410,7 @@ function(package, lib.loc = NULL, filename = NULL, name = NULL, final = FALSE)
 
     cat(unlist(Rdtxt), file = filename, sep = "\n")
 
-    message(gettextf("Created file named '%s'.", filename),
+    message(gettextf("Created file named %s.", sQuote(filename)),
             "\n",
             gettext("Edit the file and move it to the appropriate directory."),
             domain = NA)
