@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2003  Robert Gentleman, Ross Ihaka and the
- *                            R Development Core Team
+ *  Copyright (C) 1997--2011  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,11 +14,10 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole string */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -35,10 +33,13 @@
 
 #include <R_ext/Rdynload.h>
 
-static DL_FUNC User_unif_fun, User_unif_init, User_unif_nseed, 
-    User_unif_seedloc;
+static DL_FUNC User_unif_fun, User_unif_nseed,
+	User_unif_seedloc;
+typedef void (*UnifInitFun)(Int32);
 
-DL_FUNC  User_norm_fun; /* also in ../nmath/snorm.c */
+UnifInitFun User_unif_init = NULL; /* some picky compilers */
+
+DL_FUNC  User_norm_fun = NULL; /* also in ../nmath/snorm.c */
 
 
 static RNGtype RNG_kind = RNG_DEFAULT;
@@ -64,13 +65,14 @@ static
 RNGTAB RNG_Table[] =
 {
 /* kind Nkind	  name	           n_seed      i_seed */
-    { 0, 0, "Wichmann-Hill", 	        3,	dummy},
-    { 1, 0, "Marsaglia-MultiCarry",	2,	dummy},
-    { 2, 0, "Super-Duper",		2,	dummy},
-    { 3, 0, "Mersenne-Twister",	    1+624,	dummy},
-    { 4, 0, "Knuth-TAOCP",          1+100,	dummy},
-    { 5, 0, "User-supplied",            0,	dummy},
-    { 6, 0, "Knuth-TAOCP-2002",     1+100,	dummy},
+    { WICHMANN_HILL,        BUGGY_KINDERMAN_RAMAGE, "Wichmann-Hill",	     3,	dummy},
+    { MARSAGLIA_MULTICARRY, BUGGY_KINDERMAN_RAMAGE, "Marsaglia-MultiCarry",  2,	dummy},
+    { SUPER_DUPER,          BUGGY_KINDERMAN_RAMAGE, "Super-Duper",	     2,	dummy},
+    { MERSENNE_TWISTER,     BUGGY_KINDERMAN_RAMAGE, "Mersenne-Twister",  1+624,	dummy},
+    { KNUTH_TAOCP,          BUGGY_KINDERMAN_RAMAGE, "Knuth-TAOCP",       1+100,	dummy},
+    { USER_UNIF,            BUGGY_KINDERMAN_RAMAGE, "User-supplied",         0,	dummy},
+    { KNUTH_TAOCP2,         BUGGY_KINDERMAN_RAMAGE, "Knuth-TAOCP-2002",  1+100,	dummy},
+    { LECUYER_CMRG,         BUGGY_KINDERMAN_RAMAGE, "L'Ecuyer-CMRG",         6,	dummy},
 };
 
 
@@ -83,9 +85,9 @@ RNGTAB RNG_Table[] =
 #define I3 (RNG_Table[RNG_kind].i_seed[2])
 
 static void Randomize(RNGtype kind);
-static double MT_genrand();
-static Int32 KT_next();
-static void RNG_Init_KT(Int32);
+static double MT_genrand(void);
+static Int32 KT_next(void);
+static void RNG_Init_R_KT(Int32);
 static void RNG_Init_KT2(Int32);
 #define KT_pos (RNG_Table[KNUTH_TAOCP].i_seed[100])
 
@@ -135,23 +137,58 @@ double unif_rand(void)
     case USER_UNIF:
 	return *((double *) User_unif_fun());
 
+    case LECUYER_CMRG:
+    {
+	/* Based loosely on the GPL-ed version of
+	   http://www.iro.umontreal.ca/~lecuyer/myftp/streams00/c2010/RngStream.c
+	   but using int_least64_t, which C99 guarantees.
+	*/
+	int k;
+	int_least64_t p1, p2;
+
+#define II(i) (RNG_Table[RNG_kind].i_seed[i])
+#define m1    4294967087
+#define m2    4294944443
+#define normc  2.328306549295727688e-10
+#define a12     (int_least64_t)1403580
+#define a13n    (int_least64_t)810728
+#define a21     (int_least64_t)527612
+#define a23n    (int_least64_t)1370589
+
+	p1 = a12 * (unsigned int)II(1) - a13n * (unsigned int)II(0);
+	/* p1 % m1 would surely do */
+	k = p1 / m1;
+	p1 -= k * m1;
+	if (p1 < 0.0) p1 += m1;
+	II(0) = II(1); II(1) = II(2); II(2) = p1;
+
+	p2 = a21 * (unsigned int)II(5) - a23n * (unsigned int)II(3);
+	k = p2 / m2;
+	p2 -= k * m2;
+	if (p2 < 0.0) p2 += m2;
+	II(3) = II(4); II(4) = II(5); II(5) = p2;
+
+	return ((p1 > p2) ? (p1 - p2) : (p1 - p2 + m1)) * normc;
+    }
     default:
 	error(_("unif_rand: unimplemented RNG kind %d"), RNG_kind);
 	return -1.;
     }
 }
 
-static void FixupSeeds(RNGtype kind, int initial)
+/* we must mask global variable here, as I1-I3 hide RNG_kind
+   and we want the argument */
+static void FixupSeeds(RNGtype RNG_kind, int initial)
 {
 /* Depending on RNG, set 0 values to non-0, etc. */
 
     int j, notallzero = 0;
 
     /* Set 0 to 1 :
-       for(j = 0; j <= RNG_Table[kind].n_seed - 1; j++)
-       if(!RNG_Table[kind].i_seed[j]) RNG_Table[kind].i_seed[j]++; */
+       for(j = 0; j <= RNG_Table[RNG_kind].n_seed - 1; j++)
+       if(!RNG_Table[RNG_kind].i_seed[j]) RNG_Table[RNG_kind].i_seed[j]++; */
 
-    switch(kind) {
+    switch(RNG_kind) {
     case WICHMANN_HILL:
 	I1 = I1 % 30269; I2 = I2 % 30307; I3 = I3 % 30323;
 
@@ -175,14 +212,14 @@ static void FixupSeeds(RNGtype kind, int initial)
     case MERSENNE_TWISTER:
 	if(initial) I1 = 624;
 	 /* No action unless user has corrupted .Random.seed */
-	if(I1 <= 0) I1 = 624; 
+	if(I1 <= 0) I1 = 624;
 	/* check for all zeroes */
 	for (j = 1; j <= 624; j++)
-	    if(RNG_Table[kind].i_seed[j] != 0) {
+	    if(RNG_Table[RNG_kind].i_seed[j] != 0) {
 		notallzero = 1;
 		break;
 	    }
-	if(!notallzero) Randomize(kind);
+	if(!notallzero) Randomize(RNG_kind);
 	break;
 
     case KNUTH_TAOCP:
@@ -190,16 +227,36 @@ static void FixupSeeds(RNGtype kind, int initial)
 	if(KT_pos <= 0) KT_pos = 100;
 	/* check for all zeroes */
 	for (j = 0; j < 100; j++)
-	    if(RNG_Table[kind].i_seed[j] != 0) {
+	    if(RNG_Table[RNG_kind].i_seed[j] != 0) {
 		notallzero = 1;
 		break;
 	    }
-	if(!notallzero) Randomize(kind);	
+	if(!notallzero) Randomize(RNG_kind);
 	break;
     case USER_UNIF:
 	break;
+    case LECUYER_CMRG:
+	/* first set: not all zero, in [0, m1)
+	   second set: not all zero, in [0, m2) */
+    {
+	unsigned int tmp;
+	int allOK = 1;
+	for (j = 0; j < 3; j++) {
+	    tmp = RNG_Table[RNG_kind].i_seed[j];
+	    if(tmp != 0) notallzero = 1;
+	    if (tmp >= m1) allOK = 0;
+	}
+	if(!notallzero || !allOK) Randomize(RNG_kind);
+	for (j = 3; j < 6; j++) {
+	    tmp = RNG_Table[RNG_kind].i_seed[j];
+	    if(tmp != 0) notallzero = 1;
+	    if (tmp >= m2) allOK = 0;
+	}
+	if(!notallzero || !allOK) Randomize(RNG_kind);
+    }
+    break;
     default:
-	error(_("FixupSeeds: unimplemented RNG kind %d"), kind);
+	error(_("FixupSeeds: unimplemented RNG kind %d"), RNG_kind);
     }
 }
 
@@ -219,6 +276,7 @@ static void RNG_Init(RNGtype kind, Int32 seed)
     case MARSAGLIA_MULTICARRY:
     case SUPER_DUPER:
     case MERSENNE_TWISTER:
+	/* i_seed[0] is mti, *but* this is needed for historical consistency */
 	for(j = 0; j < RNG_Table[kind].n_seed; j++) {
 	    seed = (69069 * seed + 1);
 	    RNG_Table[kind].i_seed[j] = seed;
@@ -226,15 +284,22 @@ static void RNG_Init(RNGtype kind, Int32 seed)
 	FixupSeeds(kind, 1);
 	break;
     case KNUTH_TAOCP:
-	RNG_Init_KT(seed);
+	RNG_Init_R_KT(seed);
 	break;
     case KNUTH_TAOCP2:
 	RNG_Init_KT2(seed);
 	break;
+    case LECUYER_CMRG:
+	for(j = 0; j < RNG_Table[kind].n_seed; j++) {
+	    seed = (69069 * seed + 1);
+	    while(seed >= m2) seed = (69069 * seed + 1);
+	    RNG_Table[kind].i_seed[j] = seed;
+	}
+	break;
     case USER_UNIF:
 	User_unif_fun = R_FindSymbol("user_unif_rand", "", NULL);
 	if (!User_unif_fun) error(_("'user_unif_rand' not in load table"));
-	User_unif_init = R_FindSymbol("user_unif_init", "", NULL);
+	User_unif_init = (UnifInitFun) R_FindSymbol("user_unif_init", "", NULL);
 	if (User_unif_init) (void) User_unif_init(seed);
 	User_unif_nseed = R_FindSymbol("user_unif_nseed", "", NULL);
 	User_unif_seedloc = R_FindSymbol("user_unif_seedloc", "",  NULL);
@@ -258,73 +323,81 @@ static void RNG_Init(RNGtype kind, Int32 seed)
     }
 }
 
-#include <time.h>
+unsigned int TimeToSeed(void); /* datetime.c */
+
 static void Randomize(RNGtype kind)
 {
-/* Only called by  GetRNGstate() when there's no .Random.seed */
+/* Only called by  GetRNGstate() when there is no .Random.seed */
+    RNG_Init(kind, TimeToSeed());
+}
 
-    RNG_Init(kind, (Int32) time(NULL));
+static void GetRNGkind(SEXP seeds)
+{
+    /* Load RNG_kind, N01_kind from .Random.seed if present */
+    int tmp, *is;
+    RNGtype newRNG; N01type newN01;
+
+    if (isNull(seeds))
+	seeds = findVarInFrame(R_GlobalEnv, R_SeedsSymbol);
+    if (seeds == R_UnboundValue) return;
+    if (!isInteger(seeds)) {
+	if (seeds == R_MissingArg) /* How can this happen? */
+	    error(_(".Random.seed is a missing argument with no default"));
+	error(_(".Random.seed is not an integer vector but of type '%s'"),
+		type2char(TYPEOF(seeds)));
+    }
+    is = INTEGER(seeds);
+    tmp = is[0];
+    /* avoid overflow here: max current value is 705 */
+    if (tmp == NA_INTEGER || tmp < 0 || tmp > 1000)
+	error(_(".Random.seed[1] is not a valid integer"));
+    newRNG = (RNGtype) (tmp % 100);
+    newN01 = (N01type) (tmp / 100);
+    if (newN01 > KINDERMAN_RAMAGE)
+	error(_(".Random.seed[0] is not a valid Normal type"));
+    switch(newRNG) {
+    case WICHMANN_HILL:
+    case MARSAGLIA_MULTICARRY:
+    case SUPER_DUPER:
+    case MERSENNE_TWISTER:
+    case KNUTH_TAOCP:
+    case KNUTH_TAOCP2:
+    case LECUYER_CMRG:
+	break;
+    case USER_UNIF:
+	if(!User_unif_fun)
+	    error(_(".Random.seed[1] = 5 but no user-supplied generator"));
+	break;
+    default:
+	error(_(".Random.seed[1] is not a valid RNG kind (code)"));
+    }
+    RNG_kind = newRNG; N01_kind = newN01;
+    return;
 }
 
 
 void GetRNGstate()
 {
     /* Get  .Random.seed  into proper variables */
-    int len_seed, j, tmp;
+    int len_seed;
     SEXP seeds;
-    RNGtype newRNG; N01type newN01;
-    
+
     /* look only in the workspace */
     seeds = findVarInFrame(R_GlobalEnv, R_SeedsSymbol);
     if (seeds == R_UnboundValue) {
 	Randomize(RNG_kind);
-    }
-    else {
-	seeds = coerceVector(seeds, INTSXP);
-	if (seeds == R_MissingArg)
-	    error(_(".Random.seed is a missing argument with no default"));
-	if (!isVector(seeds))
-	    error(_(".Random.seed is not a vector"));
-	tmp = INTEGER(seeds)[0];
-	if (tmp == NA_INTEGER)
-	    error(_(".Random.seed[1] is not a valid integer"));
-	newRNG = tmp % 100;
-	newN01 = tmp / 100;
-	/*if (RNG_kind > USER_UNIF || RNG_kind < 0) {
-	    warning(".Random.seed was invalid: re-initializing");
-	    RNG_kind = RNG_DEFAULT;
-	    }*/
-	if (newN01 < 0 || newN01 > KINDERMAN_RAMAGE)
-	    error(_(".Random.seed[0] is not a valid Normal type"));
- 	switch(newRNG) {
- 	case WICHMANN_HILL:
- 	case MARSAGLIA_MULTICARRY:
- 	case SUPER_DUPER:
- 	case MERSENNE_TWISTER:
- 	case KNUTH_TAOCP:
- 	case KNUTH_TAOCP2:
-	    break;
- 	case USER_UNIF:
-	    if(!User_unif_fun)
-		error(_(".Random.seed[1] = 5 but no user-supplied generator"));
-	    break;
-	default:
-	    error(_(".Random.seed[1] is not a valid RNG kind (code)"));
-	}
-	RNG_kind = newRNG; N01_kind = newN01;
+    } else {
+	GetRNGkind(seeds);
 	len_seed = RNG_Table[RNG_kind].n_seed;
+	/* Not sure whether this test is needed: wrong for USER_UNIF */
 	if(LENGTH(seeds) > 1 && LENGTH(seeds) < len_seed + 1)
 	    error(_(".Random.seed has wrong length"));
-	if(LENGTH(seeds) == 1)
+	if(LENGTH(seeds) == 1 && RNG_kind != USER_UNIF)
 	    Randomize(RNG_kind);
 	else {
-	    for(j = 1; j <= len_seed; j++) {
-		tmp = INTEGER(seeds)[j];
-/* Some generators can generate NA_INTEGER as a valid integer value */
-/*		if(tmp == NA_INTEGER)
-		error(".Random.seed[%d] is not a valid integer", j+1);*/
-		RNG_Table[RNG_kind].i_seed[j - 1] = tmp;
-	    }
+	    int j, *is = INTEGER(seeds);
+	    for(j = 1; j <= len_seed; j++)
+		RNG_Table[RNG_kind].i_seed[j - 1] = is[j];
 	    FixupSeeds(RNG_kind, 0);
 	}
     }
@@ -335,15 +408,14 @@ void PutRNGstate()
     /* Copy out seeds to  .Random.seed  */
     int len_seed, j;
     SEXP seeds;
-    
-    if (RNG_kind < 0 || RNG_kind > KNUTH_TAOCP2 ||
-	N01_kind < 0 || N01_kind > KINDERMAN_RAMAGE) {
+
+    if (RNG_kind > LECUYER_CMRG || N01_kind > KINDERMAN_RAMAGE) {
 	warning("Internal .Random.seed is corrupt: not saving");
 	return;
     }
-    
+
     len_seed = RNG_Table[RNG_kind].n_seed;
-   
+
     PROTECT(seeds = allocVector(INTSXP, len_seed + 1));
 
     INTEGER(seeds)[0] = RNG_kind + 100 * N01_kind;
@@ -367,8 +439,9 @@ static void RNGkind(RNGtype newkind)
     case SUPER_DUPER:
     case MERSENNE_TWISTER:
     case KNUTH_TAOCP:
-    case KNUTH_TAOCP2:
     case USER_UNIF:
+    case KNUTH_TAOCP2:
+    case LECUYER_CMRG:
 	break;
     default:
 	error(_("RNGkind: unimplemented RNG kind %d"), newkind);
@@ -381,15 +454,17 @@ static void RNGkind(RNGtype newkind)
 
 static void Norm_kind(N01type kind)
 {
+    /* N01type is an enumeration type, so this will probably get
+       mapped to an unsigned integer type. */
     if (kind == -1) kind = N01_DEFAULT;
-    if (kind < 0 || kind > KINDERMAN_RAMAGE)
+    if (kind > KINDERMAN_RAMAGE)
 	error(_("invalid Normal type in RNGkind"));
     if (kind == USER_NORM) {
 	User_norm_fun = R_FindSymbol("user_norm_rand", "", NULL);
 	if (!User_norm_fun) error(_("'user_norm_rand' not in load table"));
     }
     GetRNGstate(); /* might not be initialized */
-    if (kind == BOX_MULLER) 
+    if (kind == BOX_MULLER)
 	BM_norm_keep = 0.0; /* zap Box-Muller history */
     N01_kind = kind;
     PutRNGstate();
@@ -398,44 +473,45 @@ static void Norm_kind(N01type kind)
 
 /*------ .Internal interface ------------------------*/
 
-SEXP do_RNGkind (SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_RNGkind (SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, rng, norm;
 
     checkArity(op,args);
+    GetRNGstate(); /* might not be initialized */
     PROTECT(ans = allocVector(INTSXP, 2));
     INTEGER(ans)[0] = RNG_kind;
     INTEGER(ans)[1] = N01_kind;
     rng = CAR(args);
     norm = CADR(args);
+    GetRNGkind(R_NilValue); /* pull from .Random.seed if present */
     if(!isNull(rng)) { /* set a new RNG kind */
-	RNGkind(asInteger(rng));
+	RNGkind((RNGtype) asInteger(rng));
     }
     if(!isNull(norm)) { /* set a new normal kind */
-	Norm_kind(asInteger(norm));
+	Norm_kind((N01type) asInteger(norm));
     }
     UNPROTECT(1);
     return ans;
 }
 
 
-SEXP do_setseed (SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_setseed (SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP skind;
+    SEXP skind, nkind;
     int seed;
-    RNGtype kind;
 
     checkArity(op,args);
     seed = asInteger(CAR(args));
     if (seed == NA_INTEGER)
 	error(_("supplied seed is not a valid integer"));
     skind = CADR(args);
-    if (!isNull(skind)) {
-	kind = asInteger(skind);
-	RNGkind(kind);
-    } else
-	kind = RNG_kind;
-    RNG_Init(RNG_kind, (Int32) seed);
+    nkind = CADDR(args);
+    GetRNGkind(R_NilValue); /* pull RNG_kind, N01_kind from 
+			       .Random.seed if present */
+    if (!isNull(skind)) RNGkind((RNGtype) asInteger(skind));
+    if (!isNull(nkind)) Norm_kind((N01type) asInteger(nkind));
+    RNG_Init(RNG_kind, (Int32) seed); /* zaps BM history */
     PutRNGstate();
     return R_NilValue;
 }
@@ -524,7 +600,7 @@ MT_sgenrand(Int32 seed)
     (seed_array[0]&UPPER_MASK), seed_array[1], ..., seed_array[N-1]
    can take any values except all zeros.                             */
 
-static double MT_genrand()
+static double MT_genrand(void)
 {
     Int32 y;
     static Int32 mag01[2]={0x0, MATRIX_A};
@@ -533,23 +609,23 @@ static double MT_genrand()
     mti = dummy[0];
 
     if (mti >= N) { /* generate N words at one time */
-        int kk;
+	int kk;
 
-        if (mti == N+1)   /* if sgenrand() has not been called, */
-            MT_sgenrand(4357); /* a default initial seed is used   */
+	if (mti == N+1)   /* if sgenrand() has not been called, */
+	    MT_sgenrand(4357); /* a default initial seed is used   */
 
-        for (kk = 0; kk < N - M; kk++) {
-            y = (mt[kk] & UPPER_MASK) | (mt[kk+1] & LOWER_MASK);
-            mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1];
-        }
-        for (; kk < N - 1; kk++) {
-            y = (mt[kk] & UPPER_MASK) | (mt[kk+1] & LOWER_MASK);
-            mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1];
-        }
-        y = (mt[N-1] & UPPER_MASK) | (mt[0] & LOWER_MASK);
-        mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1];
+	for (kk = 0; kk < N - M; kk++) {
+	    y = (mt[kk] & UPPER_MASK) | (mt[kk+1] & LOWER_MASK);
+	    mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1];
+	}
+	for (; kk < N - 1; kk++) {
+	    y = (mt[kk] & UPPER_MASK) | (mt[kk+1] & LOWER_MASK);
+	    mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1];
+	}
+	y = (mt[N-1] & UPPER_MASK) | (mt[0] & LOWER_MASK);
+	mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1];
 
-        mti = 0;
+	mti = 0;
     }
 
     y = mt[mti++];
@@ -562,45 +638,27 @@ static double MT_genrand()
     return ( (double)y * 2.3283064365386963e-10 ); /* reals: [0,1)-interval */
 }
 
+/*
+   The following code was taken from earlier versions of
+   http://www-cs-faculty.stanford.edu/~knuth/programs/rng.c-old
+   http://www-cs-faculty.stanford.edu/~knuth/programs/rng.c
+*/
+
+
 #define long Int32
-#define Void void
-#define void static void
 #define ran_arr_buf       R_KT_ran_arr_buf
 #define ran_arr_cycle     R_KT_ran_arr_cycle
 #define ran_arr_ptr       R_KT_ran_arr_ptr
 #define ran_arr_sentinel  R_KT_ran_arr_sentinel
 #define ran_x             dummy
 
-/* ===================  Knuth TAOCP ========================== */
-
-         /* Please do NOT alter this part of the code */
-
-/*    This program by D E Knuth is in the public domain and freely copyable
- *    AS LONG AS YOU MAKE ABSOLUTELY NO CHANGES!
- *    It is explained in Seminumerical Algorithms, 3rd edition, Section 3.6
- *    (or in the errata to the 2nd edition --- see
- *        http://www-cs-faculty.stanford.edu/~knuth/taocp.html
- *    in the changes to pages 171 and following of Volume 2).              */
-
-/*    If you find any bugs, please report them immediately to
- *                 taocp@cs.stanford.edu
- *    (and you will be rewarded if the bug is genuine). Thanks!            */
-
-/************ see the book for explanations and caveats! *******************/
-
-/* the old C calling conventions are used here, for reasons of portability */
-
 #define KK 100                     /* the long lag */
 #define LL  37                     /* the short lag */
 #define MM (1L<<30)                 /* the modulus */
+#define TT  70   /* guaranteed separation between streams */
 #define mod_diff(x,y) (((x)-(y))&(MM-1)) /* subtraction mod MM */
-
-/*long ran_x[KK]; */                   /* the generator state */
-
-/* void ran_array(long aa[],int n) */
-void ran_array(aa,n)    /* put n new random numbers in aa */
-  long *aa;   /* destination */
-  int n;      /* array length (must be at least KK) */
+#define is_odd(x)  ((x)&1)          /* units bit of x */
+static void ran_array(long aa[],int n)    /* put n new random numbers in aa */
 {
   register int i,j;
   for (j=0;j<KK;j++) aa[j]=ran_x[j];
@@ -608,91 +666,32 @@ void ran_array(aa,n)    /* put n new random numbers in aa */
   for (i=0;i<LL;i++,j++) ran_x[i]=mod_diff(aa[j-KK],aa[j-LL]);
   for (;i<KK;i++,j++) ran_x[i]=mod_diff(aa[j-KK],ran_x[i-LL]);
 }
-
-#define TT  70   /* guaranteed separation between streams */
-#define is_odd(x)  ((x)&1)          /* units bit of x */
-#define evenize(x) ((x)&(MM-2))   /* make x even */
-
-/* void ran_start(long seed) */
-void ran_start(seed)    /* do this before using ran_array */
-  long seed;            /* selector for different streams */
-{
-  register int t,j;
-  long x[KK+KK-1];              /* the preparation buffer */
-  register long ss=evenize(seed+2);
-  for (j=0;j<KK;j++) {
-    x[j]=ss;                      /* bootstrap the buffer */
-    ss<<=1; if (ss>=MM) ss-=MM-2; /* cyclic shift 29 bits */
-  }
-  for (;j<KK+KK-1;j++) x[j]=0;
-  x[1]++;              /* make x[1] (and only x[1]) odd */
-  ss=seed&(MM-1);
-  t=TT-1; while (t) {
-    for (j=KK-1;j>0;j--) x[j+j]=x[j];  /* "square" */
-    for (j=KK+KK-2;j>KK-LL;j-=2) x[KK+KK-1-j]=evenize(x[j]);
-    for (j=KK+KK-2;j>=KK;j--) if(is_odd(x[j])) {
-      x[j-(KK-LL)]=mod_diff(x[j-(KK-LL)],x[j]);
-      x[j-KK]=mod_diff(x[j-KK],x[j]);
-    }
-    if (is_odd(ss)) {              /* "multiply by z" */
-      for (j=KK;j>0;j--)  x[j]=x[j-1];
-      x[0]=x[KK];            /* shift the buffer cyclically */
-      if (is_odd(x[KK])) x[LL]=mod_diff(x[LL],x[KK]);
-    }
-    if (ss) ss>>=1; else t--;
-  }
-  for (j=0;j<LL;j++) ran_x[j+KK-LL]=x[j];
-  for (;j<KK;j++) ran_x[j-LL]=x[j];
-}
-
-/* the following routines are from exercise 3.6--15 */
-/* after calling ran_start, get new randoms by, e.g., "x=ran_arr_next()" */
-
 #define QUALITY 1009 /* recommended quality level for high-res use */
-long ran_arr_buf[QUALITY];
-long ran_arr_sentinel=-1;
-long *ran_arr_ptr=&ran_arr_sentinel; /* the next random number, or -1 */
+static long ran_arr_buf[QUALITY];
+static long ran_arr_sentinel=(long)-1;
+static long *ran_arr_ptr=&ran_arr_sentinel; /* the next random number, or -1 */
 
-#define ran_arr_next() (*ran_arr_ptr>=0? *ran_arr_ptr++: ran_arr_cycle())
-long ran_arr_cycle()
+static long ran_arr_cycle(void)
 {
   ran_array(ran_arr_buf,QUALITY);
-  ran_arr_buf[100]=-1;
+  ran_arr_buf[KK]=-1;
   ran_arr_ptr=ran_arr_buf+1;
   return ran_arr_buf[0];
 }
 
-/* ===================== end of Knuth's code ====================== */
-
-static Int32 KT_next()
-{
-    if(KT_pos >= 100) {
-	ran_arr_cycle();
-	KT_pos = 0;
-    }
-    return ran_x[(KT_pos)++];
-}
-
-Void RNG_Init_KT(Int32 seed)
-{
-    ran_start(seed % 1073741821);
-    KT_pos = 100;
-}
-
-#define ran_start ran_start2002
-
 /* ===================  Knuth TAOCP  2002 ========================== */
 
+/*    This program by D E Knuth is in the public domain and freely copyable.
+ *    It is explained in Seminumerical Algorithms, 3rd edition, Section 3.6
+ *    (or in the errata to the 2nd edition --- see
+ *        http://www-cs-faculty.stanford.edu/~knuth/taocp.html
+ *    in the changes to Volume 2 on pages 171 and following).              */
+
 /*    N.B. The MODIFICATIONS introduced in the 9th printing (2002) are
       included here; there's no backwards compatibility with the original. */
 
 
-#ifdef __STDC__
-void ran_start(long seed)
-#else
-void ran_start(seed)    /* do this before using ran_array */
-  long seed;            /* selector for different streams */
-#endif
+static void ran_start(long seed)
 {
   register int t,j;
   long x[KK+KK-1];              /* the preparation buffer */
@@ -702,7 +701,7 @@ void ran_start(seed)    /* do this before using ran_array */
     ss<<=1; if (ss>=MM) ss-=MM-2; /* cyclic shift 29 bits */
   }
   x[1]++;              /* make x[1] (and only x[1]) odd */
-  for (ss=seed&(MM-1),t=TT-1; t; ) {       
+  for (ss=seed&(MM-1),t=TT-1; t; ) {
     for (j=KK-1;j>0;j--) x[j+j]=x[j], x[j+j-1]=0; /* "square" */
     for (j=KK+KK-2;j>=KK;j--)
       x[j-(KK-LL)]=mod_diff(x[j-(KK-LL)],x[j]),
@@ -721,9 +720,31 @@ void ran_start(seed)    /* do this before using ran_array */
 }
 /* ===================== end of Knuth's code ====================== */
 
-
-Void RNG_Init_KT2(Int32 seed)
+static void RNG_Init_KT2(Int32 seed)
 {
     ran_start(seed % 1073741821);
     KT_pos = 100;
 }
+
+static Int32 KT_next(void)
+{
+    if(KT_pos >= 100) {
+	ran_arr_cycle();
+	KT_pos = 0;
+    }
+    return ran_x[(KT_pos)++];
+}
+
+static void RNG_Init_R_KT(Int32 seed)
+{
+    SEXP fun, sseed, call, ans;
+    fun = findVar1(install(".TAOCP1997init"), R_BaseEnv, CLOSXP, FALSE);
+    if(fun == R_UnboundValue)
+	error("function '.TAOCP1997init' is missing");
+    PROTECT(sseed = ScalarInteger(seed % 1073741821));
+    PROTECT(call = lang2(fun, sseed));
+    ans = eval(call, R_GlobalEnv);
+    memcpy(dummy, INTEGER(ans), 100*sizeof(int));
+    UNPROTECT(2);
+    KT_pos = 100;
+}
