@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1996 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2008 The R Development Core Team
+ *  Copyright (C) 1997-2011 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,9 +18,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole string, but
-   use of translateChar assumes that strings are native.
- */
 
 /*
   This is an effort to merge the 3 different dynload.c files in the
@@ -39,31 +36,31 @@
 
 /*  Dynamic Loading Support
  *
- *  This module provides support for run-time loading of shared libraries
- *  access to symbols within such libraries via .C and .Fortran.  This is
+ *  This module provides support for run-time loading of shared objects
+ *  access to symbols within such objects via .C and .Fortran.  This is
  *  done under Unix with dlopen, dlclose and dlsym (the exception is
- *  hpux, where we use compatibility code provided by Luke Tierney.
+ *  hpux, where we use compatibility code provided by Luke Tierney).
  *  There are two cases:
  *
  *
  *  1. The dlopen interface is available.
  *
- *  In this case all symbol location in packages is done using the dlopen routines.
- *  We maintain a list of currently loaded shared libraries in an array
- *  called "LoadedDLL" with the number of currenly loaded libraries
- *  being "CountDLL".  To locate a symbol, we probe the loaded libraries
- *  in order until the symbol is located.  If we do not find a symbol
- *  in the loaded libraries, we search the executable itself.  This
- *  search is not very efficient, but this probably pales into
- *  insignificance when compared with the inefficiencies in the R
- *  interpreter.
+ *  In this case all symbol location in packages is done using the
+ *  dlopen routines.  We maintain a list of currently loaded shared
+ *  objects in an array called "LoadedDLL" with the number of currently
+ *  loaded objects being "CountDLL".  To locate a symbol, we probe
+ *  the loaded objects in order until the symbol is located.  If we
+ *  do not find a symbol in the loaded objects, we search the
+ *  executable itself.  This search is not very efficient, but this
+ *  probably pales into insignificance when compared with the
+ *  inefficiencies in the R interpreter.
  *
- *  Loading and unloading of shared libraries is done via the routines
- *  AddDLL and DeleteDLL.  These routines maintain the list of currently
- *  loaded libraries.  When a library is added, any existing reference
- *  to that library are deleted and then the library is inserted at the
- *  start of the search list.  This way, symbols in more recently loaded
- *  libraries are found first.
+ *  Loading and unloading of shared objects is done via the routines
+ *  AddDLL and DeleteDLL.  These routines maintain the list of
+ *  currently loaded objects.  When an object is added, any existing
+ *  reference to that object is deleted and then the object is
+ *  inserted at the start of the search list.  This way, symbols in
+ *  more recently loaded objects are found first.
  *
  *
  *  Accessing native routines in base (the R executable).
@@ -133,7 +130,7 @@
 
 #ifdef HAVE_DYNAMIC_LOADING
 
-#ifdef CACHE_DLL_SYM
+#ifdef CACHE_DLL_SYM  /* NOT USED */
 /* keep a record of symbols that have been found */
 R_CPFun CPFun[100];
 int nCPFun = 0;
@@ -187,14 +184,6 @@ DllInfo *R_getEmbeddingDllInfo()
     return dll;
 }
 
-#ifdef UNUSED
-DllInfo *
-getBaseDllInfo()
-{
-    return(&LoadedDLL[0]);
-}
-#endif
-
 Rboolean R_useDynamicSymbols(DllInfo *info, Rboolean value)
 {
     Rboolean old;
@@ -222,13 +211,13 @@ R_addExternalRoutine(DllInfo *info,
 
 
 /*
- Returns a reference to the DllInfo object associated with the dynamic library
+ Returns a reference to the DllInfo object associated with the shared object
  with the path name `path'. This ensures uniqueness rather than having the
- undesirable situation of two libraries with the same name but in different
+ undesirable situation of two object with the same name but in different
  directories.
  This is available so that it can be called from arbitrary C routines
  that need to call R_registerRoutines(). The initialization routine
- R_init_<library name> is passed the DllInfo reference as an argument.
+ R_init_<object name> is passed the DllInfo reference as an argument.
  Other routines must explicitly request it using this routine.
  */
 DllInfo *
@@ -243,10 +232,11 @@ R_getDllInfo(const char *path)
 }
 
 /*
-  Explicitly register the native routines for use in .Call(), .C() and .Fortran()
-  functions. These registered values are used to resolve symbols in a library
-  that makes a call to this routine, rather than the usual dynamic resolution
-  done by dlsym() or the equivalent on the different platforms.
+  Explicitly register the native routines for use in .Call(), .C() and
+  .Fortran() functions. These registered values are used to resolve
+  symbols in an object that makes a call to this routine, rather than
+  the usual dynamic resolution done by dlsym() or the equivalent on
+  the different platforms.
  */
 int
 R_registerRoutines(DllInfo *info, const R_CMethodDef * const croutines,
@@ -268,7 +258,8 @@ R_registerRoutines(DllInfo *info, const R_CMethodDef * const croutines,
 
     if(croutines) {
 	for(num=0; croutines[num].name != NULL; num++) {;}
-	info->CSymbols = (Rf_DotCSymbol*)calloc(num, sizeof(Rf_DotCSymbol));
+	info->CSymbols = (Rf_DotCSymbol*)calloc((size_t) num,
+						sizeof(Rf_DotCSymbol));
 	info->numCSymbols = num;
 	for(i = 0; i < num; i++) {
 	    R_addCRoutine(info, croutines+i, info->CSymbols + i);
@@ -278,7 +269,8 @@ R_registerRoutines(DllInfo *info, const R_CMethodDef * const croutines,
     if(fortranRoutines) {
 	for(num=0; fortranRoutines[num].name != NULL; num++) {;}
 	info->FortranSymbols =
-	    (Rf_DotFortranSymbol*)calloc(num, sizeof(Rf_DotFortranSymbol));
+	    (Rf_DotFortranSymbol*)calloc((size_t) num,
+					 sizeof(Rf_DotFortranSymbol));
 	info->numFortranSymbols = num;
 
 	for(i = 0; i < num; i++) {
@@ -290,7 +282,7 @@ R_registerRoutines(DllInfo *info, const R_CMethodDef * const croutines,
     if(callRoutines) {
 	for(num=0; callRoutines[num].name != NULL; num++) {;}
 	info->CallSymbols =
-	    (Rf_DotCallSymbol*)calloc(num, sizeof(Rf_DotCallSymbol));
+	    (Rf_DotCallSymbol*)calloc((size_t) num, sizeof(Rf_DotCallSymbol));
 	info->numCallSymbols = num;
 	for(i = 0; i < num; i++) {
 	    R_addCallRoutine(info, callRoutines+i, info->CallSymbols + i);
@@ -300,7 +292,8 @@ R_registerRoutines(DllInfo *info, const R_CMethodDef * const croutines,
     if(externalRoutines) {
 	for(num=0; externalRoutines[num].name != NULL; num++) {;}
 	info->ExternalSymbols =
-	    (Rf_DotExternalSymbol*)calloc(num, sizeof(Rf_DotExternalSymbol));
+	    (Rf_DotExternalSymbol*)calloc((size_t) num,
+					  sizeof(Rf_DotExternalSymbol));
 	info->numExternalSymbols = num;
 
 	for(i = 0; i < num; i++) {
@@ -317,10 +310,12 @@ R_setPrimitiveArgTypes(const R_FortranMethodDef * const croutine,
 		       Rf_DotFortranSymbol *sym)
 {
     sym->types = (R_NativePrimitiveArgType *)
-	malloc(sizeof(R_NativePrimitiveArgType) * croutine->numArgs);
+	malloc(sizeof(R_NativePrimitiveArgType) * (size_t) croutine->numArgs);
+    if(!sym->types)
+	error("allocation failure in R_setPrimitiveArgTypes");
     if(sym->types)
 	memcpy(sym->types, croutine->types,
-	       sizeof(R_NativePrimitiveArgType) * croutine->numArgs);
+	       sizeof(R_NativePrimitiveArgType) * (size_t) croutine->numArgs);
 
 }
 
@@ -329,10 +324,12 @@ R_setArgStyles(const R_FortranMethodDef * const croutine,
 	       Rf_DotFortranSymbol *sym)
 {
     sym->styles = (R_NativeArgStyle *)
-	malloc(sizeof(R_NativeArgStyle) * croutine->numArgs);
+	malloc(sizeof(R_NativeArgStyle) * (size_t) croutine->numArgs);
+    if(!sym->styles)
+	error("allocation failure in R_setArgStyles");
     if(sym->styles)
 	memcpy(sym->styles, croutine->styles,
-	       sizeof(R_NativeArgStyle) * croutine->numArgs);
+	       sizeof(R_NativeArgStyle) * (size_t) croutine->numArgs);
 }
 
 static void
@@ -523,8 +520,8 @@ static char DLLerror[DLLerrBUFSIZE] = "";
 /* the error message; length taken from ERRBUFSIZE in ./hpdlfcn.c  */
 
 	/* Inserts the specified DLL at the head of the DLL list */
-	/* Returns 1 if the library was successfully added */
-	/* and returns 0 if the library table is full or */
+	/* Returns 1 if the DLL was successfully added */
+	/* and returns 0 if the DLL table is full or */
 	/* or if dlopen fails for some reason. */
 
 
@@ -549,7 +546,7 @@ static DllInfo* AddDLL(const char *path, int asLocal, int now,
 
     info = R_RegisterDLL(handle, path);
 
-    /* Now look for an initializing routine named R_init_<library name>.
+    /* Now look for an initializing routine named R_init_<object name>.
        If it is present, we invoke it. It should take a reference to the
        DllInfo object currently being initialized.
     */
@@ -559,10 +556,12 @@ static DllInfo* AddDLL(const char *path, int asLocal, int now,
 #ifdef HAVE_NO_SYMBOL_UNDERSCORE
 	tmp = (char*) malloc(sizeof(char)*(strlen("R_init_") +
 					   strlen(info->name)+ 1));
+	if(!tmp) error("allocation failure in AddDLL");
 	sprintf(tmp, "%s%s","R_init_", info->name);
 #else
 	tmp = (char*) malloc(sizeof(char)*(strlen("R_init_") +
 					   strlen(info->name)+ 2));
+	if(!tmp) error("allocation failure in AddDLL");
 	sprintf(tmp, "_%s%s","R_init_", info->name);
 #endif
 	f = (DllInfoInitCall) R_osDynSymbol->dlsym(info, tmp);
@@ -582,7 +581,7 @@ static DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path)
     DllInfo *info;
 
     info = &LoadedDLL[CountDLL];
-    /* default is to use old-style dynamic lookup. Library's
+    /* default is to use old-style dynamic lookup.  The object's
        initialization routine can limit access by setting this to FALSE.
     */
     info->useDynamicLookup = TRUE;
@@ -638,9 +637,11 @@ addDLL(char *dpath, char *DLLname, HINSTANCE handle)
     LoadedDLL[CountDLL].numCSymbols = 0;
     LoadedDLL[CountDLL].numCallSymbols = 0;
     LoadedDLL[CountDLL].numFortranSymbols = 0;
+    LoadedDLL[CountDLL].numExternalSymbols = 0;
     LoadedDLL[CountDLL].CSymbols = NULL;
     LoadedDLL[CountDLL].CallSymbols = NULL;
     LoadedDLL[CountDLL].FortranSymbols = NULL;
+    LoadedDLL[CountDLL].ExternalSymbols = NULL;
     CountDLL++;
 
     return(ans);
@@ -697,7 +698,6 @@ Rf_lookupRegisteredExternalSymbol(DllInfo *info, const char *name)
 static DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
 				 R_RegisteredNativeSymbol *symbol)
 {
-    int fail = 0;
     NativeSymbolType purpose = R_ANY_SYM;
 
     if(symbol)
@@ -716,7 +716,6 @@ static DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
 
 	    return((DL_FUNC) sym->fun);
 	}
-	fail = 1;
     }
 
     if((purpose == R_ANY_SYM || purpose == R_CALL_SYM) &&
@@ -731,7 +730,6 @@ static DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
 	    }
 	    return((DL_FUNC) sym->fun);
 	}
-	fail = 1;
     }
 
     if((purpose == R_ANY_SYM || purpose == R_FORTRAN_SYM) &&
@@ -746,7 +744,6 @@ static DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
 	    }
 	    return((DL_FUNC) sym->fun);
 	}
-	fail = 1;
     }
 
     if((purpose == R_ANY_SYM || purpose == R_EXTERNAL_SYM) &&
@@ -761,7 +758,6 @@ static DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
 	    }
 	    return((DL_FUNC) sym->fun);
 	}
-	fail = 1;
     }
 
     return((DL_FUNC) NULL);
@@ -771,7 +767,7 @@ DL_FUNC attribute_hidden
 R_dlsym(DllInfo *info, char const *name,
 	R_RegisteredNativeSymbol *symbol)
 {
-    char buf[MAXIDSIZE+1];
+    char buf[strlen(name) + 4]; /* up to 3 additional underscores */
     DL_FUNC f;
 
     f = R_getDLLRegisteredSymbol(info, name, symbol);
@@ -781,9 +777,9 @@ R_dlsym(DllInfo *info, char const *name,
     if(info->useDynamicLookup == FALSE) return(NULL);
 
 #ifdef HAVE_NO_SYMBOL_UNDERSCORE
-    snprintf(buf, MAXIDSIZE+1, "%s", name);
+    sprintf(buf, "%s", name);
 #else
-    snprintf(buf, MAXIDSIZE+1,"_%s", name);
+    sprintf(buf, "_%s", name);
 #endif
 
 #ifdef HAVE_F77_UNDERSCORE
@@ -809,7 +805,7 @@ R_dlsym(DllInfo *info, char const *name,
     return f;
 }
 
-	/* R_FindSymbol checks whether one of the libraries */
+	/* R_FindSymbol checks whether one of the objects */
 	/* that have been loaded contains the symbol name and */
 	/* returns a pointer to that symbol upon success. */
 
@@ -818,7 +814,7 @@ R_dlsym(DllInfo *info, char const *name,
   which will specify the nature of the symbol expected by the
   caller, specifically whether it is for a .C(), .Call(),
   .Fortran(), .External(), generic, etc. invocation. This will
-  reduce the pool of possible symbols in the case of a library
+  reduce the pool of possible symbols in the case of an object
   that registers its routines.
   This is currently done via the value in symbol.
  */
@@ -872,11 +868,11 @@ static void GetFullDLLPath(SEXP call, char *buf, const char *const path)
 }
 
 	/* do_dynload implements the R-Interface for the */
-	/* loading of shared libraries */
+	/* loading of shared objects */
 
 /*
   Extended to support 2 additional arguments (3 in total).
-  First argument is the name of the library.
+  First argument is the name of the DLL.
   Second argument is a logical indicating whether we
   want the symbols to be kept in their own local symbol table
   or added to the global symbol table of the application.
@@ -886,7 +882,7 @@ static void GetFullDLLPath(SEXP call, char *buf, const char *const path)
   the symbols as they are invoked. This is useful for
   developers so that they can ensure that all the symbols
   are available before they release, and allows users to
-  call routines from "incomplete" libraries.
+  call routines from "incomplete" DLLs.
  */
 
 SEXP attribute_hidden do_dynload(SEXP call, SEXP op, SEXP args, SEXP env)
@@ -895,14 +891,14 @@ SEXP attribute_hidden do_dynload(SEXP call, SEXP op, SEXP args, SEXP env)
     DllInfo *info;
 
     checkArity(op,args);
-    if (!isString(CAR(args)) || length(CAR(args)) < 1)
+    if (!isString(CAR(args)) || LENGTH(CAR(args)) != 1)
 	error(_("character argument expected"));
     GetFullDLLPath(call, buf, translateChar(STRING_ELT(CAR(args), 0)));
     /* AddDLL does this DeleteDLL(buf); */
     info = AddDLL(buf, LOGICAL(CADR(args))[0], LOGICAL(CADDR(args))[0],
 		  translateChar(STRING_ELT(CADDDR(args), 0)));
     if(!info)
-	error(_("unable to load shared library '%s':\n  %s"), buf, DLLerror);
+	error(_("unable to load shared object '%s':\n  %s"), buf, DLLerror);
     return(Rf_MakeDLLInfo(info));
 }
 
@@ -911,11 +907,11 @@ SEXP attribute_hidden do_dynunload(SEXP call, SEXP op, SEXP args, SEXP env)
     char buf[2 * PATH_MAX];
 
     checkArity(op,args);
-    if (!isString(CAR(args)) || length(CAR(args)) < 1)
+    if (!isString(CAR(args)) || LENGTH(CAR(args)) != 1)
 	error(_("character argument expected"));
     GetFullDLLPath(call, buf, translateChar(STRING_ELT(CAR(args), 0)));
     if(!DeleteDLL(buf))
-	error(_("dynamic/shared library '%s\' was not loaded"), buf);
+	error(_("shared object '%s\' was not loaded"), buf);
     return R_NilValue;
 }
 
@@ -935,7 +931,27 @@ int R_moduleCdynload(const char *module, int local, int now)
 #endif
     res = AddDLL(dllpath, local, now, "");
     if(!res)
-	warning(_("unable to load shared library '%s':\n  %s"),
+	warning(_("unable to load shared object '%s':\n  %s"),
+		dllpath, DLLerror);
+    return res != NULL ? 1 : 0;
+}
+
+int R_cairoCdynload(int local, int now)
+{
+    char dllpath[PATH_MAX], *p = getenv("R_HOME"), *module = "cairo";
+    DllInfo *res;
+
+    if(!p) return 0;
+#ifdef R_ARCH
+    snprintf(dllpath, PATH_MAX, "%s/library/grDevices/libs/%s/%s%s", 
+	     p, R_ARCH, module, SHLIB_EXT);
+#else
+    snprintf(dllpath, PATH_MAX, "%s/library/grDevices/libs/%s%s", 
+	     p, module, SHLIB_EXT);
+#endif
+    res = AddDLL(dllpath, local, now, "");
+    if(!res)
+	warning(_("unable to load shared object '%s':\n  %s"),
 		dllpath, DLLerror);
     return res != NULL ? 1 : 0;
 }
@@ -944,14 +960,14 @@ int R_moduleCdynload(const char *module, int local, int now)
   Creates an R object representing the value of the
   function pointer given by `f'. This object has class
   NativeSymbol and can be used to relay symbols from
-  one library to another.
+  one DLL to another.
  */
 static SEXP
 Rf_MakeNativeSymbolRef(DL_FUNC f)
 {
     SEXP ref, klass;
 
-    PROTECT(ref = R_MakeExternalPtrFn(f, Rf_install("native symbol"),
+    PROTECT(ref = R_MakeExternalPtrFn(f, install("native symbol"),
 				      R_NilValue));
     PROTECT(klass = mkString("NativeSymbol"));
     setAttrib(ref, R_ClassSymbol, klass);
@@ -981,7 +997,7 @@ Rf_MakeRegisteredNativeSymbol(R_RegisteredNativeSymbol *symbol)
     *copy = *symbol;
 
     PROTECT(ref = R_MakeExternalPtr(copy,
-				    Rf_install("registered native symbol"),
+				    install("registered native symbol"),
 				    R_NilValue));
     R_RegisterCFinalizer(ref, freeRegisteredNativeSymbolCopy);
 
@@ -998,7 +1014,7 @@ Rf_makeDllObject(HINSTANCE inst)
 {
     SEXP ans;
 
-    PROTECT(ans = R_MakeExternalPtr(inst, Rf_install("DLLHandle"),
+    PROTECT(ans = R_MakeExternalPtr(inst, install("DLLHandle"),
 				    R_NilValue));
     setAttrib(ans, R_ClassSymbol, mkString("DLLHandle"));
     UNPROTECT(1);
@@ -1011,8 +1027,8 @@ Rf_makeDllInfoReference(HINSTANCE inst)
 {
     SEXP ans;
 
-    PROTECT(ans = R_MakeExternalPtr(inst, Rf_install("DLLInfo"),
-				    Rf_install("DLLInfo")));
+    PROTECT(ans = R_MakeExternalPtr(inst, install("DLLInfo"),
+				    install("DLLInfo")));
     setAttrib(ans, R_ClassSymbol, mkString("DLLInfoReference"));
     UNPROTECT(1);
 
@@ -1061,13 +1077,13 @@ Rf_MakeDLLInfo(DllInfo *info)
     return(ref);
 }
 
-/**
+/*
   This is the routine associated with the getNativeSymbolInfo()
-  function and it takes the name of a symbol and optionally a
-  library identifier (package usually) in which to restrict the search
+  function and it takes the name of a symbol and optionally an
+  object identifier (package usually) in which to restrict the search
   for this symbol. It resolves the symbol and returns it to the caller
   giving the symbol address, the package information (i.e. name and
-  fully qualified shared library name). If the symbol was explicitly
+  fully qualified shared object name). If the symbol was explicitly
   registered (rather than dynamically resolved by R), then we pass
   back that information also, giving the number of arguments it
   expects and the interface by which it should be called.
@@ -1091,7 +1107,7 @@ R_getSymbolInfo(SEXP sname, SEXP spackage, SEXP withRegistrationInfo)
 	if(TYPEOF(spackage) == STRSXP)
 	    package = translateChar(STRING_ELT(spackage, 0));
 	else if(TYPEOF(spackage) == EXTPTRSXP &&
-		R_ExternalPtrTag(spackage) == Rf_install("DLLInfo")) {
+		R_ExternalPtrTag(spackage) == install("DLLInfo")) {
 	    f = R_dlsym((DllInfo *) R_ExternalPtrAddr(spackage), name, &symbol);
 	    package = NULL;
 	} else
@@ -1114,12 +1130,22 @@ R_getDllTable()
     int i;
     SEXP ans;
 
+ again:
     PROTECT(ans = allocVector(VECSXP, CountDLL));
     for(i = 0; i < CountDLL; i++) {
 	SET_VECTOR_ELT(ans, i, Rf_MakeDLLInfo(&(LoadedDLL[i])));
     }
     setAttrib(ans, R_ClassSymbol, mkString("DLLInfoList"));
     UNPROTECT(1);
+
+    /* There is a problem here: The allocations can cause gc, and gc
+       may result in no longer referenced DLLs being unloaded.  So
+       CountDLL can be reduced during this loop.  A simple work-around
+       is to just try again until CountDLL at the end is the same as
+       it was at the beginning.  LT */
+    if (CountDLL != LENGTH(ans))
+	goto again;
+
     return(ans);
 }
 
@@ -1262,7 +1288,7 @@ R_getRegisteredRoutines(SEXP dll)
     const char * const names[] = {".C", ".Call", ".Fortran", ".External"};
 
     if(TYPEOF(dll) != EXTPTRSXP &&
-       R_ExternalPtrTag(dll) != Rf_install("DLLInfo"))
+       R_ExternalPtrTag(dll) != install("DLLInfo"))
 	error(_("R_getRegisteredRoutines() expects a DllInfo reference"));
 
     info = (DllInfo *) R_ExternalPtrAddr(dll);
