@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-7  the R Development Core Team
+ *  Copyright (C) 2000-10  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,9 +25,8 @@
 
 /* .Internal(lapply(X, FUN)) */
 
-/* This is a special, so has unevaluated arguments.  It is called from a
-   closure wrapper, so X and FUN are promises. */
-
+/* This is a special .Internal, so has unevaluated arguments.  It is
+   called from a closure wrapper, so X and FUN are promises. */
 SEXP attribute_hidden do_lapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP R_fcall, ans, names, X, XX, FUN;
@@ -79,66 +78,153 @@ SEXP attribute_hidden do_lapply(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
-#ifdef UNUSED
-/* .Internal(apply(X, X1, FUN)) */
-/* X is a matrix, and the last dimension is the one we want to
-   loop over */
+/* .Internal(vapply(X, FUN, FUN.VALUE, USE.NAMES)) */
 
-SEXP attribute_hidden do_apply(SEXP call, SEXP op, SEXP args, SEXP rho)
+/* This is a special .Internal */
+SEXP attribute_hidden do_vapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP R_fcall, X, Xd, X1, ans, FUN;
-    int i, j, nr, nc, inr;
+    SEXP R_fcall, ans, names=R_NilValue, rowNames=R_NilValue, X, XX, FUN, value, dim_v;
+    int i, n, commonLen, useNames,
+	rnk_v = -1; // = array_rank(value) := length(dim(value))
+    Rboolean array_value;
+    SEXPTYPE commonType;
+    PROTECT_INDEX index;
 
     checkArity(op, args);
-    X = CAR(args); args = CDR(args);
-    if(!isMatrix(X))
-	error(_("first argument is not a matrix"));
-    Xd = getAttrib(X, R_DimSymbol);
-    nr = INTEGER(Xd)[0];
-    nc = INTEGER(Xd)[1];
-    X1 = CAR(args); args = CDR(args);
-    FUN = CAR(args);
+    PROTECT(X = CAR(args));
+    PROTECT(XX = eval(CAR(args), rho));
+    FUN = CADR(args);  /* must be unevaluated for use in e.g. bquote */
+    PROTECT(value = eval(CADDR(args), rho));
+    if (!isVector(value)) error(_("FUN.VALUE must be a vector"));
+    useNames = asLogical(eval(CADDDR(args), rho));
+    if (useNames == NA_LOGICAL) error(_("invalid USE.NAMES value"));
 
-    PROTECT(R_fcall = LCONS(FUN, LCONS(X1, LCONS(R_DotsSymbol, R_NilValue))));
-    PROTECT(ans = allocVector(VECSXP, nc));
-    PROTECT(X1 = allocVector(TYPEOF(X), nr));
-    SETCADR(R_fcall, X1);
-    for(i = 0; i < nc; i++) {
-	switch(TYPEOF(X)) {
-	case REALSXP:
-	    for (j = 0, inr = i*nr; j < nr; j++)
-		REAL(X1)[j] = REAL(X)[j + inr];
-	    break;
-	case INTSXP:
-	    for (j = 0, inr = i*nr; j < nr; j++)
-		INTEGER(X1)[j] = INTEGER(X)[j + inr];
-	    break;
-	case LGLSXP:
-	    for (j = 0, inr = i*nr; j < nr; j++)
-		LOGICAL(X1)[j] = LOGICAL(X)[j + inr];
-	    break;
-	case CPLXSXP:
-	    for (j = 0, inr = i*nr; j < nr; j++)
-		COMPLEX(X1)[j] = COMPLEX(X)[j + inr];
-	    break;
-	case STRSXP:
-	    for (j = 0, inr = i*nr; j < nr; j++)
-		SET_STRING_ELT(X1, j, STRING_ELT(X, j + inr));
-	    break;
-	case RAWSXP:
-	    for (j = 0, inr = i*nr; j < nr; j++)
-		RAW(X1)[j] = RAW(X)[j + inr];
-	    break;
-	default:
-	    UNIMPLEMENTED_TYPE("apply", X);
-	}
-	/* careful: we have altered X1 and might have FUN = function(x) x */
-	SET_VECTOR_ELT(ans, i, duplicate(eval(R_fcall, rho)));
+    n = length(XX);
+    if (n == NA_INTEGER) error(_("invalid length"));
+
+    commonLen = length(value);
+    commonType = TYPEOF(value);
+    dim_v = getAttrib(value, R_DimSymbol);
+    array_value = (TYPEOF(dim_v) == INTSXP && LENGTH(dim_v) >= 1) ? TRUE : FALSE;
+    PROTECT(ans = allocVector(commonType, n*commonLen));
+    if (useNames) {
+    	PROTECT(names = getAttrib(XX, R_NamesSymbol));
+    	if (isNull(names) && TYPEOF(XX) == STRSXP) {
+    	    UNPROTECT(1);
+    	    PROTECT(names = XX);
+    	}
+    	PROTECT_WITH_INDEX(rowNames = getAttrib(value,
+						array_value ? R_DimNamesSymbol
+						: R_NamesSymbol),
+			   &index);
     }
-    UNPROTECT(3);
+    /* The R level code has ensured that XX is a vector.
+       If it is atomic we can speed things up slightly by
+       using the evaluated version.
+    */
+    {
+	SEXP ind, tmp;
+	/* Build call: FUN(XX[[<ind>]], ...) */
+
+	/* Notice that it is OK to have one arg to LCONS do memory
+	   allocation and not PROTECT the result (LCONS does memory
+	   protection of its args internally), but not both of them,
+	   since the computation of one may destroy the other */
+
+	PROTECT(ind = allocVector(INTSXP, 1));
+	if(isVectorAtomic(XX))
+	    PROTECT(tmp = LCONS(R_Bracket2Symbol,
+				LCONS(XX, LCONS(ind, R_NilValue))));
+	else
+	    PROTECT(tmp = LCONS(R_Bracket2Symbol,
+				LCONS(X, LCONS(ind, R_NilValue))));
+	PROTECT(R_fcall = LCONS(FUN,
+				LCONS(tmp, LCONS(R_DotsSymbol, R_NilValue))));
+
+	for(i = 0; i < n; i++) {
+	    int j;
+	    SEXPTYPE tmpType;
+	    INTEGER(ind)[0] = i + 1;
+	    tmp = eval(R_fcall, rho);
+	    if (length(tmp) != commonLen)
+	    	error(_("values must be length %d,\n but FUN(X[[%d]]) result is length %d"),
+	               commonLen, i+1, length(tmp));
+	    tmpType = TYPEOF(tmp);
+	    if (tmpType != commonType) {
+	    	Rboolean okay = FALSE;
+	    	switch (commonType) {
+	    	case CPLXSXP: okay = (tmpType == REALSXP) || (tmpType == INTSXP)
+	    	                    || (tmpType == LGLSXP); break;
+	    	case REALSXP: okay = (tmpType == INTSXP) || (tmpType == LGLSXP); break;
+	    	case INTSXP:  okay = (tmpType == LGLSXP); break;
+	        }
+	        if (!okay)
+	            error(_("values must be type '%s',\n but FUN(X[[%d]]) result is type '%s'"),
+	            	  type2char(commonType), i+1, type2char(tmpType));
+	        tmp = coerceVector(tmp, commonType);
+	    }
+	    /* Take row names from the first result only */
+	    if (i == 0 && useNames && isNull(rowNames))
+	    	REPROTECT(rowNames = getAttrib(tmp,
+					       array_value ? R_DimNamesSymbol : R_NamesSymbol),
+			  index);
+	    for (j = 0; j < commonLen; j++) {
+	    	switch (commonType) {
+	    	case CPLXSXP: COMPLEX(ans)[i*commonLen + j] = COMPLEX(tmp)[j]; break;
+	    	case REALSXP: REAL(ans)[i*commonLen + j] = REAL(tmp)[j]; break;
+	    	case INTSXP:  INTEGER(ans)[i*commonLen + j] = INTEGER(tmp)[j]; break;
+	    	case LGLSXP:  LOGICAL(ans)[i*commonLen + j] = LOGICAL(tmp)[j]; break;
+	    	case RAWSXP:  RAW(ans)[i*commonLen + j] = RAW(tmp)[j]; break;
+	    	case STRSXP:  SET_STRING_ELT(ans, i*commonLen + j, STRING_ELT(tmp, j)); break;
+	    	case VECSXP:  SET_VECTOR_ELT(ans, i*commonLen + j, VECTOR_ELT(tmp, j)); break;
+	    	default:
+	    	    error(_("type '%s' is not supported"), type2char(commonType));
+	    	}
+	    }
+	}
+	UNPROTECT(3);
+    }
+
+    if (commonLen != 1) {
+	SEXP dim;
+	rnk_v = array_value ? LENGTH(dim_v) : 1;
+	PROTECT(dim = allocVector(INTSXP, rnk_v+1));
+	if(array_value)
+	    for(int j=0; j < rnk_v; j++)
+		INTEGER(dim)[j] = INTEGER(dim_v)[j];
+	else
+	    INTEGER(dim)[0] = commonLen;
+	INTEGER(dim)[rnk_v] = n;
+	setAttrib(ans, R_DimSymbol, dim);
+	UNPROTECT(1);
+    }
+
+    if (useNames) {
+	if (commonLen == 1) {
+	    if(!isNull(names)) setAttrib(ans, R_NamesSymbol, names);
+	} else {
+	    if (!isNull(names) || !isNull(rowNames)) {
+		SEXP dimnames;
+		PROTECT(dimnames = allocVector(VECSXP, rnk_v+1));
+		if(array_value && !isNull(rowNames)) {
+		    if(TYPEOF(rowNames) != VECSXP || LENGTH(rowNames) != rnk_v)
+			// should never happen ..
+			error(_("dimnames(<value>) is neither NULL nor list of length %d"),
+			      rnk_v);
+		    for(int j=0; j < rnk_v; j++)
+			SET_VECTOR_ELT(dimnames, j, VECTOR_ELT(rowNames, j));
+		} else
+		    SET_VECTOR_ELT(dimnames, 0, rowNames);
+
+		SET_VECTOR_ELT(dimnames, rnk_v, names);
+		setAttrib(ans, R_DimNamesSymbol, dimnames);
+		UNPROTECT(1);
+	    }
+	}
+    }
+    UNPROTECT(useNames ? 6 : 4); /* X, XX, value, ans, and maybe names and rowNames */
     return ans;
 }
-#endif
 
 static SEXP do_one(SEXP X, SEXP FUN, SEXP classes, SEXP deflt,
 		   Rboolean replace, SEXP rho)
