@@ -166,7 +166,7 @@ void attribute_hidden InitArithmetic()
 
 /* Keep these two in step */
 /* FIXME: consider using
-    tmp = (LDOUBLE)x1 - floor(q) * (LDOUBLE)x2;
+    tmp = (long double)x1 - floor(q) * (long double)x2;
  */
 static double myfmod(double x1, double x2)
 {
@@ -1372,7 +1372,7 @@ static SEXP math2B(SEXP sa, SEXP sb, double (*f)(double, double, double *),
 	if (av > amax) amax = av;
     }
     nw = 1 + (long)floor(amax);
-    work = (double *) R_alloc(nw, sizeof(double));
+    work = (double *) R_alloc((size_t) nw, sizeof(double));
 
     mod_iterate(na, nb, ia, ib) {
 	ai = a[ia];
@@ -1407,8 +1407,8 @@ SEXP attribute_hidden do_math2(SEXP call, SEXP op, SEXP args, SEXP env)
     switch (PRIMVAL(op)) {
 
     case  0: return Math2(args, atan2);
-    case 10001: return Math2(args, fround);
-    case 10004: return Math2(args, fprec); /* signif() */
+    case 10001: return Math2(args, fround);/* round(), src/nmath/fround.c */
+    case 10004: return Math2(args, fprec); /* signif(), src/nmath/fprec.c */
 
     case  2: return Math2(args, lbeta);
     case  3: return Math2(args, beta);
@@ -1735,7 +1735,7 @@ static SEXP math3B(SEXP sa, SEXP sb, SEXP sc,
 	if (av > amax) amax = av;
     }
     nw = 1 + (long)floor(amax);
-    work = (double *) R_alloc(nw, sizeof(double));
+    work = (double *) R_alloc((size_t) nw, sizeof(double));
 
     mod_iterate3 (na, nb, nc, ia, ib, ic) {
 	ai = a[ia];
