@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2007  Robert Gentleman, Ross Ihaka and the
- *			      R Development Core Team
+ *  Copyright (C) 1997--2007  The R Development Core Team
  *  Copyright (C) 2002--2005  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -20,8 +19,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole string */
-
 /* Code to handle list / vector switch */
 
 #ifdef HAVE_CONFIG_H
@@ -68,7 +65,7 @@ static int HasNames(SEXP x)
 }
 
 static void
-AnswerType(SEXP x, int recurse, int usenames, struct BindData *data)
+AnswerType(SEXP x, int recurse, int usenames, struct BindData *data, SEXP call)
 {
     switch (TYPEOF(x)) {
     case NILSXP:
@@ -108,7 +105,7 @@ AnswerType(SEXP x, int recurse, int usenames, struct BindData *data)
 	    for (i = 0; i < n; i++) {
 		if (usenames && !data->ans_nnames)
 		    data->ans_nnames = HasNames(VECTOR_ELT(x, i));
-		AnswerType(VECTOR_ELT(x, i), recurse, usenames, data);
+		AnswerType(VECTOR_ELT(x, i), recurse, usenames, data, call);
 	    }
 	}
 	else {
@@ -126,7 +123,7 @@ AnswerType(SEXP x, int recurse, int usenames, struct BindData *data)
 		    if (!isNull(TAG(x))) data->ans_nnames = 1;
 		    else data->ans_nnames = HasNames(CAR(x));
 		}
-		AnswerType(CAR(x), recurse, usenames, data);
+		AnswerType(CAR(x), recurse, usenames, data, call);
 		x = CDR(x);
 	    }
 	}
@@ -140,6 +137,16 @@ AnswerType(SEXP x, int recurse, int usenames, struct BindData *data)
 	data->ans_length += 1;
 	break;
     }
+
+    /* check for overflow in ans_length. Objects are added one at a
+       time for each call to AnswerType so it is safe to check here.
+       Since sizes are signed, positive numbers, the overflow will
+       manifest itself as a negative result (both numbers will be
+       31-bit so we cannot overflow across the 32-bit boundary). If
+       our assumption (all lengths are signed) is violated, this won't
+       work so check when switching length types! */
+    if (data->ans_length < 0)
+	errorcall(call, _("resulting vector exceeds vector length limit in '%s'"), "AnswerType");
 }
 
 
@@ -266,6 +273,11 @@ LogicalAnswer(SEXP x, struct BindData *data, SEXP call)
 	for (i = 0; i < n; i++)
 	    LOGICAL(data->ans_ptr)[data->ans_length++] = INTEGER(x)[i];
 	break;
+    case RAWSXP:
+	n = LENGTH(x);
+	for (i = 0; i < n; i++)
+	    LOGICAL(data->ans_ptr)[data->ans_length++] = (int)RAW(x)[i];
+	break;
     default:
 	errorcall(call, _("type '%s' is unimplemented in '%s'"),
 		  type2char(TYPEOF(x)), "LogicalAnswer");
@@ -301,6 +313,11 @@ IntegerAnswer(SEXP x, struct BindData *data, SEXP call)
 	for (i = 0; i < n; i++)
 	    INTEGER(data->ans_ptr)[data->ans_length++] = INTEGER(x)[i];
 	break;
+    case RAWSXP:
+	n = LENGTH(x);
+	for (i = 0; i < n; i++)
+	    INTEGER(data->ans_ptr)[data->ans_length++] = (int)RAW(x)[i];
+	break;
     default:
 	errorcall(call, _("type '%s' is unimplemented in '%s'"),
 		  type2char(TYPEOF(x)), "IntegerAnswer");
@@ -349,6 +366,11 @@ RealAnswer(SEXP x, struct BindData *data, SEXP call)
 	    else REAL(data->ans_ptr)[data->ans_length++] = xi;
 	}
 	break;
+    case RAWSXP:
+	n = LENGTH(x);
+	for (i = 0; i < n; i++)
+	    REAL(data->ans_ptr)[data->ans_length++] = (int)RAW(x)[i];
+	break;
     default:
 	errorcall(call, _("type '%s' is unimplemented in '%s'"),
 		  type2char(TYPEOF(x)), "RealAnswer");
@@ -417,6 +439,16 @@ ComplexAnswer(SEXP x, struct BindData *data, SEXP call)
 	    data->ans_length++;
 	}
 	break;
+
+    case RAWSXP:
+	n = LENGTH(x);
+	for (i = 0; i < n; i++) {
+	    COMPLEX(data->ans_ptr)[data->ans_length].r = (int)RAW(x)[i];
+	    COMPLEX(data->ans_ptr)[data->ans_length].i = 0.0;
+	    data->ans_length++;
+	}
+	break;
+
     default:
 	errorcall(call, _("type '%s' is unimplemented in '%s'"),
 		  type2char(TYPEOF(x)), "ComplexAnswer");
@@ -460,10 +492,12 @@ static SEXP NewBase(SEXP base, SEXP tag)
     base = EnsureString(base);
     tag = EnsureString(tag);
     if (*CHAR(base) && *CHAR(tag)) { /* test of length */
-	const char *sb = translateChar(base), *st = translateChar(tag);
+	const char *sb = translateCharUTF8(base), *st = translateCharUTF8(tag);
 	cbuf = R_AllocStringBuffer(strlen(st) + strlen(sb) + 1, &cbuff);
 	sprintf(cbuf, "%s.%s", sb, st);
-	ans = mkChar(cbuf);
+	/* This isn't strictly correct as we do not know that all the
+	   components of the name were correctly translated. */
+	ans = mkCharCE(cbuf, CE_UTF8);
     }
     else if (*CHAR(tag)) {
 	ans = tag;
@@ -475,38 +509,39 @@ static SEXP NewBase(SEXP base, SEXP tag)
     return ans;
 }
 
-static SEXP NewName(SEXP base, SEXP tag, int i, int n, int seqno)
+static SEXP NewName(SEXP base, SEXP tag, int seqno)
 {
 /* Construct a new Name/Tag, using
  *	base.tag
  *	base<seqno>	or
  *	tag
  *
- * NOTE: i,n   are NOT used currently */
+ */
 
     SEXP ans;
     char *cbuf;
     base = EnsureString(base);
     tag = EnsureString(tag);
     if (*CHAR(base) && *CHAR(tag)) {
-	const char *sb = translateChar(base), *st = translateChar(tag);
+	const char *sb = translateCharUTF8(base), *st = translateCharUTF8(tag);
 	cbuf = R_AllocStringBuffer(strlen(sb) + strlen(st) + 1, &cbuff);
 	sprintf(cbuf, "%s.%s", sb, st);
-	ans = mkChar(cbuf);
+	ans = mkCharCE(cbuf, CE_UTF8);
     }
     else if (*CHAR(base)) {
 	const char *sb = translateChar(base);
-	cbuf = R_AllocStringBuffer(strlen(sb) + IndexWidth(seqno), &cbuff);
+	cbuf = R_AllocStringBuffer(strlen(sb) + (size_t) IndexWidth(seqno),
+				   &cbuff);
 	sprintf(cbuf, "%s%d", sb, seqno);
-	ans = mkChar(cbuf);
+	ans = mkCharCE(cbuf, CE_UTF8);
     }
     else if (*CHAR(tag)) {
 	if(tag == NA_STRING) ans = NA_STRING;
 	else {
-	    const char *st = translateChar(tag);
+	    const char *st = translateCharUTF8(tag);
 	    cbuf = R_AllocStringBuffer(strlen(st), &cbuff);
 	    sprintf(cbuf, "%s", st);
-	    ans = mkChar(cbuf);
+	    ans = mkCharCE(cbuf, CE_UTF8);
 	}
     }
     else ans = R_BlankString;
@@ -576,7 +611,7 @@ static void NewExtractNames(SEXP v, SEXP base, SEXP tag, int recurse,
 		if (namei == R_NilValue && nameData->count == 0)
 		    nameData->firstpos = data->ans_nnames;
 		nameData->count++;
-		namei = NewName(base, namei, i, n, ++(nameData->seqno));
+		namei = NewName(base, namei, ++(nameData->seqno));
 		SET_STRING_ELT(data->ans_names, (data->ans_nnames)++, namei);
 	    }
 	    v = CDR(v);
@@ -594,7 +629,7 @@ static void NewExtractNames(SEXP v, SEXP base, SEXP tag, int recurse,
 		if (namei == R_NilValue && nameData->count == 0)
 		    nameData->firstpos = data->ans_nnames;
 		nameData->count++;
-		namei = NewName(base, namei, i, n, ++(nameData->seqno));
+		namei = NewName(base, namei, ++(nameData->seqno));
 		SET_STRING_ELT(data->ans_names, (data->ans_nnames)++, namei);
 	    }
 	}
@@ -610,7 +645,7 @@ static void NewExtractNames(SEXP v, SEXP base, SEXP tag, int recurse,
 	    if (namei == R_NilValue && nameData->count == 0)
 		nameData->firstpos = data->ans_nnames;
 	    nameData->count++;
-	    namei = NewName(base, namei, i, n, ++(nameData->seqno));
+	    namei = NewName(base, namei, ++(nameData->seqno));
 	    SET_STRING_ELT(data->ans_names, (data->ans_nnames)++, namei);
 	}
 	break;
@@ -618,7 +653,7 @@ static void NewExtractNames(SEXP v, SEXP base, SEXP tag, int recurse,
 	if (nameData->count == 0)
 	    nameData->firstpos = data->ans_nnames;
 	nameData->count++;
-	namei = NewName(base, R_NilValue, 0, 1, ++(nameData->seqno));
+	namei = NewName(base, R_NilValue, ++(nameData->seqno));
 	SET_STRING_ELT(data->ans_names, (data->ans_nnames)++, namei);
     }
     if (tag != R_NilValue) {
@@ -685,6 +720,7 @@ static SEXP ExtractOptionals(SEXP ans, int *recurse, int *usenames, SEXP call)
    argument.
 */
 
+/* This is a primitive SPECIALSXP */
 SEXP attribute_hidden do_c(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
@@ -693,7 +729,7 @@ SEXP attribute_hidden do_c(SEXP call, SEXP op, SEXP args, SEXP env)
 
     /* Attempt method dispatch. */
 
-    if (DispatchOrEval(call, op, "c", args, env, &ans, 1, 0))
+    if (DispatchOrEval(call, op, "c", args, env, &ans, 1, 1))
 	return(ans);
     return do_c_dflt(call, op, ans, env);
 }
@@ -730,7 +766,7 @@ SEXP attribute_hidden do_c_dflt(SEXP call, SEXP op, SEXP args, SEXP env)
 	    if (!isNull(TAG(t))) data.ans_nnames = 1;
 	    else data.ans_nnames = HasNames(CAR(t));
 	}
-	AnswerType(CAR(t), recurse, usenames, &data);
+	AnswerType(CAR(t), recurse, usenames, &data, call);
     }
 
     /* If a non-vector argument was encountered (perhaps a list if */
@@ -784,25 +820,13 @@ SEXP attribute_hidden do_c_dflt(SEXP call, SEXP op, SEXP args, SEXP env)
     if (data.ans_nnames && data.ans_length > 0) {
 	PROTECT(data.ans_names = allocVector(STRSXP, data.ans_length));
 	data.ans_nnames = 0;
-#ifdef EXPT
-	if (!recurse) {
-#endif
-	    while (args != R_NilValue) {
-		nameData.seqno = 0;
-		nameData.firstpos = 0;
-		nameData.count = 0;
-		NewExtractNames(CAR(args), R_NilValue, TAG(args), recurse, &data, &nameData);
-		args = CDR(args);
-	    }
-#ifdef EXPT
-	}
-	else {
+	while (args != R_NilValue) {
 	    nameData.seqno = 0;
 	    nameData.firstpos = 0;
 	    nameData.count = 0;
-	    NewExtractNames(args, R_NilValue, TAG(args), recurse);
+	    NewExtractNames(CAR(args), R_NilValue, TAG(args), recurse, &data, &nameData);
+	    args = CDR(args);
 	}
-#endif/*EXPT*/
 	setAttrib(ans, R_NamesSymbol, data.ans_names);
 	UNPROTECT(1);
     }
@@ -852,7 +876,7 @@ SEXP attribute_hidden do_unlist(SEXP call, SEXP op, SEXP args, SEXP env)
 	for (i = 0; i < n; i++) {
 	    if (usenames && !data.ans_nnames)
 		data.ans_nnames = HasNames(VECTOR_ELT(args, i));
-	    AnswerType(VECTOR_ELT(args, i), recurse, usenames, &data);
+	    AnswerType(VECTOR_ELT(args, i), recurse, usenames, &data, call);
 	}
     }
     else if (isList(args)) {
@@ -861,7 +885,7 @@ SEXP attribute_hidden do_unlist(SEXP call, SEXP op, SEXP args, SEXP env)
 		if (!isNull(TAG(t))) data.ans_nnames = 1;
 		else data.ans_nnames = HasNames(CAR(t));
 	    }
-	    AnswerType(CAR(t), recurse, usenames, &data);
+	    AnswerType(CAR(t), recurse, usenames, &data, call);
 	}
     }
     else {
@@ -958,31 +982,14 @@ SEXP attribute_hidden do_unlist(SEXP call, SEXP op, SEXP args, SEXP env)
 } /* do_unlist */
 
 
-#ifdef UNUSED
-#define LNAMBUF 100
-
-SEXP FetchMethod(char *generic, char *classname, SEXP env)
-{
-    char buf[LNAMBUF];
-    SEXP method;
-    if (strlen(generic) + strlen(classname) + 2 > LNAMBUF)
-	error(_("class name too long in '%s'"), generic);
-    sprintf(buf, "%s.%s", generic, classname);
-    method = findVar(install(buf), env);
-    if (TYPEOF(method)==PROMSXP)
-	method = eval(method, env);
-    if (TYPEOF(method) != CLOSXP)
-	method = R_NilValue;
-    return method;
-}
-#endif
-
 /* cbind(deparse.level, ...) and rbind(deparse.level, ...) : */
+/* This is a special .Internal */
 SEXP attribute_hidden do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP a, t, obj, classlist, classname, method, classmethod, rho;
     const char *generic;
-    int mode, deparse_level, compatible=1;
+    int mode, deparse_level;
+    Rboolean compatible = TRUE;
     struct BindData data;
     char buf[512];
     const char *s, *klass;
@@ -1006,7 +1013,7 @@ SEXP attribute_hidden do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
      *	  memberships from the class attribute.
      *
      * 2) We inspect each class in turn to see if there is an
-     *	  an applicable method.
+     *	  applicable method.
      *
      * 3) If we find an applicable method we make sure that it is
      *	  identical to any method determined for prior arguments.
@@ -1047,7 +1054,7 @@ SEXP attribute_hidden do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
 			if (strcmp(klass, s)) {
 			    method = R_NilValue;
 			    /* need to end both loops */
-			    compatible = 0;
+			    compatible = FALSE;
 			}
 		    }
 		    break; /* go to next parameter */
@@ -1066,17 +1073,13 @@ SEXP attribute_hidden do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
     /* Dispatch based on class membership has failed. */
     /* The default code for rbind/cbind.default follows */
     /* First, extract the evaluated arguments. */
-#ifdef OLD
-    for (a = args; a != R_NilValue; a = CDR(a))
-	CAR(a) = PRVALUE(CAR(a));
-#endif
 
     rho = env;
     data.ans_flags = 0;
     data.ans_length = 0;
     data.ans_nnames = 0;
     for (t = args; t != R_NilValue; t = CDR(t))
-	AnswerType(PRVALUE(CAR(t)), 0, 0, &data);
+	AnswerType(PRVALUE(CAR(t)), 0, 0, &data, call);
 
     /* zero-extent matrices shouldn't give NULL, but cbind(NULL) should: */
     if (!data.ans_flags && !data.ans_length) {
@@ -1212,6 +1215,7 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho,
 	    }
 	    dn = getAttrib(u, R_NamesSymbol);
 	    if (k >= lenmin && (TAG(t) != R_NilValue ||
+				(deparse_level == 2) ||
 				((deparse_level == 1) &&
 				 isSymbol(substitute(CAR(t),R_NilValue)))))
 		have_cnames = TRUE;
@@ -1254,10 +1258,12 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho,
 		case LISTSXP:
 		    PROTECT(u = coerceVector(u, mode));
 		    k = LENGTH(u);
-		    idx = (!isMatrix(u)) ? rows : k;
-		    for (i = 0; i < idx; i++)
-			SET_VECTOR_ELT(result, n++,
-				       duplicate(VECTOR_ELT(u, i % k)));
+		    if (k > 0) {
+			idx = (!isMatrix(u)) ? rows : k;
+			for (i = 0; i < idx; i++)
+			    SET_VECTOR_ELT(result, n++,
+					   duplicate(VECTOR_ELT(u, i % k)));
+		    }
 		    UNPROTECT(1);
 		    break;
 		default:
@@ -1291,13 +1297,13 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho,
 	    }
 	}
     }
-    else {
+    else { /* everything else, currently REALSXP, INTSXP, LGLSXP */
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    u = PRVALUE(CAR(t));
+	    u = PRVALUE(CAR(t)); /* type of u can be any of: RAW, LGL, INT, REAL */
 	    if (isMatrix(u) || length(u) >= lenmin) {
 		k = LENGTH(u);
 		idx = (!isMatrix(u)) ? rows : k;
-		if (TYPEOF(u) <= INTSXP) {
+		if (TYPEOF(u) <= INTSXP) { /* INT or LGL */
 		    if (mode <= INTSXP) {
 			for (i = 0; i < idx; i++)
 			    INTEGER(result)[n++] = INTEGER(u)[i % k];
@@ -1307,9 +1313,21 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho,
 			    REAL(result)[n++] = (INTEGER(u)[i % k]) == NA_INTEGER ? NA_REAL : INTEGER(u)[i % k];
 		    }
 		}
-		else {
+		else if (TYPEOF(u) == REALSXP) {
 		    for (i = 0; i < idx; i++)
 			REAL(result)[n++] = REAL(u)[i % k];
+		} 
+		else { /* RAWSXP */
+		    /* FIXME: I'm not sure what the author intended when the sequence was
+		       defined as raw < logical -- it is possible to represent logical as
+		       raw losslessly but not vice versa. So due to the way this was
+		       defined the raw -> logical conversion is bound to be lossy .. */
+		    if (mode == LGLSXP)
+			for (i = 0; i < idx; i++)
+			    LOGICAL(result)[n++] = RAW(u)[i % k] ? TRUE : FALSE;
+		    else
+			for (i = 0; i < idx; i++)
+			    INTEGER(result)[n++] = (unsigned char) RAW(u)[i % k];
 		}
 	    }
 	}
@@ -1445,8 +1463,9 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho,
 	    }
 	    dn = getAttrib(u, R_NamesSymbol);
 	    if (k >= lenmin && (TAG(t) != R_NilValue ||
-		      ((deparse_level == 1) &&
-		       isSymbol(substitute(CAR(t),R_NilValue)))))
+				(deparse_level == 2) ||
+				((deparse_level == 1) &&
+				 isSymbol(substitute(CAR(t),R_NilValue)))))
 		have_rnames = TRUE;
 	    nnames = imax2(nnames, length(dn));
 	}
@@ -1518,9 +1537,9 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho,
 	    }
 	}
     }
-    else {
+    else { /* everything else, currently REALSXP, INTSXP, LGLSXP */
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    u = PRVALUE(CAR(t));
+	    u = PRVALUE(CAR(t)); /* type of u can be any of: RAW, LGL, INT, REAL */
 	    if (isMatrix(u) || length(u) >= lenmin) {
 		k = LENGTH(u);
 		idx = (isMatrix(u)) ? nrows(u) : (k > 0);
@@ -1540,13 +1559,26 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho,
 			n += idx;
 		    }
 		}
-		else {
+		else if (TYPEOF(u) == REALSXP) {
 		    for (i = 0; i < idx; i++)
 			for (j = 0; j < cols; j++)
 			    REAL(result)[i + n + (j * rows)]
 				= REAL(u)[(i + j * idx) % k];
 		    n += idx;
 		}
+		else { /* RAWSXP */
+		    if (mode == LGLSXP) {
+			for (i = 0; i < idx; i++)
+			    for (j = 0; j < cols; j++)
+				LOGICAL(result)[i + n + (j * rows)]
+				    = RAW(u)[(i + j * idx) % k] ? TRUE : FALSE;
+		    }
+		    else
+			for (i = 0; i < idx; i++)
+			    for (j = 0; j < cols; j++)
+				INTEGER(result)[i + n + (j * rows)]
+				    = (unsigned char) RAW(u)[(i + j * idx) % k];
+		}
 	    }
 	}
     }
