@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2010  The R Development Core Team
- *  Copyright (C) 2002--2005  The R Foundation
+ *  Copyright (C) 1997--2011  The R Development Core Team
+ *  Copyright (C) 2002--2011  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,9 @@
 #endif
 
 #include <Defn.h>
+#include <float.h> /* for DBL_EPSILON etc */
 #include <Graphics.h>
+// --> R_ext/GraphicsEngine.h + Rgraphics.h
 #include <GraphicsBase.h>       /* setBaseDevice */
 #include <Rmath.h>		/* eg. fmax2() */
 
@@ -1691,6 +1693,7 @@ void GReset(pGEDevDesc dd)
 
 /*  Is the figure region too big ? */
 
+/* Why is this FLT_EPSILON? */
 static Rboolean validFigureRegion(pGEDevDesc dd)
 {
     return ((gpptr(dd)->fig[0] > 0-FLT_EPSILON) &&
@@ -1849,6 +1852,55 @@ pGEDevDesc GNewPlot(Rboolean recording)
 }
 #undef G_ERR_MSG
 
+// used in GScale(), but also ../library/grDevices/src/axis_scales.c :
+// (usr, log, n_inp) |--> (axp, n_out) :
+void GAxisPars(double *min, double *max, int *n, Rboolean log, int axis)
+{
+#define EPS_FAC_2 100
+    Rboolean swap = *min > *max;
+    double t_, min_o, max_o;
+
+    if(swap) { /* Feature: in R, something like  xlim = c(100,0)  just works */
+	t_ = *min; *min = *max; *max = t_;
+    }
+    /* save only for the extreme case (EPS_FAC_2): */
+    min_o = *min; max_o = *max;
+
+    if(log) {
+	/* Avoid infinities */
+	if(*max > 308) *max = 308;
+	if(*min < -307) *min = -307;
+	*min = pow(10., *min);
+	*max = pow(10., *max);
+	GLPretty(min, max, n);
+    }
+    else GPretty(min, max, n);
+
+    double tmp2 = EPS_FAC_2 * DBL_EPSILON;/* << prevent overflow in product below */
+    if(fabs(*max - *min) < (t_ = fmax2(fabs(*max), fabs(*min)))* tmp2) {
+	/* Treat this case somewhat similar to the (min ~= max) case above */
+	/* Too much accuracy here just shows machine differences */
+	warning(_("relative range of values =%4.0f * EPS, is small (axis %d)")
+		/*"to compute accurately"*/,
+		fabs(*max - *min) / (t_*DBL_EPSILON), axis);
+
+	/* No pretty()ing anymore */
+	*min = min_o;
+	*max = max_o;
+	double eps = .005 * fabs(*max - *min);/* .005: not to go to DBL_MIN/MAX */
+	*min += eps;
+	*max -= eps;
+	if(log) {
+	    *min = pow(10., *min);
+	    *max = pow(10., *max);
+	}
+	*n = 1;
+    }
+    if(swap) {
+	t_ = *min; *min = *max; *max = t_;
+    }
+}
+
 void GScale(double min, double max, int axis, pGEDevDesc dd)
 {
 /* GScale: used to default axis information
@@ -1856,9 +1908,8 @@ void GScale(double min, double max, int axis, pGEDevDesc dd)
  * NB: can have min > max !
  */
 #define EPS_FAC_1  16
-#define EPS_FAC_2 100
 
-    Rboolean swap, is_xaxis = (axis == 1 || axis == 3);
+    Rboolean is_xaxis = (axis == 1 || axis == 3);
     int log, n, style;
     double temp, min_o = 0., max_o = 0., tmp2 = 0.;/*-Wall*/
 
@@ -1945,57 +1996,14 @@ void GScale(double min, double max, int axis, pGEDevDesc dd)
 	}
     }
 
-    /* ------  The following : Only computation of [xy]axp[0:2] ------- */
-
     /* This is not directly needed when [xy]axt = "n",
      * but may later be different in another call to axis(), e.g.:
       > plot(1, xaxt = "n");  axis(1)
-     * In that case, do_axis() should do the following.
-     * MM: May be we should modularize and put the following into another
-     * subroutine which could be called by do_axis {when [xy]axt != 'n'} ..
+     * In that case, do_axis() should do the following:
      */
 
-    swap = min > max;
-    if(swap) { /* Feature: in R, something like  xlim = c(100,0)  just works */
-	temp = min; min = max; max = temp;
-    }
-    /* save only for the extreme case (EPS_FAC_2): */
-    min_o = min; max_o = max;
-
-    if(log) {
-	/* Avoid infinities */
-	if(max > 308) max = 308;
-	if(min < -307) min = -307;
-	min = pow(10., min);
-	max = pow(10., max);
-	GLPretty(&min, &max, &n);
-    }
-    else GPretty(&min, &max, &n);
-
-    tmp2 = EPS_FAC_2 * DBL_EPSILON;/* << prevent overflow in product below */
-    if(fabs(max - min) < (temp = fmax2(fabs(max), fabs(min)))* tmp2) {
-	/* Treat this case somewhat similar to the (min ~= max) case above */
-	/* Too much accuracy here just shows machine differences */
-	warning(_("relative range of values =%4.0f * EPS, is small (axis %d)")
-		/*"to compute accurately"*/,
-		fabs(max - min) / (temp*DBL_EPSILON), axis);
-
-	/* No pretty()ing anymore */
-	min = min_o;
-	max = max_o;
-	temp = .005 * fabs(max - min);/* .005: not to go to DBL_MIN/MAX */
-	min += temp;
-	max -= temp;
-	if(log) {
-	    min = pow(10., min);
-	    max = pow(10., max);
-	}
-	n = 1;
-    }
-
-    if(swap) {
-	temp = min; min = max; max = temp;
-    }
+    // Computation of [xy]axp[0:2] == (min,max,n) :
+    GAxisPars(&min, &max, &n, log, axis);
 
 #define G_Store_AXP(is_X)			\
     if(is_X) {					\
@@ -2522,14 +2530,10 @@ void GLine(double x1, double y1, double x2, double y2, int coords, pGEDevDesc dd
 /* Read the current "pen" position. */
 Rboolean GLocator(double *x, double *y, int coords, pGEDevDesc dd)
 {
-    if(!dd->dev->locator)
-	error(_("no locator capability in device driver"));
-    if(dd->dev->locator(x, y, dd->dev)) {
+    if(dd->dev->locator && dd->dev->locator(x, y, dd->dev)) {
 	GConvert(x, y, DEVICE, coords, dd);
 	return TRUE;
-    }
-    else
-	return FALSE;
+    } else return FALSE;
 }
 
 /* Access character font metric information.  */
@@ -2905,9 +2909,28 @@ void GRect(double x0, double y0, double x1, double y1, int coords,
     GERect(x0, y0, x1, y1, &gc, dd);
 }
 
-void GRaster(unsigned int* image, int w, int h, 
-             double x0, double y0, double x1, double y1, 
-             double angle, Rboolean interpolate, 
+void GPath(double *x, double *y,
+           int npoly, int *nper,
+           Rboolean winding,
+           int bg, int fg, pGEDevDesc dd)
+{
+    R_GE_gcontext gc; gcontextFromGP(&gc, dd);
+
+    if (gpptr(dd)->lty == LTY_BLANK)
+	fg = R_TRANWHITE; /* transparent for the border */
+
+    /*
+     * Ensure that the base clipping region is set on the device
+     */
+    GClip(dd);
+    gc.col = fg;
+    gc.fill = bg;
+    GEPath(x, y, npoly, nper, winding, &gc, dd);
+}
+
+void GRaster(unsigned int* image, int w, int h,
+             double x0, double y0, double x1, double y1,
+             double angle, Rboolean interpolate,
              pGEDevDesc dd)
 {
     R_GE_gcontext gc; gcontextFromGP(&gc, dd);
@@ -2917,7 +2940,7 @@ void GRaster(unsigned int* image, int w, int h,
      */
     GClip(dd);
 
-    GERaster(image, w, h, x0, y0, x1, y1, angle, interpolate, 
+    GERaster(image, w, h, x0, y0, x1, y1, angle, interpolate,
              &gc, dd);
 }
 
@@ -2992,11 +3015,7 @@ void GArrow(double xfrom, double yfrom, double xto, double yto, int coords,
     if((code & 3) == 0) return; /* no arrows specified */
     if(length == 0) return; /* zero-length arrow heads */
 
-#ifdef HAVE_HYPOT
     if(hypot(xfromInch - xtoInch, yfromInch - ytoInch) < eps) {
-#else
-    if(pythag(xfromInch - xtoInch, yfromInch - ytoInch) < eps) {
-#endif
 	/* effectively 0-length arrow */
 	warning(_("zero-length arrow is of indeterminate angle and so skipped"));
 	return;
@@ -3196,7 +3215,7 @@ void GMtext(const char *str, cetype_t enc, int side, double line, int outer,
 	 3 = always vertical.
 */
     double angle, xadj;
-    int coords, subcoords;
+    int coords;
 
     /* Init to keep -Wall happy: */
     angle = 0.;
@@ -3210,7 +3229,6 @@ void GMtext(const char *str, cetype_t enc, int side, double line, int outer,
 	case 3:	    coords = OMA3;	break;
 	case 4:	    coords = OMA4;	break;
 	}
-	subcoords = NIC;
     }
     else {
 	switch(side) {
@@ -3219,7 +3237,6 @@ void GMtext(const char *str, cetype_t enc, int side, double line, int outer,
 	case 3:	    coords = MAR3;	break;
 	case 4:	    coords = MAR4;	break;
 	}
-	subcoords = USER;
     }
     /* Note: I changed gpptr(dd)->yLineBias to 0.3 here. */
     /* Purely visual tuning. RI */
@@ -3228,13 +3245,21 @@ void GMtext(const char *str, cetype_t enc, int side, double line, int outer,
        a somehow fuzzy manner with respect to current side and las settings.
        Uwe L.
     */
+    /* Note from PR#14532:
+       yLineBias is the proportion of line height that is white
+       space. The manipulation of "line" below is pure visual tuning
+       such that when we plot horizontal text on side 1 (or vertical
+       text on side 4) with padj=0 (i.e. text written *above* the
+       specified y-value), it is symmetric w.r.t text written on sides
+       1 and 2 with padj=0.
+    */
     switch(side) {
     case 1:
 	if(las == 2 || las == 3) {
 	    angle = 90;
 	}
 	else {
-	    line = line + 1 - dd->dev->yLineBias;
+	    line += (1/gpptr(dd)->mex)*(1 - dd->dev->yLineBias);
 	    angle = 0;
 	}
 	break;
@@ -3243,7 +3268,7 @@ void GMtext(const char *str, cetype_t enc, int side, double line, int outer,
 	    angle = 0;
 	}
 	else {
-	    line = line + dd->dev->yLineBias;
+	    line += (1/gpptr(dd)->mex)*dd->dev->yLineBias;
 	    angle = 90;
 	}
 	break;
@@ -3252,7 +3277,7 @@ void GMtext(const char *str, cetype_t enc, int side, double line, int outer,
 	    angle = 90;
 	}
 	else {
-	    line = line + dd->dev->yLineBias;
+	    line += (1/gpptr(dd)->mex)*dd->dev->yLineBias;
 	    angle = 0;
 	}
 	break;
@@ -3261,7 +3286,7 @@ void GMtext(const char *str, cetype_t enc, int side, double line, int outer,
 	    angle = 0;
 	}
 	else {
-	    line = line + 1 - dd->dev->yLineBias;
+	    line += (1/gpptr(dd)->mex)*(1 - dd->dev->yLineBias);
 	    angle = 90;
 	}
 	break;
@@ -3326,7 +3351,7 @@ void GMathText(double x, double y, int coords, SEXP expr,
 void GMMathText(SEXP str, int side, double line, int outer,
 		double at, int las, double yadj, pGEDevDesc dd)
 {
-    int coords = 0, subcoords;
+    int coords = 0;
     double xadj, angle = 0;
 
     /* IF font metric information is not available for device */
@@ -3351,7 +3376,6 @@ void GMMathText(SEXP str, int side, double line, int outer,
 	case 3:	    coords = OMA3;	break;
 	case 4:	    coords = OMA4;	break;
 	}
-	subcoords = NIC;
     }
     else {
 	switch(side) {
@@ -3360,7 +3384,6 @@ void GMMathText(SEXP str, int side, double line, int outer,
 	case 3:	    coords = MAR3;	break;
 	case 4:	    coords = MAR4;	break;
 	}
-	subcoords = USER;
     }
     switch(side) {
     case 1:
@@ -3371,7 +3394,7 @@ void GMMathText(SEXP str, int side, double line, int outer,
 	    /*	    line = line + 1 - gpptr(dd)->yLineBias;
 		    angle = 0;
 		    yadj = NA_REAL; */
-	    line = line + 1;
+	    line += (1/gpptr(dd)->mex)*(1 - dd->dev->yLineBias);
 	    angle = 0;
 	}
 	break;
@@ -3383,6 +3406,9 @@ void GMMathText(SEXP str, int side, double line, int outer,
 	    /*	    line = line + gpptr(dd)->yLineBias;
 		    angle = 90;
 		    yadj = NA_REAL; */
+	    /* The following line is needed for symmetry with plain text
+	       but changes existing output */
+	    line += (1/gpptr(dd)->mex)*dd->dev->yLineBias;
 	    angle = 90;
 	}
 	break;
@@ -3394,6 +3420,9 @@ void GMMathText(SEXP str, int side, double line, int outer,
 	    /*   line = line + gpptr(dd)->yLineBias;
 		 angle = 0;
 		 yadj = NA_REAL; */
+	    /* The following line is needed for symmetry with plain text
+	       but changes existing output */
+	    line += (1/gpptr(dd)->mex)*dd->dev->yLineBias;
 	    angle = 0;
 	}
 	break;
@@ -3405,7 +3434,7 @@ void GMMathText(SEXP str, int side, double line, int outer,
 	    /*   line = line + 1 - gpptr(dd)->yLineBias;
 		 angle = 90;
 		 yadj = NA_REAL; */
-	    line = line + 1;
+	    line += (1/gpptr(dd)->mex)*(1 - dd->dev->yLineBias);
 	    angle = 90;
 	}
 	break;
