@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001-9  The R Development Core Team
+ *  Copyright (C) 2001-2010  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <config.h>
 #endif
 #include <Defn.h>
+/* -> Rinternals.h which exports R_compute_identical() */
 
 /* Implementation of identical(x, y) */
 
@@ -33,7 +34,7 @@ typedef enum {
     single_NA__num_eq  = 3,/* R's default: one kind of NA or NaN; for num, use '==' */
 } ne_strictness_type;
 
-/* NOTE:  ne_strict = num_eq + (single_NA * 2)  = num_eq | (single_NA << 1)   */
+/* NOTE:  ne_strict = NUM_EQ + (SINGLE_NA * 2)  = NUM_EQ | (SINGLE_NA << 1)   */
 
 static Rboolean neWithNaN(double x, double y, ne_strictness_type str);
 
@@ -41,38 +42,39 @@ static Rboolean neWithNaN(double x, double y, ne_strictness_type str);
 /* .Internal(identical(..)) */
 SEXP attribute_hidden do_identical(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    int num_eq, single_NA, attr_as_set, nargs = length(args);
+    int num_eq = 1, single_NA = 1, attr_as_set = 1, ignore_bytecode = 1, nargs = length(args), flags;
     /* avoid problems with earlier version captured in S4 methods */
     /* checkArity(op, args); */
-    if (nargs != 2 && nargs != 5)
+    if (nargs != 2 && nargs != 5 && nargs != 6)
 	error("%d arguments passed to .Internal(%s) which requires %d",
 	      length(args), PRIMNAME(op), PRIMARITY(op));
 
-    if (nargs == 5) {
+    if (nargs >= 5) {
 	num_eq      = asLogical(CADDR(args));
 	single_NA   = asLogical(CADDDR(args));
 	attr_as_set = asLogical(CAD4R(args));
-
-	if(num_eq      == NA_LOGICAL) error(_("invalid '%s' value"), "num.eq");
-	if(single_NA   == NA_LOGICAL) error(_("invalid '%s' value"), "single.NA");
-	if(attr_as_set == NA_LOGICAL) error(_("invalid '%s' value"), "attrib.as.set");
-    } else {
-	num_eq      = 1;
-	single_NA   = 1;
-	attr_as_set = 1;
     }
+    if (nargs == 6) 
+	ignore_bytecode = asLogical(CAD4R(CDR(args)));
 
-    return ScalarLogical(R_compute_identical(CAR(args), CADR(args),
-					     (Rboolean) num_eq,
-					     (Rboolean) single_NA,
-					     (Rboolean) attr_as_set));
+    if(num_eq      == NA_LOGICAL) error(_("invalid '%s' value"), "num.eq");
+    if(single_NA   == NA_LOGICAL) error(_("invalid '%s' value"), "single.NA");
+    if(attr_as_set == NA_LOGICAL) error(_("invalid '%s' value"), "attrib.as.set");
+    if(ignore_bytecode == NA_LOGICAL) error(_("invalid '%s' value"), "ignore.bytecode");
+    
+    flags = (num_eq ? 0 : 1) + (single_NA ? 0 : 2) + (attr_as_set ? 0 : 4) + (ignore_bytecode ? 0 : 8); 
+    return ScalarLogical(R_compute_identical(CAR(args), CADR(args), flags));
 }
 
+#define NUM_EQ 		(!(flags & 1))
+#define SINGLE_NA       (!(flags & 2))
+#define ATTR_AS_SET     (!(flags & 4))
+#define IGNORE_BYTECODE (!(flags & 8))
+
 /* do the two objects compute as identical?
    used in unique.c */
-Rboolean attribute_hidden
-R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
-		    Rboolean single_NA, Rboolean attr_as_set)
+Rboolean
+R_compute_identical(SEXP x, SEXP y, int flags)
 {
     SEXP ax, ay, atrx, atry;
     if(x == y) /* same pointer */
@@ -82,9 +84,16 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
     if(OBJECT(x) != OBJECT(y))
 	return FALSE;
 
+    /* Skip attribute checks for CHARSXP -- such attributes can be used for internal purposes */
+    if(TYPEOF(x) == CHARSXP)
+    {
+	/* This matches NAs */
+	return Seql(x, y);
+    }
+
     ax = ATTRIB(x); ay = ATTRIB(y);
-    if (!attr_as_set) {
-	if(!R_compute_identical(ax, ay, num_eq, single_NA, FALSE)) return FALSE;
+    if (!ATTR_AS_SET) {
+	if(!R_compute_identical(ax, ay, flags)) return FALSE;
     }
     /* Attributes are special: they should be tagged pairlists.  We
        don't test them if they are not, and we do not test the order
@@ -113,15 +122,13 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 			if(streql(tx, "row.names")) {
 			    PROTECT(atrx = getAttrib(x, R_RowNamesSymbol));
 			    PROTECT(atry = getAttrib(y, R_RowNamesSymbol));
-			    if(!R_compute_identical(atrx, atry,
-						    num_eq, single_NA, TRUE)) {
+			    if(!R_compute_identical(atrx, atry, flags)) {
 				UNPROTECT(2);
 				return FALSE;
 			    } else
 				UNPROTECT(2);
 			} else
-			    if(!R_compute_identical(CAR(elx), CAR(ely),
-						    num_eq, single_NA, TRUE))
+			    if(!R_compute_identical(CAR(elx), CAR(ely), flags))
 				return FALSE;
 			break;
 		    }
@@ -148,7 +155,7 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	if(n != length(y)) return FALSE;
 	else {
 	    double *xp = REAL(x), *yp = REAL(y);
-	    int i, ne_strict = num_eq | (single_NA << 1);
+	    int i, ne_strict = NUM_EQ | (SINGLE_NA << 1);
 	    for(i = 0; i < n; i++)
 		if(neWithNaN(xp[i], yp[i], ne_strict)) return FALSE;
 	}
@@ -160,7 +167,7 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	if(n != length(y)) return FALSE;
 	else {
 	    Rcomplex *xp = COMPLEX(x), *yp = COMPLEX(y);
-	    int i, ne_strict = num_eq | (single_NA << 1);
+	    int i, ne_strict = NUM_EQ | (SINGLE_NA << 1);
 	    for(i = 0; i < n; i++)
 		if(neWithNaN(xp[i].r, yp[i].r, ne_strict) ||
 		   neWithNaN(xp[i].i, yp[i].i, ne_strict))
@@ -182,7 +189,7 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	}
 	return TRUE;
     }
-    case CHARSXP:
+    case CHARSXP: /* Probably unreachable, but better safe than sorry... */
     {
 	/* This matches NAs */
 	return Seql(x, y);
@@ -193,8 +200,7 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	int i, n = length(x);
 	if(n != length(y)) return FALSE;
 	for(i = 0; i < n; i++)
-	    if(!R_compute_identical(VECTOR_ELT(x, i),VECTOR_ELT(y, i),
-				    num_eq, single_NA, attr_as_set))
+	    if(!R_compute_identical(VECTOR_ELT(x, i),VECTOR_ELT(y, i), flags))
 		return FALSE;
 	return TRUE;
     }
@@ -204,11 +210,9 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	while (x != R_NilValue) {
 	    if(y == R_NilValue)
 		return FALSE;
-	    if(!R_compute_identical(CAR(x), CAR(y),
-				    num_eq, single_NA, attr_as_set))
+	    if(!R_compute_identical(CAR(x), CAR(y), flags))
 		return FALSE;
-	    if(!R_compute_identical(PRINTNAME(TAG(x)), PRINTNAME(TAG(y)),
-				    num_eq, single_NA, attr_as_set))
+	    if(!R_compute_identical(PRINTNAME(TAG(x)), PRINTNAME(TAG(y)), flags))
 		return FALSE;
 	    x = CDR(x);
 	    y = CDR(y);
@@ -216,22 +220,23 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	return(y == R_NilValue);
     }
     case CLOSXP:
-	return(R_compute_identical(FORMALS(x), FORMALS(y),
-				   num_eq, single_NA, attr_as_set) &&
-	       R_compute_identical(BODY_EXPR(x), BODY_EXPR(y),
-				   num_eq, single_NA, attr_as_set) &&
-	       CLOENV(x) == CLOENV(y) ? TRUE : FALSE);
+	return(R_compute_identical(FORMALS(x), FORMALS(y), flags) &&
+	       R_compute_identical(BODY_EXPR(x), BODY_EXPR(y), flags) &&
+	       (CLOENV(x) == CLOENV(y) ? TRUE : FALSE) &&
+	       (IGNORE_BYTECODE || R_compute_identical(BODY(x), BODY(y), flags))
+	       );
     case SPECIALSXP:
     case BUILTINSXP:
 	return(PRIMOFFSET(x) == PRIMOFFSET(y) ? TRUE : FALSE);
     case ENVSXP:
     case SYMSXP:
-    case EXTPTRSXP:
     case WEAKREFSXP:
 #ifdef BYTECODE
     case BCODESXP: /**** is this the best approach? */
 #endif
 	return(x == y ? TRUE : FALSE);
+    case EXTPTRSXP:
+	return (EXTPTR_PTR(x) == EXTPTR_PTR(y) ? TRUE : FALSE);
     case RAWSXP:
 	if (length(x) != length(y)) return FALSE;
 	/* Use memcmp (which is ISO C) to speed up the comparison */
@@ -244,7 +249,7 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	/*#define PREXPR(x)	((x)->u.promsxp.expr)
 	  #define PRENV(x)	((x)->u.promsxp.env)
 	  return(R_compute_identical(subsititute(PREXPR(x), PRENV(x),
-	                             num_eq, single_NA, attr_as_set),
+	                             flags),
 	  subsititute(PREXPR(y), PRENV(y))));*/
     case S4SXP:
 	/* attributes already tested, so all slots identical */
@@ -265,11 +270,11 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
  * @param x
  * @param y  the two "number"s to be compared
  * @param str a "strictness" indicator, one of 2*2 (one|bit)_NA__num_(eq|bit)
- *  "single_NA" means: x and y differ in the case
+ *  "SINGLE_NA" means: x and y differ in the case
  *    that one, but not both are NaN.  Two NaN values are judged
  *    identical for this purpose, but NA != NaN
  *
- *  "num_eq" means: (x != y) is used when both are not NA or NaN
+ *  "NUM_EQ" means: (x != y) is used when both are not NA or NaN
  *  whereas "bit_NA" and "num_bit" use the bitwise memory comparison  memcmp();
  *  notably "*_num_bit" will differentiate '+0.' and '-0.'.
  *
