@@ -244,16 +244,16 @@ int isBasicClass(const char *ss) {
       return FALSE; /* too screwed up to do conversions */
     return findVarInFrame3(s_S3table, install(ss), FALSE) != R_UnboundValue;
 }
-    
-    
+
+
 
 int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
 	      SEXP rho, SEXP callrho, SEXP defrho, SEXP *ans)
 {
-    SEXP klass, method, sxp, t, s, matchedarg;
-    SEXP op, formals, newrho, newcall, match_obj = 0;
+    SEXP klass, method, sxp, t, s, matchedarg, sort_list;
+    SEXP op, formals, newrho, newcall;
     char buf[512];
-    int i, j, nclass, matched, S4toS3, nprotect;
+    int i, j, nclass, matched, /* S4toS3, */ nprotect;
     RCNTXT *cptr;
 
     /* Get the context which UseMethod was called from. */
@@ -291,7 +291,6 @@ int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
 	    for (t = formals; t != R_NilValue; t = CDR(t))
 	        if (TAG(t) == TAG(s)) {
 		    matched = 1;
-		    if(t == formals) match_obj = CAR(s); /* remember 1st arg */
 		}
 
 	    if (!matched) defineVar(TAG(s), CAR(s), newrho);
@@ -302,7 +301,7 @@ int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
     PROTECT(newcall = duplicate(cptr->call));
 
     PROTECT(klass = R_data_class2(obj));
-    S4toS3 = IS_S4_OBJECT(obj);
+    sort_list = install("sort.list");
 
     nclass = length(klass);
     for (i = 0; i < nclass; i++) {
@@ -313,43 +312,26 @@ int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
 	method = install(buf);
 	sxp = R_LookupMethod(method, rho, callrho, defrho);
 	if (isFunction(sxp)) {
+	    if(method == sort_list && CLOENV(sxp) == R_BaseNamespace)
+		continue; /* kludge because sort.list is not a method */
             if( RDEBUG(op) || RSTEP(op) )
                 SET_RSTEP(sxp, 1);
-	    defineVar(install(".Generic"), mkString(generic), newrho);
+	    defineVar(R_dot_Generic, mkString(generic), newrho);
 	    if (i > 0) {
 	        int ii;
 		PROTECT(t = allocVector(STRSXP, nclass - i));
 		for(j = 0, ii = i; j < length(t); j++, ii++)
 		      SET_STRING_ELT(t, j, STRING_ELT(klass, ii));
 		setAttrib(t, install("previous"), klass);
-		defineVar(install(".Class"), t, newrho);
+		defineVar(R_dot_Class, t, newrho);
 		UNPROTECT(1);
 	    } else
-		defineVar(install(".Class"), klass, newrho);
+		defineVar(R_dot_Class, klass, newrho);
 	    PROTECT(t = mkString(buf));
-	    defineVar(install(".Method"), t, newrho);
+	    defineVar(R_dot_Method, t, newrho);
 	    UNPROTECT(1);
-	    defineVar(install(".GenericCallEnv"), callrho, newrho);
-	    defineVar(install(".GenericDefEnv"), defrho, newrho);
-	    if(S4toS3 && i > 0 && isBasicClass(ss)) {
-	      SEXP S3Part; 
-	      S3Part = R_getS4DataSlot(obj, S4SXP);
-	      if(S3Part == R_NilValue && TYPEOF(obj) == S4SXP) /* could be type, e.g. "environment" */
-		S3Part = R_getS4DataSlot(obj, ANYSXP);
-	      PROTECT(S3Part); nprotect++;
-	      /* At this point S3Part is the S3 class object or
-	       an object of an abnormal type, or NULL */
-	      if(S3Part != R_NilValue) {  /* use S3Part as inherited object */
-		  obj = S3Part;
-		  if(!match_obj) /* use the first arg, for "[",e.g. */
-		    match_obj = CAR(matchedarg);
-		  if(NAMED(obj)) SET_NAMED(obj, 2);
-		  if(TYPEOF(match_obj) == PROMSXP)
-		    SET_PRVALUE(match_obj, obj); /* must have been eval'd */
-		  else /* not possible ?*/
-		    defineVar(TAG(FORMALS(sxp)), obj, newrho);
-	      } /* else, use the S4 object */
-	    }
+	    defineVar(R_dot_GenericCallEnv, callrho, newrho);
+	    defineVar(R_dot_GenericDefEnv, defrho, newrho);
 	    t = newcall;
 	    SETCAR(t, method);
 	    R_GlobalContext->callflag = CTXT_GENERIC;
@@ -367,13 +349,13 @@ int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
     if (isFunction(sxp)) {
         if( RDEBUG(op) || RSTEP(op) )
             SET_RSTEP(sxp, 1);
-	defineVar(install(".Generic"), mkString(generic), newrho);
-	defineVar(install(".Class"), R_NilValue, newrho);
+	defineVar(R_dot_Generic, mkString(generic), newrho);
+	defineVar(R_dot_Class, R_NilValue, newrho);
 	PROTECT(t = mkString(buf));
-	defineVar(install(".Method"), t, newrho);
+	defineVar(R_dot_Method, t, newrho);
 	UNPROTECT(1);
-	defineVar(install(".GenericCallEnv"), callrho, newrho);
-	defineVar(install(".GenericDefEnv"), defrho, newrho);
+	defineVar(R_dot_GenericCallEnv, callrho, newrho);
+	defineVar(R_dot_GenericDefEnv, defrho, newrho);
 	t = newcall;
 	SETCAR(t, method);
 	R_GlobalContext->callflag = CTXT_GENERIC;
@@ -389,7 +371,7 @@ int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
 
 /* Note: "do_usemethod" is not the only entry point to
    "usemethod". Things like [ and [[ call usemethod directly,
-   hence do_usemethod should just be an interface to usemethod. 
+   hence do_usemethod should just be an interface to usemethod.
 */
 
 /* This is a primitive SPECIALSXP */
@@ -405,8 +387,8 @@ SEXP attribute_hidden do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
     SET_TAG(CDR(ap), install("object"));
     PROTECT(argList =  matchArgs(ap, args, call));
     if (CAR(argList) == R_MissingArg)
-	errorcall(call, _("there must be a 'generic' argument"));	
-    else 
+	errorcall(call, _("there must be a 'generic' argument"));
+    else
 	PROTECT(generic = eval(CAR(argList), env));
     if(!isString(generic) || length(generic) != 1)
 	errorcall(call, _("'generic' argument must be a character string"));
@@ -467,7 +449,7 @@ SEXP attribute_hidden do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	char cl[1000];
 	PROTECT(klass = R_data_class2(obj));
 	nclass = length(klass);
-	if (nclass == 1) 
+	if (nclass == 1)
 	    strcpy(cl, translateChar(STRING_ELT(klass, 0)));
 	else {
 	    int i;
@@ -527,10 +509,9 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP a, group, basename;
     SEXP callenv, defenv;
     RCNTXT *cptr;
-    int i, j, cftmp;
+    int i, j;
 
     cptr = R_GlobalContext;
-    cftmp = cptr->callflag;
     cptr->callflag = CTXT_GENERIC;
 
     /* get the env NextMethod was called from */
@@ -553,18 +534,22 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
        environment (the method was called outside a method dispatch)
        then chose reasonable defaults. */
     callenv = findVarInFrame3(R_GlobalContext->sysparent,
-			      install(".GenericCallEnv"), TRUE);
+			      R_dot_GenericCallEnv, TRUE);
     if (TYPEOF(callenv) == PROMSXP)
 	callenv = eval(callenv, R_BaseEnv);
     else if (callenv == R_UnboundValue)
 	    callenv = env;
     defenv = findVarInFrame3(R_GlobalContext->sysparent,
-			     install(".GenericDefEnv"), TRUE);
+			     R_dot_GenericDefEnv, TRUE);
     if (TYPEOF(defenv) == PROMSXP) defenv = eval(defenv, R_BaseEnv);
     else if (defenv == R_UnboundValue) defenv = R_GlobalEnv;
 
     /* set up the arglist */
-    s = R_LookupMethod(CAR(cptr->call), env, callenv, defenv);
+    if (TYPEOF(CAR(cptr->call)) == CLOSXP)
+	// e.g., in do.call(function(x) NextMethod('foo'),list())
+	s = CAR(cptr->call);
+    else
+	s = R_LookupMethod(CAR(cptr->call), env, callenv, defenv);
     if (TYPEOF(s) == SYMSXP && s == R_UnboundValue)
 	error(_("no calling generic was found: was a method called directly?"));
     if (TYPEOF(s) != CLOSXP){ /* R_LookupMethod looked for a function */
@@ -652,7 +637,7 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
       isn't currently used).
     */
     klass = findVarInFrame3(R_GlobalContext->sysparent,
-			    install(".Class"), TRUE);
+			    R_dot_Class, TRUE);
 
     if (klass == R_UnboundValue) {
 	s = GetObject(cptr);
@@ -662,14 +647,14 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 
     /* the generic comes from either the sysparent or it's named */
     generic = findVarInFrame3(R_GlobalContext->sysparent,
-			      install(".Generic"), TRUE);
+			      R_dot_Generic, TRUE);
     if (generic == R_UnboundValue)
 	generic = eval(CAR(args), env);
     if( generic == R_NilValue )
 	error(_("generic function not specified"));
     PROTECT(generic);
 
-    if (!isString(generic) || length(generic) > 1)
+    if (!isString(generic) || length(generic) != 1)
 	error(_("invalid generic argument to NextMethod"));
 
     if (CHAR(STRING_ELT(generic, 0))[0] == '\0')
@@ -678,11 +663,11 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     /* determine whether we are in a Group dispatch */
 
     group = findVarInFrame3(R_GlobalContext->sysparent,
-			    install(".Group"), TRUE);
+			    R_dot_Group, TRUE);
     if (group == R_UnboundValue) PROTECT(group = mkString(""));
     else PROTECT(group);
 
-    if (!isString(group) || length(group) > 1)
+    if (!isString(group) || length(group) != 1)
 	error(_("invalid 'group' argument found in NextMethod"));
 
     /* determine the root: either the group or the generic will be it */
@@ -698,7 +683,7 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     */
 
     method = findVarInFrame3(R_GlobalContext->sysparent,
-			     install(".Method"), TRUE);
+			     R_dot_Method, TRUE);
     if( method != R_UnboundValue) {
 	const char *ss;
 	if( !isString(method) )
@@ -793,7 +778,7 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     for (j = 0; j < length(s); j++)
 	SET_STRING_ELT(s, j, duplicate(STRING_ELT(klass, i++)));
     setAttrib(s, install("previous"), klass);
-    defineVar(install(".Class"), s, m);
+    defineVar(R_dot_Class, s, m);
     /* It is possible that if a method was called directly that
 	'method' is unset */
     if (method != R_UnboundValue) {
@@ -805,15 +790,15 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
     } else
 	PROTECT(method = mkString(buf));
-    defineVar(install(".Method"), method, m);
-    defineVar(install(".GenericCallEnv"), callenv, m);
-    defineVar(install(".GenericDefEnv"), defenv, m);
+    defineVar(R_dot_Method, method, m);
+    defineVar(R_dot_GenericCallEnv, callenv, m);
+    defineVar(R_dot_GenericDefEnv, defenv, m);
 
     method = install(buf);
 
-    defineVar(install(".Generic"), generic, m);
+    defineVar(R_dot_Generic, generic, m);
 
-    defineVar(install(".Group"), group, m);
+    defineVar(R_dot_Group, group, m);
 
     SETCAR(newcall, method);
     ans = applyMethod(newcall, nextfun, matchedarg, env, m);
@@ -844,48 +829,35 @@ SEXP attribute_hidden do_unclass(SEXP call, SEXP op, SEXP args, SEXP env)
     return CAR(args);
 }
 
-static SEXP s_S4inherits;
-static SEXP do_S4inherits(SEXP obj, SEXP what, SEXP which) {
-    SEXP e, val;
-    if(!s_S4inherits)
-      s_S4inherits = install(".S4inherits");
-    PROTECT(e = allocVector(LANGSXP, 4));
-    SETCAR(e, s_S4inherits);
-    val = CDR(e);
-    SETCAR(val, obj);
-    val = CDR(val);
-    SETCAR(val, what);
-    val = CDR(val);
-    SETCAR(val, which);
-    val = eval(e, R_MethodsNamespace);
-    UNPROTECT(1);
-    return val;
-}
 
 
-SEXP attribute_hidden do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
+/* NOTE: Fast  inherits(x, what)    in ../include/Rinlinedfuns.h
+ * ----        ----------------- */
+/** C API for  R  inherits(x, what, which)
+ *
+ * @param x any R object
+ * @param what character vector
+ * @param which logical: "want vector result" ?
+ *
+ * @return if which is false, logical TRUE or FALSE
+ *	   if which is true, integer vector of length(what) ..
+ */
+SEXP inherits3(SEXP x, SEXP what, SEXP which)
 {
-    SEXP x, klass, what, which, rval = R_NilValue /* -Wall */;
-    int i, j, nwhat, isvec, nclass;
-
-    checkArity(op, args);
-
-    x = CAR(args);
+    SEXP klass, rval = R_NilValue /* -Wall */;
     if(IS_S4_OBJECT(x))
-      return do_S4inherits(x, CADR(args), CADDR(args));
+	PROTECT(klass = R_data_class2(x));
     else
-      PROTECT(klass = R_data_class(x, FALSE));
-    nclass = length(klass);
+	PROTECT(klass = R_data_class(x, FALSE));
+    int nclass = length(klass);
 
-    what = CADR(args);
     if(!isString(what))
 	error(_("'what' must be a character vector"));
-    nwhat = length(what);
+    int j, nwhat = length(what);
 
-    which = CADDR(args);
     if( !isLogical(which) || (length(which) != 1) )
 	error(_("'which' must be a length 1 logical vector"));
-    isvec = asLogical(which);
+    int isvec = asLogical(which);
 
 #ifdef _be_too_picky_
     if(IS_S4_OBJECT(x) && nwhat == 1 && !isvec &&
@@ -897,13 +869,13 @@ SEXP attribute_hidden do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
 	PROTECT(rval = allocVector(INTSXP, nwhat));
 
     for(j = 0; j < nwhat; j++) {
-	const char *ss = translateChar(STRING_ELT(what, j));
+	const char *ss = translateChar(STRING_ELT(what, j)); int i;
+	if(isvec)
+	    INTEGER(rval)[j] = 0;
 	for(i = 0; i < nclass; i++) {
-	    if(isvec)
-		INTEGER(rval)[j] = 0;
 	    if(!strcmp(translateChar(STRING_ELT(klass, i)), ss)) {
 		if(isvec)
-		   INTEGER(rval)[j] = i+1;
+		    INTEGER(rval)[j] = i+1;
 		else {
 		    UNPROTECT(1);
 		    return mkTrue();
@@ -920,6 +892,68 @@ SEXP attribute_hidden do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
     return rval;
 }
 
+SEXP attribute_hidden do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
+{
+    checkArity(op, args);
+
+    return inherits3(/* x = */ CAR(args),
+		     /* what = */ CADR(args),
+		     /* which = */ CADDR(args));
+}
+
+
+/**
+ * Return the 0-based index of an is() match in a vector of class-name
+ * strings terminated by an empty string.  Returns -1 for no match.
+ *
+ * @param x  an R object, about which we want is(x, .) information.
+ * @param valid vector of possible matches terminated by an empty string.
+ * @param rho  the environment in which the class definitions exist.
+ *
+ * @return index of match or -1 for no match
+ */
+int R_check_class_and_super(SEXP x, const char **valid, SEXP rho)
+{
+    int ans;
+    SEXP cl = getAttrib(x, R_ClassSymbol);
+    const char *class = CHAR(asChar(cl));
+    for (ans = 0; ; ans++) {
+	if (!strlen(valid[ans])) // empty string
+	    break;
+	if (!strcmp(class, valid[ans])) return ans;
+    }
+    /* if not found directly, now search the non-virtual super classes :*/
+    if(IS_S4_OBJECT(x)) {
+	/* now try the superclasses, i.e.,  try   is(x, "....") : */
+	SEXP classExts, superCl, _call;
+	static SEXP s_contains = NULL, s_selectSuperCl = NULL;
+	int i;
+	if(!s_contains) {
+	    s_contains      = install("contains");
+	    s_selectSuperCl = install(".selectSuperClasses");
+	}
+
+	PROTECT(classExts = R_do_slot(R_getClassDef(class), s_contains));
+	PROTECT(_call = lang3(s_selectSuperCl, classExts,
+			      /* dropVirtual = */ ScalarLogical(1)));
+	superCl = eval(_call, rho);
+	UNPROTECT(2);
+	PROTECT(superCl);
+	for(i=0; i < length(superCl); i++) {
+	    const char *s_class = CHAR(STRING_ELT(superCl, i));
+	    for (ans = 0; ; ans++) {
+		if (!strlen(valid[ans]))
+		    break;
+		if (!strcmp(s_class, valid[ans])) {
+		    UNPROTECT(1);
+		    return ans;
+		}
+	    }
+	}
+	UNPROTECT(1);
+    }
+    return -1;
+}
 
 /*
    ==============================================================
@@ -990,18 +1024,17 @@ static SEXP dispatchNonGeneric(SEXP name, SEXP env, SEXP fdef)
 {
     /* dispatch the non-generic definition of `name'.  Used to trap
        calls to standardGeneric during the loading of the methods package */
-    SEXP e, value, rho, fun, symbol, dot_Generic;
+    SEXP e, value, rho, fun, symbol;
     RCNTXT *cptr;
     /* find a non-generic function */
     symbol = install(translateChar(asChar(name)));
-    dot_Generic = install(".Generic");
     for(rho = ENCLOS(env); rho != R_EmptyEnv;
 	rho = ENCLOS(rho)) {
 	fun = findVarInFrame3(rho, symbol, TRUE);
 	if(fun == R_UnboundValue) continue;
 	switch(TYPEOF(fun)) {
 	case CLOSXP:
-	    value = findVarInFrame3(CLOENV(fun), dot_Generic, TRUE);
+	    value = findVarInFrame3(CLOENV(fun), R_dot_Generic, TRUE);
 	    if(value == R_UnboundValue) break;
 	case BUILTINSXP:  case SPECIALSXP:
 	default:
