@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2000  Robert Gentleman, Ross Ihaka and the
- *                            R Development Core Team
+ *  Copyright (C) 1998--2011  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,13 +23,15 @@
 #endif
 
 #include <Defn.h>
+#include <float.h>		/* for DBL_MAX */
 #include <Print.h>		/* for printRealVector() */
 #include <Rmath.h>
 #include <R_ext/Applic.h>
-#include <R_ext/RS.h>			/* for Memcpy */
+#include <R_ext/RS.h>	       	/* for Memcpy */
 
 
-/* One Dimensional Minimization --- just wrapper code for Brent's "fmin" */
+/* One Dimensional Minimization --- just wrapper for
+ * Brent's "fmin" --> ../appl/fmin.c */
 
 struct callinfo {
   SEXP R_fcall;
@@ -49,7 +50,7 @@ static double fcn1(double x, struct callinfo *info)
     case INTSXP:
 	if (length(s) != 1) goto badvalue;
 	if (INTEGER(s)[0] == NA_INTEGER) {
-	    warning("NA replaced by maximum positive value");
+	    warning(_("NA replaced by maximum positive value"));
 	    return DBL_MAX;
 	}
 	else return INTEGER(s)[0];
@@ -57,7 +58,7 @@ static double fcn1(double x, struct callinfo *info)
     case REALSXP:
 	if (length(s) != 1) goto badvalue;
 	if (!R_FINITE(REAL(s)[0])) {
-	    warning("NA/Inf replaced by maximum positive value");
+	    warning(_("NA/Inf replaced by maximum positive value"));
 	    return DBL_MAX;
 	}
 	else return REAL(s)[0];
@@ -66,54 +67,54 @@ static double fcn1(double x, struct callinfo *info)
 	goto badvalue;
     }
  badvalue:
-    error("invalid function value in 'optimize'");
+    error(_("invalid function value in 'optimize'"));
     return 0;/* for -Wall */
 }
 
 /* fmin(f, xmin, xmax tol) */
-SEXP do_fmin(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_fmin(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     double xmin, xmax, tol;
     SEXP v, res;
     struct callinfo info;
 
     checkArity(op, args);
-    PrintDefaults(rho);
+    PrintDefaults();
 
     /* the function to be minimized */
 
     v = CAR(args);
     if (!isFunction(v))
-	errorcall(call, "attempt to minimize non-function");
+	error(_("attempt to minimize non-function"));
     args = CDR(args);
 
     /* xmin */
 
     xmin = asReal(CAR(args));
     if (!R_FINITE(xmin))
-	errorcall(call, "invalid xmin value");
+	error(_("invalid '%s' value"), "xmin");
     args = CDR(args);
 
     /* xmax */
 
     xmax = asReal(CAR(args));
     if (!R_FINITE(xmax))
-	errorcall(call, "invalid xmax value");
+	error(_("invalid '%s' value"), "xmax");
     if (xmin >= xmax)
-	errorcall(call, "xmin not less than xmax");
+	error(_("'xmin' not less than 'xmax'"));
     args = CDR(args);
 
     /* tol */
 
     tol = asReal(CAR(args));
     if (!R_FINITE(tol) || tol <= 0.0)
-	errorcall(call, "invalid tol value");
+	error(_("invalid '%s' value"), "tol");
 
     info.R_env = rho;
     PROTECT(info.R_fcall = lang2(v, R_NilValue));
     PROTECT(res = allocVector(REALSXP, 1));
     SETCADR(info.R_fcall, allocVector(REALSXP, 1));
-    REAL(res)[0] = Brent_fmin(xmin, xmax, 
+    REAL(res)[0] = Brent_fmin(xmin, xmax,
 			      (double (*)(double, void*)) fcn1, &info, tol);
     UNPROTECT(2);
     return res;
@@ -123,7 +124,6 @@ SEXP do_fmin(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 /* One Dimensional Root Finding --  just wrapper code for Brent's "zeroin" */
 
-
 static double fcn2(double x, struct callinfo *info)
 {
     SEXP s;
@@ -133,7 +133,7 @@ static double fcn2(double x, struct callinfo *info)
     case INTSXP:
 	if (length(s) != 1) goto badvalue;
 	if (INTEGER(s)[0] == NA_INTEGER) {
-	    warning("NA replaced by maximum positive value");
+	    warning(_("NA replaced by maximum positive value"));
 	    return	DBL_MAX;
 	}
 	else return INTEGER(s)[0];
@@ -141,8 +141,13 @@ static double fcn2(double x, struct callinfo *info)
     case REALSXP:
 	if (length(s) != 1) goto badvalue;
 	if (!R_FINITE(REAL(s)[0])) {
-	    warning("NA/Inf replaced by maximum positive value");
-	    return DBL_MAX;
+	    if(REAL(s)[0] == R_NegInf) { // keep sign for root finding !
+		warning(_("-Inf replaced by maximally negative value"));
+		return -DBL_MAX;
+	    } else {
+		warning(_("NA/Inf replaced by maximum positive value"));
+		return DBL_MAX;
+	    }
 	}
 	else return REAL(s)[0];
 	break;
@@ -150,61 +155,63 @@ static double fcn2(double x, struct callinfo *info)
 	goto badvalue;
     }
  badvalue:
-    error("invalid function value in 'zeroin'");
+    error(_("invalid function value in 'zeroin'"));
     return 0;/* for -Wall */
 
 }
 
 /* zeroin(f, xmin, xmax, tol, maxiter) */
-SEXP do_zeroin(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_zeroin(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    double xmin, xmax, tol;
-    int iter;
-    SEXP v, res;
-    struct callinfo info;
+#define DO_ZEROIN_part_1				\
+    double xmin, xmax, tol;				\
+    int iter;						\
+    SEXP v, res;					\
+    struct callinfo info;				\
+							\
+    checkArity(op, args);				\
+    PrintDefaults();					\
+							\
+    /* the function to be minimized */			\
+    v = CAR(args);					\
+    if (!isFunction(v))					\
+	error(_("attempt to minimize non-function"));	\
+    args = CDR(args);					\
+							\
+    /* xmin */						\
+    xmin = asReal(CAR(args));				\
+    if (!R_FINITE(xmin))				\
+	error(_("invalid '%s' value"), "xmin");		\
+    args = CDR(args);					\
+							\
+    /* xmax */						\
+    xmax = asReal(CAR(args));				\
+    if (!R_FINITE(xmax))				\
+	error(_("invalid '%s' value"), "xmax");		\
+    if (xmin >= xmax)					\
+	error(_("'xmin' not less than 'xmax'"));	\
+    args = CDR(args)
 
-    checkArity(op, args);
-    PrintDefaults(rho);
+#define DO_ZEROIN_part_2						\
+    /* tol */								\
+    tol = asReal(CAR(args));						\
+    if (!R_FINITE(tol) || tol <= 0.0)					\
+	error(_("invalid '%s' value"), "tol");				\
+    args = CDR(args);							\
+									\
+    /* maxiter */							\
+    iter = asInteger(CAR(args));					\
+    if (iter <= 0)							\
+	error(_("'maxiter' must be positive"));				\
+									\
+    info.R_env = rho;							\
+    PROTECT(info.R_fcall = lang2(v, R_NilValue)); /* the info used in fcn2() */	\
+    SETCADR(info.R_fcall, allocVector(REALSXP, 1));			\
+    PROTECT(res = allocVector(REALSXP, 3))
 
-    /* the function to be minimized */
+    DO_ZEROIN_part_1;
+    DO_ZEROIN_part_2;
 
-    v = CAR(args);
-    if (!isFunction(v))
-	errorcall(call,"attempt to minimize non-function");
-    args = CDR(args);
-
-    /* xmin */
-
-    xmin = asReal(CAR(args));
-    if (!R_FINITE(xmin))
-	errorcall(call, "invalid xmin value");
-    args = CDR(args);
-
-    /* xmax */
-
-    xmax = asReal(CAR(args));
-    if (!R_FINITE(xmax))
-	errorcall(call, "invalid xmax value");
-    if (xmin >= xmax)
-	errorcall(call, "xmin not less than xmax");
-    args = CDR(args);
-
-    /* tol */
-
-    tol = asReal(CAR(args));
-    if (!R_FINITE(tol) || tol <= 0.0)
-	errorcall(call, "invalid tol value");
-    args = CDR(args);
-
-    /* maxiter */
-    iter = asInteger(CAR(args));
-    if (iter <= 0)
-	errorcall(call, "maxiter must be positive");
-
-    info.R_env = rho;
-    PROTECT(info.R_fcall = lang2(v, R_NilValue)); /* the info used in fcn2() */
-    SETCADR(info.R_fcall, allocVector(REALSXP, 1));
-    PROTECT(res = allocVector(REALSXP, 3));
     REAL(res)[0] =
 	R_zeroin(xmin, xmax,   (double (*)(double, void*)) fcn2,
 		 (void *) &info, &tol, &iter);
@@ -213,6 +220,37 @@ SEXP do_zeroin(SEXP call, SEXP op, SEXP args, SEXP rho)
     UNPROTECT(2);
     return res;
 }
+
+/* zeroin2(f, ax, bx, f.ax, f.bx, tol, maxiter) */
+SEXP attribute_hidden do_zeroin2(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    double f_ax, f_bx;
+    DO_ZEROIN_part_1;
+
+    /* f(ax) = f(xmin) */
+    f_ax = asReal(CAR(args));
+    if (ISNA(f_ax))
+	error(_("NA value for '%s' is not allowed"), "f.lower");
+    args = CDR(args);
+
+    /* f(bx) = f(xmax) */
+    f_bx = asReal(CAR(args));
+    if (ISNA(f_bx))
+	error(_("NA value for '%s' is not allowed"), "f.upper");
+    args = CDR(args);
+
+    DO_ZEROIN_part_2;
+
+    REAL(res)[0] =
+	R_zeroin2(xmin, xmax, f_ax, f_bx, (double (*)(double, void*)) fcn2,
+		 (void *) &info, &tol, &iter);
+    REAL(res)[1] = (double)iter;
+    REAL(res)[2] = tol;
+    UNPROTECT(2);
+    return res;
+}
+#undef DO_ZEROIN_part_1
+#undef DO_ZEROIN_part_2
 
 
 
@@ -229,14 +267,14 @@ typedef struct {
 } ftable;
 
 typedef struct {
-  SEXP R_fcall;       /* unevaluated call to R function */
-  SEXP R_env;         /* where to evaluate the calls */
+  SEXP R_fcall;	      /* unevaluated call to R function */
+  SEXP R_env;	      /* where to evaluate the calls */
   int have_gradient;
   int have_hessian;
 /*  int n;	      -* length of the parameter (x) vector */
-  int FT_size;        /* size of table to store computed
+  int FT_size;	      /* size of table to store computed
 			 function values */
-  int FT_last;        /* Newest entry in the table */
+  int FT_last;	      /* Newest entry in the table */
   ftable *Ftable;
 } function_info;
 
@@ -282,7 +320,7 @@ static void FT_store(int n, const double f, const double *x, const double *grad,
     state->Ftable[ind].fval = f;
     Memcpy(state->Ftable[ind].x, x, n);
     if (grad) {
-        Memcpy(state->Ftable[ind].grad, grad, n);
+	Memcpy(state->Ftable[ind].grad, grad, n);
 	if (hess) {
 	    Memcpy(state->Ftable[ind].hess, hess, n * n);
 	}
@@ -298,7 +336,7 @@ static int FT_lookup(int n, const double *x, function_info *state)
     int i, j, ind, matched;
     int FT_size, FT_last;
     ftable *Ftable;
-    
+
     FT_last = state->FT_last;
     FT_size = state->FT_size;
     Ftable = state->Ftable;
@@ -311,13 +349,13 @@ static int FT_lookup(int n, const double *x, function_info *state)
 	if (ftx) {
 	    matched = 1;
 	    for (j = 0; j < n; j++) {
-	        if (x[j] != ftx[j]) {
-	            matched = 0;
+		if (x[j] != ftx[j]) {
+		    matched = 0;
 		    break;
-	        }
+		}
 	    }
 	    if (matched) return ind;
-        }
+	}
     }
     return -1;
 }
@@ -341,15 +379,15 @@ static void fcn(int n, const double x[], double *f, function_info
 				/* calculate for a new value of x */
     s = CADR(R_fcall);
     for (i = 0; i < n; i++) {
-	if (!R_FINITE(x[i])) error("non-finite value supplied by nlm");
+	if (!R_FINITE(x[i])) error(_("non-finite value supplied by 'nlm'"));
 	REAL(s)[i] = x[i];
     }
-    s = eval(state->R_fcall, state->R_env);
+    s = PROTECT(eval(state->R_fcall, state->R_env));
     switch(TYPEOF(s)) {
     case INTSXP:
 	if (length(s) != 1) goto badvalue;
 	if (INTEGER(s)[0] == NA_INTEGER) {
-	    warning("NA replaced by maximum positive value");
+	    warning(_("NA replaced by maximum positive value"));
 	    *f = DBL_MAX;
 	}
 	else *f = INTEGER(s)[0];
@@ -357,7 +395,7 @@ static void fcn(int n, const double x[], double *f, function_info
     case REALSXP:
 	if (length(s) != 1) goto badvalue;
 	if (!R_FINITE(REAL(s)[0])) {
-	    warning("NA/Inf replaced by maximum positive value");
+	    warning(_("NA/Inf replaced by maximum positive value"));
 	    *f = DBL_MAX;
 	}
 	else *f = REAL(s)[0];
@@ -366,16 +404,17 @@ static void fcn(int n, const double x[], double *f, function_info
 	goto badvalue;
     }
     if (state->have_gradient) {
-	g = REAL(coerceVector(getAttrib(s, install("gradient")), REALSXP));
+	g = REAL(PROTECT(coerceVector(getAttrib(s, install("gradient")), REALSXP)));
 	if (state->have_hessian) {
-	    h = REAL(coerceVector(getAttrib(s, install("hessian")), REALSXP));
+	    h = REAL(PROTECT(coerceVector(getAttrib(s, install("hessian")), REALSXP)));
 	}
     }
     FT_store(n, *f, x, g, h, state);
+    UNPROTECT(1 + state->have_gradient + state->have_hessian);
     return;
 
  badvalue:
-    error("invalid function value in 'nlm' optimizer");
+    error(_("invalid function value in 'nlm' optimizer"));
 }
 
 
@@ -386,7 +425,7 @@ static void Cd1fcn(int n, const double x[], double *g, function_info *state)
     if ((ind = FT_lookup(n, x, state)) < 0) {	/* shouldn't happen */
 	fcn(n, x, g, state);
 	if ((ind = FT_lookup(n, x, state)) < 0) {
-	    error("function value caching for optimization is seriously confused.\n");
+	    error(_("function value caching for optimization is seriously confused"));
 	}
     }
     Memcpy(g, state->Ftable[ind].grad, n);
@@ -401,11 +440,11 @@ static void Cd2fcn(int nr, int n, const double x[], double *h,
     if ((ind = FT_lookup(n, x, state)) < 0) {	/* shouldn't happen */
 	fcn(n, x, h, state);
 	if ((ind = FT_lookup(n, x, state)) < 0) {
-	    error("function value caching for optimization is seriously confused.\n");
+	    error(_("function value caching for optimization is seriously confused"));
 	}
     }
     for (j = 0; j < n; j++) {  /* fill in lower triangle only */
-        Memcpy( h + j*(n + 1), state->Ftable[ind].hess + j*(n + 1), n - j);
+	Memcpy( h + j*(n + 1), state->Ftable[ind].hess + j*(n + 1), n - j);
     }
 }
 
@@ -416,15 +455,15 @@ static double *fixparam(SEXP p, int *n, SEXP call)
     int i;
 
     if (!isNumeric(p))
-	errorcall(call, "numeric parameter expected");
+	error(_("numeric parameter expected"));
 
     if (*n) {
 	if (LENGTH(p) != *n)
-	    errorcall(call, "conflicting parameter lengths");
+	    error(_("conflicting parameter lengths"));
     }
     else {
 	if (LENGTH(p) <= 0)
-	    errorcall(call, "invalid parameter length");
+	    error(_("invalid parameter length"));
 	*n = LENGTH(p);
     }
 
@@ -434,19 +473,19 @@ static double *fixparam(SEXP p, int *n, SEXP call)
     case INTSXP:
 	for (i = 0; i < *n; i++) {
 	    if (INTEGER(p)[i] == NA_INTEGER)
-		errorcall(call, "missing value in parameter");
+		error(_("missing value in parameter"));
 	    x[i] = INTEGER(p)[i];
 	}
 	break;
     case REALSXP:
 	for (i = 0; i < *n; i++) {
 	    if (!R_FINITE(REAL(p)[i]))
-		errorcall(call, "missing value in parameter");
+		error(_("missing value in parameter"));
 	    x[i] = REAL(p)[i];
 	}
 	break;
     default:
-	errorcall(call, "invalid parameter type");
+	error(_("invalid parameter type"));
     }
     return x;
 }
@@ -454,7 +493,7 @@ static double *fixparam(SEXP p, int *n, SEXP call)
 
 static void invalid_na(SEXP call)
 {
-    errorcall(call, "invalid NA value in parameter");
+    error(_("invalid NA value in parameter"));
 }
 
 
@@ -464,25 +503,25 @@ static void opterror(int nerr)
 {
     switch(nerr) {
     case -1:
-	error("non-positive number of parameters in nlm");
+	error(_("non-positive number of parameters in nlm"));
     case -2:
-	error("nlm is inefficient for 1-d problems");
+	error(_("nlm is inefficient for 1-d problems"));
     case -3:
-	error("illegal gradient tolerance in nlm");
+	error(_("invalid gradient tolerance in nlm"));
     case -4:
-	error("illegal iteration limit in nlm");
+	error(_("invalid iteration limit in nlm"));
     case -5:
-	error("minimization function has no good digits in nlm");
+	error(_("minimization function has no good digits in nlm"));
     case -6:
-	error("no analytic gradient to check in nlm!");
+	error(_("no analytic gradient to check in nlm!"));
     case -7:
-	error("no analytic Hessian to check in nlm!");
+	error(_("no analytic Hessian to check in nlm!"));
     case -21:
-	error("probable coding error in analytic gradient");
+	error(_("probable coding error in analytic gradient"));
     case -22:
-	error("probable coding error in analytic Hessian");
+	error(_("probable coding error in analytic Hessian"));
     default:
-	error("*** unknown error message (msg = %d) in nlm()\n*** should not happen!", nerr);
+	error(_("*** unknown error message (msg = %d) in nlm()\n*** should not happen!"), nerr);
     }
 }
 
@@ -493,34 +532,36 @@ static void optcode(int code)
 {
     switch(code) {
     case 1:
-	Rprintf("Relative gradient close to zero.\n");
-	Rprintf("Current iterate is probably solution.\n");
+	Rprintf(_("Relative gradient close to zero.\n"));
+	Rprintf(_("Current iterate is probably solution.\n"));
 	break;
     case 2:
-	Rprintf("Successive iterates within tolerance.\n");
-	Rprintf("Current iterate is probably solution.\n");
+	Rprintf(_("Successive iterates within tolerance.\n"));
+	Rprintf(_("Current iterate is probably solution.\n"));
 	break;
     case 3:
-	Rprintf("Last global step failed to locate a point lower than x.\n");
-	Rprintf("Either x is an approximate local minimum of the function,\n");
-	Rprintf("the function is too non-linear for this algorithm,\n");
-	Rprintf("or steptol is too large.\n");
+	Rprintf(_("Last global step failed to locate a point lower than x.\n"));
+	Rprintf(_("Either x is an approximate local minimum of the function,\n\
+the function is too non-linear for this algorithm,\n\
+or steptol is too large.\n"));
 	break;
     case 4:
-	Rprintf("Iteration limit exceeded.  Algorithm failed.\n");
+	Rprintf(_("Iteration limit exceeded.  Algorithm failed.\n"));
 	break;
     case 5:
-	Rprintf("Maximum step size exceeded 5 consecutive times.\n");
-	Rprintf("Either the function is unbounded below,\n");
-	Rprintf("becomes asymptotic to a finite value\n");
-	Rprintf("from above in some direction,\n");
-	Rprintf("or stepmx is too small.\n");
+	Rprintf(_("Maximum step size exceeded 5 consecutive times.\n\
+Either the function is unbounded below,\n\
+becomes asymptotic to a finite value\n\
+from above in some direction,\n"\
+"or stepmx is too small.\n"));
 	break;
     }
     Rprintf("\n");
 }
 
-SEXP do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
+/* NOTE: The actual Dennis-Schnabel algorithm `optif9' is in ../appl/uncmin.c */
+
+SEXP attribute_hidden do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP value, names, v, R_gradientSymbol, R_hessianSymbol;
 
@@ -530,13 +571,15 @@ SEXP do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
     int code, i, j, k, itnlim, method, iexp, omsg, msg,
 	n, ndigit, iagflg, iahflg, want_hessian, itncnt;
 
-    char *vmax;
 
+/* .Internal(
+ *	nlm(function(x) f(x, ...), p, hessian, typsize, fscale,
+ *	    msg, ndigit, gradtol, stepmax, steptol, iterlim)
+ */
     function_info *state;
 
     checkArity(op, args);
-    PrintDefaults(rho);
-    vmax = vmaxget();
+    PrintDefaults();
 
     state = (function_info *) R_alloc(1, sizeof(function_info));
 
@@ -545,33 +588,34 @@ SEXP do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
     state->R_env = rho;
     v = CAR(args);
     if (!isFunction(v))
-	error("attempt to minimize non-function");
+	error(_("attempt to minimize non-function"));
     PROTECT(state->R_fcall = lang2(v, R_NilValue));
     args = CDR(args);
 
-    /* inital parameter value */
+    /* `p' : inital parameter value */
 
     n = 0;
     x = fixparam(CAR(args), &n, call);
     args = CDR(args);
 
-    /* hessian required? */
+    /* `hessian' : H. required? */
 
     want_hessian = asLogical(CAR(args));
     if (want_hessian == NA_LOGICAL) want_hessian = 0;
     args = CDR(args);
 
-    /* typical size of parameter elements */
+    /* `typsize' : typical size of parameter elements */
 
     typsiz = fixparam(CAR(args), &n, call);
     args = CDR(args);
 
-    /* expected function size */
+    /* `fscale' : expected function size */
 
     fscale = asReal(CAR(args));
     if (ISNA(fscale)) invalid_na(call);
     args = CDR(args);
 
+    /* `msg' (bit pattern) */
     omsg = msg = asInteger(CAR(args));
     if (msg == NA_INTEGER) invalid_na(call);
     args = CDR(args);
@@ -592,6 +636,7 @@ SEXP do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (ISNA(steptol)) invalid_na(call);
     args = CDR(args);
 
+    /* `iterlim' (def. 100) */
     itnlim = asInteger(CAR(args));
     if (itnlim == NA_INTEGER) invalid_na(call);
     args = CDR(args);
@@ -617,17 +662,17 @@ SEXP do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    iagflg = 1;
 	    state->have_gradient = 1;
 	    v = getAttrib(value, R_hessianSymbol);
-	    
+
 	    if (v != R_NilValue) {
 		if (LENGTH(v) == (n * n) && (isReal(v) || isInteger(v))) {
 		    iahflg = 1;
 		    state->have_hessian = 1;
 		} else {
-		    warning("hessian supplied is of the wrong length or mode, so ignored");
+		    warning(_("hessian supplied is of the wrong length or mode, so ignored"));
 		}
 	    }
 	} else {
-	    warning("gradient supplied is of the wrong length or mode, so ignored");
+	    warning(_("gradient supplied is of the wrong length or mode, so ignored"));
 	}
     }
     if (((msg/4) % 2) && !iahflg) { /* skip check of analytic Hessian */
@@ -694,8 +739,7 @@ SEXP do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
     k = 0;
 
     SET_STRING_ELT(names, k, mkChar("minimum"));
-    SET_VECTOR_ELT(value, k, allocVector(REALSXP, 1));
-    REAL(VECTOR_ELT(value, k))[0] = fpls;
+    SET_VECTOR_ELT(value, k, ScalarReal(fpls));
     k++;
 
     SET_STRING_ELT(names, k, mkChar("estimate"));
@@ -730,7 +774,6 @@ SEXP do_nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
     k++;
 
     setAttrib(value, R_NamesSymbol, names);
-    vmaxset(vmax);
     UNPROTECT(3);
     return value;
 }
