@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2009  The R Development Core Team
+ *  Copyright (C) 1997--2011  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <Rversion.h>
 #include <R_ext/RS.h>           /* for CallocCharBuf, Free */
 #include <errno.h>
+#include <ctype.h>		/* for isspace */
 
 /* From time to time changes in R, such as the addition of a new SXP,
  * may require changes in the save file format.  Here are some
@@ -112,9 +113,9 @@
    integer.  This produces more compact output for code; it has little
    effect on data.
 
-   Environments recognized as package or name space environments are
+   Environments recognized as package or namespace environments are
    not saved directly. Instead, a STRSXP is saved that is then used to
-   attempt to find the package/name space when unserialized.  The
+   attempt to find the package/namespace when unserialized.  The
    exact mechanism for choosing the name and finding the package/name
    space from the name still has to be developed, but the
    serialization format should be able to accommodate any reasonable
@@ -128,7 +129,7 @@
 
    A mechanism is provided to allow special handling of non-system
    reference objects (all weak references and external pointers, and
-   all environments other than package environments, name space
+   all environments other than package environments, namespace
    environments, and the global environment).  The hook function
    consists of a function pointer and a data value.  The serialization
    function pointer is called with the reference object and the data
@@ -624,6 +625,21 @@ static int HashGet(SEXP item, SEXP ht)
 #define EMPTYENV_SXP	  242
 #define BASEENV_SXP	  241
 
+/* The following are needed to preserve attribute information on
+   expressions in the constant pool of byte code objects. This is
+   mainly for preserving source references attributes.  The original
+   implementation of the sharing-preserving writing and reading of yte
+   code objects did not account for the need to preserve attributes,
+   so there is now a work-around using these SXP types to flag when
+   the ATTRIB field has been written out. Object bits and S4 bits are
+   still not preserved.  It the long run in might be better to change
+   to a scheme in which all sharing is preserved and byte code objects
+   don't need to be handled as a special case.  LT */
+#ifdef BYTECODE
+#define ATTRLANGSXP       240
+#define ATTRLISTSXP       239
+#endif
+
 /*
  * Type/Flag Packing and Unpacking
  *
@@ -760,12 +776,11 @@ static int SaveSpecialHook(SEXP item)
 static void OutStringVec(R_outpstream_t stream, SEXP s, SEXP ref_table)
 {
     int i, len;
-    SEXP names;
 
     R_assert(TYPEOF(s) == STRSXP);
 
-    names = getAttrib(s, R_NamesSymbol);
 #ifdef WARN_ABOUT_NAMES_IN_PERSISTENT_STRINGS
+    SEXP names = getAttrib(s, R_NamesSymbol);
     if (names != R_NilValue)
 	warning(_("names in persistent strings are currently ignored"));
 #endif
@@ -796,6 +811,18 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
     int i;
     SEXP t;
 
+#ifdef BYTECODE
+    if (R_compile_pkgs && TYPEOF(s) == CLOSXP && TYPEOF(BODY(s)) != BCODESXP) {
+	SEXP new_s;
+	R_compile_pkgs = FALSE;
+	PROTECT(new_s = R_cmpfun(s));
+	WriteItem (new_s, ref_table, stream);
+	UNPROTECT(1);
+	R_compile_pkgs = TRUE;
+	return;
+    }
+#endif
+
  tailcall:
     R_CheckStack();
     if ((t = GetPersistentName(stream, s)) != R_NilValue) {
@@ -1045,7 +1072,16 @@ static void WriteBCLang(SEXP s, SEXP ref_table, SEXP reps,
 	    }
 	}
 	if (output) {
+	    SEXP attr = ATTRIB(s);
+	    if (attr != R_NilValue) {
+		switch(type) {
+		case LANGSXP: type = ATTRLANGSXP; break;
+		case LISTSXP: type = ATTRLISTSXP; break;
+		}
+	    }
 	    OutInteger(stream, type);
+	    if (attr != R_NilValue)
+		WriteItem(attr, ref_table, stream);
 	    WriteItem(TAG(s), ref_table, stream);
 	    WriteBCLang(CAR(s), ref_table, reps, stream);
 	    WriteBCLang(CDR(s), ref_table, reps, stream);
@@ -1198,7 +1234,6 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
     SEXPTYPE type;
     SEXP s;
     int flags, levs, objf, hasattr, hastag, length, count;
-    char *cbuf;
 
     R_assert(TYPEOF(ref_table) == LISTSXP && TYPEOF(CAR(ref_table)) == VECSXP);
 
@@ -1310,12 +1345,14 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	    break;
 	case SPECIALSXP:
 	case BUILTINSXP:
-	    /* These are all short strings */
-	    length = InInteger(stream);
-	    cbuf = alloca(length+1);
-	    InString(stream, cbuf, length);
-	    cbuf[length] = '\0';
-	    PROTECT(s = mkPRIMSXP(StrToInternal(cbuf), type == BUILTINSXP));
+	    {
+		/* These are all short strings */
+		length = InInteger(stream);
+		char cbuf[length+1];
+		InString(stream, cbuf, length);
+		cbuf[length] = '\0';
+		PROTECT(s = mkPRIMSXP(StrToInternal(cbuf), type == BUILTINSXP));
+	    }
 	    break;
 	case CHARSXP:
 	    length = InInteger(stream);
@@ -1323,18 +1360,20 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 		PROTECT(s = NA_STRING);
 	    else if (length < 1000) {
 		int enc = CE_NATIVE;
-		cbuf = alloca(length+1);
+		char cbuf[length+1];
 		InString(stream, cbuf, length);
 		cbuf[length] = '\0';
 		if (levs & UTF8_MASK) enc = CE_UTF8;
 		else if (levs & LATIN1_MASK) enc = CE_LATIN1;
+		else if (levs & BYTES_MASK) enc = CE_BYTES;
 		PROTECT(s = mkCharLenCE(cbuf, length, enc));
 	    } else {
 		int enc = CE_NATIVE;
-		cbuf = CallocCharBuf(length);
+		char *cbuf = CallocCharBuf(length);
 		InString(stream, cbuf, length);
 		if (levs & UTF8_MASK) enc = CE_UTF8;
 		else if (levs & LATIN1_MASK) enc = CE_LATIN1;
+		else if (levs & BYTES_MASK) enc = CE_BYTES;
 		PROTECT(s = mkCharLenCE(cbuf, length, enc));
 		Free(cbuf);
 	    }
@@ -1430,16 +1469,25 @@ static SEXP ReadBCLang(int type, SEXP ref_table, SEXP reps,
     case BCREPDEF:
     case LANGSXP:
     case LISTSXP:
+    case ATTRLANGSXP:
+    case ATTRLISTSXP:
 	{
 	    SEXP ans;
 	    int pos = -1;
+	    int hasattr = FALSE;
 	    if (type == BCREPDEF) {
 		pos = InInteger(stream);
 		type = InInteger(stream);
 	    }
+	    switch (type) {
+	    case ATTRLANGSXP: type = LANGSXP; hasattr = TRUE; break;
+	    case ATTRLISTSXP: type = LISTSXP; hasattr = TRUE; break;
+	    }
 	    PROTECT(ans = allocSExp(type));
 	    if (pos >= 0)
 		SET_VECTOR_ELT(reps, pos, ans);
+	    if (hasattr)
+		SET_ATTRIB(ans, ReadItem(ref_table, stream));
 	    SET_TAG(ans, ReadItem(ref_table, stream));
 	    SETCAR(ans, ReadBCLang(InInteger(stream), ref_table, reps,
 				   stream));
@@ -1469,6 +1517,8 @@ static SEXP ReadBCConsts(SEXP ref_table, SEXP reps, R_inpstream_t stream)
 	case LISTSXP:
 	case BCREPDEF:
 	case BCREPREF:
+	case ATTRLANGSXP:
+	case ATTRLISTSXP:
 	    c = ReadBCLang(type, ref_table, reps, stream);
 	    SET_VECTOR_ELT(ans, i, c);
 	    break;
@@ -1766,18 +1816,28 @@ static SEXP CallHook(SEXP x, SEXP fun)
     return val;
 }
 
+static void con_cleanup(void *data)
+{
+    Rconnection con = data;
+    if(con->isopen) con->close(con);
+}
+
+/* Used from saveRDS().
+   This became public in R 2.13.0, and that version added support for
+   connections internally */
 SEXP attribute_hidden
 do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     /* serializeToConn(object, conn, ascii, version, hook) */
 
     SEXP object, fun;
-    Rboolean ascii;
+    Rboolean ascii, wasopen;
     int version;
     Rconnection con;
     struct R_outpstream_st out;
     R_pstream_format_t type;
     SEXP (*hook)(SEXP, SEXP);
+    RCNTXT cntxt;
 
     checkArity(op, args);
 
@@ -1802,19 +1862,49 @@ do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
     fun = CAR(nthcdr(args,4));
     hook = fun != R_NilValue ? CallHook : NULL;
 
+    /* Now we need to do some sanity checking of the arguments.
+       A filename will already have been opened, so anything 
+       not open was specified as a connection directly.
+     */
+    wasopen = con->isopen;
+    if(!wasopen) {
+	char mode[5];
+	strcpy(mode, con->mode);
+	strcpy(con->mode, ascii ? "w" : "wb");
+	if(!con->open(con)) error(_("cannot open the connection"));
+	strcpy(con->mode, mode);
+	/* Set up a context which will close the connection on error */
+	begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
+		     R_NilValue, R_NilValue);
+	cntxt.cend = &con_cleanup;
+	cntxt.cenddata = con;
+    }
+    if (!ascii && con->text)
+	error(_("binary-mode connection required for ascii=FALSE"));
+    if(!con->canwrite)
+	error(_("connection not open for writing"));
+
     R_InitConnOutPStream(&out, con, type, version, hook, fun);
     R_Serialize(object, &out);
+    if(!wasopen) {endcontext(&cntxt); con->close(con);}
+
     return R_NilValue;
 }
 
-SEXP attribute_hidden do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
+/* Used from readRDS().
+   This became public in R 2.13.0, and that version added support for
+   connections internally */
+SEXP attribute_hidden 
+do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     /* unserializeFromConn(conn, hook) */
 
     struct R_inpstream_st in;
     Rconnection con;
-    SEXP fun;
+    SEXP fun, ans;
     SEXP (*hook)(SEXP, SEXP);
+    Rboolean wasopen;
+    RCNTXT cntxt;
 
     checkArity(op, args);
 
@@ -1823,8 +1913,30 @@ SEXP attribute_hidden do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP
     fun = CADR(args);
     hook = fun != R_NilValue ? CallHook : NULL;
 
+    /* Now we need to do some sanity checking of the arguments.
+       A filename will already have been opened, so anything 
+       not open was specified as a connection directly.
+     */
+    wasopen = con->isopen;
+    if(!wasopen) {
+	char mode[5];
+	strcpy(mode, con->mode);
+	strcpy(con->mode, "rb");
+	if(!con->open(con)) error(_("cannot open the connection"));
+	strcpy(con->mode, mode);
+	/* Set up a context which will close the connection on error */
+	begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
+		     R_NilValue, R_NilValue);
+	cntxt.cend = &con_cleanup;
+	cntxt.cenddata = con;
+    }
+    if(!con->canread) error(_("connection not open for reading"));
+
     R_InitConnInPStream(&in, con, R_pstream_any_format, hook, fun);
-    return R_Unserialize(&in);
+    PROTECT(ans = R_Unserialize(&in)); /* paranoia about next line */
+    if(!wasopen) {endcontext(&cntxt); con->close(con);}
+    UNPROTECT(1);
+    return ans;
 }
 
 
@@ -1883,16 +1995,25 @@ static void InitBConOutPStream(R_outpstream_t stream, bconbuf_t bb,
 }
 
 /* only for use by serialize(), with binary write to a socket connection */
-SEXP attribute_hidden R_serializeb(SEXP object, SEXP icon, SEXP fun)
+SEXP attribute_hidden 
+R_serializeb(SEXP object, SEXP icon, SEXP Sversion, SEXP fun)
 {
     struct R_outpstream_st out;
     SEXP (*hook)(SEXP, SEXP);
     struct bconbuf_st bbs;
     Rconnection con = getConnection(asInteger(icon));
+    int version;
+    
+    if (Sversion == R_NilValue)
+	version = R_DefaultSerializeVersion;
+    else version = asInteger(Sversion);
+    if (version == NA_INTEGER || version <= 0)
+	error(_("bad version value"));
 
     hook = fun != R_NilValue ? CallHook : NULL;
 
-    InitBConOutPStream(&out, &bbs, con, R_pstream_xdr_format, 0, hook, fun);
+    InitBConOutPStream(&out, &bbs, con, R_pstream_xdr_format, version, 
+		       hook, fun);
     R_Serialize(object, &out);
     flush_bcon_buffer(&bbs);
     return R_NilValue;
@@ -1909,19 +2030,27 @@ typedef struct membuf_st {
     unsigned char *buf;
 } *membuf_t;
 
+
+#define INCR MAXELTSIZE
 static void resize_buffer(membuf_t mb, R_size_t needed)
 {
     /* This used to allocate double 'needed', but that was problematic for
        large buffers */
-    R_size_t newsize = needed;
-    /* we need to store the result in a RAWSXP */
+    /* we need to store the result in a RAWSXP so limited to INT_MAX */
     if(needed > INT_MAX)
 	error(_("serialization is too large to store in a raw vector"));
-    if(needed < INT_MAX - MAXELTSIZE) needed += MAXELTSIZE;
-    mb->buf = realloc(mb->buf, newsize);
-    if (mb->buf == NULL)
+    if(needed < 10000000) /* ca 10MB */
+	needed = (1+2*needed/INCR) * INCR;
+    if(needed < 1000000000) /* ca 1GB */
+	needed = (1+1.2*needed/INCR) * INCR;
+    else if(needed < INT_MAX - INCR) 
+	needed = (1+needed/INCR) * INCR;
+    unsigned char *tmp = realloc(mb->buf, needed);
+    if (tmp == NULL) {
+	free(mb->buf); mb->buf = NULL;
 	error(_("cannot allocate buffer"));
-    mb->size = newsize;
+    } else mb->buf = tmp;
+    mb->size = needed;
 }
 
 static void OutCharMem(R_outpstream_t stream, int c)
@@ -2008,11 +2137,18 @@ static SEXP CloseMemOutPStream(R_outpstream_t stream)
 }
 
 SEXP attribute_hidden
-R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP fun)
+R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP Sversion, SEXP fun)
 {
     struct R_outpstream_st out;
     R_pstream_format_t type;
     SEXP (*hook)(SEXP, SEXP);
+    int version;
+    
+    if (Sversion == R_NilValue)
+	version = R_DefaultSerializeVersion;
+    else version = asInteger(Sversion);
+    if (version == NA_INTEGER || version <= 0)
+	error(_("bad version value"));
 
     hook = fun != R_NilValue ? CallHook : NULL;
 
@@ -2030,7 +2166,7 @@ R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP fun)
 	cntxt.cend = &free_mem_buffer;
 	cntxt.cenddata = &mbs;
 
-	InitMemOutPStream(&out, &mbs, type, 0, hook, fun);
+	InitMemOutPStream(&out, &mbs, type, version, hook, fun);
 	R_Serialize(object, &out);
 
 	val =  CloseMemOutPStream(&out);
@@ -2308,7 +2444,7 @@ R_lazyLoadDBinsertValue(SEXP value, SEXP file, SEXP ascii,
     Rboolean compress = asInteger(compsxp);
     SEXP key;
 
-    value = R_serialize(value, R_NilValue, ascii, hook);
+    value = R_serialize(value, R_NilValue, ascii, R_NilValue, hook);
     PROTECT_WITH_INDEX(value, &vpi);
     if (compress == 3)
 	REPROTECT(value = R_compress3(value), vpi);
