@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2005   The R Development Core Team.
+ *  Copyright (C) 1998-2009   The R Development Core Team
  *  Copyright (C) 2004        The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,12 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
-/* <UTF8> char here is handled as a whole string.
-   Does rely on strcoll being correct.
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,6 +25,7 @@
 
 #include <Defn.h> /* => Utils.h with the protos from here */
 #include <Rmath.h>
+#include <R_ext/RS.h>  /* for Calloc/Free */
 
 			/*--- Part I: Comparison Utilities ---*/
 
@@ -78,10 +75,11 @@ static int scmp(SEXP x, SEXP y, Rboolean nalast)
     if (x == NA_STRING && y == NA_STRING) return 0;
     if (x == NA_STRING) return nalast?1:-1;
     if (y == NA_STRING) return nalast?-1:1;
-    return STRCOLL(CHAR(x), CHAR(y));
+    if (x == y) return 0;  /* same string in cache */
+    return Scollate(x, y);
 }
 
-Rboolean isUnsorted(SEXP x)
+Rboolean isUnsorted(SEXP x, Rboolean strictly)
 {
     int n, i;
 
@@ -94,27 +92,55 @@ Rboolean isUnsorted(SEXP x)
 	    /* NOTE: x must have no NAs {is.na(.) in R};
 	       hence be faster than `rcmp()', `icmp()' for these two cases */
 
+	    /* The only difference between strictly and not is '>' vs '>='
+	       but we want the if() outside the loop */
 	case LGLSXP:
 	case INTSXP:
-	    for(i = 0; i+1 < n ; i++)
-		if(INTEGER(x)[i] > INTEGER(x)[i+1])
-		    return TRUE;
+	    if(strictly) {
+		for(i = 0; i+1 < n ; i++)
+		    if(INTEGER(x)[i] >= INTEGER(x)[i+1])
+			return TRUE;
+
+	    } else {
+		for(i = 0; i+1 < n ; i++)
+		    if(INTEGER(x)[i] > INTEGER(x)[i+1])
+			return TRUE;
+	    }
 	    break;
 	case REALSXP:
-	    for(i = 0; i+1 < n ; i++)
-		if(REAL(x)[i] > REAL(x)[i+1])
-		    return TRUE;
+	    if(strictly) {
+		for(i = 0; i+1 < n ; i++)
+		    if(REAL(x)[i] >= REAL(x)[i+1])
+			return TRUE;
+	    } else {
+		for(i = 0; i+1 < n ; i++)
+		    if(REAL(x)[i] > REAL(x)[i+1])
+			return TRUE;
+	    }
 	    break;
 	case CPLXSXP:
-	    for(i = 0; i+1 < n ; i++)
-		if(ccmp(COMPLEX(x)[i], COMPLEX(x)[i+1], TRUE) > 0)
-		    return TRUE;
+	    if(strictly) {
+		for(i = 0; i+1 < n ; i++)
+		    if(ccmp(COMPLEX(x)[i], COMPLEX(x)[i+1], TRUE) >= 0)
+			return TRUE;
+	    } else {
+		for(i = 0; i+1 < n ; i++)
+		    if(ccmp(COMPLEX(x)[i], COMPLEX(x)[i+1], TRUE) > 0)
+			return TRUE;
+	    }
 	    break;
 	case STRSXP:
-	    for(i = 0; i+1 < n ; i++)
-		if(scmp(STRING_ELT(x, i ),
-			STRING_ELT(x,i+1), TRUE) > 0)
-		    return TRUE;
+	    if(strictly) {
+		for(i = 0; i+1 < n ; i++)
+		    if(scmp(STRING_ELT(x, i ),
+			    STRING_ELT(x,i+1), TRUE) >= 0)
+			return TRUE;
+	    } else {
+		for(i = 0; i+1 < n ; i++)
+		    if(scmp(STRING_ELT(x, i ),
+			    STRING_ELT(x,i+1), TRUE) > 0)
+			return TRUE;
+	    }
 	    break;
 	default:
 	    UNIMPLEMENTED_TYPE("isUnsorted", x);
@@ -122,14 +148,30 @@ Rboolean isUnsorted(SEXP x)
     return FALSE;/* sorted */
 }
 
-SEXP do_isunsorted(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_isunsorted(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP ans;
+    int strictly, n;
+    SEXP x, ans;
+    int res = TRUE;
 
     checkArity(op, args);
-    ans = allocVector(LGLSXP, 1);
-    LOGICAL(ans)[0] = isUnsorted(CAR(args));
-    return ans;
+    x = CAR(args);
+    strictly = asLogical(CADR(args));
+    if(strictly == NA_LOGICAL)
+	errorcall(call, _("invalid '%s' argument"), "strictly");
+    n = length(x);
+    if(n < 2) return ScalarLogical(FALSE);
+    if(isVectorAtomic(x))
+	return ScalarLogical(isUnsorted(x, strictly));
+    if(isObject(x)) {
+	/* try dispatch */
+	SEXP call;
+	PROTECT(call = lang3(install(".gtn"), x, CADR(args)));
+	ans = eval(call, rho);
+	UNPROTECT(1);
+	return ans;
+    } else res = NA_LOGICAL;
+    return ScalarLogical(res);
 }
 
 
@@ -177,7 +219,8 @@ void R_csort(Rcomplex *x, int n)
 }
 
 
-void ssort(SEXP *x, int n)
+/* used in platform.c */
+void attribute_hidden ssort(SEXP *x, int n)
 {
     SEXP v;
 #define TYPE_CMP scmp
@@ -254,7 +297,7 @@ void revsort(double *a, int *ib, int n)
 }
 
 
-SEXP do_sort(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_sort(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
     Rboolean decreasing;
@@ -266,15 +309,17 @@ SEXP do_sort(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("'decreasing' must be TRUE or FALSE"));
     if(CAR(args) == R_NilValue) return R_NilValue;
     if(!isVectorAtomic(CAR(args)))
-	errorcall(call, _("only atomic vectors can be sorted"));
+	error(_("only atomic vectors can be sorted"));
     if(TYPEOF(CAR(args)) == RAWSXP)
-	errorcall(call, _("raw vectors cannot be sorted"));
-    if (decreasing || isUnsorted(CAR(args))) { /* do not duplicate if sorted */
-	ans = duplicate(CAR(args));
-	sortVector(ans, decreasing);
-	return(ans);
-    }
-    else return(CAR(args));
+	error(_("raw vectors cannot be sorted"));
+    /* we need consistent behaviour here, including dropping attibutes,
+       so as from 2.3.0 we always duplicate. */
+    PROTECT(ans = duplicate(CAR(args)));
+    SET_ATTRIB(ans, R_NilValue);  /* this is never called with names */
+    SET_OBJECT(ans, 0);		  /* we may have just stripped off the class */
+    sortVector(ans, decreasing);
+    UNPROTECT(1);
+    return(ans);
 }
 
 /* faster versions of shellsort, following Sedgewick (1986) */
@@ -347,7 +392,7 @@ static void R_csort2(Rcomplex *x, int n, Rboolean decreasing)
 	}
 }
 
-void ssort2(SEXP *x, int n, Rboolean decreasing)
+static void ssort2(SEXP *x, int n, Rboolean decreasing)
 {
     SEXP v;
     int i, j, h, t;
@@ -370,7 +415,7 @@ void ssort2(SEXP *x, int n, Rboolean decreasing)
 void sortVector(SEXP s, Rboolean decreasing)
 {
     int n = LENGTH(s);
-    if (n >= 2 && (decreasing || isUnsorted(s)))
+    if (n >= 2 && (decreasing || isUnsorted(s, FALSE)))
 	switch (TYPEOF(s)) {
 	case LGLSXP:
 	case INTSXP:
@@ -404,7 +449,7 @@ void sortVector(SEXP s, Rboolean decreasing)
     Rboolean nalast=TRUE;					\
     int L, R, i, j;						\
 								\
-    for (L = 0, R = n - 1; L < R; ) {				\
+    for (L = lo, R = hi; L < R; ) {				\
 	v = x[k];						\
 	for(i = L, j = R; i <= j;) {				\
 	    while (TYPE_CMP(x[i], v, nalast) < 0) i++;			\
@@ -416,7 +461,7 @@ void sortVector(SEXP s, Rboolean decreasing)
     }
 
 
-void iPsort(int *x, int n, int k)
+static void iPsort2(int *x, int lo, int hi, int k)
 {
     int v, w;
 #define TYPE_CMP icmp
@@ -424,7 +469,7 @@ void iPsort(int *x, int n, int k)
 #undef TYPE_CMP
 }
 
-void rPsort(double *x, int n, int k)
+static void rPsort2(double *x, int lo, int hi, int k)
 {
     double v, w;
 #define TYPE_CMP rcmp
@@ -432,7 +477,7 @@ void rPsort(double *x, int n, int k)
 #undef TYPE_CMP
 }
 
-void cPsort(Rcomplex *x, int n, int k)
+static void cPsort2(Rcomplex *x, int lo, int hi, int k)
 {
     Rcomplex v, w;
 #define TYPE_CMP ccmp
@@ -441,7 +486,7 @@ void cPsort(Rcomplex *x, int n, int k)
 }
 
 
-static void sPsort(SEXP *x, int n, int k)
+static void sPsort2(SEXP *x, int lo, int hi, int k)
 {
     SEXP v, w;
 #define TYPE_CMP scmp
@@ -449,105 +494,165 @@ static void sPsort(SEXP *x, int n, int k)
 #undef TYPE_CMP
 }
 
-static void Psort(SEXP x, int k)
+/* elements of ind are 1-based, lo and hi are 0-based */
+
+static void Psort(SEXP x, int lo, int hi, int k)
 {
+    /* Rprintf("looking for index %d in (%d, %d)\n", k, lo, hi);*/
     switch (TYPEOF(x)) {
     case LGLSXP:
     case INTSXP:
-	iPsort(INTEGER(x), LENGTH(x), k);
+	iPsort2(INTEGER(x), lo, hi, k);
 	break;
     case REALSXP:
-	rPsort(REAL(x), LENGTH(x), k);
+	rPsort2(REAL(x), lo, hi, k);
 	break;
     case CPLXSXP:
-	cPsort(COMPLEX(x), LENGTH(x), k);
+	cPsort2(COMPLEX(x), lo, hi, k);
 	break;
     case STRSXP:
-	sPsort(STRING_PTR(x), LENGTH(x), k);
+	sPsort2(STRING_PTR(x), lo, hi, k);
 	break;
     default:
 	UNIMPLEMENTED_TYPE("Psort", x);
     }
 }
 
+static void Psort0(SEXP x, int lo, int hi, int *ind, int k)
+{
+    if(k < 1 || hi-lo < 1) return;
+    if(k <= 1)
+	Psort(x, lo, hi, ind[0]-1);
+    else {
+    /* Look for index nearest the centre of the range */
+	int i, This = 0, mid = (lo+hi)/2, z;
+	for(i = 0; i < k; i++)
+	    if(ind[i]-1 <= mid) This = i;
+	z = ind[This]-1;
+	Psort(x, lo, hi, z);
+	Psort0(x, lo, z-1, ind, This);
+	Psort0(x, z+1, hi, ind + This + 1, k - This -1);
+    }
+}
+
+/* Needed for mistaken decision to put these in the API */
+void iPsort(int *x, int n, int k)
+{
+    iPsort2(x, 0, n-1, k);
+}
+
+void rPsort(double *x, int n, int k)
+{
+    rPsort2(x, 0, n-1, k);
+}
+
+void cPsort(Rcomplex *x, int n, int k)
+{
+    cPsort2(x, 0, n-1, k);
+}
+
+
+
 /* FUNCTION psort(x, indices) */
-SEXP do_psort(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_psort(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int i, k, n;
     int *l;
     checkArity(op, args);
 
     if (!isVectorAtomic(CAR(args)))
-	errorcall(call, _("only atomic vectors can be sorted"));
+	error(_("only atomic vectors can be sorted"));
     if(TYPEOF(CAR(args)) == RAWSXP)
-	errorcall(call, _("raw vectors cannot be sorted"));
+	error(_("raw vectors cannot be sorted"));
     n = LENGTH(CAR(args));
     SETCADR(args, coerceVector(CADR(args), INTSXP));
     l = INTEGER(CADR(args));
     k = LENGTH(CADR(args));
     for (i = 0; i < k; i++) {
 	if (l[i] == NA_INTEGER)
-	    errorcall(call, _("NA index"));
+	    error(_("NA index"));
 	if (l[i] < 1 || l[i] > n)
-	    errorcall(call, _("index %d outside bounds"), l[i]);
+	    error(_("index %d outside bounds"), l[i]);
     }
     SETCAR(args, duplicate(CAR(args)));
-    for (i = 0; i < k; i++)
-	Psort(CAR(args), l[i] - 1);
+    SET_ATTRIB(CAR(args), R_NilValue);  /* remove all attributes */
+    SET_OBJECT(CAR(args), 0);           /* and the object bit    */
+    Psort0(CAR(args), 0, n - 1, l, k);
     return CAR(args);
 }
 
 
 			/*--- Part IV : Rank & Order ---*/
 
-static int equal(int i, int j, SEXP x, Rboolean nalast)
+static int equal(int i, int j, SEXP x, Rboolean nalast, SEXP rho)
 {
     int c=-1;
 
-    switch (TYPEOF(x)) {
-    case LGLSXP:
-    case INTSXP:
-	c = icmp(INTEGER(x)[i], INTEGER(x)[j], nalast);
-	break;
-    case REALSXP:
-	c = rcmp(REAL(x)[i], REAL(x)[j], nalast);
-	break;
-    case CPLXSXP:
-	c = ccmp(COMPLEX(x)[i], COMPLEX(x)[j], nalast);
-	break;
-    case STRSXP:
-	c = scmp(STRING_ELT(x, i), STRING_ELT(x, j), nalast);
-	break;
-    default:
-	UNIMPLEMENTED_TYPE("equal", x);
-	break;
+    if (isObject(x) && !isNull(rho)) { /* so never any NAs */
+	/* evaluate .gt(x, i, j) */
+	SEXP si, sj, call;
+	si = ScalarInteger(i+1);
+	sj = ScalarInteger(j+1);
+	PROTECT(call = lang4(install(".gt"), x, si, sj));
+	c = asInteger(eval(call, rho));
+	UNPROTECT(1);
+    } else {
+	switch (TYPEOF(x)) {
+	case LGLSXP:
+	case INTSXP:
+	    c = icmp(INTEGER(x)[i], INTEGER(x)[j], nalast);
+	    break;
+	case REALSXP:
+	    c = rcmp(REAL(x)[i], REAL(x)[j], nalast);
+	    break;
+	case CPLXSXP:
+	    c = ccmp(COMPLEX(x)[i], COMPLEX(x)[j], nalast);
+	    break;
+	case STRSXP:
+	    c = scmp(STRING_ELT(x, i), STRING_ELT(x, j), nalast);
+	    break;
+	default:
+	    UNIMPLEMENTED_TYPE("equal", x);
+	    break;
+	}
     }
     if (c == 0)
 	return 1;
     return 0;
 }
 
-static int greater(int i, int j, SEXP x, Rboolean nalast, Rboolean decreasing)
+static int greater(int i, int j, SEXP x, Rboolean nalast, Rboolean decreasing,
+		   SEXP rho)
 {
     int c = -1;
 
-    switch (TYPEOF(x)) {
-    case LGLSXP:
-    case INTSXP:
-	c = icmp(INTEGER(x)[i], INTEGER(x)[j], nalast);
-	break;
-    case REALSXP:
-	c = rcmp(REAL(x)[i], REAL(x)[j], nalast);
-	break;
-    case CPLXSXP:
-	c = ccmp(COMPLEX(x)[i], COMPLEX(x)[j], nalast);
-	break;
-    case STRSXP:
-	c = scmp(STRING_ELT(x, i), STRING_ELT(x, j), nalast);
-	break;
-    default:
-	UNIMPLEMENTED_TYPE("greater", x);
-	break;
+    if (isObject(x) && !isNull(rho)) { /* so never any NAs */
+	/* evaluate .gt(x, i, j) */
+	SEXP si, sj, call;
+	si = ScalarInteger(i+1);
+	sj = ScalarInteger(j+1);
+	PROTECT(call = lang4(install(".gt"), x, si, sj));
+	c = asInteger(eval(call, rho));
+	UNPROTECT(1);
+    } else {
+	switch (TYPEOF(x)) {
+	case LGLSXP:
+	case INTSXP:
+	    c = icmp(INTEGER(x)[i], INTEGER(x)[j], nalast);
+	    break;
+	case REALSXP:
+	    c = rcmp(REAL(x)[i], REAL(x)[j], nalast);
+	    break;
+	case CPLXSXP:
+	    c = ccmp(COMPLEX(x)[i], COMPLEX(x)[j], nalast);
+	    break;
+	case STRSXP:
+	    c = scmp(STRING_ELT(x, i), STRING_ELT(x, j), nalast);
+	    break;
+	default:
+	    UNIMPLEMENTED_TYPE("greater", x);
+	    break;
+	}
     }
     if (decreasing) c = -c;
     if (c > 0 || (c == 0 && j < i)) return 1; else return 0;
@@ -589,8 +694,9 @@ static int listgreater(int i, int j, SEXP key, Rboolean nalast,
     if (c == 0 && i < j) return 0; else return 1;
 }
 
+/* Needs indx set to 1...n initially */
 static void orderVector(int *indx, int n, SEXP key, Rboolean nalast,
-			Rboolean decreasing, int greater_sub())
+			Rboolean decreasing, int greater_sub(int, int, SEXP, Rboolean, Rboolean))
 {
     int i, j, h, t;
     int itmp;
@@ -611,103 +717,150 @@ static void orderVector(int *indx, int n, SEXP key, Rboolean nalast,
 }
 
 #define sort2_with_index \
-            for (h = incs[t]; t < 16; h = incs[++t]) \
+	    for (h = incs[t]; t < 16; h = incs[++t]) \
 		for (i = lo + h; i <= hi; i++) { \
 		    itmp = indx[i]; \
 		    j = i; \
-		    while (j >= h && less(indx[j - h], itmp)) { \
+		    while (j >= lo + h && less(indx[j - h], itmp)) { \
 			indx[j] = indx[j - h]; j -= h; } \
 		    indx[j] = itmp; \
 		}
 
 
-static void orderVector1(int *indx, int n, SEXP key, Rboolean nalast,
-			 Rboolean decreasing)
+/* Needs indx set to 1...n initially.
+   Also used by do_options, src/gnuwin32/extra.c
+   Called with rho != R_NilValue only from do_rank, when NAs are not involved.
+ */
+void attribute_hidden
+orderVector1(int *indx, int n, SEXP key, Rboolean nalast, Rboolean decreasing,
+	     SEXP rho)
 {
     int c, i, j, h, t, lo = 0, hi = n-1;
-    int itmp, *isna, numna = 0;
-    int *ix = INTEGER(key);
-    double *x = REAL(key);
-    SEXP *sx = STRING_PTR(key);
+    int itmp, *isna = NULL, numna = 0;
+    int *ix = NULL /* -Wall */;
+    double *x = NULL /* -Wall */;
+    Rcomplex *cx = NULL /* -Wall */;
+    SEXP *sx = NULL /* -Wall */;
 
-    /* First sort NAs to one end */
-    isna = (int *) malloc(n * sizeof(int));
     switch (TYPEOF(key)) {
     case LGLSXP:
     case INTSXP:
-	for (i = 0; i < n; i++) isna[i] = (ix[i] == NA_INTEGER);
+	ix = INTEGER(key);
 	break;
     case REALSXP:
-	for (i = 0; i < n; i++) isna[i] = ISNAN(x[i]);
+	x = REAL(key);
 	break;
     case STRSXP:
-	for (i = 0; i < n; i++) isna[i] = (sx[i] == NA_STRING);
+	sx = STRING_PTR(key);
+	break;
+    case CPLXSXP:
+	cx = COMPLEX(key);
 	break;
-    default:
-	UNIMPLEMENTED_TYPE("orderVector1", key);
     }
-    for (i = 0; i < n; i++) numna += isna[i];
 
-    if(numna)
+    if(isNull(rho)) {
+	/* First sort NAs to one end */
+	isna = Calloc(n, int);
 	switch (TYPEOF(key)) {
 	case LGLSXP:
 	case INTSXP:
+	    for (i = 0; i < n; i++) isna[i] = (ix[i] == NA_INTEGER);
+	    break;
 	case REALSXP:
+	    for (i = 0; i < n; i++) isna[i] = ISNAN(x[i]);
+	    break;
 	case STRSXP:
-	    if (!nalast) for (i = 0; i < n; i++) isna[i] = !isna[i];
-	    for (t = 0; incs[t] > n; t++);
-#define less(a, b) (isna[a] > isna[b] || (isna[a] == isna[b] && a > b))
-	    sort2_with_index
-#undef less
-	    if(nalast) hi -= numna; else lo += numna;
+	    for (i = 0; i < n; i++) isna[i] = (sx[i] == NA_STRING);
+	    break;
+	case CPLXSXP:
+	    for (i = 0; i < n; i++) isna[i] = ISNAN(cx[i].r) || ISNAN(cx[i].i);
+	    break;
+	default:
+	    UNIMPLEMENTED_TYPE("orderVector1", key);
 	}
+	for (i = 0; i < n; i++) numna += isna[i];
 
+	if(numna)
+	    switch (TYPEOF(key)) {
+	    case LGLSXP:
+	    case INTSXP:
+	    case REALSXP:
+	    case STRSXP:
+	    case CPLXSXP:
+		if (!nalast) for (i = 0; i < n; i++) isna[i] = !isna[i];
+		for (t = 0; incs[t] > n; t++);
+#define less(a, b) (isna[a] > isna[b] || (isna[a] == isna[b] && a > b))
+		sort2_with_index
+#undef less
+		    if(nalast) hi -= numna; else lo += numna;
+	    }
+    }
+    
     /* Shell sort isn't stable, so add test on index */
     for (t = 0; incs[t] > hi-lo+1; t++);
-    switch (TYPEOF(key)) {
-    case LGLSXP:
-    case INTSXP:
-	if (decreasing) {
+    
+    if (isObject(key) && !isNull(rho)) {
+/* only reached from do_rank */
+#define less(a, b) greater(a, b, key, nalast^decreasing, decreasing, rho)
+	    sort2_with_index
+#undef less
+    } else {
+	switch (TYPEOF(key)) {
+	case LGLSXP:
+	case INTSXP:
+	    if (decreasing) {
 #define less(a, b) (ix[a] < ix[b] || (ix[a] == ix[b] && a > b))
-	    sort2_with_index
+		sort2_with_index
 #undef less
-        } else {
+	    } else {
 #define less(a, b) (ix[a] > ix[b] || (ix[a] == ix[b] && a > b))
-	    sort2_with_index
+		sort2_with_index
 #undef less
-        }
-	break;
-    case REALSXP:
-	if (decreasing) {
+	    }
+	    break;
+	case REALSXP:
+	    if (decreasing) {
 #define less(a, b) (x[a] < x[b] || (x[a] == x[b] && a > b))
-	    sort2_with_index
+		sort2_with_index
 #undef less
-        } else {
+	    } else {
 #define less(a, b) (x[a] > x[b] || (x[a] == x[b] && a > b))
+		sort2_with_index
+#undef less
+	    }
+	    break;
+	case CPLXSXP:
+	    if (decreasing) {
+#define less(a, b) (ccmp(cx[a], cx[b], 0) < 0 || (cx[a].r == cx[b].r && cx[a].i == cx[b].i && a > b))
+		sort2_with_index
+#undef less
+	    } else {
+#define less(a, b) (ccmp(cx[a], cx[b], 0) > 0 || (cx[a].r == cx[b].r && cx[a].i == cx[b].i && a > b))
+		sort2_with_index
+#undef less
+	    }
+	    break;
+	case STRSXP:
+	    if (decreasing)
+#define less(a, b) (c=Scollate(sx[a], sx[b]), c < 0 || (c == 0 && a > b))
+		sort2_with_index
+#undef less
+	    else
+#define less(a, b) (c=Scollate(sx[a], sx[b]), c > 0 || (c == 0 && a > b))
+		sort2_with_index
+#undef less
+	    break;
+    	default:  /* only reached from do_rank */
+#define less(a, b) greater(a, b, key, nalast^decreasing, decreasing, rho)
 	    sort2_with_index
 #undef less
-        }
-	break;
-    case STRSXP:
-	if (decreasing)
-#define less(a, b) (c=STRCOLL(CHAR(sx[a]),CHAR(sx[b])), c < 0 || (c == 0 && a > b))
-	    sort2_with_index
-#undef less
-        else
-#define less(a, b) (c=STRCOLL(CHAR(sx[a]),CHAR(sx[b])), c > 0 || (c == 0 && a > b))
-	    sort2_with_index
-#undef less
-	break;
-    default:
-#define less(a, b) greater(a, b, key, nalast^decreasing, decreasing)
-	sort2_with_index
-#undef less
+	}
     }
-    free(isna);
+    if(isna) Free(isna);
 }
 
 /* FUNCTION order(...) */
-SEXP do_order(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_order(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ap, ans;
     int i, n = -1, narg = 0;
@@ -715,7 +868,7 @@ SEXP do_order(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     nalast = asLogical(CAR(args));
     if(nalast == NA_LOGICAL)
-	error(_("'na.last' is invalid"));
+	error(_("invalid '%s' value"), "na.last");
     args = CDR(args);
     decreasing = asLogical(CAR(args));
     if(decreasing == NA_LOGICAL)
@@ -728,94 +881,94 @@ SEXP do_order(SEXP call, SEXP op, SEXP args, SEXP rho)
 	n = LENGTH(CAR(args));
     for (ap = args; ap != R_NilValue; ap = CDR(ap), narg++) {
 	if (!isVector(CAR(ap)))
-	    errorcall(call, _("argument %d is not a vector"), narg + 1);
+	    error(_("argument %d is not a vector"), narg + 1);
 	if (LENGTH(CAR(ap)) != n)
-	    errorcall(call, _("argument lengths differ"));
+	    error(_("argument lengths differ"));
     }
-    ans = allocVector(INTSXP, n);
+    /* NB: collation functions such as Scollate might allocate */
+    PROTECT(ans = allocVector(INTSXP, n));
     if (n != 0) {
 	for (i = 0; i < n; i++) INTEGER(ans)[i] = i;
 	if(narg == 1)
-	    orderVector1(INTEGER(ans), n, CAR(args), nalast, decreasing);
+	    orderVector1(INTEGER(ans), n, CAR(args), nalast, decreasing, 
+			 R_NilValue);
 	else
 	    orderVector(INTEGER(ans), n, args, nalast, decreasing, listgreater);
 	for (i = 0; i < n; i++) INTEGER(ans)[i]++;
     }
+    UNPROTECT(1);
     return ans;
 }
 
 /* FUNCTION: rank(x) */
-SEXP do_rank(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_rank(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP rank, indx, x;
-    int *in;
-    double *rk;
+    int *in, *ik = NULL /* -Wall */;
+    double *rk = NULL /* -Wall */;
     int i, j, k, n;
-    char *ties_str;
+    const char *ties_str;
     enum {AVERAGE, MAX, MIN} ties_kind = AVERAGE;
 
     checkArity(op, args);
     if (args == R_NilValue)
 	return R_NilValue;
     x = CAR(args);
-    if (!isVectorAtomic(x))
-	errorcall(call, _("argument is not an atomic vector"));
+//    if (!isVectorAtomic(x))
+//	error(_("argument is not an atomic vector"));
     if(TYPEOF(x) == RAWSXP)
-	errorcall(call, _("raw vectors cannot be sorted"));
+	error(_("raw vectors cannot be sorted"));
     n = LENGTH(x);
-    PROTECT(indx = allocVector(INTSXP, n));
-    PROTECT(rank = allocVector(REALSXP, n));
-    UNPROTECT(2);
-    ties_str = CHAR(STRING_ELT(coerceVector(CADR(args), STRSXP), 0));
+    ties_str = CHAR(asChar(CADR(args)));
     if(!strcmp(ties_str, "average"))	ties_kind = AVERAGE;
     else if(!strcmp(ties_str, "max"))	ties_kind = MAX;
     else if(!strcmp(ties_str, "min"))	ties_kind = MIN;
     else error(_("invalid ties.method for rank() [should never happen]"));
+    PROTECT(indx = allocVector(INTSXP, n));
+    if (ties_kind == AVERAGE) {
+	PROTECT(rank = allocVector(REALSXP, n));
+	rk = REAL(rank);
+    } else {
+	PROTECT(rank = allocVector(INTSXP, n));
+	ik = INTEGER(rank);
+    }
     if (n > 0) {
 	in = INTEGER(indx);
-	rk = REAL(rank);
-	for (i = 0; i < n; i++)
-	    in[i] = i;
-	orderVector1(in, n, x, TRUE, FALSE);
-	i = 0;
-	while (i < n) {
+	for (i = 0; i < n; i++) in[i] = i;
+	orderVector1(in, n, x, TRUE, FALSE, rho);
+	for (i = 0; i < n; i = j+1) {
 	    j = i;
-	    while ((j < n - 1) && equal(in[j], in[j + 1], x, TRUE))
-		j++;
-	    if (i != j) { /* ties */
-		switch(ties_kind) {
-		    case AVERAGE:
-			for (k = i; k <= j; k++)
-			    rk[in[k]] = (i + j + 2) / 2.; break;
-		    case MAX:
-			for (k = i; k <= j; k++) rk[in[k]] = j+1; break;
-		    case MIN:
-			for (k = i; k <= j; k++) rk[in[k]] = i+1; break;
-		}
+	    while ((j < n - 1) && equal(in[j], in[j + 1], x, TRUE, rho)) j++;
+	    switch(ties_kind) {
+	    case AVERAGE:
+		for (k = i; k <= j; k++)
+		    rk[in[k]] = (i + j + 2) / 2.; break;
+	    case MAX:
+		for (k = i; k <= j; k++) ik[in[k]] = j+1; break;
+	    case MIN:
+		for (k = i; k <= j; k++) ik[in[k]] = i+1; break;
 	    }
-	    else
-		rk[in[i]] = i + 1;
-	    i = j + 1;
 	}
     }
+    UNPROTECT(2);
     return rank;
 }
 
 #include <R_ext/RS.h>
 
-SEXP do_radixsort(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_radixsort(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP x, ans;
     Rboolean nalast, decreasing;
-    unsigned int *cnts;
-    int i, n, tmp, xmax = NA_INTEGER, xmin = NA_INTEGER, off, napos;
+    R_len_t i, n;
+    int tmp, xmax = NA_INTEGER, xmin = NA_INTEGER, off, napos;
 
     checkArity(op, args);
 
     x = CAR(args);
     nalast = asLogical(CADR(args));
     if(nalast == NA_LOGICAL)
-	error(_("'na.last' is invalid"));
+	error(_("invalid '%s' value"), "na.last");
     decreasing = asLogical(CADDR(args));
     if(decreasing == NA_LOGICAL)
 	error(_("'decreasing' must be TRUE or FALSE"));
@@ -825,7 +978,7 @@ SEXP do_radixsort(SEXP call, SEXP op, SEXP args, SEXP rho)
     for(i = 0; i < n; i++) {
 	tmp = INTEGER(x)[i];
 	if(tmp == NA_INTEGER) continue;
-	if(tmp < 0) errorcall(call, _("negative value in 'x'"));
+	if(tmp < 0) error(_("negative value in 'x'"));
 	if(xmax == NA_INTEGER || tmp > xmax) xmax = tmp;
 	if(xmin == NA_INTEGER || tmp < xmin) xmin = tmp;
     }
@@ -836,10 +989,11 @@ SEXP do_radixsort(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
 
     xmax -= xmin;
-    if(xmax > 100000) errorcall(call, _("too large a range of values in 'x'"));
+    if(xmax > 100000) error(_("too large a range of values in 'x'"));
     napos = off ? 0 : xmax + 1;
     off -= xmin;
-    cnts = Calloc(xmax+1, unsigned int);
+    /* automatic allocation is fine here: we know this is small */
+    R_len_t cnts[xmax+1];
 
     for(i = 0; i <= xmax+1; i++) cnts[i] = 0;
     for(i = 0; i < n; i++) {
@@ -859,7 +1013,24 @@ SEXP do_radixsort(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    INTEGER(ans)[--cnts[(tmp==NA_INTEGER) ? napos : off+tmp]] = i+1;
 	}
 
-    Free(cnts);
     UNPROTECT(1);
     return ans;
 }
+
+SEXP attribute_hidden do_xtfrm(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP fn, prargs, ans;
+
+    checkArity(op, args);
+    check1arg(args, call, "x");
+
+    if(DispatchOrEval(call, op, "xtfrm", args, rho, &ans, 0, 1)) return ans;
+    /* otherwise dispatch the default method */
+    PROTECT(fn = findFun(install("xtfrm.default"), rho));
+    PROTECT(prargs = promiseArgs(args, R_GlobalEnv));
+    SET_PRVALUE(CAR(prargs), CAR(args));
+    ans = applyClosure(call, fn, prargs, rho, R_NilValue);
+    UNPROTECT(2);
+    return ans;
+    
+}
