@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1996   Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2008   Robert Gentleman, Ross Ihaka
- *                            and the R Development Core Team
+ *  Copyright (C) 1997-2011   The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,8 +18,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -53,11 +50,26 @@
 # include <sys/stat.h>
 #endif
 
-#if HAVE_AQUA
-extern int (*ptr_CocoaSystem)(char*);
+#ifdef HAVE_AQUA
+int (*ptr_CocoaSystem)(char*);
 extern	Rboolean useaqua;
 #endif
 
+#ifdef Win32
+Rboolean attribute_hidden R_FileExists(const char *path)
+{
+    struct _stati64 sb;
+    return _stati64(R_ExpandFileName(path), &sb) == 0;
+}
+
+double attribute_hidden R_FileMtime(const char *path)
+{
+    struct _stati64 sb;
+    if (_stati64(R_ExpandFileName(path), &sb) != 0)
+	error(_("cannot determine file modification time of '%s'"), path);
+    return sb.st_mtime;
+}
+#else
 Rboolean attribute_hidden R_FileExists(const char *path)
 {
     struct stat sb;
@@ -71,6 +83,7 @@ double attribute_hidden R_FileMtime(const char *path)
 	error(_("cannot determine file modification time of '%s'"), path);
     return sb.st_mtime;
 }
+#endif
 
     /*
      *  Unix file names which begin with "." are invisible.
@@ -136,9 +149,10 @@ FILE *R_fopen(const char *filename, const char *mode)
 
 #if defined(Win32)
 
+#define BSIZE 100000 
 wchar_t *filenameToWchar(const SEXP fn, const Rboolean expand)
 {
-    static wchar_t filename[PATH_MAX+1];
+    static wchar_t filename[BSIZE+1];
     void *obj;
     const char *from = "", *inbuf;
     char *outbuf;
@@ -150,18 +164,20 @@ wchar_t *filenameToWchar(const SEXP fn, const Rboolean expand)
     }
     if(IS_LATIN1(fn)) from = "latin1";
     if(IS_UTF8(fn)) from = "UTF-8";
+    if(IS_BYTES(fn)) error(_("encoding of a filename cannot be 'bytes'"));
     obj = Riconv_open("UCS-2LE", from);
     if(obj == (void *)(-1))
-	error(_("unsupported conversion in 'filenameToWchar'"));
+	error(_("unsupported conversion from '%s' in codepage %d"), 
+	      from, localeCP);
 
     if(expand) inbuf = R_ExpandFileName(CHAR(fn)); else inbuf = CHAR(fn);
 
-    inb = strlen(inbuf)+1; outb = 2*PATH_MAX+1;
+    inb = strlen(inbuf)+1; outb = 2*BSIZE;
     outbuf = (char *) filename;
     res = Riconv(obj, &inbuf , &inb, &outbuf, &outb);
     Riconv_close(obj);
-    if(res == -1) error(_("file name conversion problem"));
     if(inb > 0) error(_("file name conversion problem -- name too long?"));
+    if(res == -1) error(_("file name conversion problem"));
 
     return filename;
 }
@@ -201,9 +217,10 @@ char *R_HomeDir(void)
     return getenv("R_HOME");
 }
 
-
+/* This is a primitive (with no arguments) */
 SEXP attribute_hidden do_interactive(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
+    checkArity(op, args);
     return ScalarLogical( (R_Interactive) ? 1 : 0 );
 }
 
@@ -216,29 +233,36 @@ SEXP attribute_hidden do_tempdir(SEXP call, SEXP op, SEXP args, SEXP env)
 
 SEXP attribute_hidden do_tempfile(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP  ans, pattern, tempdir;
-    const char *tn, *td;
+    SEXP  ans, pattern, fileext, tempdir;
+    const char *tn, *td, *te;
     char *tm;
-    int i, n1, n2, slen;
+    int i, n1, n2, n3, slen;
 
     checkArity(op, args);
-    pattern = CAR(args); n1 = length(pattern);
-    tempdir = CADR(args); n2 = length(tempdir);
+    pattern = CAR(args); n1 = length(pattern); args = CDR(args);
+    tempdir = CAR(args); n2 = length(tempdir); args = CDR(args);
+    fileext = CAR(args); n3 = length(fileext);
     if (!isString(pattern))
 	error(_("invalid filename pattern"));
     if (!isString(tempdir))
 	error(_("invalid '%s' value"), "tempdir");
+    if (!isString(fileext))
+	error(_("invalid file extension"));
     if (n1 < 1)
 	error(_("no 'pattern'"));
     if (n2 < 1)
 	error(_("no 'tempdir'"));
+    if (n3 < 1)
+        error(_("no 'fileext'"));
     slen = (n1 > n2) ? n1 : n2;
+    slen = (n3 > slen) ? n3 : slen;
     PROTECT(ans = allocVector(STRSXP, slen));
     for(i = 0; i < slen; i++) {
 	tn = translateChar( STRING_ELT( pattern , i%n1 ) );
 	td = translateChar( STRING_ELT( tempdir , i%n2 ) );
+	te = translateChar( STRING_ELT( fileext , i%n3 ) );
 	/* try to get a new file name */
-	tm = R_tmpnam(tn, td);
+	tm = R_tmpnam2(tn, td, te);
 	SET_STRING_ELT(ans, i, mkChar(tm));
 	if(tm) free(tm);
     }
@@ -246,7 +270,6 @@ SEXP attribute_hidden do_tempfile(SEXP call, SEXP op, SEXP args, SEXP env)
     return (ans);
 }
 
-#ifdef HAVE_POPEN
 FILE *R_popen(const char *command, const char *type)
 {
     FILE *fp;
@@ -263,11 +286,14 @@ FILE *R_popen(const char *command, const char *type)
 #endif
     return fp;
 }
-#endif /* HAVE_POPEN */
+
+#ifdef HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
 
 int R_system(const char *command)
 {
-    int val;
+    int res;
 #ifdef __APPLE_CC__
     /* Luke recommends this to fix PR#1140 */
     sigset_t ss;
@@ -279,16 +305,23 @@ int R_system(const char *command)
     if(useaqua) {
 	/* FIXME, is Cocoa's interface not const char*? */
 	cmdcpy = acopy_string(command);
-	val = ptr_CocoaSystem(cmdcpy);
+	res = ptr_CocoaSystem(cmdcpy);
     }
     else
 #endif
-    val = system(command);
+    res = system(command);
     sigprocmask(SIG_UNBLOCK, &ss, NULL);
 #else
-    val = system(command);
+    res = system(command);
 #endif
-    return val;
+#ifdef HAVE_SYS_WAIT_H
+    if (WIFEXITED(res)) res = WEXITSTATUS(res);
+    else res = 0;
+#else
+    /* assume that this is shifted if a multiple of 256 */
+    if ((res % 256) == 0) res = res/256;
+#endif
+    return res;
 }
 
 #if defined(__APPLE__)
@@ -299,14 +332,9 @@ extern char ** environ;
 #endif
 
 #ifdef Win32
-# define WC_ENVIRON
-#endif
-
-#ifdef WC_ENVIRON
 /* _wenviron is declared in stdlib.h */
 # define WIN32_LEAN_AND_MEAN 1
 # include <windows.h> /* _wgetenv etc */
-const wchar_t *wtransChar(SEXP x);
 #endif
 
 SEXP attribute_hidden do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
@@ -324,14 +352,13 @@ SEXP attribute_hidden do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
 
     i = LENGTH(CAR(args));
     if (i == 0) {
-#ifdef WC_ENVIRON
-	char *buf;
+#ifdef Win32
 	int n = 0, N;
 	wchar_t **w;
 	for (i = 0, w = _wenviron; *w != NULL; i++, w++)
 	    n = max(n, wcslen(*w));
-	N = 3*n+1; buf = alloca(N);
-	R_CheckStack();
+	N = 3*n+1; 
+	char buf[N];
 	PROTECT(ans = allocVector(STRSXP, i));
 	for (i = 0, w = _wenviron; *w != NULL; i++, w++) {
 	    wcstoutf8(buf, *w, N); buf[N-1] = '\0';
@@ -347,14 +374,14 @@ SEXP attribute_hidden do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
     } else {
 	PROTECT(ans = allocVector(STRSXP, i));
 	for (j = 0; j < i; j++) {
-#ifdef WC_ENVIRON
+#ifdef Win32
 	    const wchar_t *wnm = wtransChar(STRING_ELT(CAR(args), j));
 	    wchar_t *w = _wgetenv(wnm);
 	    if (w == NULL)
 		SET_STRING_ELT(ans, j, STRING_ELT(CADR(args), 0));
 	    else {
 		int n = wcslen(w), N = 3*n+1; /* UCS-2 maps to <=3 UTF-8 */
-		char *buf = alloca(N);
+		char buf[N];
 		R_CheckStack();
 		wcstoutf8(buf, w, N); buf[N-1] = '\0'; /* safety */
 		SET_STRING_ELT(ans, j, mkCharCE(buf, CE_UTF8));
@@ -377,13 +404,14 @@ SEXP attribute_hidden do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
     return (ans);
 }
 
-#ifdef WC_ENVIRON
+#ifdef Win32
 static int Rwputenv(const wchar_t *nm, const wchar_t *val)
 {
     wchar_t *buf;
     buf = (wchar_t *) malloc((wcslen(nm) + wcslen(val) + 2) * sizeof(wchar_t));
     if(!buf) return 1;
-    wsprintfW(buf, L"%s=%s", nm, val);
+    /* previously wsprintfW, which had a limit of 1024 chars */
+    wcscpy(buf, nm); wcscat(buf, L"="); wcscat(buf, val);
     if(_wputenv(buf)) return 1;
     /* no free here: storage remains in use */
     return 0;
@@ -424,7 +452,7 @@ SEXP attribute_hidden do_setenv(SEXP call, SEXP op, SEXP args, SEXP env)
 	LOGICAL(ans)[i] = setenv(translateChar(STRING_ELT(nm, i)),
 				 translateChar(STRING_ELT(vars, i)),
 				 1) == 0;
-#elif defined(WC_ENVIRON)
+#elif defined(Win32)
     for (i = 0; i < n; i++)
 	LOGICAL(ans)[i] = Rwputenv(wtransChar(STRING_ELT(nm, i)),
 				   wtransChar(STRING_ELT(vars, i))) == 0;
@@ -462,11 +490,10 @@ SEXP attribute_hidden do_unsetenv(SEXP call, SEXP op, SEXP args, SEXP env)
 	putenv(buf);
     }
 #elif defined(HAVE_PUTENV_UNSET2)
-# ifdef WC_ENVIRON
+# ifdef Win32
     for (i = 0; i < n; i++) {
 	const wchar_t *w = wtransChar(STRING_ELT(vars, i));
-	wchar_t *buf = (wchar_t *) alloca(2*wcslen(w));
-	R_CheckStack();
+	wchar_t buf[2*wcslen(w)];
 	wcscpy(buf, w);
 	wcscat(buf, L"=");
 	_wputenv(buf);
@@ -502,9 +529,7 @@ SEXP attribute_hidden do_unsetenv(SEXP call, SEXP op, SEXP args, SEXP env)
     return ans;
 }
 
-#if defined(HAVE_ICONV_H) && defined(ICONV_LATIN1)
-# include <iconv.h>
-#endif
+#include <iconv.h>
 
 #ifdef HAVE_ICONVLIST
 static unsigned int cnt;
@@ -530,10 +555,9 @@ write_one (unsigned int namescount, const char * const *names, void *data)
 
 #include "RBufferUtils.h"
 
-/* iconv(x, from, to, sub) */
+/* iconv(x, from, to, sub, mark) */
 SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-#if defined(HAVE_ICONV) && defined(ICONV_LATIN1)
     SEXP ans, x = CAR(args), si;
     void * obj;
     int i, j, nout;
@@ -542,6 +566,7 @@ SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
     const char *sub;
     size_t inb, outb, res;
     R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
+    Rboolean isRawlist = FALSE;
 
     checkArity(op, args);
     if(isNull(x)) {  /* list locales */
@@ -555,21 +580,31 @@ SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 	PROTECT(ans = R_NilValue);
 #endif
     } else {
+	int mark, toRaw;
 	const char *from, *to;
 	Rboolean isLatin1 = FALSE, isUTF8 = FALSE;
 
-	if(TYPEOF(x) != STRSXP)
-	    error(_("'x' must be a character vector"));
-	if(!isString(CADR(args)) || length(CADR(args)) != 1)
+	args = CDR(args);
+	if(!isString(CAR(args)) || length(CAR(args)) != 1)
 	    error(_("invalid '%s' argument"), "from");
-	if(!isString(CADDR(args)) || length(CADDR(args)) != 1)
+	from = CHAR(STRING_ELT(CAR(args), 0)); /* ASCII */
+	args = CDR(args);
+	if(!isString(CAR(args)) || length(CAR(args)) != 1)
 	    error(_("invalid '%s' argument"), "to");
-	if(!isString(CADDDR(args)) || length(CADDDR(args)) != 1)
+	to = CHAR(STRING_ELT(CAR(args), 0));
+	args = CDR(args);
+	if(!isString(CAR(args)) || length(CAR(args)) != 1)
 	    error(_("invalid '%s' argument"), "sub");
-	if(STRING_ELT(CADDDR(args), 0) == NA_STRING) sub = NULL;
-	else sub = translateChar(STRING_ELT(CADDDR(args), 0));
-	from = CHAR(STRING_ELT(CADR(args), 0)); /* ASCII */
-	to = CHAR(STRING_ELT(CADDR(args), 0));
+	if(STRING_ELT(CAR(args), 0) == NA_STRING) sub = NULL;
+	else sub = translateChar(STRING_ELT(CAR(args), 0));
+	args = CDR(args);
+	mark = asLogical(CAR(args));
+	if(mark == NA_LOGICAL)
+	    error(_("invalid '%s' argument"), "mark");	
+	args = CDR(args);
+	toRaw = asLogical(CAR(args));
+	if(toRaw == NA_LOGICAL)
+	    error(_("invalid '%s' argument"), "toRaw");	
 	/* some iconv's allow "UTF8", but libiconv does not */
 	if(streql(from, "UTF8") || streql(from, "utf8") ) from = "UTF-8";
 	if(streql(to, "UTF8") || streql(from, "utf8") ) to = "UTF-8";
@@ -581,13 +616,48 @@ SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 	if(streql(to, "") && known_to_be_utf8) isUTF8 = TRUE;
 	obj = Riconv_open(to, from);
 	if(obj == (iconv_t)(-1))
-	    error(_("unsupported conversion"));
-	PROTECT(ans = duplicate(x));
+#ifdef Win32
+	    error(_("unsupported conversion from '%s' to '%s' in codepage %d"), 
+		  from, to, localeCP);
+#else
+	    error(_("unsupported conversion from '%s' to '%s'"), from, to);
+#endif
+	isRawlist = (TYPEOF(x) == VECSXP);
+	if(isRawlist) {
+	    if(toRaw)
+		PROTECT(ans = duplicate(x));
+	    else {
+		PROTECT(ans = allocVector(STRSXP, LENGTH(x)));
+		DUPLICATE_ATTRIB(ans, x);
+	    }
+	} else {   
+	    if(TYPEOF(x) != STRSXP)
+		error(_("'x' must be a character vector"));
+	    if(toRaw) {
+		PROTECT(ans = allocVector(VECSXP, LENGTH(x)));
+		DUPLICATE_ATTRIB(ans, x);
+	    } else 
+		PROTECT(ans = duplicate(x));
+	}
 	R_AllocStringBuffer(0, &cbuff);  /* 0 -> default */
 	for(i = 0; i < LENGTH(x); i++) {
-	    si = STRING_ELT(x, i);
+	    if (isRawlist) {
+		si = VECTOR_ELT(x, i);
+		if (TYPEOF(si) == NILSXP) {
+		    if (!toRaw) SET_STRING_ELT(ans, i, NA_STRING);
+		    continue;
+		} else if (TYPEOF(si) != RAWSXP)
+		    error(_("'x' must be a list of NULL or raw vectors"));
+	    } else {
+		si = STRING_ELT(x, i);
+		if (si == NA_STRING) {
+		    if(!toRaw) SET_STRING_ELT(ans, i, NA_STRING);
+		    continue;
+		}
+	    }
 	top_of_loop:
-	    inbuf = CHAR(si); inb = LENGTH(si);
+	    inbuf = isRawlist ? (const char *) RAW(si) : CHAR(si); 
+	    inb = LENGTH(si);
 	    outbuf = cbuff.data; outb = cbuff.bufsize - 1;
 	    /* First initialize output */
 	    Riconv (obj, NULL, NULL, &outbuf, &outb);
@@ -600,7 +670,8 @@ SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 	    if(res == -1 && errno == E2BIG) {
 		R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
 		goto top_of_loop;
-	    } else if(res == -1 && errno == EILSEQ && sub) {
+	    } else if(res == -1 && sub && 
+		      (errno == EILSEQ || errno == EINVAL)) {
 		/* it seems this gets thrown for non-convertible input too */
 		if(strcmp(sub, "byte") == 0) {
 		    if(outb < 5) {
@@ -622,69 +693,69 @@ SEXP attribute_hidden do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 		goto next_char;
 	    }
 
-	    if(res != -1 && inb == 0) {
-		/* we can currently only put the result in the CHARSXP
-		   cache if it does not contain nuls. */
-		Rboolean has_nul = FALSE;
-		char *p = cbuff.data;
-
-		nout = cbuff.bufsize - 1 - outb;
-		for(j = 0; j < nout; j++) if(!*p++) {has_nul = TRUE; break;}
-		if(has_nul) {
-		    si = mkCharLen(cbuff.data, nout);
-		} else {
-		    if(isLatin1) si = mkCharCE(cbuff.data, CE_LATIN1);
-		    else if(isUTF8) si = mkCharCE(cbuff.data, CE_UTF8);
-		    else si = mkChar(cbuff.data);
-		}
-		SET_STRING_ELT(ans, i, si);
+	    if(toRaw) {
+		if(res != -1 && inb == 0) {
+		    nout = cbuff.bufsize - 1 - outb;
+		    SEXP el = allocVector(RAWSXP, nout);
+		    memcpy(RAW(el), cbuff.data, nout);
+		    SET_VECTOR_ELT(ans, i, el);
+		} /* otherwise is already NULL */
+	    } else {
+		if(res != -1 && inb == 0) {
+		    cetype_t ienc = CE_NATIVE;
+		    
+		    nout = cbuff.bufsize - 1 - outb;
+		    if(mark) {
+			if(isLatin1) ienc = CE_LATIN1;
+			else if(isUTF8) ienc = CE_UTF8;
+		    }
+		    SET_STRING_ELT(ans, i, mkCharLenCE(cbuff.data, nout, ienc));
+		} else SET_STRING_ELT(ans, i, NA_STRING);
 	    }
-	    else SET_STRING_ELT(ans, i, NA_STRING);
 	}
 	Riconv_close(obj);
 	R_FreeStringBuffer(&cbuff);
     }
     UNPROTECT(1);
     return ans;
-#else
-    error(_("'iconv' is not available on this system"));
-    return R_NilValue; /* -Wall */
-#endif
 }
 
 cetype_t getCharCE(SEXP x)
 {
     if(TYPEOF(x) != CHARSXP)
-	error(_("'%s' must be called on a CHARSXP"), "getEncChar");
+	error(_("'%s' must be called on a CHARSXP"), "getCharCE");
     if(IS_UTF8(x)) return CE_UTF8;
     else if(IS_LATIN1(x)) return CE_LATIN1;
+    else if(IS_BYTES(x)) return CE_BYTES;
     else return CE_NATIVE;
 }
 
 
-#if defined(HAVE_ICONV) && defined(ICONV_LATIN1)
 void * Riconv_open (const char* tocode, const char* fromcode)
 {
 #if defined Win32 || __APPLE__
-    const char *cp = "UTF-8";
 # ifdef Win32
-#  ifndef SUPPORT_UTF8_WIN32
-    cp = locale2charset(NULL);
+    const char *cp = "ASCII";
+#  ifndef SUPPORT_UTF8_WIN32 /* Always, at present */
+    char to[20] = "";
+    if (localeCP > 0) {snprintf(to, 20, "CP%d", localeCP); cp = to;}
 #  endif
 # else /* __APPLE__ */
+    const char *cp = "UTF-8";
     if (latin1locale) cp = "ISO-8859-1";
     else if (!utf8locale) cp = locale2charset(NULL);
 # endif
     if (!*tocode && !*fromcode) return iconv_open(cp, cp);
-    if(strcmp(tocode, "") == 0)  return iconv_open(cp, fromcode);
-    else if(strcmp(fromcode, "") == 0) return iconv_open(tocode, cp);
+    if(!*tocode)  return iconv_open(cp, fromcode);
+    else if(!*fromcode) return iconv_open(tocode, cp);
     else return iconv_open(tocode, fromcode);
 #else
     return iconv_open(tocode, fromcode);
 #endif
 }
 
-/* Should be defined in config.h */
+/* Should be defined in config.h, but prior to 2.13.0 was only checked
+   if the NLS was enabled  */
 #ifndef ICONV_CONST
 # define ICONV_CONST
 #endif
@@ -702,7 +773,8 @@ int Riconv_close (void *cd)
     return iconv_close((iconv_t) cd);
 }
 
-static void *latin1_obj = NULL, *utf8_obj=NULL, *ucsmb_obj=NULL;
+static void *latin1_obj = NULL, *utf8_obj=NULL, *ucsmb_obj=NULL,
+    *ucsutf8_obj=NULL;
 
 const char *translateChar(SEXP x)
 {
@@ -710,14 +782,14 @@ const char *translateChar(SEXP x)
     const char *inbuf, *ans = CHAR(x);
     char *outbuf, *p;
     size_t inb, outb, res;
-#ifdef SUPPORT_MBCS
     cetype_t ienc = getCharCE(x);
-#endif
     R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
 
     if(TYPEOF(x) != CHARSXP)
 	error(_("'%s' must be called on a CHARSXP"), "translateChar");
     if(x == NA_STRING || !(ENC_KNOWN(x))) return ans;
+    if(IS_BYTES(x))
+	error(_("translating strings with \"bytes\" encoding is not allowed"));
     if(utf8locale && IS_UTF8(x)) return ans;
     if(latin1locale && IS_LATIN1(x)) return ans;
     if(strIsASCII(CHAR(x))) return ans;
@@ -726,7 +798,14 @@ const char *translateChar(SEXP x)
 	if(!latin1_obj) {
 	    obj = Riconv_open("", "latin1");
 	    /* should never happen */
-	    if(obj == (void *)(-1)) error(_("unsupported conversion"));
+	    if(obj == (void *)(-1))
+#ifdef Win32
+		error(_("unsupported conversion from '%s' in codepage %d"),
+		      "latin1", localeCP);
+#else
+	        error(_("unsupported conversion from '%s' to '%s'"),
+		      "latin1", "");
+#endif
 	    latin1_obj = obj;
 	}
 	obj = latin1_obj;
@@ -734,7 +813,14 @@ const char *translateChar(SEXP x)
 	if(!utf8_obj) {
 	    obj = Riconv_open("", "UTF-8");
 	    /* should never happen */
-	    if(obj == (void *)(-1)) error(_("unsupported conversion"));
+	    if(obj == (void *)(-1)) 
+#ifdef Win32
+		error(_("unsupported conversion from '%s' in codepage %d"),
+		      "latin1", localeCP);
+#else
+	        error(_("unsupported conversion from '%s' to '%s'"),
+		      "latin1", "");
+#endif
 	    utf8_obj = obj;
 	}
 	obj = utf8_obj;
@@ -752,12 +838,11 @@ next_char:
     if(res == -1 && errno == E2BIG) {
 	R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
 	goto top_of_loop;
-    } else if(res == -1 && errno == EILSEQ) {
+    } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
 	if(outb < 13) {
 	    R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
 	    goto top_of_loop;
 	}
-#ifdef SUPPORT_MBCS
 	if (ienc == CE_UTF8) {
 	    /* if starting in UTF-8, use \uxxxx */
 	    /* This must be the first byte */
@@ -782,9 +867,7 @@ next_char:
 		outbuf += 4; outb -= 4;
 		inbuf++; inb--;
 	    }
-	} else
-#endif
-	{
+	} else {
 	    snprintf(outbuf, 5, "<%02x>", (unsigned char)*inbuf);
 	    outbuf += 4; outb -= 4;
 	    inbuf++; inb--;
@@ -799,6 +882,14 @@ next_char:
     return p;
 }
 
+const char *translateChar0(SEXP x)
+{
+    if(TYPEOF(x) != CHARSXP)
+	error(_("'%s' must be called on a CHARSXP"), "translateChar0");
+    if(IS_BYTES(x)) return CHAR(x);
+    return translateChar(x);
+}
+
 const char *translateCharUTF8(SEXP x)
 {
     void *obj;
@@ -812,9 +903,17 @@ const char *translateCharUTF8(SEXP x)
     if(x == NA_STRING) return ans;
     if(IS_UTF8(x)) return ans;
     if(strIsASCII(CHAR(x))) return ans;
+    if(IS_BYTES(x))
+	error(_("translating strings with \"bytes\" encoding is not allowed"));
 
     obj = Riconv_open("UTF-8", IS_LATIN1(x) ? "latin1" : "");
-    if(obj == (void *)(-1)) error(_("unsupported conversion"));
+    if(obj == (void *)(-1)) 
+#ifdef Win32
+	error(_("unsupported conversion from '%s' in codepage %d"),
+	      "latin1", localeCP);
+#else
+       error(_("unsupported conversion from '%s' to '%s'"), "latin1", "UTF-8");
+#endif
     R_AllocStringBuffer(0, &cbuff);
 top_of_loop:
     inbuf = ans; inb = strlen(inbuf);
@@ -827,7 +926,7 @@ next_char:
     if(res == -1 && errno == E2BIG) {
 	R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
 	goto top_of_loop;
-    } else if(res == -1 && errno == EILSEQ) {
+    } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
 	if(outb < 5) {
 	    R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
 	    goto top_of_loop;
@@ -846,12 +945,24 @@ next_char:
     return p;
 }
 
+
 #ifdef Win32
+static const char TO_WCHAR[] = "UCS-2LE";
+#else
+# ifdef WORDS_BIGENDIAN
+static const char TO_WCHAR[] = "UCS-4BE";
+# else
+static const char TO_WCHAR[] = "UCS-4LE";
+# endif
+#endif
+
 static void *latin1_wobj = NULL, *utf8_wobj=NULL;
 
-/* Translate from current encoding to wchar_t = UCS-2 on Windows
-   (not using surrogates). NB: this is not general.
+/* Translate from current encoding to wchar_t = UCS-2/4
+   NB: that wchar_t is UCS-4 is an assumption, but not easy to avoid.
 */
+
+attribute_hidden /* but not hidden on Windows, where it was used in tcltk.c */
 const wchar_t *wtransChar(SEXP x)
 {
     void * obj;
@@ -865,25 +976,38 @@ const wchar_t *wtransChar(SEXP x)
     if(TYPEOF(x) != CHARSXP)
 	error(_("'%s' must be called on a CHARSXP"), "wtransChar");
 
+    if(IS_BYTES(x))
+	error(_("translating strings with \"bytes\" encoding is not allowed"));
+
     if(IS_LATIN1(x)) {
 	if(!latin1_wobj) {
-	    obj = Riconv_open("UCS-2LE", "latin1");
-	    if(obj == (void *)(-1)) error(_("unsupported conversion"));
+	    obj = Riconv_open(TO_WCHAR, "latin1");
+	    if(obj == (void *)(-1))
+		error(_("unsupported conversion from '%s' to '%s'"),
+		      "latin1", TO_WCHAR);
 	    latin1_wobj = obj;
 	} else
 	    obj = latin1_wobj;
 	knownEnc = TRUE;
     } else if(IS_UTF8(x)) {
 	if(!utf8_wobj) {
-	    obj = Riconv_open("UCS-2LE", "UTF-8");
-	    if(obj == (void *)(-1)) error(_("unsupported conversion"));
+	    obj = Riconv_open(TO_WCHAR, "UTF-8");
+	    if(obj == (void *)(-1)) 
+		error(_("unsupported conversion from '%s' to '%s'"),
+		      "latin1", TO_WCHAR);
 	    utf8_wobj = obj;
 	} else
 	    obj = utf8_wobj;
 	knownEnc = TRUE;
     } else {
-	obj = Riconv_open("UCS-2LE", "");
-	if(obj == (void *)(-1)) error(_("unsupported conversion"));
+	obj = Riconv_open(TO_WCHAR, "");
+	if(obj == (void *)(-1))
+#ifdef Win32
+	    error(_("unsupported conversion to '%s' from codepage %d"),
+		  TO_WCHAR, localeCP);
+#else
+	    error(_("unsupported conversion from '%s' to '%s'"), "", TO_WCHAR);
+#endif
     }
 
     R_AllocStringBuffer(0, &cbuff);
@@ -892,24 +1016,34 @@ top_of_loop:
     outbuf = cbuff.data; top = outb = cbuff.bufsize - 1;
     /* First initialize output */
     Riconv (obj, NULL, NULL, &outbuf, &outb);
-    /* Then convert input: should always work  */
+next_char:
+    /* Then convert input  */
     res = Riconv(obj, &inbuf , &inb, &outbuf, &outb);
     if(res == -1 && errno == E2BIG) {
 	R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
 	goto top_of_loop;
-    } else if(res == -1 && errno == EILSEQ) {
-	if(!knownEnc) Riconv_close(obj);
-	error(_("invalid input in wtransChar"));
+    } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
+	if(outb < 5) {
+	    R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
+	    goto top_of_loop;
+	}
+	snprintf(outbuf, 5, "<%02x>", (unsigned char)*inbuf);
+	outbuf += 4; outb -= 4;
+	inbuf++; inb--;
+	goto next_char;
+	/* if(!knownEnc) Riconv_close(obj);
+	   error(_("invalid input in wtransChar")); */
     }
     if(!knownEnc) Riconv_close(obj);
     res = (top - outb);
-    p = (wchar_t *) R_alloc(res+2, 1);
-    memset(p, 0, res+2);
+    /* terminator is 2 or 4 null bytes */
+    p = (wchar_t *) R_alloc(res+4, 1);
+    memset(p, 0, res+4);
     memcpy(p, cbuff.data, res);
     R_FreeStringBuffer(&cbuff);
     return p;
 }
-#endif
+
 
 extern void *Rf_AdobeSymbol2utf8(char* work, const char *c0, int nwork); /* from util.c */
 
@@ -962,7 +1096,17 @@ const char *reEnc(const char *x, cetype_t ce_in, cetype_t ce_out, int subst)
     }
 
     switch(ce_out) {
+ #ifdef Win32
+    case CE_NATIVE:
+	{
+	    /* avoid possible misidentification of CP1250 as LATIN-2 */
+	    sprintf(buf, "CP%d", localeCP);
+	    tocode = buf;
+	    break;
+	}
+#else
     case CE_NATIVE: tocode = ""; break;
+#endif
     case CE_LATIN1: tocode = "latin1"; break;
     case CE_UTF8:   tocode = "UTF-8"; break;
     default: return x;
@@ -982,7 +1126,7 @@ next_char:
     if(res == -1 && errno == E2BIG) {
 	R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
 	goto top_of_loop;
-    } else if(res == -1 && errno == EILSEQ) {
+    } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
 	switch(subst) {
 	case 1: /* substitute hex */
 	    if(outb < 5) {
@@ -1002,6 +1146,14 @@ next_char:
 	    *outbuf++ = '.'; inbuf++; outb--; inb--;
 	    goto next_char;
 	    break;
+	case 3: /* substitute ? */
+	    if(outb < 1) {
+		R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
+		goto top_of_loop;
+	    }
+	    *outbuf++ = '?'; inbuf++; outb--; inb--;
+	    goto next_char;
+	    break;
 	default: /* skip byte */
 	    inbuf++; inb--;
 	    goto next_char;
@@ -1119,6 +1271,7 @@ mbtoucs(unsigned int *wc, const char *s, size_t n)
 	    return (size_t) -1;
 	}
     }
+    Riconv_close(cd);
     *wc = wcs[0];
     return (size_t) 1;
 }
@@ -1140,9 +1293,16 @@ size_t ucstoutf8(char *s, const unsigned int wc)
     memset(buf, 0, sizeof(buf));
     wcs[0] = wc; wcs[1] = 0;
 
-    if((void *)(-1) == (cd = Riconv_open("UTF-8", UNICODE)))
-	return (size_t)(-1);
-    status = Riconv(cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
+    if(ucsutf8_obj == NULL) {
+	if((void *)(-1) == (cd = Riconv_open("UTF-8", UNICODE))) {
+	    error(_("unsupported conversion from '%s' to '%s'"),
+		  UNICODE, "UTF-8");
+	    return (size_t)(-1);
+	}
+	ucsutf8_obj = cd;
+    }
+    
+    status = Riconv(ucsutf8_obj, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
 
     if (status == (size_t) -1) {
 	switch(errno){
@@ -1162,51 +1322,6 @@ size_t ucstoutf8(char *s, const unsigned int wc)
     return strlen(buf);
 }
 
-#else
-void * Riconv_open (const char* tocode, const char* fromcode)
-{
-    error(_("'iconv' is not available on this system"));
-    return (void *)-1;
-}
-
-size_t Riconv (void *cd, const char **inbuf, size_t *inbytesleft,
-	       char **outbuf, size_t *outbytesleft)
-{
-    error(_("'iconv' is not available on this system"));
-    return 0;
-}
-
-int Riconv_close (void * cd)
-{
-    error(_("'iconv' is not available on this system"));
-    return -1;
-}
-
-const char *translateChar(SEXP x)
-{
-    return CHAR(x);
-}
-
-const char *translateCharUTF8(SEXP x)
-{
-    return CHAR(x);
-}
-
-const char *reEnc(const char *x, cetype_t ce_in, cetype_t ce_out, int subst)
-{
-    return x;
-}
-
-void attribute_hidden
-invalidate_cached_recodings(void)
-{
-}
-size_t
-ucstoutf8(char *s, const unsigned int wc)
-{
-}
-#endif
-
 /* moved from src/unix/sys-unix.c and src/gnuwin32/extra.c */
 
 #ifdef HAVE_STAT
@@ -1227,10 +1342,18 @@ ucstoutf8(char *s, const unsigned int wc)
 
 static int isDir(char *path)
 {
+#ifdef Win32
+    struct _stati64 sb;
+#else
     struct stat sb;
+#endif
     int isdir = 0;
     if(!path) return 0;
+#ifdef Win32
+    if(_stati64(path, &sb) == 0) {
+#else
     if(stat(path, &sb) == 0) {
+#endif
 	isdir = (sb.st_mode & S_IFDIR) > 0; /* is a directory */
 #ifdef HAVE_ACCESS
 	/* We want to know if the directory is writable by this user,
@@ -1251,6 +1374,10 @@ static int isDir(char *path)
 extern char * mkdtemp (char *template);
 #endif
 
+#ifdef Win32
+# include <ctype.h>
+#endif
+
 void attribute_hidden InitTempDir()
 {
     char *tmp, *tm, tmp1[PATH_MAX+11], *p;
@@ -1322,8 +1449,17 @@ void attribute_hidden InitTempDir()
 
 char * R_tmpnam(const char * prefix, const char * tempdir)
 {
-    char tm[PATH_MAX], tmp1[PATH_MAX], *res;
-    unsigned int n, done = 0;
+    return R_tmpnam2(prefix, tempdir, "");
+}
+
+/* NB for use with multicore: parent and all children share the same
+   session directory and run in parallel.   
+   So as from 2.14.1, we make sure getpic() is part of the process.
+*/
+char * R_tmpnam2(const char *prefix, const char *tempdir, const char *fileext)
+{
+    char tm[PATH_MAX], *res;
+    unsigned int n, done = 0, pid = getpid();
 #ifdef Win32
     char filesep[] = "\\";
 #else
@@ -1331,14 +1467,23 @@ char * R_tmpnam(const char * prefix, const char * tempdir)
 #endif
 
     if(!prefix) prefix = "";	/* NULL */
-    if(strlen(tempdir) >= PATH_MAX) error(_("invalid 'tempdir' in R_tmpnam"));
-    strcpy(tmp1, tempdir);
+    if(!fileext) fileext = "";  /*  "   */
+    
+#if RAND_MAX > 16777215
+#define RAND_WIDTH 8
+#else
+#define RAND_WIDTH 12
+#endif
+    
+    if(strlen(tempdir) + 1 + strlen(prefix) + RAND_WIDTH + strlen(fileext) >= PATH_MAX) 
+    	error(_("temporary name too long"));
+    	
     for (n = 0; n < 100; n++) {
 	/* try a random number at the end.  Need at least 6 hex digits */
 #if RAND_MAX > 16777215
-	sprintf(tm, "%s%s%s%x", tmp1, filesep, prefix, rand());
+	snprintf(tm, PATH_MAX, "%s%s%s%x%x%s", tempdir, filesep, prefix, pid, rand(), fileext);
 #else
-	sprintf(tm, "%s%s%s%x%x", tmp1, filesep, prefix, rand(), rand());
+	snprintf(tm, PATH_MAX, "%s%s%s%x%x%x%s", tempdir, filesep, prefix, pid, rand(), rand(), fileext);
 #endif
 	if(!R_FileExists(tm)) {
 	    done = 1;
@@ -1348,14 +1493,17 @@ char * R_tmpnam(const char * prefix, const char * tempdir)
     if(!done)
 	error(_("cannot find unused tempfile name"));
     res = (char *) malloc((strlen(tm)+1) * sizeof(char));
+    if(!res)
+	error(_("allocation failed in R_tmpnam2"));
     strcpy(res, tm);
     return res;
 }
 
 SEXP attribute_hidden do_proctime(SEXP call, SEXP op, SEXP args, SEXP env)
-#ifdef _R_HAVE_TIMING_
 {
     SEXP ans, nm;
+
+    checkArity(op, args);
     PROTECT(ans = allocVector(REALSXP, 5));
     PROTECT(nm = allocVector(STRSXP, 5));
     R_getProcTime(REAL(ans));
@@ -1369,9 +1517,158 @@ SEXP attribute_hidden do_proctime(SEXP call, SEXP op, SEXP args, SEXP env)
     UNPROTECT(2);
     return ans;
 }
-#else
+
+void attribute_hidden resetTimeLimits()
 {
-    error(_("proc.time() is not implemented on this system"));
-    return R_NilValue;		/* -Wall */
-}
+    double data[5];
+    R_getProcTime(data);
+
+    elapsedLimit = (elapsedLimitValue > 0) ? data[2] + elapsedLimitValue : -1.0;
+    if (elapsedLimit2 > 0.0 &&
+	(elapsedLimit <= 0.0 || elapsedLimit2 < elapsedLimit))
+	elapsedLimit = elapsedLimit2;
+
+#ifdef Win32
+    cpuLimit = (cpuLimitValue > 0) ? data[0] + data[1] + cpuLimitValue : -1.0;
+#else
+    cpuLimit = (cpuLimitValue > 0) ? data[0] + data[1] + data[3] + data[4] + cpuLimitValue : -1.0;
 #endif
+    if (cpuLimit2 > 0.0 && (cpuLimit <= 0.0 || cpuLimit2 < cpuLimit))
+	cpuLimit = cpuLimit2;
+}
+
+SEXP attribute_hidden
+do_setTimeLimit(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    double cpu, elapsed, old_cpu = cpuLimitValue,
+	old_elapsed = elapsedLimitValue;
+    int transient;
+
+    checkArity(op, args);
+    cpu = asReal(CAR(args));
+    elapsed = asReal(CADR(args));
+    transient = asLogical(CADDR(args));
+
+    if (R_FINITE(cpu) && cpu > 0) cpuLimitValue = cpu; else cpuLimitValue = -1;
+
+    if (R_FINITE(elapsed) && elapsed > 0) elapsedLimitValue = elapsed;
+    else elapsedLimitValue = -1;
+
+    resetTimeLimits();
+
+    if (transient == TRUE) {
+	cpuLimitValue = old_cpu;
+	elapsedLimitValue = old_elapsed;
+    }
+
+    return R_NilValue;
+}
+
+SEXP attribute_hidden
+do_setSessionTimeLimit(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    double cpu, elapsed, data[5];
+
+    checkArity(op, args);
+    cpu = asReal(CAR(args));
+    elapsed = asReal(CADR(args));
+    R_getProcTime(data);
+
+    if (R_FINITE(cpu) && cpu > 0)
+#ifdef Win32
+	cpuLimit2 = cpu + data[0] + data[1];
+#else
+	cpuLimit2 = cpu + data[0] + data[1] + data[3] + data[4];
+#endif
+    else cpuLimit2 = -1;
+
+    if (R_FINITE(elapsed) && elapsed > 0) elapsedLimit2 = elapsed + data[2];
+    else elapsedLimit2 = -1;
+
+    return R_NilValue;
+}
+
+/* moved from character.c in 2.10.0: configure requires this */
+
+#ifdef HAVE_GLOB_H
+# include <glob.h>
+#endif
+#ifdef Win32
+# include <dos_wglob.h>
+# define globfree dos_wglobfree
+# define glob_t wglob_t
+#else
+# ifndef GLOB_QUOTE
+#  define GLOB_QUOTE 0
+# endif
+#endif
+SEXP attribute_hidden do_glob(SEXP call, SEXP op, SEXP args, SEXP env)
+{
+    SEXP x, ans;
+    int i, n, res, dirmark;
+    glob_t globbuf;
+#ifdef Win32
+    R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
+#endif
+
+    checkArity(op, args);
+    if (!isString(x = CAR(args)))
+	error(_("invalid '%s' argument"), "paths");
+    if (!LENGTH(x)) return allocVector(STRSXP, 0);
+    dirmark = asLogical(CADR(args));
+    if (dirmark == NA_LOGICAL)
+	error(_("invalid '%s' argument"), "dirmark");
+#ifndef GLOB_MARK
+    if (dirmark)
+	error(_("'dirmark = TRUE' is not supported on this platform"));
+#endif
+
+    for (i = 0; i < LENGTH(x); i++) {
+	SEXP el = STRING_ELT(x, i);
+	if (el == NA_STRING) continue;
+#ifdef Win32
+	res = dos_wglob(filenameToWchar(el, FALSE),
+			(dirmark ? GLOB_MARK : 0) |
+			GLOB_QUOTE | (i ? GLOB_APPEND : 0),
+			NULL, &globbuf);
+	if (res == GLOB_NOSPACE)
+	    error(_("internal out-of-memory condition"));
+#else
+	res = glob(translateChar(el),
+# ifdef GLOB_MARK
+		   (dirmark ? GLOB_MARK : 0) |
+# endif
+		   GLOB_QUOTE | (i ? GLOB_APPEND : 0),
+		   NULL, &globbuf);
+# ifdef GLOB_ABORTED
+	if (res == GLOB_ABORTED)
+	    warning(_("read error on '%s'"), translateChar(el));
+# endif
+# ifdef GLOB_NOSPACE
+	if (res == GLOB_NOSPACE)
+	    error(_("internal out-of-memory condition"));
+# endif
+#endif
+    }
+    n = globbuf.gl_pathc;
+    PROTECT(ans = allocVector(STRSXP, n));
+    for (i = 0; i < n; i++)
+#ifdef Win32
+    {
+	wchar_t *w = globbuf.gl_pathv[i];
+	char *buf;
+	int nb = wcstoutf8(NULL, w, 0);
+	buf = R_AllocStringBuffer(nb+1, &cbuff);
+	wcstoutf8(buf, w, nb+1); buf[nb] = '\0'; /* safety check */
+	SET_STRING_ELT(ans, i, mkCharCE(buf, CE_UTF8));
+    }
+#else
+	SET_STRING_ELT(ans, i, mkChar(globbuf.gl_pathv[i]));
+#endif
+    UNPROTECT(1);
+#ifdef Win32
+    R_FreeStringBufferL(&cbuff);
+#endif
+    globfree(&globbuf);
+    return ans;
+}
