@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2008  Robert Gentleman, Ross Ihaka and the
- *                            R Development Core Team
+ *  Copyright (C) 1997--2011  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,19 +18,14 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8>
-   char here is mainly either ASCII or handled as a whole.
-   isBlankString has been improved.
-   do_basename and do_dirname now work in chars.
-*/
-
-
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <Defn.h>
+#include <ctype.h>		/* for isspace */
+
 #undef COMPILING_R
 
 #define imax2(x, y) ((x < y) ? y : x)
@@ -42,7 +36,7 @@
 #endif
 
 #ifdef Win32
-static void R_UTF8fixslash(char *s);
+void R_UTF8fixslash(char *s);
 static void R_wfixslash(wchar_t *s);
 #endif
 
@@ -159,9 +153,11 @@ SEXP asChar(SEXP x)
 		sprintf(buf, "%d", INTEGER(x)[0]);
 		return mkChar(buf);
 	    case REALSXP:
+		PrintDefaults();
 		formatReal(REAL(x), 1, &w, &d, &e, 0);
 		return mkChar(EncodeReal(REAL(x)[0], w, d, e, OutDec));
 	    case CPLXSXP:
+		PrintDefaults();
 		formatComplex(COMPLEX(x), 1, &w, &d, &e, &wi, &di, &ei, 0);
 		return mkChar(EncodeComplex(COMPLEX(x)[0], w, d, e, wi, di, ei, OutDec));
 	    case STRSXP:
@@ -236,7 +232,7 @@ SEXPTYPE str2type(const char *s)
     int i;
     for (i = 0; TypeTable[i].str; i++) {
 	if (!strcmp(s, TypeTable[i].str))
-	    return TypeTable[i].type;
+	    return (SEXPTYPE) TypeTable[i].type;
     }
     /* SEXPTYPE is an unsigned int, so the compiler warns us w/o the cast. */
     return (SEXPTYPE) -1;
@@ -297,7 +293,6 @@ void UNIMPLEMENTED_TYPE(const char *s, SEXP x)
     UNIMPLEMENTED_TYPEt(s, TYPEOF(x));
 }
 
-#if defined(SUPPORT_MBCS)
 # include <R_ext/Riconv.h>
 # include <sys/param.h>
 # include <errno.h>
@@ -334,12 +329,12 @@ size_t mbcsToUcs2(const char *in, ucs2_t *out, int nout, int enc)
     i_buf = (char *)in;
     i_len = strlen(in); /* not including terminator */
     o_buf = (char *)out;
-    o_len = nout * sizeof(ucs2_t);
+    o_len = ((size_t) nout) * sizeof(ucs2_t);
     status = Riconv(cd, &i_buf, (size_t *)&i_len, &o_buf, (size_t *)&o_len);
-
+    int serrno = errno;
     Riconv_close(cd);
     if (status == (size_t)-1) {
-	switch(errno){
+	switch(serrno){
 	case EINVAL:
 	    return (size_t) -2;
 	case EILSEQ:
@@ -353,26 +348,21 @@ size_t mbcsToUcs2(const char *in, ucs2_t *out, int nout, int enc)
     }
     return wc_len; /* status would be better? */
 }
-#endif /* SUPPORT_MBCS */
 
 
-#ifdef SUPPORT_MBCS
 #include <wctype.h>
-#endif
 
 /* This one is not in Rinternals.h, but is used in internet module */
 Rboolean isBlankString(const char *s)
 {
-#ifdef SUPPORT_MBCS
     if(mbcslocale) {
-	wchar_t wc; int used; mbstate_t mb_st;
+	wchar_t wc; size_t used; mbstate_t mb_st;
 	mbs_init(&mb_st);
 	while( (used = Mbrtowc(&wc, s, MB_CUR_MAX, &mb_st)) ) {
-	    if(!iswspace(wc)) return FALSE;
+	    if(!iswspace((wint_t) wc)) return FALSE;
 	    s += used;
 	}
     } else
-#endif
 	while (*s)
 	    if (!isspace((int)*s++)) return FALSE;
     return TRUE;
@@ -404,6 +394,7 @@ Rboolean StringFalse(const char *name)
     return FALSE;
 }
 
+/* used in bind.c and options.c */
 SEXP attribute_hidden EnsureString(SEXP s)
 {
     switch(TYPEOF(s)) {
@@ -429,37 +420,54 @@ void Rf_checkArityCall(SEXP op, SEXP args, SEXP call)
 {
     if (PRIMARITY(op) >= 0 && PRIMARITY(op) != length(args)) {
 	if (PRIMINTERNAL(op))
-	    error(P_("%d argument passed to .Internal(%s) which requires %d",
+	    error(ngettext("%d argument passed to .Internal(%s) which requires %d",
 		     "%d arguments passed to .Internal(%s) which requires %d",
-		     length(args)),
+			   (unsigned long) length(args)),
 		  length(args), PRIMNAME(op), PRIMARITY(op));
 	else
-	    errorcall(call, P_("%d argument passed to '%s' which requires %d",
+	    errorcall(call,
+		      ngettext("%d argument passed to '%s' which requires %d",
 			       "%d arguments passed to '%s' which requires %d",
-			       length(args)),
+			       (unsigned long) length(args)),
 		      length(args), PRIMNAME(op), PRIMARITY(op));
     }
 }
 
+void attribute_hidden Rf_check1arg(SEXP arg, SEXP call, const char *formal)
+{
+    SEXP tag = TAG(arg);
+    const char *supplied;
+    size_t ns;
+    if (tag == R_NilValue) return;
+    supplied = CHAR(PRINTNAME(tag)); ns = strlen(supplied);
+    if (ns > strlen(formal) || strncmp(supplied, formal, ns))
+	errorcall(call, _("supplied argument name '%s' does not match '%s'"),
+		  supplied, formal);
+}
+
+
 SEXP nthcdr(SEXP s, int n)
 {
     if (isList(s) || isLanguage(s) || isFrame(s) || TYPEOF(s) == DOTSXP ) {
 	while( n-- > 0 ) {
 	    if (s == R_NilValue)
-		error(_("\"nthcdr\" list shorter than %d"), n);
+		error(_("'nthcdr' list shorter than %d"), n);
 	    s = CDR(s);
 	}
 	return s;
     }
-    else error(_("\"nthcdr\" needs a list to CDR down"));
+    else error(_("'nthcdr' needs a list to CDR down"));
     return R_NilValue;/* for -Wall */
 }
 
 
+/* This is a primitive (with no arguments) */
 SEXP attribute_hidden do_nargs(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     RCNTXT *cptr;
     int nargs = NA_INTEGER;
+
+    checkArity(op, args);
     for (cptr = R_GlobalContext; cptr != NULL; cptr = cptr->nextcontext) {
 	if ((cptr->callflag & CTXT_FUNCTION) && cptr->cloenv == rho) {
 	    nargs = length(cptr->promargs);
@@ -554,10 +562,11 @@ static void isort_with_index(int *x, int *indx, int n)
 */
 SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP xi, yi, ansx, ansy, ans, ansnames, x_lone, y_lone;
+    SEXP xi, yi, ansx, ansy, ans, x_lone, y_lone;
     int nx = 0, ny = 0, i, j, k, nans = 0, nx_lone = 0, ny_lone = 0;
     int all_x = 0, all_y = 0, ll = 0/* "= 0" : for -Wall */;
     int *ix, *iy, tmp, nnx, nny, i0, j0;
+    const char *nms[] = {"xi", "yi", "x.alone", "y.alone", ""};
 
     checkArity(op, args);
     xi = CAR(args);
@@ -572,8 +581,8 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("'all.y' must be TRUE or FALSE"));
 
     /* 0. sort the indices */
-    ix = (int *) R_alloc(nx, sizeof(int));
-    iy = (int *) R_alloc(ny, sizeof(int));
+    ix = (int *) R_alloc((size_t) nx, sizeof(int));
+    iy = (int *) R_alloc((size_t) ny, sizeof(int));
     for(i = 0; i < nx; i++) ix[i] = i+1;
     for(i = 0; i < ny; i++) iy[i] = i+1;
     isort_with_index(INTEGER(xi), ix, nx);
@@ -595,7 +604,8 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 
     /* 2. allocate and store result components */
-    PROTECT(ans = allocVector(VECSXP, 4));
+
+    PROTECT(ans = mkNamed(VECSXP, nms));
     ansx = allocVector(INTSXP, nans);    SET_VECTOR_ELT(ans, 0, ansx);
     ansy = allocVector(INTSXP, nans);    SET_VECTOR_ELT(ans, 1, ansy);
 
@@ -625,13 +635,7 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    }
     }
 
-    PROTECT(ansnames = allocVector(STRSXP, 4));
-    SET_STRING_ELT(ansnames, 0, mkChar("xi"));
-    SET_STRING_ELT(ansnames, 1, mkChar("yi"));
-    SET_STRING_ELT(ansnames, 2, mkChar("x.alone"));
-    SET_STRING_ELT(ansnames, 3, mkChar("y.alone"));
-    setAttrib(ans, R_NamesSymbol, ansnames);
-    UNPROTECT(2);
+    UNPROTECT(1);
     return ans;
 }
 
@@ -654,12 +658,13 @@ SEXP static intern_getwd(void)
 	if(res > 0) {
 	    wcstoutf8(buf, wbuf, PATH_MAX+1);
 	    R_UTF8fixslash(buf);
-	    rval = allocVector(STRSXP, 1);
+	    PROTECT(rval = allocVector(STRSXP, 1));
 	    SET_STRING_ELT(rval, 0, mkCharCE(buf, CE_UTF8));
+	    UNPROTECT(1);
 	}
     }
-#elif defined(HAVE_GETCWD)
-    char *res = getcwd(buf, PATH_MAX);
+#else
+    char *res = getcwd(buf, PATH_MAX); /* can return NULL */
     if(res) rval = mkString(buf);
 #endif
     return(rval);
@@ -700,9 +705,7 @@ SEXP attribute_hidden do_setwd(SEXP call, SEXP op, SEXP args, SEXP rho)
     {
 	const char *path
 	    = R_ExpandFileName(translateChar(STRING_ELT(s, 0)));
-# ifdef HAVE_CHDIR
     if(chdir(path) < 0)
-# endif
 	error(_("cannot change working directory"));
     }
 #endif
@@ -864,6 +867,76 @@ SEXP attribute_hidden do_dirname(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 #endif
 
+
+#ifndef Win32 /* Windows version is in src/gnuwin32/extra.c */
+#ifndef HAVE_DECL_REALPATH
+extern char *realpath(const char *path, char *resolved_path);
+#endif
+
+SEXP attribute_hidden do_normalizepath(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP ans, paths = CAR(args);
+    int i, n = LENGTH(paths);
+    const char *path;
+    char abspath[PATH_MAX+1];
+
+    checkArity(op, args);
+    if (!isString(paths))
+	error(_("'path' must be a character vector"));
+
+    int mustWork = asLogical(CADDR(args)); /* 1, NA_LOGICAL or 0 */
+
+/* Does any platform not have this? */
+#ifdef HAVE_REALPATH
+    PROTECT(ans = allocVector(STRSXP, n));
+    for (i = 0; i < n; i++) {
+	path = translateChar(STRING_ELT(paths, i));
+	char *res = realpath(path, abspath);
+	if (res) 
+	    SET_STRING_ELT(ans, i, mkChar(abspath));
+	else {
+	    SET_STRING_ELT(ans, i, STRING_ELT(paths, i));
+	    /* and report the problem */
+	    if (mustWork == 1)
+		error("path[%d]=\"%s\": %s", i+1, path, strerror(errno));
+	    else if (mustWork == NA_LOGICAL)
+		warning("path[%d]=\"%s\": %s", i+1, path, strerror(errno));
+	}
+    }
+#else
+    Rboolean OK;
+    warning("this platform does not have realpath so the results may not be canonical");
+    PROTECT(ans = allocVector(STRSXP, n));
+    for (i = 0; i < n; i++) {
+	path = translateChar(STRING_ELT(paths, i));
+	OK = strlen(path) <= PATH_MAX;
+	if (OK) {
+	    if (path[0] == '/') strncpy(abspath, path, PATH_MAX);
+	    else {
+		OK = getcwd(abspath, PATH_MAX) != NULL;
+		OK = OK && (strlen(path) + strlen(abspath) + 1 <= PATH_MAX);
+		if (OK) {strcat(abspath, "/"); strcat(abspath, path);}
+	    }
+	}
+	/* we need to check that this exists */
+	if (OK) OK = (access(abspath, 0 /* F_OK */) == 0);
+	if (OK) SET_STRING_ELT(ans, i, mkChar(abspath));
+	else {
+	    SET_STRING_ELT(ans, i, STRING_ELT(paths, i));
+	    /* and report the problem */
+	    if (mustWork == 1)
+		error("path[%d]=\"%s\": %s", i+1, path, strerror(errno));
+	    else if (mustWork == NA_LOGICAL)
+		warning("path[%d]=\"%s\": %s", i+1, path, strerror(errno));
+	}
+    }
+#endif
+    UNPROTECT(1);
+    return ans;
+}
+#endif
+
+
 /* encodeString(x, w, quote, justify) */
 SEXP attribute_hidden do_encodeString(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
@@ -928,7 +1001,8 @@ SEXP attribute_hidden do_encoding(SEXP call, SEXP op, SEXP args, SEXP rho)
     n = LENGTH(x);
     PROTECT(ans = allocVector(STRSXP, n));
     for (i = 0; i < n; i++) {
-	if(IS_LATIN1(STRING_ELT(x, i))) tmp = "latin1";
+	if(IS_BYTES(STRING_ELT(x, i))) tmp = "bytes";
+	else if(IS_LATIN1(STRING_ELT(x, i))) tmp = "latin1";
 	else if(IS_UTF8(STRING_ELT(x, i))) tmp = "UTF-8";
 	else tmp = "unknown";
 	SET_STRING_ELT(ans, i, mkChar(tmp));
@@ -959,12 +1033,14 @@ SEXP attribute_hidden do_setencoding(SEXP call, SEXP op, SEXP args, SEXP rho)
 	this = CHAR(STRING_ELT(enc, i % m)); /* ASCII */
 	if(streql(this, "latin1")) ienc = CE_LATIN1;
 	else if(streql(this, "UTF-8")) ienc = CE_UTF8;
+	else if(streql(this, "bytes")) ienc = CE_BYTES;
 	tmp = STRING_ELT(x, i);
 	if(tmp == NA_STRING) continue;
 	if (! ((ienc == CE_LATIN1 && IS_LATIN1(tmp)) ||
 	       (ienc == CE_UTF8 && IS_UTF8(tmp)) ||
+	       (ienc == CE_BYTES && IS_BYTES(tmp)) ||
 	       (ienc == CE_NATIVE && ! IS_LATIN1(tmp) && ! IS_UTF8(tmp))))
-	    SET_STRING_ELT(x, i, mkCharCE(CHAR(tmp), ienc));
+	    SET_STRING_ELT(x, i, mkCharLenCE(CHAR(tmp), LENGTH(tmp), ienc));
     }
     UNPROTECT(1);
     return x;
@@ -988,7 +1064,6 @@ Rboolean strIsASCII(const char *str)
     return TRUE;
 }
 
-#ifdef SUPPORT_MBCS
 /* Number of additional bytes */
 static const unsigned char utf8_table4[] = {
   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
@@ -1003,8 +1078,8 @@ int attribute_hidden utf8clen(char c)
     return 1 + utf8_table4[c & 0x3f];
 }
 
-/* This returns the result in wchar_t, but does not assume
-   wchar_t is UCS-2/4 and so is for internal use only */
+/* These return the result in wchar_t, but does assume
+   wchar_t is UCS-2/4 and so are for internal use only */
 size_t attribute_hidden
 utf8toucs(wchar_t *wc, const char *s)
 {
@@ -1020,63 +1095,64 @@ utf8toucs(wchar_t *wc, const char *s)
 	*w = (wchar_t) byte;
 	return 1;
     } else if (byte < 0xE0) {
-	if(strlen(s) < 2) return -2;
+	if(strlen(s) < 2) return (size_t)-2;
 	if ((s[1] & 0xC0) == 0x80) {
 	    *w = (wchar_t) (((byte & 0x1F) << 6) | (s[1] & 0x3F));
 	    return 2;
-	} else return -1;
+	} else return (size_t)-1;
     } else if (byte < 0xF0) {
-	if(strlen(s) < 3) return -2;
+	if(strlen(s) < 3) return (size_t)-2;
 	if (((s[1] & 0xC0) == 0x80) && ((s[2] & 0xC0) == 0x80)) {
 	    *w = (wchar_t) (((byte & 0x0F) << 12)
-		    | ((s[1] & 0x3F) << 6) | (s[2] & 0x3F));
-	    byte = *w;
+			    | (unsigned int) ((s[1] & 0x3F) << 6)
+			    | (s[2] & 0x3F));
+	    byte = (unsigned int) *w;
 	    /* Surrogates range */
-	    if(byte >= 0xD800 && byte <= 0xDFFF) return -1;
-	    if(byte == 0xFFFE || byte == 0xFFFF) return -1;
+	    if(byte >= 0xD800 && byte <= 0xDFFF) return (size_t)-1;
+	    if(byte == 0xFFFE || byte == 0xFFFF) return (size_t)-1;
 	    return 3;
-	} else return -1;
+	} else return (size_t)-1;
     }
-    if(sizeof(wchar_t) < 4) return -2;
+    if(sizeof(wchar_t) < 4) return (size_t)-2;
     /* So now handle 4,5.6 byte sequences with no testing */
     if (byte < 0xf8) {
-	if(strlen(s) < 4) return -2;
+	if(strlen(s) < 4) return (size_t)-2;
 	*w = (wchar_t) (((byte & 0x0F) << 18)
-			| ((s[1] & 0x3F) << 12)
-			| ((s[2] & 0x3F) << 6)
+			| (unsigned int) ((s[1] & 0x3F) << 12)
+			| (unsigned int) ((s[2] & 0x3F) << 6)
 			| (s[3] & 0x3F));
 	return 4;
     } else if (byte < 0xFC) {
-	if(strlen(s) < 5) return -2;
+	if(strlen(s) < 5) return (size_t)-2;
 	*w = (wchar_t) (((byte & 0x0F) << 24)
-			| ((s[1] & 0x3F) << 12)
-			| ((s[2] & 0x3F) << 12)
-			| ((s[3] & 0x3F) << 6)
+			| (unsigned int) ((s[1] & 0x3F) << 12)
+			| (unsigned int) ((s[2] & 0x3F) << 12)
+			| (unsigned int) ((s[3] & 0x3F) << 6)
 			| (s[4] & 0x3F));
 	return 5;
     } else {
-	if(strlen(s) < 6) return -2;
+	if(strlen(s) < 6) return (size_t)-2;
 	*w = (wchar_t) (((byte & 0x0F) << 30)
-			| ((s[1] & 0x3F) << 24)
-			| ((s[2] & 0x3F) << 18)
-			| ((s[3] & 0x3F) << 12)
-			| ((s[4] & 0x3F) << 6)
+			| (unsigned int) ((s[1] & 0x3F) << 24)
+			| (unsigned int) ((s[2] & 0x3F) << 18)
+			| (unsigned int) ((s[3] & 0x3F) << 12)
+			| (unsigned int) ((s[4] & 0x3F) << 6)
 			| (s[5] & 0x3F));
 	return 6;
     }
 }
 
-size_t 
+size_t
 utf8towcs(wchar_t *wc, const char *s, size_t n)
 {
-    int m, res = 0;
+    ssize_t m, res = 0;
     const char *t;
     wchar_t *p;
     wchar_t local;
 
     if(wc)
 	for(p = wc, t = s; ; p++, t += m) {
-	    m  = utf8toucs(p, t);
+	    m  = (ssize_t) utf8toucs(p, t);
 	    if (m < 0) error(_("invalid input '%s' in 'utf8towcs'"), s);
 	    if (m == 0) break;
 	    res ++;
@@ -1084,22 +1160,22 @@ utf8towcs(wchar_t *wc, const char *s, size_t n)
 	}
     else
 	for(t = s; ; res++, t += m) {
-	    m  = utf8toucs(&local, t);
+	    m  = (ssize_t) utf8toucs(&local, t);
 	    if (m < 0) error(_("invalid input '%s' in 'utf8towcs'"), s);
 	    if (m == 0) break;
 	}
-    return res;
+    return (size_t) res;
 }
 
 /* based on pcre.c */
-static const int utf8_table1[] =
+static const unsigned int utf8_table1[] =
   { 0x7f, 0x7ff, 0xffff, 0x1fffff, 0x3ffffff, 0x7fffffff};
-static const int utf8_table2[] = { 0, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc};
+static const unsigned int utf8_table2[] = { 0, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc};
 
 static size_t Rwcrtomb(char *s, const wchar_t wc)
 {
-    register int i, j;
-    unsigned int cvalue = wc;
+    register size_t i, j;
+    unsigned int cvalue = (unsigned int) wc;
     char buf[10], *b;
 
     b = s ? s : buf;
@@ -1108,22 +1184,22 @@ static size_t Rwcrtomb(char *s, const wchar_t wc)
 	if (cvalue <= utf8_table1[i]) break;
     b += i;
     for (j = i; j > 0; j--) {
-	*b-- = 0x80 | (cvalue & 0x3f);
+	*b-- = (char) (0x80 | (cvalue & 0x3f));
 	cvalue >>= 6;
     }
-    *b = utf8_table2[i] | cvalue;
+    *b = (char) (utf8_table2[i] | cvalue);
     return i + 1;
 }
 
 /* attribute_hidden? */
 size_t wcstoutf8(char *s, const wchar_t *wc, size_t n)
 {
-    int m, res=0;
+    ssize_t m, res=0;
     char *t;
     const wchar_t *p;
     if(s) {
 	for(p = wc, t = s; ; p++) {
-	    m  = Rwcrtomb(t, *p);
+	    m  = (ssize_t) Rwcrtomb(t, *p);
 	    if(m <= 0) break;
 	    res += m;
 	    if(res >= n) break;
@@ -1131,14 +1207,15 @@ size_t wcstoutf8(char *s, const wchar_t *wc, size_t n)
 	}
     } else {
 	for(p = wc; ; p++) {
-	    m  = Rwcrtomb(NULL, *p);
+	    m  = (ssize_t) Rwcrtomb(NULL, *p);
 	    if(m <= 0) break;
 	    res += m;
 	}
     }
-    return res;
+    return (size_t) res;
 }
 
+
 /* A version that reports failure as an error */
 size_t Mbrtowc(wchar_t *wc, const char *s, size_t n, mbstate_t *ps)
 {
@@ -1147,8 +1224,10 @@ size_t Mbrtowc(wchar_t *wc, const char *s, size_t n, mbstate_t *ps)
     if(n <= 0 || !*s) return (size_t)0;
     used = mbrtowc(wc, s, n, ps);
     if((int) used < 0) {
+	/* This gets called from the menu setup in RGui */
+	if (!R_Is_Running) return (size_t)-1;
 	/* let's try to print out a readable version */
-	char *err = alloca(4*strlen(s) + 1), *q;
+	char err[4*strlen(s) + 1], *q;
 	const char *p;
 	R_CheckStack();
 	for(p = s, q = err; *p; ) {
@@ -1177,61 +1256,32 @@ Rboolean mbcsValid(const char *str)
     return  ((int)mbstowcs(NULL, str, 0) >= 0);
 }
 
-#ifdef UNUSED
-/* We do this conversion ourselves to do our own error recovery */
-void mbcsToLatin1(const char *in, char *out)
-{
-    wchar_t *wbuff;
-    int i;
-    size_t res = mbstowcs(NULL, in, 0), mres;
+#include "pcre.h"
+/* This changed at 8.13: we don't allow < 8.0 */
+#if  PCRE_MAJOR > 8 || PCRE_MINOR >= 13
+extern int _pcre_valid_utf8(const char *string, int length, int *erroroffset);
 
-    if(res == (size_t)(-1)) {
-	/* let's try to print out a readable version */
-	size_t used, n = strlen(in);
-	char *err = alloca(4*n + 1), *q;
-	const char *p;
-	mbstate_t ps;
-	R_CheckStack();
-	for(p = in, q = err; *p; ) {
-	    used = mbrtowc(NULL, p, n, &ps);
-	    if(used == 0) break;
-	    else if((int) used > 0) {
-		memcpy(q, p, used);
-		p += used;
-		q += used;
-		n -= used;
-	    } else {
-		sprintf(q, "<%02x>", (unsigned char) *p++);
-		q += 4;
-		n--;
-	    }
-	}
-	*q = '\0';
-	warning(_("invalid input '%s' in mbcsToLatin1: omitted"), err);
-	*out = '\0';
-	return;
-    }
-    wbuff = (wchar_t *) alloca((res+1) * sizeof(wchar_t));
-    R_CheckStack();
-    if(!wbuff) error(_("allocation failure in '%s'"), "mbcsToLatin1");
-    mres = mbstowcs(wbuff, in, res+1);
-    if(mres == (size_t)-1) /* we checked above, so should not get here */
-	error("invalid input in 'mbcsToLatin1'");
-    for(i = 0; i < res; i++) {
-	/* here we do assume Unicode wchars */
-	if(wbuff[i] > 0xFF) out[i] = '.';
-	else out[i] = (char) wbuff[i];
-    }
-    out[res] = '\0';
+Rboolean utf8Valid(const char *str)
+{
+    int errp;
+    return  (_pcre_valid_utf8(str, (int) strlen(str), &errp) == 0);
+}
+#else
+extern int _pcre_valid_utf8(const char *string, int length);
+
+Rboolean utf8Valid(const char *str)
+{
+    return  (_pcre_valid_utf8(str, (int) strlen(str)) < 0);
 }
 #endif
 
+
 /* MBCS-aware versions of common comparisons.  Only used for ASCII c */
 char *Rf_strchr(const char *s, int c)
 {
     char *p = (char *)s;
     mbstate_t mb_st;
-    int used;
+    size_t used;
 
     if(!mbcslocale || utf8locale) return strchr(s, c);
     mbs_init(&mb_st);
@@ -1246,7 +1296,7 @@ char *Rf_strrchr(const char *s, int c)
 {
     char *p = (char *)s, *plast = NULL;
     mbstate_t mb_st;
-    int used;
+    size_t used;
 
     if(!mbcslocale || utf8locale) return strrchr(s, c);
     mbs_init(&mb_st);
@@ -1256,27 +1306,12 @@ char *Rf_strrchr(const char *s, int c)
     }
     return plast;
 }
-#else
-/* Dummy entry points so R.dll always has them */
-int utf8clen(char c) { return 1;}
-size_t Mbrtowc(wchar_t *wc, const char *s, size_t n, void *ps)
-{ return (size_t)(-1);}
-Rboolean mbcsValid(const char *str) { return TRUE; }
-#ifdef UNUSED
-void mbcsToLatin1(char *in, char *out) {}
-#endif
-#undef Rf_strchr
-char *Rf_strchr(const char *s, int c) {return strchr(s, c);}
-#undef Rf_strrchr
-char *Rf_strrchr(const char *s, int c) {return strrchr(s, c);}
-#endif
 
 #ifdef Win32
 void R_fixslash(char *s)
 {
     char *p = s;
 
-#ifdef SUPPORT_MBCS
     if(mbcslocale) {
 	mbstate_t mb_st; int used;
 	mbs_init(&mb_st);
@@ -1285,13 +1320,12 @@ void R_fixslash(char *s)
 	    p += used;
 	}
     } else
-#endif
 	for (; *p; p++) if (*p == '\\') *p = '/';
 	/* preserve network shares */
 	if(s[0] == '/' && s[1] == '/') s[0] = s[1] = '\\';
 }
 
-static void R_UTF8fixslash(char *s)
+void R_UTF8fixslash(char *s)
 {
     char *p = s;
 
@@ -1314,7 +1348,6 @@ void R_fixbackslash(char *s)
 {
     char *p = s;
 
-#ifdef SUPPORT_MBCS
     if(mbcslocale) {
 	mbstate_t mb_st; int used;
 	mbs_init(&mb_st);
@@ -1323,7 +1356,6 @@ void R_fixbackslash(char *s)
 	    p += used;
 	}
     } else
-#endif
 	for (; *p; p++) if (*p == '/') *p = '\\';
 }
 #endif
@@ -1336,7 +1368,7 @@ void F77_SYMBOL(rexitc)(char *msg, int *nchar)
 	warning(_("error message truncated to 255 chars"));
 	nc = 255;
     }
-    strncpy(buf, msg, nc);
+    strncpy(buf, msg, (size_t) nc);
     buf[nc] = '\0';
     error("%s", buf);
 }
@@ -1349,7 +1381,7 @@ void F77_SYMBOL(rwarnc)(char *msg, int *nchar)
 	warning(_("warning message truncated to 255 chars"));
 	nc = 255;
     }
-    strncpy(buf, msg, nc);
+    strncpy(buf, msg, (size_t) nc);
     buf[nc] = '\0';
     warning("%s", buf);
 }
@@ -1363,9 +1395,9 @@ void F77_SYMBOL(rchkusr)(void)
 char *acopy_string(const char *in)
 {
     char *out;
-    int len = strlen(in);
+    size_t len = strlen(in);
     if (len > 0) {
-	out = (char *) R_alloc(1+strlen(in), sizeof(char));
+	out = (char *) R_alloc(1 + len, sizeof(char));
 	strcpy(out, in);
     } else
 	out = "";
@@ -1373,52 +1405,6 @@ char *acopy_string(const char *in)
 }
 
 
-/* FIXME: consider inlining here */
-#ifdef Win32
-
-static int Rstrcoll(const char *s1, const char *s2)
-{
-    wchar_t *w1, *w2;
-    w1 = (wchar_t *) alloca((strlen(s1)+1)*sizeof(wchar_t));
-    w2 = (wchar_t *) alloca((strlen(s2)+1)*sizeof(wchar_t));
-    R_CheckStack();
-    utf8towcs(w1, s1, strlen(s1));
-    utf8towcs(w2, s2, strlen(s2));
-    return wcscoll(w1, w2);
-}
-
-int Scollate(SEXP a, SEXP b)
-{
-    if(getCharCE(a) == CE_UTF8 || getCharCE(b) == CE_UTF8)
-	return Rstrcoll(translateCharUTF8(a), translateCharUTF8(b));
-    else
-	return strcoll(translateChar(a), translateChar(b));
-}
-
-int Seql(SEXP a, SEXP b)
-{
-    return (a == b) || !strcmp(translateCharUTF8(a), translateCharUTF8(b));
-}
-
-#else
-
-# ifdef HAVE_STRCOLL
-#  define STRCOLL strcoll
-# else
-#  define STRCOLL strcmp
-# endif
-
-int Scollate(SEXP a, SEXP b)
-{
-    return STRCOLL(translateChar(a), translateChar(b));
-}
-
-int Seql(SEXP a, SEXP b)
-{
-    return (a == b) || !strcmp(translateChar(a), translateChar(b));
-}
-
-#endif
 
 
 /* Table from
@@ -1463,15 +1449,15 @@ void *Rf_AdobeSymbol2utf8(char *work, const char *c0, int nwork)
     while (*c) {
 	if (*c < 32) *t++ = ' ';
 	else {
-	    unsigned int u = s2u[*c - 32];
-	    if (u < 128) *t++ = u;
+	    unsigned int u = (unsigned int) s2u[*c - 32];
+	    if (u < 128) *t++ = (unsigned char) u;
 	    else if (u < 0x800) {
-		*t++ = 0xc0 | (u >> 6);
-		*t++ = 0x80 | (u & 0x3f);
+		*t++ = (unsigned char) (0xc0 | (u >> 6));
+		*t++ = (unsigned char) (0x80 | (u & 0x3f));
 	    } else {
-		*t++ = 0xe0 | (u >> 12);
-		*t++ = 0x80 | ((u >> 6) & 0x3f);
-		*t++ = 0x80 | (u & 0x3f);
+		*t++ = (unsigned char) (0xe0 | (u >> 12));
+		*t++ = (unsigned char) (0x80 | ((u >> 6) & 0x3f));
+		*t++ = (unsigned char) (0x80 | (u & 0x3f));
 	    }
 	}
 	if (t+6 > (unsigned char *)(work + nwork)) break;
@@ -1489,8 +1475,8 @@ int attribute_hidden Rf_AdobeSymbol2ucs2(int n)
 
 double R_strtod4(const char *str, char **endptr, char dec, Rboolean NA)
 {
-    LDOUBLE ans = 0.0, p10 = 10.0, fac = 1.0;
-    int n, expn = 0, sign = 1, ndigits = 0;
+    long double ans = 0.0, p10 = 10.0, fac = 1.0;
+    int n, expn = 0, sign = 1, ndigits = 0, exph = -1;
     const char *p = str;
 
     /* optional whitespace */
@@ -1530,7 +1516,9 @@ double R_strtod4(const char *str, char **endptr, char dec, Rboolean NA)
 	    if('0' <= *p && *p <= '9') ans = 16*ans + (*p -'0');
 	    else if('a' <= *p && *p <= 'f') ans = 16*ans + (*p -'a' + 10);
 	    else if('A' <= *p && *p <= 'F') ans = 16*ans + (*p -'A' + 10);
+	    else if(*p == dec) {exph = 0; continue;}
 	    else break;
+	    if (exph >= 0) exph += 4;
 	}
 	if (*p == 'p' || *p == 'P') {
 	    int expsign = 1;
@@ -1542,6 +1530,7 @@ double R_strtod4(const char *str, char **endptr, char dec, Rboolean NA)
 	    }
 	    for (n = 0; *p >= '0' && *p <= '9'; p++) n = n * 10 + (*p - '0');
 	    expn += expsign * n;
+	    if(exph > 0) expn -= exph;
 	    if (expn < 0) {
 		for (n = -expn, fac = 1.0; n; n >>= 1, p2 *= p2)
 		    if (n & 1) fac *= p2;
@@ -1611,3 +1600,282 @@ double R_atof(const char *str)
 {
     return R_strtod4(str, NULL, '.', FALSE);
 }
+
+/* enc2native and enc2utf8, but they are the same in a UTF-8 locale */
+/* primitive */
+SEXP attribute_hidden do_enc2(SEXP call, SEXP op, SEXP args, SEXP env)
+{
+    SEXP ans, el;
+    int i;
+    Rboolean duped = FALSE;
+
+    checkArity(op, args);
+    check1arg(args, call, "x");
+
+    if (!isString(CAR(args)))
+	errorcall(call, "argumemt is not a character vector");
+    ans = CAR(args);
+    for (i = 0; i < LENGTH(ans); i++) {
+	el = STRING_ELT(ans, i);
+	if(PRIMVAL(op) && !known_to_be_utf8) { /* enc2utf8 */
+	    if(!IS_UTF8(el) && !strIsASCII(CHAR(el))) {
+		if (!duped) { PROTECT(ans = duplicate(ans)); duped = TRUE; }
+		SET_STRING_ELT(ans, i, 
+			       mkCharCE(translateCharUTF8(el), CE_UTF8));
+	    }
+	} else { /* enc2native */
+	    if((known_to_be_latin1 && IS_UTF8(el)) ||
+	       (known_to_be_utf8 && IS_LATIN1(el)) ||
+	       ENC_KNOWN(el)) {
+		if (!duped) { PROTECT(ans = duplicate(ans)); duped = TRUE; }
+		SET_STRING_ELT(ans, i, mkChar(translateChar(el)));
+	    }
+	}
+    }
+    if(duped) UNPROTECT(1);
+    return ans;
+}
+
+#ifdef USE_ICU
+# include <locale.h>
+#ifdef USE_ICU_APPLE
+/* Mac OS X is missing the headers */
+typedef int UErrorCode; /* really an enum these days */
+struct UCollator;
+typedef struct UCollator UCollator;
+
+typedef enum {
+  UCOL_EQUAL    = 0,
+  UCOL_GREATER    = 1,
+  UCOL_LESS    = -1
+} UCollationResult ;
+
+typedef enum {
+  UCOL_DEFAULT = -1,
+  UCOL_PRIMARY = 0,
+  UCOL_SECONDARY = 1,
+  UCOL_TERTIARY = 2,
+  UCOL_DEFAULT_STRENGTH = UCOL_TERTIARY,
+  UCOL_CE_STRENGTH_LIMIT,
+  UCOL_QUATERNARY=3,
+  UCOL_IDENTICAL=15,
+  UCOL_STRENGTH_LIMIT,
+  UCOL_OFF = 16,
+  UCOL_ON = 17,
+  UCOL_SHIFTED = 20,
+  UCOL_NON_IGNORABLE = 21,
+  UCOL_LOWER_FIRST = 24,
+  UCOL_UPPER_FIRST = 25,
+  UCOL_ATTRIBUTE_VALUE_COUNT
+} UColAttributeValue;
+
+typedef UColAttributeValue UCollationStrength;
+
+typedef enum {
+      UCOL_FRENCH_COLLATION,
+      UCOL_ALTERNATE_HANDLING,
+      UCOL_CASE_FIRST,
+      UCOL_CASE_LEVEL,
+      UCOL_NORMALIZATION_MODE,
+      UCOL_DECOMPOSITION_MODE = UCOL_NORMALIZATION_MODE,
+      UCOL_STRENGTH,
+      UCOL_HIRAGANA_QUATERNARY_MODE,
+      UCOL_NUMERIC_COLLATION,
+      UCOL_ATTRIBUTE_COUNT
+} UColAttribute;
+
+/* UCharIterator struct has to be defined since we use its instances as
+   local variables, but we don't actually use any of its members. */
+typedef struct UCharIterator {
+  const void *context;
+  int32_t length, start, index, limit, reservedField;
+  void *fns[16]; /* we overshoot here (there is just 10 fns in ICU 3.6),
+		    but we have to make sure that enough stack space
+		    is allocated when used as a local var in future
+		    versions */
+} UCharIterator;
+
+UCollator* ucol_open(const char *loc, UErrorCode *status);
+void ucol_close(UCollator *coll);
+void ucol_setAttribute(UCollator *coll, UColAttribute attr,
+		       UColAttributeValue value, UErrorCode *status);
+void ucol_setStrength(UCollator *coll, UCollationStrength strength);
+UCollationResult ucol_strcollIter(const UCollator *coll,
+				  UCharIterator *sIter,
+				  UCharIterator *tIter,
+				  UErrorCode *status);
+void uiter_setUTF8(UCharIterator *iter, const char *s, int32_t length);
+
+void uloc_setDefault(const char* localeID, UErrorCode* status);
+
+#define U_ZERO_ERROR 0
+#define U_FAILURE(x) ((x)>U_ZERO_ERROR)
+
+#else
+#include <unicode/utypes.h>
+#include <unicode/ucol.h>
+#include <unicode/uloc.h>
+#include <unicode/uiter.h>
+#endif
+
+static UCollator *collator = NULL;
+
+/* called from platform.c */
+void attribute_hidden resetICUcollator(void)
+{
+    if (collator) ucol_close(collator);
+    collator = NULL;
+}
+
+static const struct {
+    const char * const str;
+    int val;
+} ATtable[] = {
+    { "case_first", UCOL_CASE_FIRST },
+    { "upper", UCOL_UPPER_FIRST },
+    { "lower", UCOL_LOWER_FIRST },
+    { "default ", UCOL_DEFAULT },
+    { "strength", 999 },
+    { "primary ", UCOL_PRIMARY },
+    { "secondary ", UCOL_SECONDARY },
+    { "teritary ", UCOL_TERTIARY },
+    { "guaternary ", UCOL_QUATERNARY },
+    { "identical ", UCOL_IDENTICAL },
+    { "french_collation", UCOL_FRENCH_COLLATION },
+    { "on", UCOL_ON },
+    { "off", UCOL_OFF },
+    { "normalization", UCOL_NORMALIZATION_MODE },
+    { "alternate_handling", UCOL_ALTERNATE_HANDLING },
+    { "non_ignorable", UCOL_NON_IGNORABLE },
+    { "shifted", UCOL_SHIFTED },
+    { "case_level", UCOL_CASE_LEVEL },
+    { "hiragana_quaternary", UCOL_HIRAGANA_QUATERNARY_MODE },
+    { NULL,  0 }
+};
+
+
+SEXP attribute_hidden do_ICUset(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP x;
+    UErrorCode  status = U_ZERO_ERROR;
+
+    for (; args != R_NilValue; args = CDR(args)) {
+	const char *this = CHAR(PRINTNAME(TAG(args)));
+	const char *s;
+
+	x = CAR(args);
+	if (!isString(x) || LENGTH(x) != 1)
+	    error(_("invalid '%s' argument"), this);
+	s = CHAR(STRING_ELT(x, 0));
+	if (streql(this, "locale")) {
+	    if (collator) ucol_close(collator);
+	    uloc_setDefault(s, &status);
+	    if(U_FAILURE(status))
+		error("failed to set ICU locale");
+	    collator = ucol_open(NULL, &status);
+	    if (U_FAILURE(status)) error("failed to open ICU collator");
+	} else {
+	    int i, at = -1, val = -1;
+	    for (i = 0; ATtable[i].str; i++)
+		if (streql(this, ATtable[i].str)) {
+		    at = ATtable[i].val;
+		    break;
+		}
+	    for (i = 0; ATtable[i].str; i++)
+		if (streql(s, ATtable[i].str)) {
+		    val = ATtable[i].val;
+		    break;
+		}
+	    if (collator && at == 999 && val >= 0) {
+		ucol_setStrength(collator, val);
+	    } else if (collator && at >= 0 && val >= 0) {
+		ucol_setAttribute(collator, at, val, &status);
+		if (U_FAILURE(status))
+		    error("failed to set ICU collator attribute");
+	    }
+	}
+    }
+
+    return R_NilValue;
+}
+
+
+/* NB: strings can have equal collation weight without being identical */
+int Scollate(SEXP a, SEXP b)
+{
+    int result = 0;
+    UErrorCode  status = U_ZERO_ERROR;
+    UCharIterator aIter, bIter;
+    const char *as = translateCharUTF8(a), *bs = translateCharUTF8(b);
+    size_t len1 = strlen(as), len2 = strlen(bs);
+
+    if (collator == NULL && strcmp("C", setlocale(LC_COLLATE, NULL)) ) {
+	/* do better later */
+	uloc_setDefault(setlocale(LC_COLLATE, NULL), &status);
+	if(U_FAILURE(status))
+	    error("failed to set ICU locale");
+	collator = ucol_open(NULL, &status);
+	if (U_FAILURE(status)) error("failed to open ICU collator");
+    }
+    if (collator == NULL)
+	return strcoll(translateChar(a), translateChar(b));
+
+    uiter_setUTF8(&aIter, as, len1);
+    uiter_setUTF8(&bIter, bs, len2);
+    result = ucol_strcollIter(collator, &aIter, &bIter, &status);
+    if (U_FAILURE(status)) error("could not collate");
+    return result;
+}
+
+#else /* not USE_ICU */
+
+SEXP attribute_hidden do_ICUset(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    warning(_("ICU is not supported on this build"));
+    return R_NilValue;
+}
+
+void attribute_hidden resetICUcollator(void) {}
+
+# ifdef Win32
+
+static int Rstrcoll(const char *s1, const char *s2)
+{
+    wchar_t w1[strlen(s1)+1], w2[strlen(s2)+1];
+    R_CheckStack();
+    utf8towcs(w1, s1, strlen(s1));
+    utf8towcs(w2, s2, strlen(s2));
+    return wcscoll(w1, w2);
+}
+
+int Scollate(SEXP a, SEXP b)
+{
+    if(getCharCE(a) == CE_UTF8 || getCharCE(b) == CE_UTF8)
+	return Rstrcoll(translateCharUTF8(a), translateCharUTF8(b));
+    else
+	return strcoll(translateChar(a), translateChar(b));
+}
+
+# else
+int Scollate(SEXP a, SEXP b)
+{
+    return strcoll(translateChar(a), translateChar(b));
+}
+
+# endif
+#endif
+
+#include <lzma.h>
+
+SEXP crc64ToString(SEXP in)
+{
+    uint64_t crc = 0;
+    char ans[17];
+    if (!isString(in)) error("input must be a character string");
+    const char *str = CHAR(STRING_ELT(in, 0));
+
+    /* Seems this is realy 64-bit only on 64-bit platforms */
+    crc = lzma_crc64((uint8_t *)str, strlen(str), crc);
+    snprintf(ans, 17, "%lx", (long unsigned int) crc);
+    return mkString(ans);
+}
