@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001--2006  The R Development Core Team.
- *  Copyright (C) 2003-5      The R Foundation
+ *  Copyright (C) 2001--2010  The R Development Core Team.
+ *  Copyright (C) 2003--2010  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> All chars are ASCII */
@@ -25,28 +25,70 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
+
 #include <Defn.h>
+#include <ctype.h> /* for toupper */
 
 #include "Lapack.h"
 
+/* FIXME: Would want to make these available to packages;
+ * BUT:  1) cannot be in libRlapack {since that may be external}
+ *       2) Pkgs cannot get it from the C-Lapack interface code {lapack.so}
+ *          since that is R-internal
+ */
+char La_norm_type(const char *typstr)
+{
+    char typup;
+
+    if (strlen(typstr) != 1)
+	error(
+	    _("argument type[1]='%s' must be a character string of string length 1"),
+	    typstr);
+    typup = toupper(*typstr);
+    if (typup == '1')
+	typup = 'O'; /* aliases */
+    else if (typup == 'E')
+	typup = 'F';
+    else if (typup != 'M' && typup != 'O' && typup != 'I' && typup != 'F')
+	error(_("argument type[1]='%s' must be one of 'M','1','O','I','F' or 'E'"),
+	      typstr);
+    return typup;
+}
+
+/* Lapack condition number approximation: currently only supports _1 or _Inf norm : */
+char La_rcond_type(const char *typstr)
+{
+    char typup;
+
+    if (strlen(typstr) != 1)
+	error(_("argument type[1]='%s' must be a character string of string length 1"),
+	      typstr);
+    typup = toupper(*typstr);
+    if (typup == '1')
+	typup = 'O'; /* alias */
+    else if (typup != 'O' && typup != 'I')
+	error(_("argument type[1]='%s' must be one of '1','O', or 'I'"),
+	      typstr);
+    return typup; /* 'O' or 'I' */
+}
+
+
 static SEXP modLa_svd(SEXP jobu, SEXP jobv, SEXP x, SEXP s, SEXP u, SEXP v,
 		      SEXP method)
 {
     int *xdims, n, p, lwork, info = 0;
     double *work, *xvals, tmp;
     SEXP val, nm;
-    char *meth;
 
     if (!(isString(jobu) && isString(jobv)))
 	error(_("'jobu' and 'jobv' must be character strings"));
     if (!isString(method))
 	error(_("'method' must be a character string"));
-    meth = CHAR(STRING_ELT(method, 0));
     xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     n = xdims[0]; p = xdims[1];
     xvals = (double *) R_alloc(n * p, sizeof(double));
     /* work on a copy of x */
-    Memcpy(xvals, REAL(x), (size_t) (n * p));
+    Memcpy(xvals, REAL(x), n * p);
 
     {
 	int ldu = INTEGER(getAttrib(u, R_DimSymbol))[0],
@@ -106,7 +148,7 @@ static SEXP modLa_rs(SEXP xin, SEXP only_values)
     if (n != xdims[1])
 	error(_("'x' must be a square numeric matrix"));
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
+    if (ov == NA_LOGICAL) error(_("invalid '%s' argument"), "only.values");
     if (ov) jobv[0] = 'N'; else jobv[0] = 'V';
 
     PROTECT(values = allocVector(REALSXP, n));
@@ -194,9 +236,9 @@ static SEXP modLa_rg(SEXP x, SEXP only_values)
 
     xvals = (double *) R_alloc(n * n, sizeof(double));
     /* work on a copy of x */
-    Memcpy(xvals, REAL(x), (size_t) (n * n));
+    Memcpy(xvals, REAL(x), n * n);
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
+    if (ov == NA_LOGICAL) error(_("invalid '%s' argument"), "only.values");
     vectors = !ov;
     jobVL[0] = jobVR[0] = 'N';
     left = right = (double *) 0;
@@ -258,7 +300,222 @@ static SEXP modLa_rg(SEXP x, SEXP only_values)
     return ret;
 }
 
+static SEXP modLa_dlange(SEXP A, SEXP type)
+{
+    SEXP x, val;
+    int *xdims, m, n, nprot = 0;
+    double *work = NULL;
+    char typNorm[] = {'\0', '\0'};
+
+    if (!isString(type))
+	error(_("'type' must be a character string"));
+    if (!isReal(A) && isNumeric(A)) {
+	x = PROTECT(coerceVector(A, REALSXP)); nprot++;
+    } else
+	x = A;
+    if (!(isMatrix(x) && isReal(x))) {
+	UNPROTECT(nprot);
+	error(_("'A' must be a numeric matrix"));
+    }
+
+    xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
+    m = xdims[0];
+    n = xdims[1]; /* m x n  matrix {using Lapack naming convention} */
+
+    typNorm[0] = La_norm_type(CHAR(asChar(type)));
+
+    val = PROTECT(allocVector(REALSXP, 1)); nprot++;
+    if(*typNorm == 'I') work = (double *) R_alloc(m, sizeof(double));
+    REAL(val)[0] = F77_CALL(dlange)(typNorm, &m, &n, REAL(x), &m, work);
+
+    UNPROTECT(nprot);
+    return val;
+}
+
+
 /* ------------------------------------------------------------ */
+static SEXP modLa_dgecon(SEXP A, SEXP norm)
+{
+    SEXP x, val;
+    int *xdims, m, n, info, *iwork;
+    double anorm, *work;
+    char typNorm[] = {'\0', '\0'};
+
+    if (!isString(norm))
+	error(_("'norm' must be a character string"));
+    if (!isReal(A) && isNumeric(A))
+	x = PROTECT(coerceVector(A, REALSXP));
+    else
+	x = PROTECT(duplicate(A)); /* will be overwritten by LU */
+    if (!(isMatrix(x) && isReal(x))) {
+	UNPROTECT(1);
+	error(_("'A' must be a numeric matrix"));
+    }
+
+    xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
+    m = xdims[0];
+    n = xdims[1]; /* m x n  matrix {using Lapack naming convention} */
+
+    typNorm[0] = La_rcond_type(CHAR(asChar(norm)));
+
+    val = PROTECT(allocVector(REALSXP, 1));
+    work = (double *) R_alloc((*typNorm == 'I' && m > 4*n) ? m : 4*n,
+			      sizeof(double));
+    iwork = (int *) R_alloc(m, sizeof(int));
+
+    anorm = F77_CALL(dlange)(typNorm, &m, &n, REAL(x), &m, work);
+
+    /* Compute the LU-decomposition and overwrite 'x' with result :*/
+    F77_CALL(dgetrf)(&m, &n, REAL(x), &m, iwork, &info);
+    if (info) {
+	if (info < 0) {
+	    UNPROTECT(2);
+	    error(_("error [%d] from Lapack 'dgetrf()'"), info);
+	}
+	else { /* i := info > 0:  LU decomp. is completed, but  U[i,i] = 0
+		* <==> singularity */
+#if 0
+	    warning(_("exact singularity: U[%d,%d] = 0 in LU-decomposition {Lapack 'dgetrf()'}"),
+		    info,info);
+#endif
+	    REAL(val)[0] = 0.; /* rcond = 0 <==> singularity */
+	    UNPROTECT(2);
+	    return val;
+	}
+    }
+    F77_CALL(dgecon)(typNorm, &n, REAL(x), &n, &anorm,
+		     /* rcond = */ REAL(val),
+		     work, iwork, &info);
+    UNPROTECT(2);
+    if (info) error(_("error [%d] from Lapack 'dgecon()'"), info);
+    return val;
+}
+
+static SEXP modLa_dtrcon(SEXP A, SEXP norm)
+{
+    SEXP x, val;
+    int *xdims, n, nprot = 0, info;
+    char typNorm[] = {'\0', '\0'};
+
+    if (!isString(norm))
+	error(_("'norm' must be a character string"));
+    if (!isReal(A) && isNumeric(A)) {
+	nprot++;
+	x = PROTECT(coerceVector(A, REALSXP));
+    } else
+	x = A; /* no copy needed */
+    if (!(isMatrix(x) && isReal(x))) {
+	UNPROTECT(nprot);
+	error(_("'A' must be a numeric matrix"));
+    }
+    xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
+    n = xdims[0];
+    if(n != xdims[1]) {
+	UNPROTECT(nprot);
+	error(_("'A' must be a *square* matrix"));
+    }
+
+    typNorm[0] = La_rcond_type(CHAR(asChar(norm)));
+
+    nprot++;
+    val = PROTECT(allocVector(REALSXP, 1));
+
+    F77_CALL(dtrcon)(typNorm, "U", "N", &n, REAL(x), &n,
+		     REAL(val),
+		     /* work : */ (double *) R_alloc(3*n, sizeof(double)),
+		     /* iwork: */ (int *)    R_alloc(n, sizeof(int)),
+		     &info);
+    UNPROTECT(nprot);
+    if (info) error(_("error [%d] from Lapack 'dtrcon()'"), info);
+    return val;
+}
+
+static SEXP modLa_zgecon(SEXP A, SEXP norm)
+{
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
+    SEXP val;
+    Rcomplex *avals; /* copy of A, to be modified */
+    int *dims, n, info;
+    double anorm, *rwork;
+    char typNorm[] = {'\0', '\0'};
+
+    if (!isString(norm))
+	error(_("'norm' must be a character string"));
+    if (!(isMatrix(A) && isComplex(A)))
+	error(_("'A' must be a complex matrix"));
+    dims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
+    n = dims[0];
+    if(n != dims[1])
+	error(_("'A' must be a *square* matrix"));
+
+    typNorm[0] = La_rcond_type(CHAR(asChar(norm)));
+
+    val = PROTECT(allocVector(REALSXP, 1));
+
+    rwork = (double *) R_alloc(2*n, sizeof(Rcomplex));
+    anorm = F77_CALL(zlange)(typNorm, &n, &n, COMPLEX(A), &n, rwork);
+
+    /* Compute the LU-decomposition and overwrite 'x' with result;
+     * working on a copy of A : */
+    avals = (Rcomplex *) R_alloc(n * n, sizeof(Rcomplex));
+    Memcpy(avals, COMPLEX(A), n * n);
+    F77_CALL(zgetrf)(&n, &n, avals, &n,
+		     /* iwork: */(int *) R_alloc(n, sizeof(int)),
+		     &info);
+    if (info) {
+	UNPROTECT(1);
+	error(_("error [%d] from Lapack 'zgetrf()'"), info);
+    }
+
+    F77_CALL(zgecon)(typNorm, &n, avals, &n, &anorm,
+		     /* rcond = */ REAL(val),
+		     /* work : */ (Rcomplex *) R_alloc(4*n, sizeof(Rcomplex)),
+		     rwork, &info);
+    UNPROTECT(1);
+    if (info) error(_("error [%d] from Lapack 'zgecon()'"), info);
+    return val;
+
+#else
+    error(_("Fortran complex functions are not available on this platform"));
+    return R_NilValue; /* -Wall */
+#endif
+}
+
+static SEXP modLa_ztrcon(SEXP A, SEXP norm)
+{
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
+    SEXP val;
+    int *dims, n, info;
+    char typNorm[] = {'\0', '\0'};
+
+    if (!isString(norm))
+	error(_("'norm' must be a character string"));
+    if (!(isMatrix(A) && isComplex(A)))
+	error(_("'A' must be a complex matrix"));
+    dims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
+    n = dims[0];
+    if(n != dims[1])
+	error(_("'A' must be a *square* matrix"));
+
+    typNorm[0] = La_rcond_type(CHAR(asChar(norm)));
+
+    val = PROTECT(allocVector(REALSXP, 1));
+
+    F77_CALL(ztrcon)(typNorm, "U", "N", &n, COMPLEX(A), &n,
+		     REAL(val),
+		     /* work : */ (Rcomplex *) R_alloc(2*n, sizeof(Rcomplex)),
+		     /* rwork: */ (double *)   R_alloc(n, sizeof(double)),
+		     &info);
+    UNPROTECT(1);
+    if (info) error(_("error [%d] from Lapack 'ztrcon()'"), info);
+    return val;
+#else
+    error(_("Fortran complex functions are not available on this platform"));
+    return R_NilValue; /* -Wall */
+#endif
+}
+
+
 
 static SEXP modLa_zgesv(SEXP A, SEXP Bin)
 {
@@ -287,7 +544,7 @@ static SEXP modLa_zgesv(SEXP A, SEXP Bin)
 
     avals = (Rcomplex *) R_alloc(n * n, sizeof(Rcomplex));
     /* work on a copy of x */
-    Memcpy(avals, COMPLEX(A), (size_t) (n * n));
+    Memcpy(avals, COMPLEX(A), n * n);
     F77_CALL(zgesv)(&n, &p, avals, &n, ipiv, COMPLEX(B), &n, &info);
     if (info < 0)
 	error(_("argument %d of Lapack routine %s had invalid value"),
@@ -334,8 +591,7 @@ static SEXP modLa_zgeqp3(SEXP Ain)
 	error(_("error code %d from Lapack routine '%s'"), info, "zgeqp3");
     val = PROTECT(allocVector(VECSXP, 4));
     nm = PROTECT(allocVector(STRSXP, 4));
-    rank = PROTECT(allocVector(INTSXP, 1));
-    INTEGER(rank)[0] = m < n ? m : n;
+    rank = PROTECT(ScalarInteger(m < n ? m : n));
     SET_STRING_ELT(nm, 0, mkChar("qr"));
     SET_STRING_ELT(nm, 1, mkChar("rank"));
     SET_STRING_ELT(nm, 2, mkChar("qraux"));
@@ -407,7 +663,7 @@ static SEXP modqr_qy_cmplx(SEXP Q, SEXP Bin, SEXP trans)
     if (!(isMatrix(Bin) && isComplex(Bin)))
 	error(_("'b' must be a complex matrix"));
     tr = asLogical(trans);
-    if(tr == NA_LOGICAL) error(_("invalid 'trans' parameter"));
+    if(tr == NA_LOGICAL) error(_("invalid '%s' argument"), "trans");
 
     PROTECT(B = duplicate(Bin));
     Qdims = INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP));
@@ -503,7 +759,7 @@ static SEXP modLa_rs_cmplx(SEXP xin, SEXP only_values)
     if (n != xdims[1])
 	error(_("'x' must be a square numeric matrix"));
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
+    if (ov == NA_LOGICAL) error(_("invalid '%s' argument"), "only.values");
     if (ov) jobv[0] = 'N'; else jobv[0] = 'V';
 
     PROTECT(values = allocVector(REALSXP, n));
@@ -558,9 +814,9 @@ static SEXP modLa_rg_cmplx(SEXP x, SEXP only_values)
 
     xvals = (Rcomplex *) R_alloc(n * n, sizeof(Rcomplex));
     /* work on a copy of x */
-    Memcpy(xvals, COMPLEX(x), (size_t) (n * n));
+    Memcpy(xvals, COMPLEX(x), n * n);
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
+    if (ov == NA_LOGICAL) error(_("invalid '%s' argument"), "only.values");
     jobVL[0] = jobVR[0] = 'N';
     left = right = (Rcomplex *) 0;
     if (!ov) {
@@ -641,27 +897,35 @@ static SEXP modLa_chol(SEXP A)
 static SEXP modLa_chol2inv(SEXP A, SEXP size)
 {
     int sz = asInteger(size);
-    if (sz == NA_INTEGER || sz < 1)
+    if (sz == NA_INTEGER || sz < 1) {
 	error(_("'size' argument must be a positive integer"));
-    if (isMatrix(A)) {
-	SEXP Amat = PROTECT(coerceVector(A, REALSXP));
-	SEXP ans;
-	SEXP adims = getAttrib(A, R_DimSymbol);
-	int m = INTEGER(adims)[0];
-	int n = INTEGER(adims)[1];
-	int i, j;
+	return R_NilValue; /* -Wall */
+    } else {
+	SEXP ans, Amat = A; /* -Wall: we initialize here as for the 1x1 case */
+	int m = 1, n = 1, i, j, nprot = 0;
 
-	if (sz > n) error(_("'size' cannot exceed ncol(x) = %d"), n);
-	if (sz > m) error(_("'size' cannot exceed nrow(x) = %d"), m);
-	ans = PROTECT(allocMatrix(REALSXP, sz, sz));
+	if (sz == 1 && !isMatrix(A) && isReal(A)) {
+	    /* nothing to do; m = n = 1; ... */
+	} else if (isMatrix(A)) {
+	    SEXP adims = getAttrib(A, R_DimSymbol);
+	    Amat = PROTECT(coerceVector(A, REALSXP)); nprot++;
+	    m = INTEGER(adims)[0];
+	    n = INTEGER(adims)[1];
+	} else error(_("'a' must be a numeric matrix"));
+
+	if (sz > n) { UNPROTECT(nprot); error(_("'size' cannot exceed ncol(x) = %d"), n); }
+	if (sz > m) { UNPROTECT(nprot); error(_("'size' cannot exceed nrow(x) = %d"), m); }
+	ans = PROTECT(allocMatrix(REALSXP, sz, sz)); nprot++;
 	for (j = 0; j < sz; j++) {
 	    for (i = 0; i <= j; i++)
 		REAL(ans)[i + j * sz] = REAL(Amat)[i + j * m];
 	}
 	F77_CALL(dpotri)("Upper", &sz, REAL(ans), &sz, &i);
 	if (i != 0) {
+	    UNPROTECT(nprot);
 	    if (i > 0)
-		error(_("element (%d, %d) is zero, so the inverse cannot be computed"), i, i);
+		error(_("element (%d, %d) is zero, so the inverse cannot be computed"),
+		      i, i);
 	    error(_("argument %d of Lapack routine %s had invalid value"),
 		  -i, "dpotri");
 	}
@@ -669,11 +933,9 @@ static SEXP modLa_chol2inv(SEXP A, SEXP size)
 	    for (i = j+1; i < sz; i++)
 		REAL(ans)[i + j * sz] = REAL(ans)[j + i * sz];
 	}
-	unprotect(2);
+	UNPROTECT(nprot);
 	return ans;
     }
-    else error(_("'a' must be a numeric matrix"));
-    return R_NilValue; /* -Wall */
 }
 
 /* ------------------------------------------------------------ */
@@ -704,7 +966,7 @@ static SEXP modLa_dgesv(SEXP A, SEXP Bin, SEXP tolin)
 
     avals = (double *) R_alloc(n * n, sizeof(double));
 				/* work on a copy of A */
-    Memcpy(avals, REAL(A), (size_t) (n * n));
+    Memcpy(avals, REAL(A), n * n);
     F77_CALL(dgesv)(&n, &p, avals, &n, ipiv, REAL(B), &n, &info);
     if (info < 0)
 	error(_("argument %d of Lapack routine %s had invalid value"),
@@ -750,8 +1012,7 @@ static SEXP modLa_dgeqp3(SEXP Ain)
 	error(_("error code %d from Lapack routine '%s'"), info, "dgeqp3");
     val = PROTECT(allocVector(VECSXP, 4));
     nm = PROTECT(allocVector(STRSXP, 4));
-    rank = PROTECT(allocVector(INTSXP, 1));
-    INTEGER(rank)[0] = m < n ? m : n;
+    rank = PROTECT(ScalarInteger(m < n ? m : n));
     SET_STRING_ELT(nm, 0, mkChar("qr"));
     SET_STRING_ELT(nm, 1, mkChar("rank"));
     SET_STRING_ELT(nm, 2, mkChar("qraux"));
@@ -813,7 +1074,7 @@ static SEXP modqr_qy_real(SEXP Q, SEXP Bin, SEXP trans)
     if (!(isMatrix(Bin) && isReal(Bin)))
 	error(_("'b' must be a numeric matrix"));
     tr = asLogical(trans);
-    if(tr == NA_LOGICAL) error(_("invalid 'trans' parameter"));
+    if(tr == NA_LOGICAL) error(_("invalid '%s' argument"), "trans");
 
     PROTECT(B = duplicate(Bin));
     Qdims = INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP));
@@ -839,6 +1100,7 @@ static SEXP modqr_qy_real(SEXP Q, SEXP Bin, SEXP trans)
     return B;
 }
 
+/* TODO : add  a *complex* version, using  LAPACK ZGETRF() */
 static SEXP moddet_ge_real(SEXP Ain, SEXP logarithm)
 {
     int i, n, *Adims, info, *jpvt, sign, useLog;
@@ -861,7 +1123,12 @@ static SEXP moddet_ge_real(SEXP Ain, SEXP logarithm)
 	error(_("error code %d from Lapack routine '%s'"), info, "dgetrf");
     else if (info > 0) { /* Singular matrix:  U[i,i] (i := info) is 0 */
 	/*warning("Lapack dgetrf(): singular matrix: U[%d,%d]=0", info,info);*/
-	modulus = (useLog ? R_NegInf : 0.);
+	modulus =
+#ifdef _not_quite_/* unfortunately does not work -- FIXME */
+	    (ISNAN(REAL(A)[(info-1)*n + (info-1)])) /* pivot is NA/NaN */
+	    ? R_NaN :
+#endif
+	    (useLog ? R_NegInf : 0.);
     }
     else {
 	for (i = 0; i < n; i++) if (jpvt[i] != (i + 1))
@@ -910,6 +1177,11 @@ R_init_lapack(DllInfo *info)
     tmp->svd = modLa_svd;
     tmp->rs = modLa_rs;
     tmp->rg = modLa_rg;
+    tmp->dlange = modLa_dlange;
+    tmp->dgecon = modLa_dgecon;
+    tmp->dtrcon = modLa_dtrcon;
+    tmp->zgecon = modLa_zgecon;
+    tmp->ztrcon = modLa_ztrcon;
     tmp->zgesv = modLa_zgesv;
     tmp->zgeqp3 = modLa_zgeqp3;
     tmp->qr_coef_cmplx = modqr_coef_cmplx;
