@@ -23,8 +23,8 @@
 /* From http://www.netlib.org/specfun/ribesl	Fortran translated by f2c,...
  *	------------------------------=#----	Martin Maechler, ETH Zurich
  */
-#include "bessel.h"
 #include "nmath.h"
+#include "bessel.h"
 
 #ifndef MATHLIB_STANDALONE
 #include <R_ext/Memory.h>
@@ -56,7 +56,7 @@ double bessel_i(double x, double alpha, double expo)
 	/* Using Abramowitz & Stegun  9.6.2 & 9.6.6
 	 * this may not be quite optimal (CPU and accuracy wise) */
 	return(bessel_i(x, -alpha, expo) +
-	       ((alpha == na) ? 0 :
+	       ((alpha == na) ? /* sin(pi * alpha) = 0 */ 0 :
 		bessel_k(x, -alpha, expo) *
 		((ize == 1)? 2. : 2.*exp(-2.*x))/M_PI * sin(-M_PI * alpha)));
     }
@@ -67,7 +67,7 @@ double bessel_i(double x, double alpha, double expo)
     if (!bi) MATHLIB_ERROR("%s", _("bessel_i allocation error"));
 #else
     vmax = vmaxget();
-    bi = (double *) R_alloc(nb, sizeof(double));
+    bi = (double *) R_alloc((size_t) nb, sizeof(double));
 #endif
     I_bessel(&x, &alpha, &nb, &ize, bi, &ncalc);
     if(ncalc != nb) {/* error input */
@@ -87,6 +87,46 @@ double bessel_i(double x, double alpha, double expo)
     return x;
 }
 
+/* modified version of bessel_i that accepts a work array instead of
+   allocating one. */
+double bessel_i_ex(double x, double alpha, double expo, double *bi)
+{
+    long nb, ncalc, ize;
+    double na;
+
+#ifdef IEEE_754
+    /* NaNs propagated correctly */
+    if (ISNAN(x) || ISNAN(alpha)) return x + alpha;
+#endif
+    if (x < 0) {
+	ML_ERROR(ME_RANGE, "bessel_i");
+	return ML_NAN;
+    }
+    ize = (long)expo;
+    na = floor(alpha);
+    if (alpha < 0) {
+	/* Using Abramowitz & Stegun  9.6.2 & 9.6.6
+	 * this may not be quite optimal (CPU and accuracy wise) */
+	return(bessel_i_ex(x, -alpha, expo, bi) +
+	       ((alpha == na) ? 0 :
+		bessel_k_ex(x, -alpha, expo, bi) *
+		((ize == 1)? 2. : 2.*exp(-2.*x))/M_PI * sin(-M_PI * alpha)));
+    }
+    nb = 1 + (long)na;/* nb-1 <= alpha < nb */
+    alpha -= (nb-1);
+    I_bessel(&x, &alpha, &nb, &ize, bi, &ncalc);
+    if(ncalc != nb) {/* error input */
+	if(ncalc < 0)
+	    MATHLIB_WARNING4(_("bessel_i(%g): ncalc (=%ld) != nb (=%ld); alpha=%g. Arg. out of range?\n"),
+			     x, ncalc, nb, alpha);
+	else
+	    MATHLIB_WARNING2(_("bessel_i(%g,nu=%g): precision lost in result\n"),
+			     x, alpha+nb-1);
+    }
+    x = bi[nb-1];
+    return x;
+}
+
 static void I_bessel(double *x, double *alpha, long *nb,
 		     long *ize, double *bi, long *ncalc)
 {
@@ -214,7 +254,7 @@ static void I_bessel(double *x, double *alpha, long *nb,
 	    return;
 	}
 	intx = (long) (*x);/* fine, since *x <= xlrg_BESS_IJ <<< LONG_MAX */
-	if (*x >= rtnsig_BESS) { /* "non-small" x */
+	if (*x >= rtnsig_BESS) { /* "non-small" x ( >= 1e-4 ) */
 /* -------------------------------------------------------------------
    Initialize the forward sweep, the P-sequence of Olver
    ------------------------------------------------------------------- */
@@ -436,33 +476,33 @@ L230:
 		    bi[n] /= sum;
 	    }
 	    return;
-	} else {
+	} else { /* small x  < 1e-4 */
 	    /* -----------------------------------------------------------
 	       Two-term ascending series for small X.
 	       -----------------------------------------------------------*/
 	    aa = 1.;
 	    empal = 1. + nu;
-	    if (*x > enmten_BESS)
+#ifdef IEEE_754
+	    /* No need to check for underflow */
+	    halfx = .5 * *x;
+#else
+	    if (*x > enmten_BESS) */
 		halfx = .5 * *x;
 	    else
-		halfx = 0.;
+	    	halfx = 0.;
+#endif
 	    if (nu != 0.)
 		aa = pow(halfx, nu) / gamma_cody(empal);
 	    if (*ize == 2)
 		aa *= exp(-(*x));
-	    if (*x + 1. > 1.)
-		bb = halfx * halfx;
-	    else
-		bb = 0.;
-
+	    bb = halfx * halfx;
 	    bi[1] = aa + aa * bb / empal;
 	    if (*x != 0. && bi[1] == 0.)
 		*ncalc = 0;
 	    if (*nb > 1) {
 		if (*x == 0.) {
-		    for (n = 2; n <= *nb; ++n) {
+		    for (n = 2; n <= *nb; ++n)
 			bi[n] = 0.;
-		    }
 		} else {
 		    /* -------------------------------------------------
 		       Calculate higher-order functions.
