@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000-2001, 2005 The R Development Core Team
+ *  Copyright (C) 2000-11 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,17 +28,20 @@
  *
  */
 
-#include <config.h> /* needed for HAVE_RINT and USE_BUILTIN_RINT */
+#include <config.h> /* needed for HAVE_*, IEEE_754 */
 #include "nmath.h"
 
-/* USE_BUILTIN_RINT could also be defined by a configure test */
-#ifndef HAVE_RINT
-#define USE_BUILTIN_RINT
-#endif
 
-#ifdef USE_BUILTIN_RINT
-#define R_rint private_rint
+/*  nearbyint is C99, so all platforms should have it (and AFAIK, all do) */
+#ifdef HAVE_NEARBYINT
+# define R_rint nearbyint
+#elif defined(HAVE_RINT)
+# define R_rint rint
+#else
+# define R_rint private_rint
+extern double private_rint(double x);
 
+/* also used potentially in fprec.c and main/format.c */
 double attribute_hidden private_rint(double x)
 {
     double tmp, sgn = 1.0;
@@ -63,25 +66,23 @@ double attribute_hidden private_rint(double x)
     }
     return sgn * tmp;
 }
-#else
-#define R_rint rint
 #endif
 
 double fround(double x, double digits) {
 #define MAX_DIGITS DBL_MAX_10_EXP
     /* = 308 (IEEE); was till R 0.99: (DBL_DIG - 1) */
     /* Note that large digits make sense for very small numbers */
-    double pow10, sgn, intx;
+    long double pow10, sgn, intx;
     int dig;
 
-#ifdef IEEE_754
     if (ISNAN(x) || ISNAN(digits))
 	return x + digits;
     if(!R_FINITE(x)) return x;
-#endif
 
-    if (digits > MAX_DIGITS)
-	digits = MAX_DIGITS;
+    if(digits == ML_POSINF) return x;
+    else if(digits == ML_NEGINF) return 0.0;
+
+    if (digits > MAX_DIGITS) digits = MAX_DIGITS;
     dig = (int)floor(digits + 0.5);
     if(x < 0.) {
 	sgn = -1.;
