@@ -1662,4 +1662,36 @@ stopifnot(identical(unstack(DF4), DF[1])) # was a list
 stopifnot(!is.na(qchisq(p=0.025, df=0.00991)))
 ## NaN in 2.13.2
 
+
+## nobs() for zero-weight glm fits:
+DF <- data.frame(x1=log(1:10), x2=c(1/(1:9), NA), y=1:10,
+                 wt=c(0,2,0,4,0,6,7,8,9,10))
+stopifnot(nobs(lm(y ~ x1 + x2, weights = wt, data=DF)) ==
+          nobs(glm(y ~ x1 + x2, weights = wt, data = DF)))
+## was 6 and 9 in R < 2.14.1.
+
+
+## anyDuplicated(*, MARGIN=0)
+m. <- m <- cbind(M = c(3,2,7,2),
+                 F = c(6,2,7,2))
+rownames(m.) <- LETTERS[1:4]; m.
+stopifnot(identical(attributes(dm <- duplicated(m., MARGIN=0)),
+		    attributes(m.)),
+	  (dvm <- duplicated(as.vector(m.))) == dm, # all TRUE
+	  identical(anyDuplicated(	    m.,	 MARGIN=0),
+		    anyDuplicated(as.vector(m.), MARGIN=0)))
+## gave error in R < 2.14.1
+
+
+## PR#14739
+stopifnot(!is.nan(pbinom(10, 1e6, 0.01, log.p=TRUE)))
+## was NaN thanks to Maechler's misuse of toms708 in 2.11.0.
+
+
+## PR14742
+stopifnot(identical(duplicated(data.frame(c(1, 1)), fromLast = TRUE),
+                    duplicated(c(1, 1), fromLast = TRUE)))
+## first ignored fromLast in 2.14.0.
+
+
 proc.time()
