@@ -1,7 +1,7 @@
 ## Regression tests for which the printed output is the issue
 ### _and_ must work (no Recommended packages, please)
 
-postscript("reg-tests-2.ps", encoding = "ISOLatin1.enc")
+pdf("reg-tests-2.pdf", encoding = "ISOLatin1.enc")
 
 ## force standard handling for data frames
 options(stringsAsFactors=TRUE)
@@ -204,7 +204,7 @@ is.na(paste(c(1,NA)))
 is.na(list())# logical(0)
 ll <- list(pi,"C",NaN,Inf, 1:3, c(0,NA), NA)
 is.na (ll)
-is.nan(ll)
+lapply(ll, is.nan)  # is.nan no longer works on lists
 ## end of moved from NA.Rd
 
 ## is.na was returning unset values on nested lists
@@ -481,10 +481,8 @@ rowsum(matrix(1:12, 3,4), c("Y","X","Y"))
 
 ## PR#1115 (saving strings with ascii=TRUE)
 x <- y <- unlist(as.list(
-    parse(text=paste("\"\\",
-          as.character(structure(1:255,class="octmode")),
-             "\"",sep=""))))
-save(x, ascii=T, file=(fn <- tempfile()))
+    parse(text=paste("\"\\", as.character(as.octmode(1:255)), "\"",sep=""))))
+save(x, ascii=TRUE, file=(fn <- tempfile()))
 load(fn)
 all(x==y)
 unlink(fn)
@@ -1152,7 +1150,11 @@ binom.test(c(800,10))# p-value < epsilon
 
 ## Misleading error messages on integer overflow
 ## Uwe Ligges, R-devel, 2004-02-19
-try(numeric(2^31))
+## (modified to make printed result the same whether numeric() is
+##  compiled or interpreted)
+## try(numeric(2^31))
+tryCatch(numeric(2^31),
+         error = function(e) paste("Error:", conditionMessage(e)))
 try(matrix( , 2^31, 1))
 try(matrix( , 2^31/10, 100))
 try(array(dim=c(2^31/10, 100)))
@@ -1584,8 +1586,14 @@ df1[df1 == 2] # this is first coerced to a matrix, and drops to a vector
 df3 <- data.frame(a=1:2, b=2:3)
 df3[df3 == 2]            # had spurious names
 # but not allowed
-try(df2[df2 == 2] <- 1:2)
-try(m2[m2 == 2] <- 1:2)
+## (modified to make printed result the same whether numeric() is
+##  compiled or interpreted)
+## try(df2[df2 == 2] <- 1:2)
+## try(m2[m2 == 2] <- 1:2)
+tryCatch(df2[df2 == 2] <- 1:2,
+         error = function(e) paste("Error:", conditionMessage(e)))
+tryCatch(m2[m2 == 2] <- 1:2,
+         error = function(e) paste("Error:", conditionMessage(e)))
 ##
 
 
@@ -1626,7 +1634,7 @@ x # list
 
 
 ## printing of a kernel:
-kernel(1, 0)
+kernel(1)
 ## printed wrongly in R <= 2.1.1
 
 
@@ -1859,11 +1867,11 @@ x[2]
 x[[2]]
 stopifnot(identical(x[2], x[[2]]))
 as.list(x)
-unlist(as.list(x))
-stopifnot(identical(x, unlist(as.list(x))))
+(xx <- unlist(as.list(x)))
+stopifnot(identical(x, xx))
 as.vector(x, "list")
-sapply(x, na.pass)
-stopifnot(identical(x, sapply(x, na.pass)))
+(sx <- sapply(x, function(.).))
+stopifnot(identical(x, sx))
 ## changed in 2.4.0
 
 
@@ -1964,6 +1972,7 @@ fit <- lm(y ~ 1, na.action="na.exclude")
 summary(fit)
 ## failed < 2.4.0
 
+RNGkind("default","default")## reset to default - ease  R core
 
 ## prettyNum lost attributes (PR#8695)
 format(matrix(1:16, 4), big.mark = ",")
@@ -2096,15 +2105,16 @@ all.equal(as.raw(1:3), as.raw(3:1))
 ## tests of deparsing
 # if we run this from stdin, we will have no source, so fake it
 f <- function(x, xm  = max(1L, x)) {xx <- 0L; yy <- NA_real_}
-attr(f, "source") <-
-    "function(x, xm  = max(1L, x)) {xx <- 0L; yy <- NA_real_}"
+attr(f, "srcref") <- srcref(srcfilecopy("",
+    "function(x, xm  = max(1L, x)) {xx <- 0L; yy <- NA_real_}"),
+    c(1L, 1L, 1L, 56L))
 f # uses the source
 dput(f) # not source
 dput(f, control="all") # uses the source
 cat(deparse(f), sep="\n")
 dump("f", file="")
 # remove the source
-attr(f, "source") <- NULL
+attr(f, "srcref") <- NULL
 f
 dput(f, control="all")
 dump("f", file="")
@@ -2355,7 +2365,7 @@ stopifnot(identical(of2, of),
 ## ^^ would have failed up to R 2.9.x
 foo
 print(foo, useSource = FALSE)
-attr(foo, "source") <- NULL
+attr(foo, "srcref") <- NULL
 foo
 (f <- structure(function(){}, note = "just a note",
                 yada = function() "not the same"))
@@ -2414,3 +2424,181 @@ print(m22, na.print="<missing value>")
 ## https://stat.ethz.ch/pipermail/r-devel/2009-July/054184.html
 update(`a: b` ~ x, ~ . + y)
 ## 2.9.1 dropped backticks
+
+
+## print(ls.str(.)) did evaluate calls
+E <- new.env(); E$cl <- call("print", "Boo !")
+ls.str(E)
+## 2.10.0 did print..
+
+
+## complete.cases with no input
+try(complete.cases())
+try(complete.cases(list(), list()))
+## gave unhelpful messages in 2.10.0, silly results in pre-2.10.1
+
+
+## error messages from (C-level) evalList
+tst <- function(y) { stopifnot(is.numeric(y)); y+ 1 }
+try(tst())
+try(c(1,,2))
+## change in 2.8.0 made these less clear
+
+
+## empty levels from cut.Date (cosmetic, PR#14162)
+x <- as.Date(c("2009-03-21","2009-03-31"))
+cut(x, breaks= "quarter") # had two levels in 2.10.1
+cut(as.POSIXlt(x), breaks= "quarter")
+## remove empty final level
+
+
+## tests of error conditions in switch()
+switch("a", a=, b=, c=, 4)
+switch("a", a=, b=, c=, )
+.Last.value
+switch("a", a=, b=, c=, invisible(4))
+.Last.value
+## visiblilty changed in 2.11.0
+
+
+## rounding error in aggregate.ts
+## https://stat.ethz.ch/pipermail/r-devel/2010-April/057225.html
+x <- rep(6:10, 1:5)
+aggregate(as.ts(x), FUN = mean, ndeltat = 5)
+x <- rep(6:10, 1:5)
+aggregate(as.ts(x), FUN = mean, nfrequency = 0.2)
+## platform-dependent in 2.10.1
+
+
+## wish of PR#9574
+a <- c(0.1, 0.3, 0.4, 0.5, 0.3, 0.0001)
+format.pval(a, eps=0.01)
+format.pval(a, eps=0.01, nsmall =2)
+## granted in 2.12.0
+
+
+## printing fractional dates
+as.Date(0.5, origin="1969-12-31")
+## changed to round down in 2.12.1
+
+
+## printing data frames with  ""  colnames
+dfr <- data.frame(x=1:6, CC=11:16, f = gl(3,2)); colnames(dfr)[2] <- ""
+dfr
+## now prints the same as data.matrix(dfr) does here
+
+
+## format(., zero.print) --> prettyNum()
+set.seed(9); m <- matrix(local({x <- rnorm(40)
+                                sign(x)*round(exp(2*x))/10}), 8,5)
+noquote(format(m, zero.print= "."))
+## used to print  ". 0" instead of ".  "
+
+
+## tests of NA having precedence over NaN -- all must print "NA"
+min(c(NaN, NA))
+min(c(NA, NaN)) # NaN in 2.12.2
+min(NaN, NA_real_)  # NaN in 2.12.2
+min(NA_real_, NaN)
+max(c(NaN, NA))
+max(c(NA, NaN))  # NaN in 2.12.2
+max(NaN, NA_real_)  # NaN in 2.12.2
+max(NA_real_, NaN)
+## might depend on compiler < 2.13.0
+
+
+## PR#14514
+# Data are from Conover, "Nonparametric Statistics", 3rd Ed, p. 197,
+# re-arranged to make a lower-tail test the issue of relevance:  we
+# want to see if pregnant nurses exposed to nitrous oxide have higher
+# rates of miscarriage, stratifying on the type of nurse.
+Nitrous <- array(c(32,210,8,26,18,21,3,3,7,75,0,10), dim = c(2,2,3),
+                 dimnames = list(c("Exposed","NotExposed"),
+                 c("FullTerm","Miscarriage"),
+                 c("DentalAsst","OperRoomNurse","OutpatientNurse")))
+mantelhaen.test(Nitrous, exact=TRUE, alternative="less")
+mantelhaen.test(Nitrous, exact=FALSE, alternative="less")
+## exact = FALSE gave the wrong tail in 2.12.2.
+
+
+## scan(strip.white=TRUE) could strip trailing (but not leading) space
+## inside quoted strings.
+writeLines(' "  A  "; "B" ;"C";" D ";"E ";  F  ;G  ', "foo")
+cat(readLines("foo"), sep = "\n")
+scan('foo', list(""), sep=";")[[1]]
+scan('foo', "", sep=";")
+scan('foo', list(""), sep=";", strip.white = TRUE)[[1]]
+scan('foo', "", sep=";", strip.white = TRUE)
+unlink('foo')
+
+writeLines(' "  A  "\n "B" \n"C"\n" D "\n"E "\n  F  \nG  ', "foo2")
+scan('foo2', "")
+scan('foo2', "", strip.white=TRUE) # documented to be ignored ...
+unlink('foo2')
+## Changed for 2.13.0, found when investigating non-bug PR#14522.
+
+
+## PR#14488: missing values in rank correlations
+set.seed(1)
+x <- runif(10)
+y <- runif(10)
+x[3] <- NA; y[5] <- NA
+xy <- cbind(x, y)
+
+cor(x, y, method = "spearman", use = "complete.obs")
+cor(x, y, method = "spearman", use = "pairwise.complete.obs")
+cor(na.omit(xy),  method = "spearman", use = "complete.obs")
+cor(xy,  method = "spearman", use = "complete.obs")
+cor(xy,  method = "spearman", use = "pairwise.complete.obs")
+## inconsistent in R < 2.13.0
+
+
+## integer overflow in rowsum() went undetected
+# https://stat.ethz.ch/pipermail/r-devel/2011-March/060304.html
+x <- 2e9L
+rowsum(c(x, x), c("a", "a"))
+rowsum(data.frame(z = c(x, x)), c("a", "a"))
+## overflow in R < 2.13.0.
+
+
+## method dispatch in [[.data.frame:
+## https://stat.ethz.ch/pipermail/r-devel/2011-April/060409.html
+d <- data.frame(num = 1:4,
+          fac = factor(letters[11:14], levels = letters[1:15]),
+          date = as.Date("2011-04-01") + (0:3),
+          pv = package_version(c("1.2-3", "4.5", "6.7", "8.9-10")))
+for (i in seq_along(d)) print(d[[1, i]])
+## did not dispatch in R < 2.14.0
+
+
+## some tests of 24:00 as midnight
+as.POSIXlt("2011-05-16 24:00:00", tz = "GMT")
+as.POSIXlt("2010-01-31 24:00:00", tz = "GMT")
+as.POSIXlt("2011-02-28 24:00:00", tz = "GMT")
+as.POSIXlt("2008-02-28 24:00:00", tz = "GMT")
+as.POSIXlt("2008-02-29 24:00:00", tz = "GMT")
+as.POSIXlt("2010-12-31 24:00:00", tz = "GMT")
+## new in 2.14.0
+
+
+## Unwarranted conversion of logical values
+try(double(FALSE))
+x <- 1:3
+try(length(x) <- TRUE)
+## coerced to integer in 2.13.x
+
+
+## filter(recursive = TRUE) on input with NAs
+# https://stat.ethz.ch/pipermail/r-devel/2011-July/061547.html
+x <- c(1:4, NA, 6:9)
+cbind(x, "1"=filter(x, 0.5, method="recursive"),
+         "2"=filter(x, c(0.5, 0.0), method="recursive"),
+         "3"=filter(x, c(0.5, 0.0, 0.0), method="recursive"))
+## NAs in wrong place in R <= 2.13.1.
+
+
+## PR#14679.  Format depends if TZ is set.
+x <- as.POSIXlt(c("2010-02-27 22:30:33", "2009-08-09 06:01:03",
+                  "2010-07-23 17:29:59"))
+stopifnot(!is.na(trunc(x, units = "days")[1:3]))
+## gave NAs after the first in R < 2.13.2
